@@ -17,7 +17,7 @@
  */
 
 /*
- * Copyright (C) 2006-2023 Oracle and/or its affiliates.
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
  * This file is part of VirtualBox base platform packages, as
  * available from https://www.virtualbox.org.
@@ -71,15 +71,15 @@
 #define VBOX_SHCL_MODE_BIDIRECTIONAL 3
 /** @}  */
 
-/** @name VBOX_SHCL_TRANSFER_MODE_XXX - The Shared Clipboard file transfer mode (bit field).
+/** @name VBOX_SHCL_TRANSFER_MODE_XXX - The Shared Clipboard file transfer mode flags.
  * @{
  */
 /** Shared Clipboard file transfers are disabled. */
-#define VBOX_SHCL_TRANSFER_MODE_DISABLED     UINT32_C(0)
+#define VBOX_SHCL_TRANSFER_MODE_F_NONE         UINT32_C(0)
 /** Shared Clipboard file transfers are enabled. */
-#define VBOX_SHCL_TRANSFER_MODE_ENABLED      RT_BIT(0)
+#define VBOX_SHCL_TRANSFER_MODE_F_ENABLED      RT_BIT(0)
 /** Shared Clipboard file transfer mode valid mask. */
-#define VBOX_SHCL_TRANSFER_MODE_VALID_MASK   UINT32_C(0x1)
+#define VBOX_SHCL_TRANSFER_MODE_F_VALID_MASK   UINT32_C(0x1)
 /** @} */
 
 
@@ -163,71 +163,71 @@
 #define VBOX_SHCL_HOST_MSG_READ_DATA_CID                    5
 
 /** Sends a transfer status to the guest side.
- * @since   6.1.?
+ * @since   7.1
  */
 #define VBOX_SHCL_HOST_MSG_TRANSFER_STATUS                   50
 /** Reads the root list header from the guest.
- * @since   6.1.?
+ * @since   7.1
  */
 #define VBOX_SHCL_HOST_MSG_TRANSFER_ROOT_LIST_HDR_READ       51
 /** Writes the root list header to the guest.
- * @since   6.1.?
+ * @since   7.1
  */
 #define VBOX_SHCL_HOST_MSG_TRANSFER_ROOT_LIST_HDR_WRITE      52
 /** Reads a root list entry from the guest.
- * @since   6.1.?
+ * @since   7.1
  */
 #define VBOX_SHCL_HOST_MSG_TRANSFER_ROOT_LIST_ENTRY_READ     53
 /** Writes a root list entry to the guest.
- * @since   6.1.?
+ * @since   7.1
  */
 #define VBOX_SHCL_HOST_MSG_TRANSFER_ROOT_LIST_ENTRY_WRITE    54
 /** Open a transfer list on the guest side.
- * @since   6.1.?
+ * @since   7.1
  */
 #define VBOX_SHCL_HOST_MSG_TRANSFER_LIST_OPEN                55
 /** Closes a formerly opened transfer list on the guest side.
- * @since   6.1.?
+ * @since   7.1
  */
 #define VBOX_SHCL_HOST_MSG_TRANSFER_LIST_CLOSE               56
 /** Reads a list header from the guest.
- * @since   6.1.?
+ * @since   7.1
  */
 #define VBOX_SHCL_HOST_MSG_TRANSFER_LIST_HDR_READ            57
 /** Writes a list header to the guest.
- * @since   6.1.?
+ * @since   7.1
  */
 #define VBOX_SHCL_HOST_MSG_TRANSFER_LIST_HDR_WRITE           58
 /** Reads a list entry from the guest.
- * @since   6.1.?
+ * @since   7.1
  */
 #define VBOX_SHCL_HOST_MSG_TRANSFER_LIST_ENTRY_READ          59
 /** Writes a list entry to the guest.
- * @since   6.1.?
+ * @since   7.1
  */
 #define VBOX_SHCL_HOST_MSG_TRANSFER_LIST_ENTRY_WRITE         60
 /** Open a transfer object on the guest side.
- * @since   6.1.?
+ * @since   7.1
  */
 #define VBOX_SHCL_HOST_MSG_TRANSFER_OBJ_OPEN                 61
 /** Closes a formerly opened transfer object on the guest side.
- * @since   6.1.?
+ * @since   7.1
  */
 #define VBOX_SHCL_HOST_MSG_TRANSFER_OBJ_CLOSE                62
 /** Reads from an object on the guest side.
- * @since   6.1.?
+ * @since   7.1
  */
 #define VBOX_SHCL_HOST_MSG_TRANSFER_OBJ_READ                 63
 /** Writes to an object on the guest side.
- * @since   6.1.?
+ * @since   7.1
  */
 #define VBOX_SHCL_HOST_MSG_TRANSFER_OBJ_WRITE                64
 /** Indicates that the host has canceled a transfer.
- * @since   6.1.?
+ * @since   7.1
  */
 #define VBOX_SHCL_HOST_MSG_TRANSFER_CANCEL                   65
 /** Indicates that the an unrecoverable error on the host occurred.
- * @since   6.1.?
+ * @since   7.1
  */
 #define VBOX_SHCL_HOST_MSG_TRANSFER_ERROR                    66
 /** @} */
@@ -329,7 +329,7 @@
  * @retval  VERR_INVALID_CLIENT_ID
  * @retval  VERR_WRONG_PARAMETER_COUNT
  * @retval  VERR_WRONG_PARAMETER_TYPE
- * @since   6.1.0
+ * @since   7.1.0
  */
 #define VBOX_SHCL_GUEST_FN_REPORT_FEATURES          6
 /** Query the host ones feature masks.
@@ -342,7 +342,7 @@
  * @retval  VERR_INVALID_CLIENT_ID
  * @retval  VERR_WRONG_PARAMETER_COUNT
  * @retval  VERR_WRONG_PARAMETER_TYPE
- * @since   6.1.0
+ * @since   7.1.0
  */
 #define VBOX_SHCL_GUEST_FN_QUERY_FEATURES           7
 /** Peeks at the next message, returning immediately.
@@ -364,7 +364,7 @@
  * @retval  VERR_INVALID_CLIENT_ID
  * @retval  VERR_WRONG_PARAMETER_COUNT
  * @retval  VERR_WRONG_PARAMETER_TYPE
- * @since   6.1.0
+ * @since   7.1.0
  */
 #define VBOX_SHCL_GUEST_FN_MSG_PEEK_NOWAIT          8
 /** Peeks at the next message, waiting for one to arrive.
@@ -392,7 +392,7 @@
  * @retval  VERR_WRONG_PARAMETER_COUNT
  * @retval  VERR_WRONG_PARAMETER_TYPE
  * @note    This replaces VBOX_SHCL_GUEST_FN_MSG_OLD_GET_WAIT.
- * @since   6.1.0
+ * @since   7.1.0
  */
 #define VBOX_SHCL_GUEST_FN_MSG_PEEK_WAIT            9
 /** Gets the next message, returning immediately.
@@ -439,134 +439,134 @@
  * @retval  VERR_WRONG_PARAMETER_TYPE
  * @since   6.1.x
  */
-#define VBOX_SHCL_GUEST_FN_REPLY                  11
-/** Gets the root list header from the host.
+#define VBOX_SHCL_GUEST_FN_REPLY                 11
+/** Gets the transfer root list header from the host.
  *
  * @retval  VINF_SUCCESS on success.
  * @retval  VERR_INVALID_CLIENT_ID
  * @retval  VERR_WRONG_PARAMETER_COUNT
  * @retval  VERR_WRONG_PARAMETER_TYPE
- * @since   6.1.x
+ * @since   7.1.x
  */
-#define VBOX_SHCL_GUEST_FN_ROOT_LIST_HDR_READ     12
-/** Sends the root list header to the host.
+#define VBOX_SHCL_GUEST_FN_ROOT_LIST_HDR_READ    12
+/** Sends the transfer root list header to the host.
  *
  * @retval  VINF_SUCCESS on success.
  * @retval  VERR_INVALID_CLIENT_ID
  * @retval  VERR_WRONG_PARAMETER_COUNT
  * @retval  VERR_WRONG_PARAMETER_TYPE
- * @since   6.1.x
+ * @since   7.1.x
  */
-#define VBOX_SHCL_GUEST_FN_ROOT_LIST_HDR_WRITE    13
-/** Gets a root list root entry from the host.
+#define VBOX_SHCL_GUEST_FN_ROOT_LIST_HDR_WRITE   13
+/** Gets a transfer root list root entry from the host.
  *
  * @retval  VINF_SUCCESS on success.
  * @retval  VERR_INVALID_CLIENT_ID
  * @retval  VERR_WRONG_PARAMETER_COUNT
  * @retval  VERR_WRONG_PARAMETER_TYPE
- * @since   6.1.x
+ * @since   7.1.x
  */
-#define VBOX_SHCL_GUEST_FN_ROOT_LIST_ENTRY_READ   14
-/** Sends a root list root entry to the host.
+#define VBOX_SHCL_GUEST_FN_ROOT_LIST_ENTRY_READ  14
+/** Sends a transfer root list root entry to the host.
  *
  * @retval  VINF_SUCCESS on success.
  * @retval  VERR_INVALID_CLIENT_ID
  * @retval  VERR_WRONG_PARAMETER_COUNT
  * @retval  VERR_WRONG_PARAMETER_TYPE
- * @since   6.1.x
+ * @since   7.1.x
  */
-#define VBOX_SHCL_GUEST_FN_ROOT_LIST_ENTRY_WRITE  15
-/** Opens / gets a list handle from the host.
+#define VBOX_SHCL_GUEST_FN_ROOT_LIST_ENTRY_WRITE 15
+/** Opens / gets a transfer list handle from the host.
  *
  * @retval  VINF_SUCCESS on success.
  * @retval  VERR_INVALID_CLIENT_ID
  * @retval  VERR_WRONG_PARAMETER_COUNT
  * @retval  VERR_WRONG_PARAMETER_TYPE
- * @since   6.1.x
+ * @since   7.1.x
  */
-#define VBOX_SHCL_GUEST_FN_LIST_OPEN              16
-/** Closes a list handle from the host.
+#define VBOX_SHCL_GUEST_FN_LIST_OPEN             16
+/** Closes a transfer list handle from the host.
  *
  * @retval  VINF_SUCCESS on success.
  * @retval  VERR_INVALID_CLIENT_ID
  * @retval  VERR_WRONG_PARAMETER_COUNT
  * @retval  VERR_WRONG_PARAMETER_TYPE
- * @since   6.1.x
+ * @since   7.1.x
  */
-#define VBOX_SHCL_GUEST_FN_LIST_CLOSE             17
-/** Reads a list header from the host.
+#define VBOX_SHCL_GUEST_FN_LIST_CLOSE            17
+/** Reads a transfer list header from the host.
  *
  * @retval  VINF_SUCCESS on success.
  * @retval  VERR_INVALID_CLIENT_ID
  * @retval  VERR_WRONG_PARAMETER_COUNT
  * @retval  VERR_WRONG_PARAMETER_TYPE
- * @since   6.1.x
+ * @since   7.1.x
  */
-#define VBOX_SHCL_GUEST_FN_LIST_HDR_READ          18
-/** Writes a list header to the host.
+#define VBOX_SHCL_GUEST_FN_LIST_HDR_READ         18
+/** Writes a transfer list header to the host.
  *
  * @retval  VINF_SUCCESS on success.
  * @retval  VERR_INVALID_CLIENT_ID
  * @retval  VERR_WRONG_PARAMETER_COUNT
  * @retval  VERR_WRONG_PARAMETER_TYPE
- * @since   6.1.x
+ * @since   7.1.x
  */
-#define VBOX_SHCL_GUEST_FN_LIST_HDR_WRITE         19
-/** Reads a list entry from the host.
+#define VBOX_SHCL_GUEST_FN_LIST_HDR_WRITE        19
+/** Reads a transfer list entry from the host.
  *
  * @retval  VINF_SUCCESS on success.
  * @retval  VERR_INVALID_CLIENT_ID
  * @retval  VERR_WRONG_PARAMETER_COUNT
  * @retval  VERR_WRONG_PARAMETER_TYPE
- * @since   6.1.x
+ * @since   7.1.x
  */
-#define VBOX_SHCL_GUEST_FN_LIST_ENTRY_READ        20
-/** Sends a list entry to the host.
+#define VBOX_SHCL_GUEST_FN_LIST_ENTRY_READ       20
+/** Sends a transfer list entry to the host.
  *
  * @retval  VINF_SUCCESS on success.
  * @retval  VERR_INVALID_CLIENT_ID
  * @retval  VERR_WRONG_PARAMETER_COUNT
  * @retval  VERR_WRONG_PARAMETER_TYPE
- * @since   6.1.x
+ * @since   7.1.x
  */
-#define VBOX_SHCL_GUEST_FN_LIST_ENTRY_WRITE       21
-/** Opens an object on the host.
+#define VBOX_SHCL_GUEST_FN_LIST_ENTRY_WRITE      21
+/** Opens a transfer object on the host.
  *
  * @retval  VINF_SUCCESS on success.
  * @retval  VERR_INVALID_CLIENT_ID
  * @retval  VERR_WRONG_PARAMETER_COUNT
  * @retval  VERR_WRONG_PARAMETER_TYPE
- * @since   6.1.x
+ * @since   7.1.x
  */
-#define VBOX_SHCL_GUEST_FN_OBJ_OPEN               22
-/** Closes an object on the host.
+#define VBOX_SHCL_GUEST_FN_OBJ_OPEN              22
+/** Closes a transfer object on the host.
  *
  * @retval  VINF_SUCCESS on success.
  * @retval  VERR_INVALID_CLIENT_ID
  * @retval  VERR_WRONG_PARAMETER_COUNT
  * @retval  VERR_WRONG_PARAMETER_TYPE
- * @since   6.1.x
+ * @since   7.1.x
  */
-#define VBOX_SHCL_GUEST_FN_OBJ_CLOSE              23
-/** Reads from an object on the host.
+#define VBOX_SHCL_GUEST_FN_OBJ_CLOSE             23
+/** Reads from a transfer object on the host.
  *
  * @retval  VINF_SUCCESS on success.
  * @retval  VERR_INVALID_CLIENT_ID
  * @retval  VERR_WRONG_PARAMETER_COUNT
  * @retval  VERR_WRONG_PARAMETER_TYPE
- * @since   6.1.x
+ * @since   7.1.x
  */
-#define VBOX_SHCL_GUEST_FN_OBJ_READ               24
-/** Writes to an object on the host.
+#define VBOX_SHCL_GUEST_FN_OBJ_READ              24
+/** Writes to a transfer object on the host.
  *
  * @retval  VINF_SUCCESS on success.
  * @retval  VERR_INVALID_CLIENT_ID
  * @retval  VERR_WRONG_PARAMETER_COUNT
  * @retval  VERR_WRONG_PARAMETER_TYPE
- * @since   6.1.x
+ * @since   7.1.x
  */
-#define VBOX_SHCL_GUEST_FN_OBJ_WRITE              25
-/** Reports an error to the host.
+#define VBOX_SHCL_GUEST_FN_OBJ_WRITE             25
+/** Reports a transfer error to the host.
  *
  * @todo r=bird: Smells like GUEST_MSG_SKIP
  *
@@ -576,7 +576,7 @@
  * @retval  VERR_WRONG_PARAMETER_TYPE
  * @since   6.1
  */
-#define VBOX_SHCL_GUEST_FN_ERROR                  27
+#define VBOX_SHCL_GUEST_FN_ERROR                 27
 
 /** For negotiating a chunk size between the guest and host.
  *
@@ -592,7 +592,7 @@
  * @retval  VERR_WRONG_PARAMETER_TYPE
  * @retval  VERR_INVALID_PARAMETER if the 2nd parameter is larger than the
  *          first one
- * @since   6.1
+ * @since   7.1
  */
 #define VBOX_SHCL_GUEST_FN_NEGOTIATE_CHUNK_SIZE     28
 
@@ -637,7 +637,7 @@
 #define VBOX_SHCL_HF_0_CONTEXT_ID                 RT_BIT_64(0)
 /** The host can copy & paste files and directories.
  *  This includes messages like
- * @since 6.1.? */
+ * @since 7.1 */
 #define VBOX_SHCL_HF_0_TRANSFERS                  RT_BIT_64(1)
 /** @} */
 
@@ -797,18 +797,25 @@ typedef struct _VBoxShClGetHostMsg
 #define VBOX_SHCL_CPARMS_GET_HOST_MSG 3
 
 /** No listing flags specified. */
-#define VBOX_SHCL_LIST_FLAG_NONE          0
+#define VBOX_SHCL_LIST_F_NONE               0
 /** Only returns one entry per read. */
-#define VBOX_SHCL_LIST_FLAG_RETURN_ONE    RT_BIT(0)
+#define VBOX_SHCL_LIST_F_RETURN_ONE         RT_BIT(0)
 /** Restarts reading a list from the beginning. */
-#define VBOX_SHCL_LIST_FLAG_RESTART       RT_BIT(1)
+#define VBOX_SHCL_LIST_F_RESTART            RT_BIT(1)
+/** Listing flags valid mask. */
+#define VBOX_SHCL_LIST_F_VALID_MASK         0x3
 
-#define VBOX_SHCL_LISTHDR_FLAG_NONE       0
+/** No list header flags specified. */
+#define VBOX_SHCL_LISTHDR_F_NONE            0
+/** List header flags valid mask. */
+#define VBOX_SHCL_LISTHDR_F_VALID_MASK      0x0
 
 /** No additional information provided. */
-#define VBOX_SHCL_INFO_FLAG_NONE          0
+#define VBOX_SHCL_INFO_F_NONE               0
 /** Get object information of type SHCLFSOBJINFO. */
-#define VBOX_SHCL_INFO_FLAG_FSOBJINFO     RT_BIT(0)
+#define VBOX_SHCL_INFO_F_FSOBJINFO          RT_BIT(0)
+/** Info flags valid mask. */
+#define VBOX_SHCL_INFO_F_VALID_MASK         0x1
 
 /**
  * Status message for lists and objects.
@@ -828,17 +835,17 @@ typedef struct _VBoxShClStatusMsg
 #define VBOX_SHCL_CPARMS_STATUS 3
 
 /** Invalid message type, do not use. */
-#define VBOX_SHCL_REPLYMSGTYPE_INVALID           0
+#define VBOX_SHCL_TX_REPLYMSGTYPE_INVALID           0
 /** Replies a transfer status. */
-#define VBOX_SHCL_REPLYMSGTYPE_TRANSFER_STATUS   1
+#define VBOX_SHCL_TX_REPLYMSGTYPE_TRANSFER_STATUS   1
 /** Replies a list open status. */
-#define VBOX_SHCL_REPLYMSGTYPE_LIST_OPEN         2
+#define VBOX_SHCL_TX_REPLYMSGTYPE_LIST_OPEN         2
 /** Replies a list close status. */
-#define VBOX_SHCL_REPLYMSGTYPE_LIST_CLOSE        3
+#define VBOX_SHCL_TX_REPLYMSGTYPE_LIST_CLOSE        3
 /** Replies an object open status. */
-#define VBOX_SHCL_REPLYMSGTYPE_OBJ_OPEN          4
+#define VBOX_SHCL_TX_REPLYMSGTYPE_OBJ_OPEN          4
 /** Replies an object close status. */
-#define VBOX_SHCL_REPLYMSGTYPE_OBJ_CLOSE         5
+#define VBOX_SHCL_TX_REPLYMSGTYPE_OBJ_CLOSE         5
 
 /**
  * Generic reply message.
@@ -849,7 +856,7 @@ typedef struct _VBoxShClReplyMsg
 
     /** uint64_t, out: Context ID. */
     HGCMFunctionParameter uContext;
-    /** uint32_t, out: Message type of type VBOX_SHCL_REPLYMSGTYPE_XXX. */
+    /** uint32_t, out: Message type of type VBOX_SHCL_TX_REPLYMSGTYPE_XXX. */
     HGCMFunctionParameter enmType;
     /** uint32_t, out: IPRT result of overall operation. */
     HGCMFunctionParameter rc;
@@ -926,9 +933,9 @@ typedef struct _VBoxShClRootListEntryParms
 {
     /** uint64_t, in: Context ID. */
     HGCMFunctionParameter uContext;
-    /** uint32_t, in: VBOX_SHCL_INFO_FLAG_XXX. */
+    /** uint32_t, in: VBOX_SHCL_INFO_F_XXX. */
     HGCMFunctionParameter fInfo;
-    /** uint32_t, in: Index of root list entry to get (zero-based). */
+    /** uint64_t, in: Index of root list entry to get (zero-based). */
     HGCMFunctionParameter uIndex;
 } VBoxShClRootListEntryParms;
 
@@ -956,7 +963,8 @@ typedef struct _VBoxShClRootListEntryMsg
 
     /** in/out: Request parameters. */
     VBoxShClRootListEntryParms Parms;
-    /** pointer, in/out: Entry name. */
+    /** pointer, in/out: Entry name.
+     *  Up to SHCLLISTENTRY_MAX_NAME. */
     HGCMFunctionParameter      szName;
     /** uint32_t, out: Bytes to be used for information/How many bytes were used.  */
     HGCMFunctionParameter      cbInfo;
@@ -981,7 +989,8 @@ typedef struct _VBoxShClListOpenMsg
     HGCMFunctionParameter fList;
     /** pointer, in: Filter string. */
     HGCMFunctionParameter pvFilter;
-    /** pointer, in: Listing poth. If empty or NULL the listing's root path will be opened. */
+    /** pointer, in: Listing path. If empty or NULL the listing's root path will be opened.
+     *  We always use UNIX-style paths. */
     HGCMFunctionParameter pvPath;
     /** uint64_t, out: List handle. */
     HGCMFunctionParameter uHandle;
@@ -1052,7 +1061,7 @@ typedef struct _VBoxShClListEntryReqParms
     HGCMFunctionParameter uContext;
     /** uint64_t, in: List handle. */
     HGCMFunctionParameter uHandle;
-    /** uint32_t, in: VBOX_SHCL_INFO_FLAG_XXX. */
+    /** uint32_t, in: VBOX_SHCL_INFO_F_XXX. */
     HGCMFunctionParameter fInfo;
 } VBoxShClListEntryReqParms;
 
@@ -1101,7 +1110,8 @@ typedef struct _VBoxShClObjOpenMsg
     HGCMFunctionParameter uContext;
     /** uint64_t, out: Object handle. */
     HGCMFunctionParameter uHandle;
-    /** pointer, in: Absoulte path of object to open/create. */
+    /** pointer, in: Absoulte path of object to open/create.
+     *  We always use UNIX-style paths. */
     HGCMFunctionParameter szPath;
     /** uint32_t in: Open / Create flags of type SHCL_OBJ_CF_. */
     HGCMFunctionParameter fCreate;
