@@ -3,28 +3,41 @@
  */
 
 /*
- * Copyright (C) 2011-2016 Oracle Corporation
+ * Copyright (C) 2011-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
-#ifndef ___VBox_VD_Interfaces_h
-#define ___VBox_VD_Interfaces_h
+#ifndef VBOX_INCLUDED_vd_ifs_h
+#define VBOX_INCLUDED_vd_ifs_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include <iprt/assert.h>
 #include <iprt/string.h>
@@ -38,6 +51,9 @@
 
 RT_C_DECLS_BEGIN
 
+/** @addtogroup grp_vd
+ * @{ */
+
 /** Interface header magic. */
 #define VDINTERFACE_MAGIC UINT32_C(0x19701015)
 
@@ -153,9 +169,7 @@ DECLINLINE(int) VDInterfaceAdd(PVDINTERFACE pInterface, const char *pszName, VDI
                     && enmInterface < VDINTERFACETYPE_INVALID,
                     ("enmInterface=%u", enmInterface), VERR_INVALID_PARAMETER);
 
-    AssertMsgReturn(VALID_PTR(ppVDIfs),
-                    ("pInterfaceList=%#p", ppVDIfs),
-                    VERR_INVALID_PARAMETER);
+    AssertPtrReturn(ppVDIfs, VERR_INVALID_PARAMETER);
 
     /* Fill out interface descriptor. */
     pInterface->u32Magic         = VDINTERFACE_MAGIC;
@@ -183,13 +197,8 @@ DECLINLINE(int) VDInterfaceRemove(PVDINTERFACE pInterface, PVDINTERFACE *ppVDIfs
     int rc = VERR_NOT_FOUND;
 
     /* Argument checks. */
-    AssertMsgReturn(VALID_PTR(pInterface),
-                    ("pInterface=%#p", pInterface),
-                    VERR_INVALID_PARAMETER);
-
-    AssertMsgReturn(VALID_PTR(ppVDIfs),
-                    ("pInterfaceList=%#p", ppVDIfs),
-                    VERR_INVALID_PARAMETER);
+    AssertPtrReturn(pInterface, VERR_INVALID_PARAMETER);
+    AssertPtrReturn(ppVDIfs, VERR_INVALID_PARAMETER);
 
     if (*ppVDIfs)
     {
@@ -211,7 +220,8 @@ DECLINLINE(int) VDInterfaceRemove(PVDINTERFACE pInterface, PVDINTERFACE *ppVDIfs
         }
         else if (pCurr)
         {
-            pPrev = pCurr->pNext;
+            Assert(pPrev->pNext == pCurr);
+            pPrev->pNext = pCurr->pNext;
             rc = VINF_SUCCESS;
         }
     }
@@ -296,6 +306,12 @@ DECLINLINE(int) RT_IPRT_FORMAT_ATTR(6, 7) vdIfError(PVDINTERFACEERROR pIfError,
     if (pIfError)
         pIfError->pfnError(pIfError->Core.pvUser, rc, RT_SRC_POS_ARGS, pszFormat, va);
     va_end(va);
+
+#if defined(LOG_ENABLED) && defined(Log)
+    va_start(va, pszFormat);
+    Log(("vdIfError: %N\n", pszFormat, &va));
+    va_end(va);
+#endif
     return rc;
 }
 
@@ -315,6 +331,12 @@ DECLINLINE(int) RT_IPRT_FORMAT_ATTR(2, 3) vdIfErrorMessage(PVDINTERFACEERROR pIf
     if (pIfError && pIfError->pfnMessage)
         rc = pIfError->pfnMessage(pIfError->Core.pvUser, pszFormat, va);
     va_end(va);
+
+#if defined(LOG_ENABLED) && defined(Log)
+    va_start(va, pszFormat);
+    Log(("vdIfErrorMessage: %N\n", pszFormat, &va));
+    va_end(va);
+#endif
     return rc;
 }
 
@@ -326,7 +348,7 @@ DECLINLINE(int) RT_IPRT_FORMAT_ATTR(2, 3) vdIfErrorMessage(PVDINTERFACEERROR pIf
  * @param   pvUser          Opaque user data which is passed on request submission.
  * @param   rcReq           Status code of the completed request.
  */
-typedef DECLCALLBACK(int) FNVDCOMPLETED(void *pvUser, int rcReq);
+typedef DECLCALLBACKTYPE(int, FNVDCOMPLETED,(void *pvUser, int rcReq));
 /** Pointer to FNVDCOMPLETED() */
 typedef FNVDCOMPLETED *PFNVDCOMPLETED;
 
@@ -356,21 +378,21 @@ typedef struct VDINTERFACEIO
      *                          of the request initiator (ie the one who calls
      *                          VDAsyncRead or VDAsyncWrite) in pvCompletion
      *                          if this is NULL.
-     * @param   ppStorage       Where to store the opaque storage handle.
+     * @param   ppvStorage      Where to store the opaque storage handle.
      */
     DECLR3CALLBACKMEMBER(int, pfnOpen, (void *pvUser, const char *pszLocation,
                                         uint32_t fOpen,
                                         PFNVDCOMPLETED pfnCompleted,
-                                        void **ppStorage));
+                                        void **ppvStorage));
 
     /**
      * Close callback.
      *
      * @return  VBox status code.
      * @param   pvUser          The opaque data passed on container creation.
-     * @param   pStorage        The opaque storage handle to close.
+     * @param   pvStorage       The opaque storage handle to close.
      */
-    DECLR3CALLBACKMEMBER(int, pfnClose, (void *pvUser, void *pStorage));
+    DECLR3CALLBACKMEMBER(int, pfnClose, (void *pvUser, void *pvStorage));
 
     /**
      * Delete callback.
@@ -418,10 +440,10 @@ typedef struct VDINTERFACEIO
      *
      * @return  VBox status code.
      * @param   pvUser          The opaque data passed on container creation.
-     * @param   pStorage        The opaque storage handle to close.
-     * @param   pcbSize         Where to store the size of the storage backend.
+     * @param   pvStorage       The opaque storage handle to get the size from.
+     * @param   pcb             Where to store the size of the storage backend.
      */
-    DECLR3CALLBACKMEMBER(int, pfnGetSize, (void *pvUser, void *pStorage, uint64_t *pcbSize));
+    DECLR3CALLBACKMEMBER(int, pfnGetSize, (void *pvUser, void *pvStorage, uint64_t *pcb));
 
     /**
      * Sets the size of the opened storage backend if possible.
@@ -429,8 +451,8 @@ typedef struct VDINTERFACEIO
      * @return  VBox status code.
      * @retval  VERR_NOT_SUPPORTED if the backend does not support this operation.
      * @param   pvUser          The opaque data passed on container creation.
-     * @param   pStorage        The opaque storage handle to close.
-     * @param   cbSize          The new size of the image.
+     * @param   pvStorage       The opaque storage handle to set the size for.
+     * @param   cb              The new size of the image.
      *
      * @note Depending on the host the underlying storage (backing file, etc.)
      *       might not have all required storage allocated (sparse file) which
@@ -440,7 +462,7 @@ typedef struct VDINTERFACEIO
      *       Use VDINTERFACEIO::pfnSetAllocationSize to make sure the storage is
      *       really alloacted.
      */
-    DECLR3CALLBACKMEMBER(int, pfnSetSize, (void *pvUser, void *pStorage, uint64_t cbSize));
+    DECLR3CALLBACKMEMBER(int, pfnSetSize, (void *pvUser, void *pvStorage, uint64_t cb));
 
     /**
      * Sets the size of the opened storage backend making sure the given size
@@ -450,12 +472,12 @@ typedef struct VDINTERFACEIO
      * @retval VERR_NOT_SUPPORTED if the implementer of the interface doesn't support
      *         this method.
      * @param  pvUser          The opaque data passed on container creation.
-     * @param  pStorage        The storage handle.
+     * @param  pvStorage       The storage handle.
      * @param  cbSize          The new size of the image.
      * @param  fFlags          Flags for controlling the allocation strategy.
      *                         Reserved for future use, MBZ.
      */
-    DECLR3CALLBACKMEMBER(int, pfnSetAllocationSize, (void *pvUser, void *pStorage,
+    DECLR3CALLBACKMEMBER(int, pfnSetAllocationSize, (void *pvUser, void *pvStorage,
                                                      uint64_t cbSize, uint32_t fFlags));
 
     /**
@@ -463,44 +485,44 @@ typedef struct VDINTERFACEIO
      *
      * @return  VBox status code.
      * @param   pvUser          The opaque data passed on container creation.
-     * @param   pStorage        The storage handle to use.
-     * @param   uOffset         The offset to start from.
-     * @param   pvBuffer        Pointer to the bits need to be written.
-     * @param   cbBuffer        How many bytes to write.
+     * @param   pvStorage       The storage handle to use.
+     * @param   off             The offset to start from.
+     * @param   pvBuf           Pointer to the bits need to be written.
+     * @param   cbToWrite       How many bytes to write.
      * @param   pcbWritten      Where to store how many bytes were actually written.
      */
-    DECLR3CALLBACKMEMBER(int, pfnWriteSync, (void *pvUser, void *pStorage, uint64_t uOffset,
-                                             const void *pvBuffer, size_t cbBuffer, size_t *pcbWritten));
+    DECLR3CALLBACKMEMBER(int, pfnWriteSync, (void *pvUser, void *pvStorage, uint64_t off,
+                                             const void *pvBuf, size_t cbToWrite, size_t *pcbWritten));
 
     /**
      * Synchronous read callback.
      *
      * @return  VBox status code.
      * @param   pvUser          The opaque data passed on container creation.
-     * @param   pStorage        The storage handle to use.
-     * @param   uOffset         The offset to start from.
-     * @param   pvBuffer        Where to store the read bits.
-     * @param   cbBuffer        How many bytes to read.
+     * @param   pvStorage       The storage handle to use.
+     * @param   off             The offset to start from.
+     * @param   pvBuf           Where to store the read bits.
+     * @param   cbToRead        How many bytes to read.
      * @param   pcbRead         Where to store how many bytes were actually read.
      */
-    DECLR3CALLBACKMEMBER(int, pfnReadSync, (void *pvUser, void *pStorage, uint64_t uOffset,
-                                            void *pvBuffer, size_t cbBuffer, size_t *pcbRead));
+    DECLR3CALLBACKMEMBER(int, pfnReadSync, (void *pvUser, void *pvStorage, uint64_t off,
+                                            void *pvBuf, size_t cbToRead, size_t *pcbRead));
 
     /**
      * Flush data to the storage backend.
      *
      * @return  VBox status code.
      * @param   pvUser          The opaque data passed on container creation.
-     * @param   pStorage        The storage handle to flush.
+     * @param   pvStorage       The storage handle to flush.
      */
-    DECLR3CALLBACKMEMBER(int, pfnFlushSync, (void *pvUser, void *pStorage));
+    DECLR3CALLBACKMEMBER(int, pfnFlushSync, (void *pvUser, void *pvStorage));
 
     /**
      * Initiate an asynchronous read request.
      *
      * @return  VBox status code.
      * @param   pvUser         The opaque user data passed on container creation.
-     * @param   pStorage       The storage handle.
+     * @param   pvStorage      The storage handle.
      * @param   uOffset        The offset to start reading from.
      * @param   paSegments     Scatter gather list to store the data in.
      * @param   cSegments      Number of segments in the list.
@@ -508,7 +530,7 @@ typedef struct VDINTERFACEIO
      * @param   pvCompletion   The opaque user data which is returned upon completion.
      * @param   ppTask         Where to store the opaque task handle.
      */
-    DECLR3CALLBACKMEMBER(int, pfnReadAsync, (void *pvUser, void *pStorage, uint64_t uOffset,
+    DECLR3CALLBACKMEMBER(int, pfnReadAsync, (void *pvUser, void *pvStorage, uint64_t uOffset,
                                              PCRTSGSEG paSegments, size_t cSegments,
                                              size_t cbRead, void *pvCompletion,
                                              void **ppTask));
@@ -518,7 +540,7 @@ typedef struct VDINTERFACEIO
      *
      * @return  VBox status code.
      * @param   pvUser         The opaque user data passed on conatiner creation.
-     * @param   pStorage       The storage handle.
+     * @param   pvStorage      The storage handle.
      * @param   uOffset        The offset to start writing to.
      * @param   paSegments     Scatter gather list of the data to write
      * @param   cSegments      Number of segments in the list.
@@ -526,7 +548,7 @@ typedef struct VDINTERFACEIO
      * @param   pvCompletion   The opaque user data which is returned upon completion.
      * @param   ppTask         Where to store the opaque task handle.
      */
-    DECLR3CALLBACKMEMBER(int, pfnWriteAsync, (void *pvUser, void *pStorage, uint64_t uOffset,
+    DECLR3CALLBACKMEMBER(int, pfnWriteAsync, (void *pvUser, void *pvStorage, uint64_t uOffset,
                                               PCRTSGSEG paSegments, size_t cSegments,
                                               size_t cbWrite, void *pvCompletion,
                                               void **ppTask));
@@ -536,11 +558,11 @@ typedef struct VDINTERFACEIO
      *
      * @return  VBox status code.
      * @param   pvUser          The opaque data passed on container creation.
-     * @param   pStorage        The storage handle to flush.
+     * @param   pvStorage       The storage handle to flush.
      * @param   pvCompletion    The opaque user data which is returned upon completion.
      * @param   ppTask          Where to store the opaque task handle.
      */
-    DECLR3CALLBACKMEMBER(int, pfnFlushAsync, (void *pvUser, void *pStorage,
+    DECLR3CALLBACKMEMBER(int, pfnFlushAsync, (void *pvUser, void *pvStorage,
                                               void *pvCompletion, void **ppTask));
 
 } VDINTERFACEIO, *PVDINTERFACEIO;
@@ -650,6 +672,8 @@ DECLINLINE(int) vdIfIoFileFlushSync(PVDINTERFACEIO pIfIo, void *pStorage)
  */
 VBOXDDU_DECL(int) VDIfCreateVfsStream(PVDINTERFACEIO pVDIfsIo, void *pvStorage, uint32_t fFlags, PRTVFSIOSTREAM phVfsIos);
 
+struct VDINTERFACEIOINT;
+
 /**
  * Create a VFS file handle around a VD I/O interface.
  *
@@ -667,7 +691,8 @@ VBOXDDU_DECL(int) VDIfCreateVfsStream(PVDINTERFACEIO pVDIfsIo, void *pvStorage,
  * @param   fFlags          RTFILE_O_XXX, access mask requied.
  * @param   phVfsFile       Where to return the VFS file handle on success.
  */
-VBOXDDU_DECL(int) VDIfCreateVfsFile(PVDINTERFACEIO pVDIfs, struct VDINTERFACEIOINT *pVDIfsInt, void *pvStorage, uint32_t fFlags, PRTVFSFILE phVfsFile);
+VBOXDDU_DECL(int) VDIfCreateVfsFile(PVDINTERFACEIO pVDIfs, struct VDINTERFACEIOINT *pVDIfsInt, void *pvStorage,
+                                    uint32_t fFlags, PRTVFSFILE phVfsFile);
 
 /**
  * Creates an VD I/O interface wrapper around an IPRT VFS I/O stream.
@@ -699,9 +724,9 @@ VBOXDDU_DECL(int) VDIfDestroyFromVfsStream(PVDINTERFACEIO pIoIf);
  *
  * @return  VBox status code.
  * @param   pvUser          The opaque user data associated with this interface.
- * @param   uPercent        Completion percentage.
+ * @param   uPercentage     Completion percentage.
  */
-typedef DECLCALLBACK(int) FNVDPROGRESS(void *pvUser, unsigned uPercentage);
+typedef DECLCALLBACKTYPE(int, FNVDPROGRESS,(void *pvUser, unsigned uPercentage));
 /** Pointer to FNVDPROGRESS() */
 typedef FNVDPROGRESS *PFNVDPROGRESS;
 
@@ -724,6 +749,9 @@ typedef struct VDINTERFACEPROGRESS
 
 } VDINTERFACEPROGRESS, *PVDINTERFACEPROGRESS;
 
+/** Initializer for VDINTERFACEPROGRESS.  */
+#define VDINTERFACEPROGRESS_INITALIZER(a_pfnProgress) { { 0, NULL, NULL, VDINTERFACETYPE_INVALID, 0, NULL }, a_pfnProgress }
+
 /**
  * Get progress interface from interface list.
  *
@@ -743,6 +771,19 @@ DECLINLINE(PVDINTERFACEPROGRESS) VDIfProgressGet(PVDINTERFACE pVDIfs)
     return (PVDINTERFACEPROGRESS)pIf;
 }
 
+/**
+ * Signal new progress information to the frontend.
+ *
+ * @returns VBox status code.
+ * @param   pIfProgress        The progress interface.
+ * @param   uPercentage        Completion percentage.
+ */
+DECLINLINE(int) vdIfProgress(PVDINTERFACEPROGRESS pIfProgress, unsigned uPercentage)
+{
+    if (pIfProgress)
+        return pIfProgress->pfnProgress(pIfProgress->Core.pvUser, uPercentage);
+    return VINF_SUCCESS;
+}
 
 /**
  * Configuration information interface
@@ -806,6 +847,19 @@ typedef struct VDINTERFACECONFIG
      */
     DECLR3CALLBACKMEMBER(int, pfnQueryBytes, (void *pvUser, const char *pszName, void *ppvData, size_t cbData));
 
+    /**
+     * Set a named property to a specified string value, optionally creating if it doesn't exist.
+     *
+     * @return  VBox status code.
+     *          VERR_CFGM_VALUE_NOT_FOUND means that the key is not known and fCreate flag was not set.
+     * @param   pvUser          The opaque user data associated with this interface.
+     * @param   fCreate         Create property if it doesn't exist (if property exists, it is not an error)
+     * @param   pszName         Name of the key to query.
+     * @param   pszValue        String value to set the name property to.
+     */
+    DECLR3CALLBACKMEMBER(int, pfnUpdate, (void *pvUser, bool fCreate,
+            const char *pszName, const char *pszValue));
+
 } VDINTERFACECONFIG, *PVDINTERFACECONFIG;
 
 /**
@@ -1089,6 +1143,42 @@ DECLINLINE(int) VDCFGQueryBytesAlloc(PVDINTERFACECONFIG pCfgIf,
     return rc;
 }
 
+/**
+ * Set property value to string (optionally create if non-existent).
+ *
+ * @return  VBox status code.
+ * @param   pCfgIf      Pointer to configuration callback table.
+ * @param   fCreate     Create the property if it doesn't exist
+ * @param   pszName     Name of property
+ * @param   pszValue    String value to assign to property
+ */
+DECLINLINE(int) VDCFGUpdate(PVDINTERFACECONFIG pCfgIf, bool fCreate, const char *pszName, const char *pszValue)
+{
+    int rc = pCfgIf->pfnUpdate(pCfgIf->Core.pvUser, fCreate, pszName, pszValue);
+    return rc;
+}
+
+/**
+ * Set property value to Unsigned Int 64-bit (optionally create if non-existent).
+ *
+ * @return  VBox status code.
+ * @param   pCfgIf      Pointer to configuration callback table.
+ * @param   fCreate     Create the property if it doesn't exist
+ * @param   pszName     Name of property
+ * @param   u64Value    64-bit unsigned value to save with property.
+ */
+
+DECLINLINE(int) VDCFGUpdateU64(PVDINTERFACECONFIG pCfgIf, bool fCreate, const char *pszName, uint64_t u64Value)
+{
+     int rc = 0;
+     char pszValue[21];
+     (void) RTStrPrintf(pszValue, sizeof(pszValue), "%RU64", u64Value);
+     rc = VDCFGUpdate(pCfgIf, fCreate, pszName, pszValue);
+     return rc;
+}
+
+
+
 /** Forward declaration of a VD socket. */
 typedef struct VDSOCKETINT *VDSOCKET;
 /** Pointer to a VD socket. */
@@ -1138,23 +1228,23 @@ typedef struct VDINTERFACETCPNET
      * @return  iprt status code.
      * @retval  VERR_NOT_SUPPORTED if the combination of flags is not supported.
      * @param   fFlags    Combination of the VD_INTERFACETCPNET_CONNECT_* \#defines.
-     * @param   pSock     Where to store the handle.
+     * @param   phVdSock  Where to store the handle.
      */
-    DECLR3CALLBACKMEMBER(int, pfnSocketCreate, (uint32_t fFlags, PVDSOCKET pSock));
+    DECLR3CALLBACKMEMBER(int, pfnSocketCreate, (uint32_t fFlags, PVDSOCKET phVdSock));
 
     /**
      * Destroys the socket.
      *
      * @return iprt status code.
-     * @param  Sock       Socket handle (/ pointer).
+     * @param  hVdSock    Socket handle (/ pointer).
      */
-    DECLR3CALLBACKMEMBER(int, pfnSocketDestroy, (VDSOCKET Sock));
+    DECLR3CALLBACKMEMBER(int, pfnSocketDestroy, (VDSOCKET hVdSock));
 
     /**
      * Connect as a client to a TCP port.
      *
      * @return  iprt status code.
-     * @param   Sock            Socket handle (/ pointer)..
+     * @param   hVdSock         Socket handle (/ pointer)..
      * @param   pszAddress      The address to connect to.
      * @param   uPort           The port to connect to.
      * @param   cMillies        Number of milliseconds to wait for the connect attempt to complete.
@@ -1162,135 +1252,135 @@ typedef struct VDINTERFACETCPNET
      *                          Use RT_SOCKETCONNECT_DEFAULT_WAIT to wait for the default time
      *                          configured on the running system.
      */
-    DECLR3CALLBACKMEMBER(int, pfnClientConnect, (VDSOCKET Sock, const char *pszAddress, uint32_t uPort,
+    DECLR3CALLBACKMEMBER(int, pfnClientConnect, (VDSOCKET hVdSock, const char *pszAddress, uint32_t uPort,
                                                  RTMSINTERVAL cMillies));
 
     /**
      * Close a TCP connection.
      *
      * @return  iprt status code.
-     * @param   Sock            Socket handle (/ pointer).
+     * @param   hVdSock         Socket handle (/ pointer).
      */
-    DECLR3CALLBACKMEMBER(int, pfnClientClose, (VDSOCKET Sock));
+    DECLR3CALLBACKMEMBER(int, pfnClientClose, (VDSOCKET hVdSock));
 
     /**
      * Returns whether the socket is currently connected to the client.
      *
      * @returns true if the socket is connected.
      *          false otherwise.
-     * @param   Sock        Socket handle (/ pointer).
+     * @param   hVdSock     Socket handle (/ pointer).
      */
-    DECLR3CALLBACKMEMBER(bool, pfnIsClientConnected, (VDSOCKET Sock));
+    DECLR3CALLBACKMEMBER(bool, pfnIsClientConnected, (VDSOCKET hVdSock));
 
     /**
      * Socket I/O multiplexing.
      * Checks if the socket is ready for reading.
      *
      * @return  iprt status code.
-     * @param   Sock        Socket handle (/ pointer).
+     * @param   hVdSock     Socket handle (/ pointer).
      * @param   cMillies    Number of milliseconds to wait for the socket.
      *                      Use RT_INDEFINITE_WAIT to wait for ever.
      */
-    DECLR3CALLBACKMEMBER(int, pfnSelectOne, (VDSOCKET Sock, RTMSINTERVAL cMillies));
+    DECLR3CALLBACKMEMBER(int, pfnSelectOne, (VDSOCKET hVdSock, RTMSINTERVAL cMillies));
 
     /**
      * Receive data from a socket.
      *
      * @return  iprt status code.
-     * @param   Sock        Socket handle (/ pointer).
+     * @param   hVdSock     Socket handle (/ pointer).
      * @param   pvBuffer    Where to put the data we read.
      * @param   cbBuffer    Read buffer size.
      * @param   pcbRead     Number of bytes read.
      *                      If NULL the entire buffer will be filled upon successful return.
      *                      If not NULL a partial read can be done successfully.
      */
-    DECLR3CALLBACKMEMBER(int, pfnRead, (VDSOCKET Sock, void *pvBuffer, size_t cbBuffer, size_t *pcbRead));
+    DECLR3CALLBACKMEMBER(int, pfnRead, (VDSOCKET hVdSock, void *pvBuffer, size_t cbBuffer, size_t *pcbRead));
 
     /**
      * Send data to a socket.
      *
      * @return  iprt status code.
-     * @param   Sock        Socket handle (/ pointer).
+     * @param   hVdSock     Socket handle (/ pointer).
      * @param   pvBuffer    Buffer to write data to socket.
      * @param   cbBuffer    How much to write.
      */
-    DECLR3CALLBACKMEMBER(int, pfnWrite, (VDSOCKET Sock, const void *pvBuffer, size_t cbBuffer));
+    DECLR3CALLBACKMEMBER(int, pfnWrite, (VDSOCKET hVdSock, const void *pvBuffer, size_t cbBuffer));
 
     /**
      * Send data from scatter/gather buffer to a socket.
      *
      * @return  iprt status code.
-     * @param   Sock        Socket handle (/ pointer).
-     * @param   pSgBuffer   Scatter/gather buffer to write data to socket.
+     * @param   hVdSock     Socket handle (/ pointer).
+     * @param   pSgBuf      Scatter/gather buffer to write data to socket.
      */
-    DECLR3CALLBACKMEMBER(int, pfnSgWrite, (VDSOCKET Sock, PCRTSGBUF pSgBuffer));
+    DECLR3CALLBACKMEMBER(int, pfnSgWrite, (VDSOCKET hVdSock, PCRTSGBUF pSgBuf));
 
     /**
      * Receive data from a socket - not blocking.
      *
      * @return  iprt status code.
-     * @param   Sock        Socket handle (/ pointer).
+     * @param   hVdSock     Socket handle (/ pointer).
      * @param   pvBuffer    Where to put the data we read.
      * @param   cbBuffer    Read buffer size.
      * @param   pcbRead     Number of bytes read.
      */
-    DECLR3CALLBACKMEMBER(int, pfnReadNB, (VDSOCKET Sock, void *pvBuffer, size_t cbBuffer, size_t *pcbRead));
+    DECLR3CALLBACKMEMBER(int, pfnReadNB, (VDSOCKET hVdSock, void *pvBuffer, size_t cbBuffer, size_t *pcbRead));
 
     /**
      * Send data to a socket - not blocking.
      *
      * @return  iprt status code.
-     * @param   Sock        Socket handle (/ pointer).
+     * @param   hVdSock     Socket handle (/ pointer).
      * @param   pvBuffer    Buffer to write data to socket.
      * @param   cbBuffer    How much to write.
      * @param   pcbWritten  Number of bytes written.
      */
-    DECLR3CALLBACKMEMBER(int, pfnWriteNB, (VDSOCKET Sock, const void *pvBuffer, size_t cbBuffer, size_t *pcbWritten));
+    DECLR3CALLBACKMEMBER(int, pfnWriteNB, (VDSOCKET hVdSock, const void *pvBuffer, size_t cbBuffer, size_t *pcbWritten));
 
     /**
      * Send data from scatter/gather buffer to a socket - not blocking.
      *
      * @return  iprt status code.
-     * @param   Sock        Socket handle (/ pointer).
-     * @param   pSgBuffer   Scatter/gather buffer to write data to socket.
+     * @param   hVdSock     Socket handle (/ pointer).
+     * @param   pSgBuf      Scatter/gather buffer to write data to socket.
      * @param   pcbWritten  Number of bytes written.
      */
-    DECLR3CALLBACKMEMBER(int, pfnSgWriteNB, (VDSOCKET Sock, PRTSGBUF pSgBuffer, size_t *pcbWritten));
+    DECLR3CALLBACKMEMBER(int, pfnSgWriteNB, (VDSOCKET hVdSock, PRTSGBUF pSgBuf, size_t *pcbWritten));
 
     /**
      * Flush socket write buffers.
      *
      * @return  iprt status code.
-     * @param   Sock        Socket handle (/ pointer).
+     * @param   hVdSock     Socket handle (/ pointer).
      */
-    DECLR3CALLBACKMEMBER(int, pfnFlush, (VDSOCKET Sock));
+    DECLR3CALLBACKMEMBER(int, pfnFlush, (VDSOCKET hVdSock));
 
     /**
      * Enables or disables delaying sends to coalesce packets.
      *
      * @return  iprt status code.
-     * @param   Sock        Socket handle (/ pointer).
+     * @param   hVdSock     Socket handle (/ pointer).
      * @param   fEnable     When set to true enables coalescing.
      */
-    DECLR3CALLBACKMEMBER(int, pfnSetSendCoalescing, (VDSOCKET Sock, bool fEnable));
+    DECLR3CALLBACKMEMBER(int, pfnSetSendCoalescing, (VDSOCKET hVdSock, bool fEnable));
 
     /**
      * Gets the address of the local side.
      *
      * @return  iprt status code.
-     * @param   Sock        Socket handle (/ pointer).
+     * @param   hVdSock     Socket handle (/ pointer).
      * @param   pAddr       Where to store the local address on success.
      */
-    DECLR3CALLBACKMEMBER(int, pfnGetLocalAddress, (VDSOCKET Sock, PRTNETADDR pAddr));
+    DECLR3CALLBACKMEMBER(int, pfnGetLocalAddress, (VDSOCKET hVdSock, PRTNETADDR pAddr));
 
     /**
      * Gets the address of the other party.
      *
      * @return  iprt status code.
-     * @param   Sock        Socket handle (/ pointer).
+     * @param   hVdSock     Socket handle (/ pointer).
      * @param   pAddr       Where to store the peer address on success.
      */
-    DECLR3CALLBACKMEMBER(int, pfnGetPeerAddress, (VDSOCKET Sock, PRTNETADDR pAddr));
+    DECLR3CALLBACKMEMBER(int, pfnGetPeerAddress, (VDSOCKET hVdSock, PRTNETADDR pAddr));
 
     /**
      * Socket I/O multiplexing - extended version which can be woken up.
@@ -1304,7 +1394,7 @@ typedef struct VDINTERFACETCPNET
      * @param   cMillies    Number of milliseconds to wait for the socket.
      *                      Use RT_INDEFINITE_WAIT to wait for ever.
      */
-    DECLR3CALLBACKMEMBER(int, pfnSelectOneEx, (VDSOCKET Sock, uint32_t fEvents,
+    DECLR3CALLBACKMEMBER(int, pfnSelectOneEx, (VDSOCKET hVdSock, uint32_t fEvents,
                                                uint32_t *pfEvents, RTMSINTERVAL cMillies));
 
     /**
@@ -1313,7 +1403,7 @@ typedef struct VDINTERFACETCPNET
      * @return iprt status code.
      * @param  hVdSock      VD Socket handle(/pointer).
      */
-    DECLR3CALLBACKMEMBER(int, pfnPoke, (VDSOCKET Sock));
+    DECLR3CALLBACKMEMBER(int, pfnPoke, (VDSOCKET hVdSock));
 
 } VDINTERFACETCPNET, *PVDINTERFACETCPNET;
 
@@ -1672,4 +1762,4 @@ RT_C_DECLS_END
 
 /** @} */
 
-#endif
+#endif /* !VBOX_INCLUDED_vd_ifs_h */
