@@ -3,24 +3,34 @@
  */
 
 /*
- * Copyright (C) 2006-2019 Oracle Corporation
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 #ifndef IPRT_INCLUDED_fs_h
@@ -227,6 +237,7 @@ typedef enum RTFSTYPE
     RTFSTYPE_PROC,
     RTFSTYPE_OCFS2,
     RTFSTYPE_BTRFS,
+    RTFSTYPE_ECRYPTFS,
 
     /* Windows: */
     /** New Technology File System. */
@@ -238,6 +249,8 @@ typedef enum RTFSTYPE
     RTFSTYPE_FAT,
     /** Extended File Allocation Table, main target are flash drives. */
     RTFSTYPE_EXFAT,
+    /** Resilient File System. */
+    RTFSTYPE_REFS,
 
     /* Solaris: */
     /** Zettabyte File System.  */
@@ -251,6 +264,7 @@ typedef enum RTFSTYPE
     /** Hierarchical File System. */
     RTFSTYPE_HFS,
     /** @todo RTFSTYPE_HFS_PLUS? */
+    RTFSTYPE_APFS,
     RTFSTYPE_AUTOFS,
     RTFSTYPE_DEVFS,
 
@@ -575,7 +589,7 @@ typedef struct RTFSPROPERTIES
      * False if it can't. */
     bool    fSupportsUnicode;
 
-    /** True if the filesystem is compresses.
+    /** True if the filesystem is compressed.
      * False if it isn't or we don't know. */
     bool    fCompressed;
 
@@ -620,7 +634,7 @@ RTR3DECL(bool) RTFsIsCaseSensitive(const char *pszFsPath);
  * @param   pszMountpoint   The mountpoint name.
  * @param   pvUser          The user argument.
  */
-typedef DECLCALLBACK(int) FNRTFSMOUNTPOINTENUM(const char *pszMountpoint, void *pvUser);
+typedef DECLCALLBACKTYPE(int, FNRTFSMOUNTPOINTENUM,(const char *pszMountpoint, void *pvUser));
 /** Pointer to a FNRTFSMOUNTPOINTENUM(). */
 typedef FNRTFSMOUNTPOINTENUM *PFNRTFSMOUNTPOINTENUM;
 
