@@ -1,5 +1,5 @@
 /**********************************************************
- * Copyright 2008-2023 VMware, Inc.  All rights reserved.
+ * Copyright 2008-2009 VMware, Inc.  All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -24,15 +24,15 @@
  **********************************************************/
 
 #include "git_sha1.h" /* For MESA_GIT_SHA1 */
-#include "compiler/nir/nir.h"
 #include "util/format/u_format.h"
 #include "util/u_memory.h"
 #include "util/u_inlines.h"
-#include "util/u_process.h"
 #include "util/u_screen.h"
 #include "util/u_string.h"
 #include "util/u_math.h"
 
+#include "os/os_process.h"
+
 #include "svga_winsys.h"
 #include "svga_public.h"
 #include "svga_context.h"
@@ -74,9 +74,6 @@ static const struct debug_named_value svga_debug_flags[] = {
    { "streamout",   DEBUG_STREAMOUT, NULL },
    { "query",       DEBUG_QUERY, NULL },
    { "samplers",    DEBUG_SAMPLERS, NULL },
-   { "image",       DEBUG_IMAGE, NULL },
-   { "uav",         DEBUG_UAV, NULL },
-   { "retry",       DEBUG_RETRY, NULL },
    DEBUG_NAMED_VALUE_END
 };
 #endif
@@ -105,7 +102,7 @@ svga_get_name( struct pipe_screen *pscreen )
 #else
    build = "build: RELEASE;";
 #endif
-#if DRAW_LLVM_AVAILABLE
+#ifdef DRAW_LLVM_AVAILABLE
    llvm = "LLVM;";
 #endif
 
@@ -141,9 +138,9 @@ get_uint_cap(struct svga_winsys_screen *sws, SVGA3dDevCapIndex cap,
 
 
 /** Helper for querying boolean-valued device cap */
-static bool
+static boolean
 get_bool_cap(struct svga_winsys_screen *sws, SVGA3dDevCapIndex cap,
-             bool defaultVal)
+             boolean defaultVal)
 {
    SVGA3dDevCapResult result;
    if (sws->get_cap(sws, cap, &result))
@@ -160,22 +157,14 @@ svga_get_paramf(struct pipe_screen *screen, enum pipe_capf param)
    struct svga_winsys_screen *sws = svgascreen->sws;
 
    switch (param) {
-   case PIPE_CAPF_MIN_LINE_WIDTH:
-   case PIPE_CAPF_MIN_LINE_WIDTH_AA:
-   case PIPE_CAPF_MIN_POINT_SIZE:
-   case PIPE_CAPF_MIN_POINT_SIZE_AA:
-      return 1;
-   case PIPE_CAPF_POINT_SIZE_GRANULARITY:
-   case PIPE_CAPF_LINE_WIDTH_GRANULARITY:
-      return 0.1;
    case PIPE_CAPF_MAX_LINE_WIDTH:
       return svgascreen->maxLineWidth;
    case PIPE_CAPF_MAX_LINE_WIDTH_AA:
       return svgascreen->maxLineWidthAA;
 
-   case PIPE_CAPF_MAX_POINT_SIZE:
+   case PIPE_CAPF_MAX_POINT_WIDTH:
       FALLTHROUGH;
-   case PIPE_CAPF_MAX_POINT_SIZE_AA:
+   case PIPE_CAPF_MAX_POINT_WIDTH_AA:
       return svgascreen->maxPointSize;
 
    case PIPE_CAPF_MAX_TEXTURE_ANISOTROPY:
@@ -219,15 +208,14 @@ svga_get_param(struct pipe_screen *screen, enum pipe_cap param)
       return sws->have_vgpu10 ? 1 : 0;
    case PIPE_CAP_ANISOTROPIC_FILTER:
       return 1;
+   case PIPE_CAP_POINT_SPRITE:
+      return 1;
    case PIPE_CAP_MAX_RENDER_TARGETS:
       return svgascreen->max_color_buffers;
    case PIPE_CAP_OCCLUSION_QUERY:
       return 1;
    case PIPE_CAP_TEXTURE_BUFFER_OBJECTS:
       return sws->have_vgpu10;
-   case PIPE_CAP_TEXTURE_BUFFER_OFFSET_ALIGNMENT:
-      return sws->have_vgpu10 ? 16 : 0;
-
    case PIPE_CAP_TEXTURE_SWIZZLE:
       return 1;
    case PIPE_CAP_CONSTANT_BUFFER_OFFSET_ALIGNMENT:
@@ -253,7 +241,12 @@ svga_get_param(struct pipe_screen *screen, enum pipe_cap param)
       return MIN2(util_logbase2(result.u) + 1, SVGA_MAX_TEXTURE_LEVELS);
 
    case PIPE_CAP_MAX_TEXTURE_CUBE_LEVELS:
-      return util_last_bit(screen->get_param(screen, PIPE_CAP_MAX_TEXTURE_2D_SIZE));
+      /*
+       * No mechanism to query the host, and at least limited to 2048x2048 on
+       * certain hardware.
+       */
+      return MIN2(util_last_bit(screen->get_param(screen, PIPE_CAP_MAX_TEXTURE_2D_SIZE)),
+                  12 /* 2048x2048 */);
 
    case PIPE_CAP_MAX_TEXTURE_ARRAY_LAYERS:
       return sws->have_sm5 ? SVGA3D_SM5_MAX_SURFACE_ARRAYSIZE :
@@ -262,11 +255,11 @@ svga_get_param(struct pipe_screen *screen, enum pipe_cap param)
    case PIPE_CAP_BLEND_EQUATION_SEPARATE: /* req. for GL 1.5 */
       return 1;
 
-   case PIPE_CAP_FS_COORD_ORIGIN_UPPER_LEFT:
+   case PIPE_CAP_TGSI_FS_COORD_ORIGIN_UPPER_LEFT:
       return 1;
-   case PIPE_CAP_FS_COORD_PIXEL_CENTER_HALF_INTEGER:
+   case PIPE_CAP_TGSI_FS_COORD_PIXEL_CENTER_HALF_INTEGER:
       return sws->have_vgpu10;
-   case PIPE_CAP_FS_COORD_PIXEL_CENTER_INTEGER:
+   case PIPE_CAP_TGSI_FS_COORD_PIXEL_CENTER_INTEGER:
       return !sws->have_vgpu10;
 
    case PIPE_CAP_VERTEX_COLOR_UNCLAMPED:
@@ -274,11 +267,11 @@ svga_get_param(struct pipe_screen *screen, enum pipe_cap param)
    case PIPE_CAP_VERTEX_COLOR_CLAMPED:
       return sws->have_vgpu10;
 
+   case PIPE_CAP_MIXED_COLORBUFFER_FORMATS:
+      return 1; /* expected for GL_ARB_framebuffer_object */
+
    case PIPE_CAP_GLSL_FEATURE_LEVEL:
-   case PIPE_CAP_GLSL_FEATURE_LEVEL_COMPATIBILITY:
-      if (sws->have_gl43) {
-         return 430;
-      } else if (sws->have_sm5) {
+      if (sws->have_sm5) {
          return 410;
       } else if (sws->have_vgpu10) {
          return 330;
@@ -286,18 +279,22 @@ svga_get_param(struct pipe_screen *screen, enum pipe_cap param)
          return 120;
       }
 
-   case PIPE_CAP_TEXTURE_TRANSFER_MODES:
+   case PIPE_CAP_GLSL_FEATURE_LEVEL_COMPATIBILITY:
+      return sws->have_sm5 ? 410 : (sws->have_vgpu10 ? 330 : 120);
+
+   case PIPE_CAP_PREFER_BLIT_BASED_TEXTURE_TRANSFER:
       return 0;
 
    case PIPE_CAP_FRAGMENT_SHADER_TEXTURE_LOD:
    case PIPE_CAP_FRAGMENT_SHADER_DERIVATIVES:
+   case PIPE_CAP_VERTEX_SHADER_SATURATE:
       return 1;
 
    case PIPE_CAP_DEPTH_CLIP_DISABLE:
    case PIPE_CAP_INDEP_BLEND_ENABLE:
    case PIPE_CAP_CONDITIONAL_RENDER:
    case PIPE_CAP_QUERY_TIMESTAMP:
-   case PIPE_CAP_VS_INSTANCEID:
+   case PIPE_CAP_TGSI_INSTANCEID:
    case PIPE_CAP_VERTEX_ELEMENT_INSTANCE_DIVISOR:
    case PIPE_CAP_SEAMLESS_CUBE_MAP:
    case PIPE_CAP_FAKE_SW_MSAA:
@@ -317,7 +314,7 @@ svga_get_param(struct pipe_screen *screen, enum pipe_cap param)
    case PIPE_CAP_TEXTURE_MULTISAMPLE:
       return svgascreen->ms_samples ? 1 : 0;
 
-   case PIPE_CAP_MAX_TEXEL_BUFFER_ELEMENTS_UINT:
+   case PIPE_CAP_MAX_TEXTURE_BUFFER_SIZE:
       /* convert bytes to texels for the case of the largest texel
        * size: float[4].
        */
@@ -367,32 +364,12 @@ svga_get_param(struct pipe_screen *screen, enum pipe_cap param)
    case PIPE_CAP_MAX_VERTEX_STREAMS:
       return sws->have_sm5 ? 4 : 0;
    case PIPE_CAP_COMPUTE:
-      return sws->have_gl43;
+      return 0;
    case PIPE_CAP_MAX_VARYINGS:
-      /* According to the spec, max varyings does not include the components
-       * for position, so remove one count from the max for position.
-       */
-      return sws->have_vgpu10 ? VGPU10_MAX_FS_INPUTS-1 : 10;
+      return sws->have_vgpu10 ? VGPU10_MAX_FS_INPUTS : 10;
    case PIPE_CAP_BUFFER_MAP_PERSISTENT_COHERENT:
       return sws->have_coherent;
 
-   case PIPE_CAP_START_INSTANCE:
-      return sws->have_sm5;
-   case PIPE_CAP_ROBUST_BUFFER_ACCESS_BEHAVIOR:
-      return sws->have_sm5;
-
-   case PIPE_CAP_SAMPLER_VIEW_TARGET:
-      return sws->have_gl43;
-
-   case PIPE_CAP_FRAMEBUFFER_NO_ATTACHMENT:
-      return sws->have_gl43;
-
-   case PIPE_CAP_CLIP_HALFZ:
-#ifndef VBOX_WITH_MESA3D_SVGA_HALFZ
-      return sws->have_gl43;
-#else
-      return 1;
-#endif
    case PIPE_CAP_SHAREABLE_SHADERS:
       return 0;
 
@@ -401,25 +378,12 @@ svga_get_param(struct pipe_screen *screen, enum pipe_cap param)
    case PIPE_CAP_PCI_DEVICE:
    case PIPE_CAP_PCI_FUNCTION:
       return 0;
-   case PIPE_CAP_SHADER_BUFFER_OFFSET_ALIGNMENT:
-      return sws->have_gl43 ? 16 : 0;
-
-   case PIPE_CAP_MAX_COMBINED_SHADER_OUTPUT_RESOURCES:
-   case PIPE_CAP_MAX_COMBINED_SHADER_BUFFERS:
-      return sws->have_gl43 ? SVGA_MAX_SHADER_BUFFERS : 0;
-   case PIPE_CAP_MAX_COMBINED_HW_ATOMIC_COUNTERS:
-   case PIPE_CAP_MAX_COMBINED_HW_ATOMIC_COUNTER_BUFFERS:
-      return sws->have_gl43 ? SVGA_MAX_ATOMIC_BUFFERS : 0;
    case PIPE_CAP_MIN_MAP_BUFFER_ALIGNMENT:
       return 64;
    case PIPE_CAP_VERTEX_BUFFER_STRIDE_4BYTE_ALIGNED_ONLY:
-      return sws->have_vgpu10 ? 0 : 1;
-   case PIPE_CAP_VERTEX_ATTRIB_ELEMENT_ALIGNED_ONLY:
-      /* This CAP cannot be used with any other alignment-requiring CAPs */
-      return sws->have_vgpu10 ? 1 : 0;
    case PIPE_CAP_VERTEX_BUFFER_OFFSET_4BYTE_ALIGNED_ONLY:
    case PIPE_CAP_VERTEX_ELEMENT_SRC_OFFSET_4BYTE_ALIGNED_ONLY:
-      return sws->have_vgpu10 ? 0 : 1;
+      return 1;  /* need 4-byte alignment for all offsets and strides */
    case PIPE_CAP_MAX_VERTEX_ATTRIB_STRIDE:
       return 2048;
    case PIPE_CAP_MAX_VIEWPORTS:
@@ -433,11 +397,7 @@ svga_get_param(struct pipe_screen *screen, enum pipe_cap param)
    case PIPE_CAP_VENDOR_ID:
       return 0x15ad; /* VMware Inc. */
    case PIPE_CAP_DEVICE_ID:
-      if (sws->device_id) {
-         return sws->device_id;
-      } else {
-         return 0x0405; /* assume SVGA II */
-      }
+      return 0x0405; /* assume SVGA II */
    case PIPE_CAP_ACCELERATED:
       return 0; /* XXX: */
    case PIPE_CAP_VIDEO_MEMORY:
@@ -449,28 +409,27 @@ svga_get_param(struct pipe_screen *screen, enum pipe_cap param)
 #endif
    case PIPE_CAP_COPY_BETWEEN_COMPRESSED_AND_PLAIN_FORMATS:
       return sws->have_vgpu10;
+   case PIPE_CAP_CLEAR_TEXTURE:
+      return sws->have_vgpu10;
    case PIPE_CAP_DOUBLES:
       return sws->have_sm5;
    case PIPE_CAP_UMA:
+   case PIPE_CAP_GLSL_OPTIMIZE_CONSERVATIVELY:
    case PIPE_CAP_ALLOW_MAPPED_BUFFERS_DURING_EXECUTION:
       return 0;
    case PIPE_CAP_TGSI_DIV:
       return 1;
    case PIPE_CAP_MAX_GS_INVOCATIONS:
       return 32;
-   case PIPE_CAP_MAX_SHADER_BUFFER_SIZE_UINT:
+   case PIPE_CAP_MAX_SHADER_BUFFER_SIZE:
       return 1 << 27;
    /* Verify this once protocol is finalized. Setting it to minimum value. */
    case PIPE_CAP_MAX_SHADER_PATCH_VARYINGS:
       return sws->have_sm5 ? 30 : 0;
-   case PIPE_CAP_TEXTURE_FLOAT_LINEAR:
-      return 1;
-   case PIPE_CAP_TEXTURE_HALF_FLOAT_LINEAR:
-      return 1;
-   case PIPE_CAP_TGSI_TEXCOORD:
-      return sws->have_vgpu10 ? 1 : 0;
-   case PIPE_CAP_IMAGE_STORE_FORMATTED:
-      return sws->have_gl43;
+#ifdef VBOX_WITH_MESA3D_SVGA_HALFZ
+   case PIPE_CAP_CLIP_HALFZ:
+       return 1;
+#endif
    default:
       return u_pipe_screen_get_param_defaults(screen, param);
    }
@@ -507,7 +466,7 @@ vgpu9_get_shader_param(struct pipe_screen *screen,
          return 10;
       case PIPE_SHADER_CAP_MAX_OUTPUTS:
          return svgascreen->max_color_buffers;
-      case PIPE_SHADER_CAP_MAX_CONST_BUFFER0_SIZE:
+      case PIPE_SHADER_CAP_MAX_CONST_BUFFER_SIZE:
          return 224 * sizeof(float[4]);
       case PIPE_SHADER_CAP_MAX_CONST_BUFFERS:
          return 1;
@@ -522,7 +481,7 @@ vgpu9_get_shader_param(struct pipe_screen *screen,
           * does it is better to defer loop unrolling to the gallium frontend.
           */
          return 0;
-      case PIPE_SHADER_CAP_CONT_SUPPORTED:
+      case PIPE_SHADER_CAP_TGSI_CONT_SUPPORTED:
          return 0;
       case PIPE_SHADER_CAP_TGSI_SQRT_SUPPORTED:
          return 0;
@@ -544,14 +503,24 @@ vgpu9_get_shader_param(struct pipe_screen *screen,
       case PIPE_SHADER_CAP_MAX_TEXTURE_SAMPLERS:
       case PIPE_SHADER_CAP_MAX_SAMPLER_VIEWS:
          return 16;
+      case PIPE_SHADER_CAP_PREFERRED_IR:
+         return PIPE_SHADER_IR_TGSI;
       case PIPE_SHADER_CAP_SUPPORTED_IRS:
-         return (1 << PIPE_SHADER_IR_TGSI) | (1 << PIPE_SHADER_IR_NIR);
+         return 1 << PIPE_SHADER_IR_TGSI;
+      case PIPE_SHADER_CAP_TGSI_DROUND_SUPPORTED:
+      case PIPE_SHADER_CAP_TGSI_DFRACEXP_DLDEXP_SUPPORTED:
+      case PIPE_SHADER_CAP_TGSI_LDEXP_SUPPORTED:
+      case PIPE_SHADER_CAP_TGSI_FMA_SUPPORTED:
       case PIPE_SHADER_CAP_TGSI_ANY_INOUT_DECL_RANGE:
       case PIPE_SHADER_CAP_MAX_SHADER_BUFFERS:
       case PIPE_SHADER_CAP_MAX_SHADER_IMAGES:
+      case PIPE_SHADER_CAP_LOWER_IF_THRESHOLD:
+      case PIPE_SHADER_CAP_TGSI_SKIP_MERGE_REGISTERS:
       case PIPE_SHADER_CAP_MAX_HW_ATOMIC_COUNTERS:
       case PIPE_SHADER_CAP_MAX_HW_ATOMIC_COUNTER_BUFFERS:
          return 0;
+      case PIPE_SHADER_CAP_MAX_UNROLL_ITERATIONS_HINT:
+         return 32;
       }
       /* If we get here, we failed to handle a cap above */
       debug_printf("Unexpected fragment shader query %u\n", param);
@@ -573,14 +542,14 @@ vgpu9_get_shader_param(struct pipe_screen *screen,
          return 16;
       case PIPE_SHADER_CAP_MAX_OUTPUTS:
          return 10;
-      case PIPE_SHADER_CAP_MAX_CONST_BUFFER0_SIZE:
+      case PIPE_SHADER_CAP_MAX_CONST_BUFFER_SIZE:
          return 256 * sizeof(float[4]);
       case PIPE_SHADER_CAP_MAX_CONST_BUFFERS:
          return 1;
       case PIPE_SHADER_CAP_MAX_TEMPS:
          val = get_uint_cap(sws, SVGA3D_DEVCAP_MAX_VERTEX_SHADER_TEMPS, 32);
          return MIN2(val, SVGA3D_TEMPREG_MAX);
-      case PIPE_SHADER_CAP_CONT_SUPPORTED:
+      case PIPE_SHADER_CAP_TGSI_CONT_SUPPORTED:
          return 0;
       case PIPE_SHADER_CAP_TGSI_SQRT_SUPPORTED:
          return 0;
@@ -605,14 +574,24 @@ vgpu9_get_shader_param(struct pipe_screen *screen,
       case PIPE_SHADER_CAP_MAX_TEXTURE_SAMPLERS:
       case PIPE_SHADER_CAP_MAX_SAMPLER_VIEWS:
          return 0;
+      case PIPE_SHADER_CAP_PREFERRED_IR:
+         return PIPE_SHADER_IR_TGSI;
       case PIPE_SHADER_CAP_SUPPORTED_IRS:
-         return (1 << PIPE_SHADER_IR_TGSI) | (1 << PIPE_SHADER_IR_NIR);
+         return 1 << PIPE_SHADER_IR_TGSI;
+      case PIPE_SHADER_CAP_TGSI_DROUND_SUPPORTED:
+      case PIPE_SHADER_CAP_TGSI_DFRACEXP_DLDEXP_SUPPORTED:
+      case PIPE_SHADER_CAP_TGSI_LDEXP_SUPPORTED:
+      case PIPE_SHADER_CAP_TGSI_FMA_SUPPORTED:
       case PIPE_SHADER_CAP_TGSI_ANY_INOUT_DECL_RANGE:
       case PIPE_SHADER_CAP_MAX_SHADER_BUFFERS:
       case PIPE_SHADER_CAP_MAX_SHADER_IMAGES:
+      case PIPE_SHADER_CAP_LOWER_IF_THRESHOLD:
+      case PIPE_SHADER_CAP_TGSI_SKIP_MERGE_REGISTERS:
       case PIPE_SHADER_CAP_MAX_HW_ATOMIC_COUNTERS:
       case PIPE_SHADER_CAP_MAX_HW_ATOMIC_COUNTER_BUFFERS:
          return 0;
+      case PIPE_SHADER_CAP_MAX_UNROLL_ITERATIONS_HINT:
+         return 32;
       }
       /* If we get here, we failed to handle a cap above */
       debug_printf("Unexpected vertex shader query %u\n", param);
@@ -623,9 +602,6 @@ vgpu9_get_shader_param(struct pipe_screen *screen,
    case PIPE_SHADER_TESS_EVAL:
       /* no support for geometry, tess or compute shaders at this time */
       return 0;
-   case PIPE_SHADER_MESH:
-   case PIPE_SHADER_TASK:
-      return 0;
    default:
       debug_printf("Unexpected shader type (%u) query\n", shader);
       return 0;
@@ -645,14 +621,11 @@ vgpu10_get_shader_param(struct pipe_screen *screen,
    assert(sws->have_vgpu10);
    (void) sws;  /* silence unused var warnings in non-debug builds */
 
-   if (shader == PIPE_SHADER_MESH || shader == PIPE_SHADER_TASK)
-      return 0;
-
    if ((!sws->have_sm5) &&
        (shader == PIPE_SHADER_TESS_CTRL || shader == PIPE_SHADER_TESS_EVAL))
       return 0;
 
-   if ((!sws->have_gl43) && (shader == PIPE_SHADER_COMPUTE))
+   if (shader == PIPE_SHADER_COMPUTE)
       return 0;
 
    /* NOTE: we do not query the device for any caps/limits at this time */
@@ -670,13 +643,13 @@ vgpu10_get_shader_param(struct pipe_screen *screen,
       if (shader == PIPE_SHADER_FRAGMENT)
          return VGPU10_MAX_FS_INPUTS;
       else if (shader == PIPE_SHADER_GEOMETRY)
-         return svgascreen->max_gs_inputs;
+         return VGPU10_MAX_GS_INPUTS;
       else if (shader == PIPE_SHADER_TESS_CTRL)
          return VGPU11_MAX_HS_INPUT_CONTROL_POINTS;
       else if (shader == PIPE_SHADER_TESS_EVAL)
          return VGPU11_MAX_DS_INPUT_CONTROL_POINTS;
       else
-         return svgascreen->max_vs_inputs;
+         return VGPU10_MAX_VS_INPUTS;
    case PIPE_SHADER_CAP_MAX_OUTPUTS:
       if (shader == PIPE_SHADER_FRAGMENT)
          return VGPU10_MAX_FS_OUTPUTS;
@@ -687,9 +660,8 @@ vgpu10_get_shader_param(struct pipe_screen *screen,
       else if (shader == PIPE_SHADER_TESS_EVAL)
          return VGPU11_MAX_DS_OUTPUTS;
       else
-         return svgascreen->max_vs_outputs;
-
-   case PIPE_SHADER_CAP_MAX_CONST_BUFFER0_SIZE:
+         return VGPU10_MAX_VS_OUTPUTS;
+   case PIPE_SHADER_CAP_MAX_CONST_BUFFER_SIZE:
       return VGPU10_MAX_CONSTANT_BUFFER_ELEMENT_COUNT * sizeof(float[4]);
    case PIPE_SHADER_CAP_MAX_CONST_BUFFERS:
       return svgascreen->max_const_buffers;
@@ -699,40 +671,40 @@ vgpu10_get_shader_param(struct pipe_screen *screen,
    case PIPE_SHADER_CAP_INDIRECT_OUTPUT_ADDR:
    case PIPE_SHADER_CAP_INDIRECT_TEMP_ADDR:
    case PIPE_SHADER_CAP_INDIRECT_CONST_ADDR:
-      return true; /* XXX verify */
-   case PIPE_SHADER_CAP_CONT_SUPPORTED:
+      return TRUE; /* XXX verify */
+   case PIPE_SHADER_CAP_TGSI_CONT_SUPPORTED:
    case PIPE_SHADER_CAP_TGSI_SQRT_SUPPORTED:
    case PIPE_SHADER_CAP_SUBROUTINES:
    case PIPE_SHADER_CAP_INTEGERS:
-      return true;
+      return TRUE;
    case PIPE_SHADER_CAP_FP16:
    case PIPE_SHADER_CAP_FP16_DERIVATIVES:
    case PIPE_SHADER_CAP_FP16_CONST_BUFFERS:
    case PIPE_SHADER_CAP_INT16:
    case PIPE_SHADER_CAP_GLSL_16BIT_CONSTS:
-      return false;
+      return FALSE;
    case PIPE_SHADER_CAP_MAX_TEXTURE_SAMPLERS:
    case PIPE_SHADER_CAP_MAX_SAMPLER_VIEWS:
-      return sws->have_gl43 ? PIPE_MAX_SAMPLERS : SVGA3D_DX_MAX_SAMPLERS;
+      return SVGA3D_DX_MAX_SAMPLERS;
+   case PIPE_SHADER_CAP_PREFERRED_IR:
+      return PIPE_SHADER_IR_TGSI;
    case PIPE_SHADER_CAP_SUPPORTED_IRS:
-      if (sws->have_gl43)
-         return (1 << PIPE_SHADER_IR_TGSI) | (1 << PIPE_SHADER_IR_NIR);
-      else
-         return 0;
-
-   case PIPE_SHADER_CAP_MAX_SHADER_IMAGES:
-      return sws->have_gl43 ? SVGA_MAX_IMAGES : 0;
-
+      return 1 << PIPE_SHADER_IR_TGSI;
+   case PIPE_SHADER_CAP_TGSI_DROUND_SUPPORTED:
+   case PIPE_SHADER_CAP_TGSI_DFRACEXP_DLDEXP_SUPPORTED:
+   case PIPE_SHADER_CAP_TGSI_LDEXP_SUPPORTED:
+   case PIPE_SHADER_CAP_TGSI_FMA_SUPPORTED:
+   case PIPE_SHADER_CAP_TGSI_ANY_INOUT_DECL_RANGE:
    case PIPE_SHADER_CAP_MAX_SHADER_BUFFERS:
-      return sws->have_gl43 ? SVGA_MAX_SHADER_BUFFERS : 0;
-
+   case PIPE_SHADER_CAP_MAX_SHADER_IMAGES:
+   case PIPE_SHADER_CAP_LOWER_IF_THRESHOLD:
+   case PIPE_SHADER_CAP_TGSI_SKIP_MERGE_REGISTERS:
+   case PIPE_SHADER_CAP_INT64_ATOMICS:
    case PIPE_SHADER_CAP_MAX_HW_ATOMIC_COUNTERS:
    case PIPE_SHADER_CAP_MAX_HW_ATOMIC_COUNTER_BUFFERS:
-      return sws->have_gl43 ? SVGA_MAX_ATOMIC_BUFFERS : 0;
-
-   case PIPE_SHADER_CAP_TGSI_ANY_INOUT_DECL_RANGE:
-   case PIPE_SHADER_CAP_INT64_ATOMICS:
       return 0;
+   case PIPE_SHADER_CAP_MAX_UNROLL_ITERATIONS_HINT:
+      return 32;
    default:
       debug_printf("Unexpected vgpu10 shader query %u\n", param);
       return 0;
@@ -740,72 +712,6 @@ vgpu10_get_shader_param(struct pipe_screen *screen,
    return 0;
 }
 
-#define COMMON_OPTIONS                                                        \
-   .lower_extract_byte = true,                                                \
-   .lower_extract_word = true,                                                \
-   .lower_insert_byte = true,                                                 \
-   .lower_insert_word = true,                                                 \
-   .lower_int64_options = nir_lower_imul_2x32_64 | nir_lower_divmod64,        \
-   .lower_fdph = true,                                                        \
-   .lower_flrp64 = true,                                                      \
-   .lower_ldexp = true,                                                       \
-   .lower_uniforms_to_ubo = true,                                             \
-   .lower_vector_cmp = true,                                                  \
-   .lower_cs_local_index_to_id = true,                                        \
-   .max_unroll_iterations = 32,                                               \
-   .use_interpolated_input_intrinsics = true
-
-#define VGPU10_OPTIONS                                                        \
-   .lower_doubles_options = nir_lower_dfloor | nir_lower_dsign | nir_lower_dceil | nir_lower_dtrunc | nir_lower_dround_even, \
-   .lower_fmod = true,                                                        \
-   .lower_fpow = true
-
-static const nir_shader_compiler_options svga_vgpu9_fragment_compiler_options = {
-   COMMON_OPTIONS,
-   .lower_bitops = true,
-   .force_indirect_unrolling = nir_var_all,
-   .force_indirect_unrolling_sampler = true,
-};
-
-static const nir_shader_compiler_options svga_vgpu9_vertex_compiler_options = {
-   COMMON_OPTIONS,
-   .lower_bitops = true,
-   .force_indirect_unrolling = nir_var_function_temp,
-   .force_indirect_unrolling_sampler = true,
-};
-
-static const nir_shader_compiler_options svga_vgpu10_compiler_options = {
-   COMMON_OPTIONS,
-   VGPU10_OPTIONS,
-   .force_indirect_unrolling_sampler = true,
-};
-
-static const nir_shader_compiler_options svga_gl4_compiler_options = {
-   COMMON_OPTIONS,
-   VGPU10_OPTIONS,
-};
-
-static const void *
-svga_get_compiler_options(struct pipe_screen *pscreen,
-                          enum pipe_shader_ir ir,
-                          enum pipe_shader_type shader)
-{
-   struct svga_screen *svgascreen = svga_screen(pscreen);
-   struct svga_winsys_screen *sws = svgascreen->sws;
-
-   assert(ir == PIPE_SHADER_IR_NIR);
-
-   if (sws->have_gl43 || sws->have_sm5)
-      return &svga_gl4_compiler_options;
-   else if (sws->have_vgpu10)
-      return &svga_vgpu10_compiler_options;
-   else {
-      if (shader == PIPE_SHADER_FRAGMENT)
-         return &svga_vgpu9_fragment_compiler_options;
-      else
-         return &svga_vgpu9_vertex_compiler_options;
-   }
-}
 
 static int
 svga_get_shader_param(struct pipe_screen *screen, enum pipe_shader_type shader,
@@ -822,44 +728,6 @@ svga_get_shader_param(struct pipe_screen *screen, enum pipe_shader_type shader,
 }
 
 
-static int
-svga_sm5_get_compute_param(struct pipe_screen *screen,
-                           enum pipe_shader_ir ir_type,
-                           enum pipe_compute_cap param,
-                           void *ret)
-{
-   ASSERTED struct svga_screen *svgascreen = svga_screen(screen);
-   ASSERTED struct svga_winsys_screen *sws = svgascreen->sws;
-   uint64_t *iret = (uint64_t *)ret;
-
-   assert(sws->have_gl43);
-
-   switch (param) {
-   case PIPE_COMPUTE_CAP_MAX_GRID_SIZE:
-      iret[0] = 65535;
-      iret[1] = 65535;
-      iret[2] = 65535;
-      return 3 * sizeof(uint64_t);
-   case PIPE_COMPUTE_CAP_MAX_BLOCK_SIZE:
-      iret[0] = 1024;
-      iret[1] = 1024;
-      iret[2] = 64;
-      return 3 * sizeof(uint64_t);
-   case PIPE_COMPUTE_CAP_MAX_THREADS_PER_BLOCK:
-      *iret = 1024;
-      return sizeof(uint64_t);
-   case PIPE_COMPUTE_CAP_MAX_LOCAL_SIZE:
-      *iret = 32768;
-      return sizeof(uint64_t);
-   case PIPE_COMPUTE_CAP_MAX_VARIABLE_THREADS_PER_BLOCK:
-      *iret = 0;
-      return sizeof(uint64_t);
-   default:
-      debug_printf("Unexpected compute param %u\n", param);
-   }
-   return 0;
-}
-
 static void
 svga_fence_reference(struct pipe_screen *screen,
                      struct pipe_fence_handle **ptr,
@@ -886,7 +754,7 @@ svga_fence_finish(struct pipe_screen *screen,
    }
    else {
       SVGA_DBG(DEBUG_DMA|DEBUG_PERF, "%s fence_ptr %p\n",
-               __func__, fence);
+               __FUNCTION__, fence);
 
       retVal = sws->fence_finish(sws, fence, timeout, 0) == 0;
    }
@@ -903,7 +771,7 @@ svga_fence_get_fd(struct pipe_screen *screen,
 {
    struct svga_winsys_screen *sws = svga_screen(screen)->sws;
 
-   return sws->fence_get_fd(sws, fence, true);
+   return sws->fence_get_fd(sws, fence, TRUE);
 }
 
 
@@ -1008,9 +876,9 @@ init_logging(struct pipe_screen *screen)
    /* If the SVGA_EXTRA_LOGGING env var is set, log the process's command
     * line (program name and arguments).
     */
-   if (debug_get_bool_option("SVGA_EXTRA_LOGGING", false)) {
+   if (debug_get_bool_option("SVGA_EXTRA_LOGGING", FALSE)) {
       char cmdline[1000];
-      if (util_get_command_line(cmdline, sizeof(cmdline))) {
+      if (os_get_command_line(cmdline, sizeof(cmdline))) {
          snprintf(host_log, sizeof(host_log) - strlen(log_prefix),
                   "%s%s\n", log_prefix, cmdline);
          svgascreen->sws->host_log(svgascreen->sws, host_log);
@@ -1045,15 +913,6 @@ svga_destroy_screen( struct pipe_screen *screen )
 }
 
 
-static int
-svga_screen_get_fd( struct pipe_screen *screen )
-{
-   struct svga_winsys_screen *sws = svga_screen(screen)->sws;
-
-   return sws->get_fd(sws);
-}
-
-
 /**
  * Create a new svga_screen object
  */
@@ -1072,17 +931,17 @@ svga_screen_create(struct svga_winsys_screen *sws)
       goto error1;
 
    svgascreen->debug.force_level_surface_view =
-      debug_get_bool_option("SVGA_FORCE_LEVEL_SURFACE_VIEW", false);
+      debug_get_bool_option("SVGA_FORCE_LEVEL_SURFACE_VIEW", FALSE);
    svgascreen->debug.force_surface_view =
-      debug_get_bool_option("SVGA_FORCE_SURFACE_VIEW", false);
+      debug_get_bool_option("SVGA_FORCE_SURFACE_VIEW", FALSE);
    svgascreen->debug.force_sampler_view =
-      debug_get_bool_option("SVGA_FORCE_SAMPLER_VIEW", false);
+      debug_get_bool_option("SVGA_FORCE_SAMPLER_VIEW", FALSE);
    svgascreen->debug.no_surface_view =
-      debug_get_bool_option("SVGA_NO_SURFACE_VIEW", false);
+      debug_get_bool_option("SVGA_NO_SURFACE_VIEW", FALSE);
    svgascreen->debug.no_sampler_view =
-      debug_get_bool_option("SVGA_NO_SAMPLER_VIEW", false);
+      debug_get_bool_option("SVGA_NO_SAMPLER_VIEW", FALSE);
    svgascreen->debug.no_cache_index_buffers =
-      debug_get_bool_option("SVGA_NO_CACHE_INDEX_BUFFERS", false);
+      debug_get_bool_option("SVGA_NO_CACHE_INDEX_BUFFERS", FALSE);
 
    screen = &svgascreen->screen;
 
@@ -1090,10 +949,8 @@ svga_screen_create(struct svga_winsys_screen *sws)
    screen->get_name = svga_get_name;
    screen->get_vendor = svga_get_vendor;
    screen->get_device_vendor = svga_get_vendor; // TODO actual device vendor
-   screen->get_screen_fd = svga_screen_get_fd;
    screen->get_param = svga_get_param;
    screen->get_shader_param = svga_get_shader_param;
-   screen->get_compiler_options = svga_get_compiler_options;
    screen->get_paramf = svga_get_paramf;
    screen->get_timestamp = NULL;
    screen->is_format_supported = svga_is_format_supported;
@@ -1103,9 +960,6 @@ svga_screen_create(struct svga_winsys_screen *sws)
    screen->fence_get_fd = svga_fence_get_fd;
 
    screen->get_driver_query_info = svga_get_driver_query_info;
-
-   screen->get_compute_param = svga_sm5_get_compute_param;
-
    svgascreen->sws = sws;
 
    svga_init_screen_resource_functions(svgascreen);
@@ -1123,29 +977,7 @@ svga_screen_create(struct svga_winsys_screen *sws)
       goto error2;
    }
 
-   if (sws->have_gl43) {
-      svgascreen->forcedSampleCount =
-         get_uint_cap(sws, SVGA3D_DEVCAP_MAX_FORCED_SAMPLE_COUNT, 0);
-
-      sws->have_gl43 = sws->have_gl43 && (svgascreen->forcedSampleCount >= 4);
-
-      /* Allow a temporary environment variable to enable/disable GL43 support.
-       */
-      sws->have_gl43 =
-         debug_get_bool_option("SVGA_GL43", sws->have_gl43);
-
-      svgascreen->debug.sampler_state_mapping =
-         debug_get_bool_option("SVGA_SAMPLER_STATE_MAPPING", false);
-   }
-   else {
-      /* sampler state mapping code is only enabled with GL43
-       * due to the limitation in SW Renderer. (VMware bug 2825014)
-       */
-      svgascreen->debug.sampler_state_mapping = false;
-   }
-
    debug_printf("%s enabled\n",
-                sws->have_gl43 ? "SM5+" :
                 sws->have_sm5 ? "SM5" :
                 sws->have_sm4_1 ? "SM4_1" :
                 sws->have_vgpu10 ? "VGPU10" : "VGPU9");
@@ -1163,7 +995,7 @@ svga_screen_create(struct svga_winsys_screen *sws)
     */
 
    {
-      bool has_df16, has_df24, has_d24s8_int;
+      boolean has_df16, has_df24, has_d24s8_int;
       SVGA3dSurfaceFormatCaps caps;
       SVGA3dSurfaceFormatCaps mask;
       mask.value = 0;
@@ -1202,53 +1034,36 @@ svga_screen_create(struct svga_winsys_screen *sws)
     */
    if (sws->have_vgpu10) {
       svgascreen->haveProvokingVertex
-         = get_bool_cap(sws, SVGA3D_DEVCAP_DX_PROVOKING_VERTEX, false);
-      svgascreen->haveLineSmooth = true;
+         = get_bool_cap(sws, SVGA3D_DEVCAP_DX_PROVOKING_VERTEX, FALSE);
+      svgascreen->haveLineSmooth = TRUE;
       svgascreen->maxPointSize = 80.0F;
       svgascreen->max_color_buffers = SVGA3D_DX_MAX_RENDER_TARGETS;
 
       /* Multisample samples per pixel */
-      if (sws->have_sm4_1 && debug_get_bool_option("SVGA_MSAA", true)) {
-         if (get_bool_cap(sws, SVGA3D_DEVCAP_MULTISAMPLE_2X, false))
+      if (sws->have_sm4_1 && debug_get_bool_option("SVGA_MSAA", TRUE)) {
+         if (get_bool_cap(sws, SVGA3D_DEVCAP_MULTISAMPLE_2X, FALSE))
             svgascreen->ms_samples |= 1 << 1;
-         if (get_bool_cap(sws, SVGA3D_DEVCAP_MULTISAMPLE_4X, false))
+         if (get_bool_cap(sws, SVGA3D_DEVCAP_MULTISAMPLE_4X, FALSE))
             svgascreen->ms_samples |= 1 << 3;
       }
 
-      if (sws->have_sm5 && debug_get_bool_option("SVGA_MSAA", true)) {
-         if (get_bool_cap(sws, SVGA3D_DEVCAP_MULTISAMPLE_8X, false))
+      if (sws->have_sm5 && debug_get_bool_option("SVGA_MSAA", TRUE)) {
+         if (get_bool_cap(sws, SVGA3D_DEVCAP_MULTISAMPLE_8X, FALSE))
             svgascreen->ms_samples |= 1 << 7;
       }
 
       /* Maximum number of constant buffers */
-      if (sws->have_gl43) {
-         svgascreen->max_const_buffers = SVGA_MAX_CONST_BUFS;
-      }
-      else {
-         svgascreen->max_const_buffers =
-            get_uint_cap(sws, SVGA3D_DEVCAP_DX_MAX_CONSTANT_BUFFERS, 1);
-         svgascreen->max_const_buffers = MIN2(svgascreen->max_const_buffers,
-                                              SVGA_MAX_CONST_BUFS);
-      }
+      svgascreen->max_const_buffers =
+         get_uint_cap(sws, SVGA3D_DEVCAP_DX_MAX_CONSTANT_BUFFERS, 1);
+      svgascreen->max_const_buffers = MIN2(svgascreen->max_const_buffers,
+                                           SVGA_MAX_CONST_BUFS);
 
       svgascreen->haveBlendLogicops =
-         get_bool_cap(sws, SVGA3D_DEVCAP_LOGIC_BLENDOPS, false);
+         get_bool_cap(sws, SVGA3D_DEVCAP_LOGIC_BLENDOPS, FALSE);
 
       screen->is_format_supported = svga_is_dx_format_supported;
 
       svgascreen->max_viewports = SVGA3D_DX_MAX_VIEWPORTS;
-
-      /* Shader limits */
-      if (sws->have_sm4_1) {
-         svgascreen->max_vs_inputs  = VGPU10_1_MAX_VS_INPUTS;
-         svgascreen->max_vs_outputs = VGPU10_1_MAX_VS_OUTPUTS;
-         svgascreen->max_gs_inputs  = VGPU10_1_MAX_GS_INPUTS;
-      }
-      else {
-         svgascreen->max_vs_inputs  = VGPU10_MAX_VS_INPUTS;
-         svgascreen->max_vs_outputs = VGPU10_MAX_VS_OUTPUTS;
-         svgascreen->max_gs_inputs  = VGPU10_MAX_GS_INPUTS;
-      }
    }
    else {
       /* VGPU9 */
@@ -1262,10 +1077,10 @@ svga_screen_create(struct svga_winsys_screen *sws)
          goto error2;
       }
 
-      svgascreen->haveProvokingVertex = false;
+      svgascreen->haveProvokingVertex = FALSE;
 
       svgascreen->haveLineSmooth =
-         get_bool_cap(sws, SVGA3D_DEVCAP_LINE_AA, false);
+         get_bool_cap(sws, SVGA3D_DEVCAP_LINE_AA, FALSE);
 
       svgascreen->maxPointSize =
          get_float_cap(sws, SVGA3D_DEVCAP_MAX_POINT_SIZE, 1.0f);
@@ -1286,16 +1101,11 @@ svga_screen_create(struct svga_winsys_screen *sws)
 
       /* Only one viewport */
       svgascreen->max_viewports = 1;
-
-      /* Shader limits */
-      svgascreen->max_vs_inputs  = 16;
-      svgascreen->max_vs_outputs = 10;
-      svgascreen->max_gs_inputs  = 0;
    }
 
    /* common VGPU9 / VGPU10 caps */
    svgascreen->haveLineStipple =
-      get_bool_cap(sws, SVGA3D_DEVCAP_LINE_STIPPLE, false);
+      get_bool_cap(sws, SVGA3D_DEVCAP_LINE_STIPPLE, FALSE);
 
    svgascreen->maxLineWidth =
       MAX2(1.0, get_float_cap(sws, SVGA3D_DEVCAP_MAX_LINE_WIDTH, 1.0f));
@@ -1315,11 +1125,11 @@ svga_screen_create(struct svga_winsys_screen *sws)
    }
 
    (void) mtx_init(&svgascreen->tex_mutex, mtx_plain);
-   (void) mtx_init(&svgascreen->swc_mutex, mtx_plain | mtx_recursive);
+   (void) mtx_init(&svgascreen->swc_mutex, mtx_recursive);
 
    svga_screen_cache_init(svgascreen);
 
-   if (debug_get_bool_option("SVGA_NO_LOGGING", false) == true) {
+   if (debug_get_bool_option("SVGA_NO_LOGGING", FALSE) == TRUE) {
       svgascreen->sws->host_log = nop_host_log;
    } else {
       init_logging(screen);
