@@ -28,7 +28,7 @@
 #include "pipe/p_state.h"
 #include "pipe/p_defines.h"
 #include "util/u_inlines.h"
-#include "util/u_thread.h"
+#include "os/os_thread.h"
 #include "util/u_bitmask.h"
 #include "util/format/u_format.h"
 #include "util/u_math.h"
@@ -189,13 +189,13 @@ svga_texture_view_surface(struct svga_context *svga,
                           int layer_pick,
                           unsigned num_layers,
                           int zslice_pick,
-                          bool cacheable,
+                          boolean cacheable,
                           struct svga_host_surface_cache_key *key) /* OUT */
 {
    struct svga_screen *ss = svga_screen(svga->pipe.screen);
    struct svga_winsys_surface *handle = NULL;
-   bool invalidated;
-   bool needCopyResource;
+   boolean validated;
+   boolean needCopyResource;
 
    SVGA_DBG(DEBUG_PERF,
             "svga: Create surface view: layer %d zslice %d mips %d..%d\n",
@@ -245,8 +245,8 @@ svga_texture_view_surface(struct svga_context *svga,
    } else {
       SVGA_DBG(DEBUG_DMA, "surface_create for texture view\n");
       handle = svga_screen_surface_create(ss, bind_flags, PIPE_USAGE_DEFAULT,
-                                          &invalidated, key);
-      needCopyResource = true;
+                                          &validated, key);
+      needCopyResource = TRUE;
 
       if (cacheable && !tex->backed_handle) {
          tex->backed_handle = handle;
@@ -288,7 +288,7 @@ static struct pipe_surface *
 svga_create_surface_view(struct pipe_context *pipe,
                          struct pipe_resource *pt,
                          const struct pipe_surface *surf_tmpl,
-                         bool clone_resource)
+                         boolean clone_resource)
 {
    struct svga_context *svga = svga_context(pipe);
    struct svga_texture *tex = svga_texture(pt);
@@ -400,7 +400,7 @@ svga_create_surface_view(struct pipe_context *pipe,
                                             tex->key.format,
                                             surf_tmpl->u.tex.level, 1,
                                             layer, nlayers, zslice,
-                                            true, &s->key);
+                                            TRUE, &s->key);
       if (!s->handle) {
          FREE(s);
          goto done;
@@ -440,22 +440,22 @@ svga_create_surface(struct pipe_context *pipe,
    struct svga_context *svga = svga_context(pipe);
    struct pipe_screen *screen = pipe->screen;
    struct pipe_surface *surf = NULL;
-   bool view = false;
+   boolean view = FALSE;
 
    SVGA_STATS_TIME_PUSH(svga_sws(svga), SVGA_STATS_TIME_CREATESURFACE);
 
    if (svga_screen(screen)->debug.force_surface_view)
-      view = true;
+      view = TRUE;
 
    if (surf_tmpl->u.tex.level != 0 &&
        svga_screen(screen)->debug.force_level_surface_view)
-      view = true;
+      view = TRUE;
 
    if (pt->target == PIPE_TEXTURE_3D)
-      view = true;
+      view = TRUE;
 
    if (svga_have_vgpu10(svga) || svga_screen(screen)->debug.no_surface_view)
-      view = false;
+      view = FALSE;
 
    surf = svga_create_surface_view(pipe, pt, surf_tmpl, view);
 
@@ -470,7 +470,7 @@ svga_create_surface(struct pipe_context *pipe,
  */
 static struct svga_surface *
 create_backed_surface_view(struct svga_context *svga, struct svga_surface *s,
-                           bool clone_resource)
+                           boolean clone_resource)
 {
    struct svga_texture *tex = svga_texture(s->base.texture);
 
@@ -555,15 +555,15 @@ svga_validate_surface_view(struct svga_context *svga, struct svga_surface *s)
     * associated resource. We will then use the cloned surface view for
     * render target.
     */
-   for (shader = PIPE_SHADER_VERTEX; shader <= PIPE_SHADER_COMPUTE; shader++) {
+   for (shader = PIPE_SHADER_VERTEX; shader <= PIPE_SHADER_TESS_EVAL; shader++) {
       if (svga_check_sampler_view_resource_collision(svga, s->handle, shader)) {
          SVGA_DBG(DEBUG_VIEWS,
                   "same resource used in shaderResource and renderTarget 0x%x\n",
                   s->handle);
-         s = create_backed_surface_view(svga, s, true);
+         s = create_backed_surface_view(svga, s, TRUE);
 
          if (s)
-            svga->state.hw_draw.has_backed_views = true;
+            svga->state.hw_draw.has_backed_views = TRUE;
 
          /* s may be null here if the function failed */
          break;
@@ -575,10 +575,10 @@ svga_validate_surface_view(struct svga_context *svga, struct svga_surface *s)
     * view was created for another context.
     */
    if (s && s->base.context != &svga->pipe) {
-      s = create_backed_surface_view(svga, s, false);
+      s = create_backed_surface_view(svga, s, FALSE);
 
       if (s)
-         svga->state.hw_draw.has_backed_views = true;
+         svga->state.hw_draw.has_backed_views = TRUE;
    }
 
    if (s && s->view_id == SVGA3D_INVALID_ID) {
@@ -586,7 +586,7 @@ svga_validate_surface_view(struct svga_context *svga, struct svga_surface *s)
       SVGA3dRenderTargetViewDesc desc;
       struct svga_texture *stex = svga_texture(s->base.texture);
 
-      if (stex->surface_state < SVGA_SURFACE_STATE_INVALIDATED) {
+      if (stex->validated == FALSE) {
          assert(stex->handle);
 
          /* We are about to render into a surface that has not been validated.
@@ -595,7 +595,7 @@ svga_validate_surface_view(struct svga_context *svga, struct svga_surface *s)
           * content when the associated mob is first bound to the surface.
           */
          SVGA_RETRY(svga, SVGA3D_InvalidateGBSurface(svga->swc, stex->handle));
-         stex->surface_state = SVGA_SURFACE_STATE_INVALIDATED;
+         stex->validated = TRUE;
       }
 
       desc.tex.mipSlice = s->real_level;
@@ -603,48 +603,38 @@ svga_validate_surface_view(struct svga_context *svga, struct svga_surface *s)
       desc.tex.arraySize =
          s->base.u.tex.last_layer - s->base.u.tex.first_layer + 1;
 
+      s->view_id = util_bitmask_add(svga->surface_view_id_bm);
+
       resType = svga_resource_type(s->base.texture->target);
 
       if (util_format_is_depth_or_stencil(s->base.format)) {
-
-         /* Create depth stencil view only if the resource is created
-          * with depth stencil bind flag.
-          */
-         if (stex->key.flags & SVGA3D_SURFACE_BIND_DEPTH_STENCIL) {
-            s->view_id = util_bitmask_add(svga->surface_view_id_bm);
-            ret = SVGA3D_vgpu10_DefineDepthStencilView(svga->swc,
-                                                       s->view_id,
-                                                       s->handle,
-                                                       s->key.format,
-                                                       resType,
-                                                       &desc);
-         }
+         ret = SVGA3D_vgpu10_DefineDepthStencilView(svga->swc,
+                                                    s->view_id,
+                                                    s->handle,
+                                                    s->key.format,
+                                                    resType,
+                                                    &desc);
       }
       else {
-         /* Create render target view only if the resource is created
-          * with render target bind flag.
+         SVGA3dSurfaceFormat view_format = s->key.format;
+         const struct svga_texture *stex = svga_texture(s->base.texture);
+
+         /* Can't create RGBA render target view of a RGBX surface so adjust
+          * the view format.  We do something similar for texture samplers in
+          * svga_validate_pipe_sampler_view().
           */
-         if (stex->key.flags & SVGA3D_SURFACE_BIND_RENDER_TARGET) {
-            SVGA3dSurfaceFormat view_format = s->key.format;
-
-            /* Can't create RGBA render target view of a RGBX surface so adjust
-             * the view format.  We do something similar for texture samplers in
-             * svga_validate_pipe_sampler_view().
-             */
-            if (view_format == SVGA3D_B8G8R8A8_UNORM &&
-                (stex->key.format == SVGA3D_B8G8R8X8_UNORM ||
-                 stex->key.format == SVGA3D_B8G8R8X8_TYPELESS)) {
-               view_format = SVGA3D_B8G8R8X8_UNORM;
-            }
-
-            s->view_id = util_bitmask_add(svga->surface_view_id_bm);
-            ret = SVGA3D_vgpu10_DefineRenderTargetView(svga->swc,
-                                                       s->view_id,
-                                                       s->handle,
-                                                       view_format,
-                                                       resType,
-                                                       &desc);
+         if (view_format == SVGA3D_B8G8R8A8_UNORM &&
+             (stex->key.format == SVGA3D_B8G8R8X8_UNORM ||
+              stex->key.format == SVGA3D_B8G8R8X8_TYPELESS)) {
+            view_format = SVGA3D_B8G8R8X8_UNORM;
          }
+
+         ret = SVGA3D_vgpu10_DefineRenderTargetView(svga->swc,
+                                                    s->view_id,
+                                                    s->handle,
+                                                    view_format,
+                                                    resType,
+                                                    &desc);
       }
 
       if (ret != PIPE_OK) {
@@ -683,9 +673,7 @@ svga_surface_destroy(struct pipe_context *pipe,
     */
    if (s->handle != t->handle && s->handle != t->backed_handle) {
       SVGA_DBG(DEBUG_DMA, "unref sid %p (tex surface)\n", s->handle);
-      svga_screen_surface_destroy(ss, &s->key,
-                                  svga_was_texture_rendered_to(t),
-                                  &s->handle);
+      svga_screen_surface_destroy(ss, &s->key, &s->handle);
    }
 
    if (s->view_id != SVGA3D_INVALID_ID) {
@@ -727,7 +715,7 @@ svga_mark_surface_dirty(struct pipe_surface *surf)
    struct svga_texture *tex = svga_texture(surf->texture);
 
    if (!s->dirty) {
-      s->dirty = true;
+      s->dirty = TRUE;
 
       if (s->handle == tex->handle) {
          /* hmm so 3d textures always have all their slices marked ? */
@@ -783,7 +771,7 @@ svga_mark_surfaces_dirty(struct svga_context *svga)
  */
 void
 svga_propagate_surface(struct svga_context *svga, struct pipe_surface *surf,
-                       bool reset)
+                       boolean reset)
 {
    struct svga_surface *s = svga_surface(surf);
    struct svga_texture *tex = svga_texture(surf->texture);
@@ -897,12 +885,12 @@ svga_propagate_rendertargets(struct svga_context *svga)
    for (i = 0; i < svga->state.hw_clear.num_rendertargets; i++) {
       struct pipe_surface *s = svga->state.hw_clear.rtv[i];
       if (s) {
-         svga_propagate_surface(svga, s, false);
+         svga_propagate_surface(svga, s, FALSE);
       }
    }
 
    if (svga->state.hw_clear.dsv) {
-      svga_propagate_surface(svga, svga->state.hw_clear.dsv, false);
+      svga_propagate_surface(svga, svga->state.hw_clear.dsv, FALSE);
    }
 }
 
@@ -910,7 +898,7 @@ svga_propagate_rendertargets(struct svga_context *svga)
 /**
  * Check if we should call svga_propagate_surface on the surface.
  */
-bool
+boolean
 svga_surface_needs_propagation(const struct pipe_surface *surf)
 {
    const struct svga_surface *s = svga_surface_const(surf);
