@@ -1276,6 +1276,75 @@ NTSTATUS vboxWddmDrvCfgInit(PUNICODE_STRING pRegStr)
     return Status;
 }
 
+NTSTATUS vboxWddmLoggerCreate(PUNICODE_STRING pRegStr)
+{
+    HANDLE hKey;
+    OBJECT_ATTRIBUTES ObjAttr;
+
+    InitializeObjectAttributes(&ObjAttr, pRegStr, OBJ_CASE_INSENSITIVE | OBJ_KERNEL_HANDLE, NULL, NULL);
+
+    NTSTATUS Status = ZwOpenKey(&hKey, GENERIC_READ, &ObjAttr);
+    if (!NT_SUCCESS(Status))
+    {
+        WARN(("ZwOpenKey for settings key failed, Status 0x%x", Status));
+        return Status;
+    }
+
+    union
+    {
+        KEY_VALUE_PARTIAL_INFORMATION   PartialInfo;
+        uint8_t                         abPadding[sizeof(KEY_VALUE_PARTIAL_INFORMATION) + sizeof(WCHAR) * 64];
+        uint64_t                        uAlign;
+    } uBuf;
+
+    UNICODE_STRING ValueName;
+    ULONG cbActual = 0;
+
+    RtlInitUnicodeString(&ValueName, L"VBOXWDDM_RELEASE_LOG");
+    Status = ZwQueryValueKey(hKey, &ValueName, KeyValuePartialInformation, &uBuf, sizeof(uBuf) - sizeof(WCHAR), &cbActual);
+
+    if (!NT_SUCCESS(Status))
+    {
+        ZwClose(hKey);
+        return Status;
+    }
+
+    if (uBuf.PartialInfo.Type != REG_SZ)
+    {
+        ZwClose(hKey);
+        return Status;
+    }
+
+    UNICODE_STRING Value;
+    Value.Buffer = (WCHAR *)uBuf.PartialInfo.Data;
+    Value.Length = uBuf.PartialInfo.DataLength;
+    if (Value.Length >= sizeof(WCHAR) && Value.Buffer[Value.Length / sizeof(WCHAR) - 1] == '\0')
+        Value.Length -= sizeof(WCHAR);
+    Value.MaximumLength = Value.Length + sizeof(WCHAR);
+    Value.Buffer[uBuf.PartialInfo.DataLength / sizeof(WCHAR)] = '\0';
+
+    ANSI_STRING ansiValue;
+
+    Status = RtlUnicodeStringToAnsiString(&ansiValue, &Value, TRUE);
+
+    if (Status == STATUS_SUCCESS)
+    {
+        static const char * const s_apszGroups[] = VBOX_LOGGROUP_NAMES;
+        PRTLOGGER pRelLogger; // +drv_miniport.e.l.l2 is expected
+        int rc = RTLogCreate(&pRelLogger, 0 /*fFlags*/, ansiValue.Buffer, "VBOXWDDM_RELEASE_LOG", RT_ELEMENTS(s_apszGroups), s_apszGroups,
+                        RTLOGDEST_USER, NULL);
+
+        if (RT_SUCCESS(rc))
+            RTLogRelSetDefaultInstance(pRelLogger);
+
+        RtlFreeAnsiString(&ansiValue);
+    }
+
+    ZwClose(hKey);
+
+    return Status;
+}
+
 NTSTATUS vboxWddmThreadCreate(PKTHREAD * ppThread, PKSTART_ROUTINE pStartRoutine, PVOID pStartContext)
 {
     NTSTATUS fStatus;
