@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2016-2023 Oracle and/or its affiliates.
+ * Copyright (C) 2016-2024 Oracle and/or its affiliates.
  *
  * This file is part of VirtualBox base platform packages, as
  * available from https://www.virtualbox.org.
@@ -240,10 +240,17 @@ NTSTATUS GaContextDestroy(PVBOXWDDM_EXT_GA pGaDevExt,
                 void *pvCmd = SvgaCmdBuf3dCmdReserve(pSvga, SVGA_3D_CMD_DX_SET_COTABLE, sizeof(SVGA3dCmdDXSetCOTable), SVGA3D_INVALID_ID);
                 if (pvCmd)
                 {
+                    SVGACOTableType enmType;
+                    if (i < SVGA_COTABLE_MAX)
+                        enmType = (SVGACOTableType)i;
+                    else if (i < SVGA_COTABLE_MAX + (VBSVGA_COTABLE_MAX - VBSVGA_COTABLE_MIN))
+                        enmType = (SVGACOTableType)(i - SVGA_COTABLE_MAX + VBSVGA_COTABLE_MIN);
+                    else
+                        AssertFailedBreak();
                     SVGA3dCmdDXSetCOTable *pCmd = (SVGA3dCmdDXSetCOTable *)pvCmd;
                     pCmd->cid              = pSvgaContext->u32Cid;
                     pCmd->mobid            = SVGA3D_INVALID_ID;
-                    pCmd->type             = (SVGACOTableType)i;
+                    pCmd->type             = enmType;
                     pCmd->validSizeInBytes = 0;
                     SvgaCmdBufCommit(pSvga, sizeof(*pCmd));
                 }
