@@ -275,12 +275,12 @@ DECLCALLBACK(int) vbtrIPCInit(const PVBOXTRAYSVCENV pEnv, void **ppvInstance)
 /**
  * @interface_method_impl{VBOXTRAYSVCDESC,pfnStop}
  */
-DECLCALLBACK(void) VBoxIPCStop(void *pvInstance)
+DECLCALLBACK(int) vbtrIPCStop(void *pvInstance)
 {
     /* Can be NULL if VBoxIPCInit failed. */
     if (!pvInstance)
-        return;
-    AssertPtrReturnVoid(pvInstance);
+        return VINF_SUCCESS;
+    AssertPtrReturn(pvInstance, VERR_INVALID_POINTER);
 
      VBoxTrayInfo("IPC: Stopping worker thread ...\n");
 
@@ -310,7 +310,11 @@ DECLCALLBACK(void) VBoxIPCStop(void *pvInstance)
                 /* Keep going. */
             }
         }
+
+        RTCritSectLeave(&pCtx->CritSect);
     }
+
+    return VINF_SUCCESS;
 }
 
 /**
@@ -468,7 +472,7 @@ static DECLCALLBACK(int) vboxIPCSessionThread(RTTHREAD hThreadSelf, void *pvSess
             break;
         }
         else
-            VBoxTrayError("IPC: Session %#x: Waiting for session data failed with rc=%Rrc\n", hSession, rc);
+            VBoxTrayVerbose(1, "*** Error: IPC: Session %#x: Waiting for session data failed with rc=%Rrc\n", hSession, rc);
     }
 
     VBoxTrayVerbose(1, "IPC: Session thread %#x ended with rc=%Rrc\n", hSession, rc);
@@ -641,6 +645,6 @@ VBOXTRAYSVCDESC g_SvcDescIPC =
     vbtrIPCOption,
     vbtrIPCInit,
     vbtrIPCWorker,
-    NULL /* pfnStop */,
+    vbtrIPCStop,
     vbtrIPCDestroy
 };
