@@ -45,6 +45,11 @@
 #include <iprt/time.h>
 #include <iprt/utf16.h>
 
+#include <package-generated.h>
+#include "product-generated.h"
+
+#include <VBox/version.h>
+
 
 /*********************************************************************************************************************************
 *   Defined Constants And Macros                                                                                                 *
@@ -993,6 +998,18 @@ static RTEXITCODE cmdAddTrustedPublisher(int argc, char **argv)
 }
 
 
+/**
+ * Shows the logo.
+ *
+ * @param   pStream             Output stream to show logo on.
+ */
+static void showLogo(PRTSTREAM pStream)
+{
+    RTStrmPrintf(pStream, VBOX_PRODUCT " VBoxCertUtil Version " VBOX_VERSION_STRING " - r%s\n"
+                 "Copyright (C) " VBOX_C_YEAR " " VBOX_VENDOR "\n\n", RTBldCfgRevisionStr());
+}
+
+
 /**
  * Displays the usage info.
  */
@@ -1044,6 +1061,8 @@ int main(int argc, char **argv)
         VCU_COMMON_OPTION_DEFINITIONS(),
     };
 
+    showLogo(g_pStdOut);
+
     RTGETOPTUNION   ValueUnion;
     RTGETOPTSTATE   GetState;
     RTGetOptInit(&GetState, argc, argv, s_aOptions, RT_ELEMENTS(s_aOptions), 1, 0);
