@@ -43,8 +43,16 @@
 # include <drm/drm_probe_helper.h>
 #endif
 
-#if RTLNX_VER_MIN(5,14,0) || RTLNX_RHEL_RANGE(8,6, 8,99)
-# include <drm/drm_aperture.h>
+#if RTLNX_VER_MIN(6,13,0) && defined(CONFIG_APERTURE_HELPERS) || RTLNX_RHEL_RANGE(9,7, 9,99)
+# include <linux/aperture.h>
+#endif
+
+#if RTLNX_VER_RANGE(5,14,0, 6,13,0) || RTLNX_RHEL_RANGE(8,6, 8,99)
+/** RHEL 9.7 is based on 5.14, however this header is not available
+ * there due to backports from newer kernels. */
+# if !RTLNX_RHEL_RANGE(9,7, 9,99)
+#  include <drm/drm_aperture.h>
+# endif
 #endif
 
 #include "version-generated.h"
@@ -84,7 +92,19 @@ static int vbox_pci_probe(struct pci_dev *pdev, const struct pci_device_id *ent)
 	struct drm_device *dev = NULL;
 	int ret = 0;
 
-# if RTLNX_VER_MIN(5,14,0) || RTLNX_RHEL_RANGE(8,6, 8,99)
+#if RTLNX_VER_MIN(6,0,0)
+	static bool fWarned = false;
+	if (!fWarned)
+	{
+		printk(KERN_ERR "vboxvideo: VM is using legacy graphics controller, "
+				"please consider to configure this guest to use VMSVGA instead\n");
+		fWarned = true;
+	}
+#endif
+
+# if RTLNX_VER_MIN(6,13,0) && defined(CONFIG_APERTURE_HELPERS) || RTLNX_RHEL_RANGE(9,7, 9,99)
+	ret = aperture_remove_conflicting_pci_devices(pdev, driver.name);
+# elif RTLNX_VER_RANGE(5,14,0, 6,13,0) || RTLNX_RHEL_RANGE(8,6, 8,99)
 #  if RTLNX_VER_MIN(5,15,0) || RTLNX_RHEL_RANGE(8,7, 8,99) || RTLNX_RHEL_MIN(9,1) || RTLNX_SUSE_MAJ_PREREQ(15,4)
 	ret = drm_aperture_remove_conflicting_pci_framebuffers(pdev, &driver);
 #  else
@@ -294,6 +314,9 @@ static const struct file_operations vbox_fops = {
 	.compat_ioctl = drm_compat_ioctl,
 #endif
 	.read = drm_read,
+#if RTLNX_VER_MIN(6,12,0)
+	.fop_flags = FOP_UNSIGNED_OFFSET,
+#endif
 };
 
 #if RTLNX_VER_MIN(5,9,0) || RTLNX_RHEL_MIN(8,4) || RTLNX_SUSE_MAJ_PREREQ(15,3)
@@ -359,7 +382,7 @@ static struct drm_driver driver = {
 	.load = vbox_driver_load,
 	.unload = vbox_driver_unload,
 #endif
-#if RTLNX_VER_MAX(6,12,0)
+#if RTLNX_VER_MAX(6,12,0) && !RTLNX_RHEL_RANGE(9,6, 9,99)
 	.lastclose = vbox_driver_lastclose,
 #endif
 	.master_set = vbox_master_set,
@@ -376,7 +399,12 @@ static struct drm_driver driver = {
 #endif
 	.name = DRIVER_NAME,
 	.desc = DRIVER_DESC,
+#if RTLNX_VER_MAX(6,14,0) && !RTLNX_RHEL_RANGE(9,7, 9,99)
 	.date = DRIVER_DATE,
+#endif
+#if RTLNX_VER_MIN(6,15,0) || RTLNX_RHEL_RANGE(9,7, 9,99)
+	.fbdev_probe = vboxfb_create,
+#endif
 	.major = DRIVER_MAJOR,
 	.minor = DRIVER_MINOR,
 	.patchlevel = DRIVER_PATCHLEVEL,
@@ -396,7 +424,9 @@ static struct drm_driver driver = {
 	.prime_fd_to_handle = drm_gem_prime_fd_to_handle,
 #endif
 	.gem_prime_import = drm_gem_prime_import,
+#if RTLNX_VER_MAX(6,15,0) && !RTLNX_RHEL_RANGE(9,7, 9,99)
 	.gem_prime_import_sg_table = vbox_gem_prime_import_sg_table,
+#endif
 #if RTLNX_VER_MAX(6,6,0) && !RTLNX_RHEL_RANGE(9,4, 9,99) && !RTLNX_SUSE_MAJ_PREREQ(15, 6)
 	.gem_prime_mmap = vbox_gem_prime_mmap,
 #endif
