@@ -1,3 +1,4 @@
+/* $XFree86: xc/programs/Xserver/hw/xfree86/ramdac/IBMPriv.h,v 1.1.2.2 1998/07/18 17:54:01 dawes Exp $ */
 
 #ifdef HAVE_XORG_CONFIG_H
 #include <xorg-config.h>
@@ -6,7 +7,7 @@
 #include "IBM.h"
 
 typedef struct {
-	const char *DeviceName;
+	char *DeviceName;
 } xf86IBMramdacInfo;
 
 extern xf86IBMramdacInfo IBMramdacDeviceInfo[];
