@@ -1,3 +1,4 @@
+/* $XFree86: xc/programs/Xserver/hw/xfree86/common/xf86Bus.h,v 1.23 2003/08/24 17:36:50 dawes Exp $ */
 
 /*
  * Copyright (c) 1997-2003 by The XFree86 Project, Inc.
@@ -40,7 +41,7 @@
 #define _XF86_BUS_H
 
 #include "xf86pciBus.h"
-#if defined(__sparc__) || defined(__sparc)
+#ifdef __sparc__
 #include "xf86sbusBus.h"
 #endif
 
@@ -74,6 +75,15 @@ typedef struct {
     IOADDRESS                   domainIO;
 } EntityRec, *EntityPtr;
 
+/* asynchronous event handling */
+#ifdef async
+typedef struct _AsyncQRec {
+    void (*func)(pointer);
+    pointer arg;
+    struct _AsyncQRec *next;
+} AsyncQRec, *AsyncQPtr;
+#endif
+
 #define NO_SEPARATE_IO_FROM_MEM 0x0001
 #define NO_SEPARATE_MEM_FROM_IO 0x0002
 #define NEED_VGA_ROUTED 0x0004
@@ -141,9 +151,12 @@ int xf86AllocateEntity(void);
 BusType StringToBusType(const char* busID, const char **retID);
 memType ChkConflict(resRange *rgp, resPtr res, xf86State state);
 Bool xf86IsSubsetOf(resRange range, resPtr list);
+Bool xf86IsListSubsetOf(resPtr list, resPtr BaseList);
 resPtr xf86ExtractTypeFromList(resPtr list, unsigned long type);
+resPtr findIntersect(resRange Range, resPtr list);
 resPtr xf86FindIntersect(resRange Range, resPtr list);
 void RemoveOverlaps(resPtr target, resPtr list, Bool pow2Alignment,
 		    Bool useEstimated);
+void xf86ConvertListToHost(int entityIndex, resPtr list);
 
 #endif /* _XF86_BUS_H */
