@@ -1,3 +1,4 @@
+/* $XFree86: xc/programs/Xserver/hw/xfree86/ramdac/xf86CursorPriv.h,v 1.4tsi Exp $ */
 
 #ifdef HAVE_XORG_CONFIG_H
 #include <xorg-config.h>
@@ -27,8 +28,9 @@ typedef struct {
     Bool			PalettedCursor;
     ColormapPtr			pInstalledMap;
     Bool                	(*SwitchMode)(int, DisplayModePtr,int);
-    xf86EnableDisableFBAccessProc *EnableDisableFBAccess;
-    CursorPtr                   SavedCursor;
+    Bool                	(*EnterVT)(int, int);
+    void                	(*LeaveVT)(int, int);
+    int				(*SetDGAMode)(int, int, DGADevicePtr);
 
     /* Number of requests to force HW cursor */
     int				ForceHWCursorCount;
@@ -45,6 +47,6 @@ Bool xf86InitHardwareCursor(ScreenPtr pScreen, xf86CursorInfoPtr infoPtr);
 
 CARD32 xf86ReverseBitOrder(CARD32 data);
 
-extern DevPrivateKey xf86CursorScreenKey;
+extern int xf86CursorScreenIndex;
 
 #endif /* _XF86CURSORPRIV_H */
