@@ -340,7 +340,7 @@ xf86TriState;
 
 typedef struct
 {
-	struct xorg_list entry;
+	struct list entry;
 	char **values;
 }
 xf86MatchGroup;
@@ -350,15 +350,15 @@ typedef struct
 	GenericListRec list;
 	char *identifier;
 	char *driver;
-	struct xorg_list match_product;
-	struct xorg_list match_vendor;
-	struct xorg_list match_device;
-	struct xorg_list match_os;
-	struct xorg_list match_pnpid;
-	struct xorg_list match_usbid;
-	struct xorg_list match_driver;
-	struct xorg_list match_tag;
-	struct xorg_list match_layout;
+	struct list match_product;
+	struct list match_vendor;
+	struct list match_device;
+	struct list match_os;
+	struct list match_pnpid;
+	struct list match_usbid;
+	struct list match_driver;
+	struct list match_tag;
+	struct list match_layout;
 	xf86TriState is_keyboard;
 	xf86TriState is_pointer;
 	xf86TriState is_joystick;
@@ -479,7 +479,7 @@ XF86ConfigRec, *XF86ConfigPtr;
 typedef struct
 {
 	int token;			/* id of the token */
-	const char *name;		/* pointer to the LOWERCASED name */
+	char *name;			/* pointer to the LOWERCASED name */
 }
 xf86ConfigSymTabRec, *xf86ConfigSymTabPtr;
 
@@ -487,10 +487,10 @@ xf86ConfigSymTabRec, *xf86ConfigSymTabPtr;
  * prototypes for public functions
  */
 extern void xf86initConfigFiles(void);
-extern char *xf86openConfigFile(const char *path, const char *cmdline,
-				const char *projroot);
-extern char *xf86openConfigDirFiles(const char *path, const char *cmdline,
-				    const char *projroot);
+extern const char *xf86openConfigFile(const char *path, const char *cmdline,
+				      const char *projroot);
+extern const char *xf86openConfigDirFiles(const char *path, const char *cmdline,
+					  const char *projroot);
 extern void xf86setBuiltinConfig(const char *config[]);
 extern XF86ConfigPtr xf86readConfigFile(void);
 extern void xf86closeConfigFile(void);
