@@ -29,9 +29,8 @@
 
 #include "damage.h"
 #include "gcstruct.h"
-#ifdef RENDER
+#include "privates.h"
 # include "picturestr.h"
-#endif
 
 typedef struct _damage {
     DamagePtr		pNext;
@@ -45,10 +44,15 @@ typedef struct _damage {
     DrawablePtr		pDrawable;
     
     DamageReportFunc	damageReport;
+    DamageReportFunc	damageReportPostRendering;
     DamageDestroyFunc	damageDestroy;
+    DamageMarkerFunc	damageMarker;
 
     Bool		reportAfter;
-    RegionRec		pendingDamage;
+    RegionRec		pendingDamage; /* will be flushed post submission at the latest */
+    RegionRec		backupDamage; /* for use with damageMarker */
+    ScreenPtr		pScreen;
+    PrivateRec		*devPrivates;
 } DamageRec;
 
 typedef struct _damageScrPriv {
@@ -60,19 +64,18 @@ typedef struct _damageScrPriv {
      */
     DamagePtr			pScreenDamage;
 
-    PaintWindowBackgroundProcPtr PaintWindowBackground;
-    PaintWindowBorderProcPtr	PaintWindowBorder;
     CopyWindowProcPtr		CopyWindow;
     CloseScreenProcPtr		CloseScreen;
     CreateGCProcPtr		CreateGC;
     DestroyPixmapProcPtr	DestroyPixmap;
     SetWindowPixmapProcPtr	SetWindowPixmap;
     DestroyWindowProcPtr	DestroyWindow;
-#ifdef RENDER
     CompositeProcPtr		Composite;
     GlyphsProcPtr		Glyphs;
-#endif
-    BSFuncRec			BackingStoreFuncs;
+    AddTrapsProcPtr		AddTraps;
+
+    /* Table of wrappable function pointers */
+    DamageScreenFuncsRec	funcs;
 } DamageScrPrivRec, *DamageScrPrivPtr;
 
 typedef struct _damageGCPriv {
@@ -81,31 +84,31 @@ typedef struct _damageGCPriv {
 } DamageGCPrivRec, *DamageGCPrivPtr;
 
 /* XXX should move these into damage.c, damageScrPrivateIndex is static */
-#define damageGetScrPriv(pScr) \
-    ((DamageScrPrivPtr) (pScr)->devPrivates[damageScrPrivateIndex].ptr)
+#define damageGetScrPriv(pScr) ((DamageScrPrivPtr) \
+    dixLookupPrivate(&(pScr)->devPrivates, damageScrPrivateKey))
 
 #define damageScrPriv(pScr) \
     DamageScrPrivPtr    pScrPriv = damageGetScrPriv(pScr)
 
 #define damageGetPixPriv(pPix) \
-    ((DamagePtr) (pPix)->devPrivates[damagePixPrivateIndex].ptr)
+    dixLookupPrivate(&(pPix)->devPrivates, damagePixPrivateKey)
 
 #define damgeSetPixPriv(pPix,v) \
-    ((pPix)->devPrivates[damagePixPrivateIndex].ptr = (pointer ) (v))
+    dixSetPrivate(&(pPix)->devPrivates, damagePixPrivateKey, v)
 
 #define damagePixPriv(pPix) \
     DamagePtr	    pDamage = damageGetPixPriv(pPix)
 
 #define damageGetGCPriv(pGC) \
-    ((DamageGCPrivPtr) (pGC)->devPrivates[damageGCPrivateIndex].ptr)
+    dixLookupPrivate(&(pGC)->devPrivates, damageGCPrivateKey)
 
 #define damageGCPriv(pGC) \
     DamageGCPrivPtr  pGCPriv = damageGetGCPriv(pGC)
 
 #define damageGetWinPriv(pWin) \
-    ((DamagePtr) (pWin)->devPrivates[damageWinPrivateIndex].ptr)
+    ((DamagePtr)dixLookupPrivate(&(pWin)->devPrivates, damageWinPrivateKey))
 
 #define damageSetWinPriv(pWin,d) \
-    ((pWin)->devPrivates[damageWinPrivateIndex].ptr = (d))
+    dixSetPrivate(&(pWin)->devPrivates, damageWinPrivateKey, d)
 
 #endif /* _DAMAGESTR_H_ */
