@@ -1,4 +1,3 @@
-/* $XFree86$ */
 /*
  * Copyright 2001-2004 Red Hat Inc., Durham, North Carolina.
  *
@@ -44,23 +43,21 @@
 
 /** Picture private structure */
 typedef struct _dmxPictPriv {
-    Picture  pict;		/**< Picture ID from back-end server */
-    Mask     savedMask;         /**< Mask of picture attributes saved for
+    Picture pict;               /**< Picture ID from back-end server */
+    Mask savedMask;             /**< Mask of picture attributes saved for
 				 *   lazy window creation. */
 } dmxPictPrivRec, *dmxPictPrivPtr;
 
-
 /** Glyph Set private structure */
 typedef struct _dmxGlyphPriv {
-    GlyphSet  *glyphSets; /**< Glyph Set IDs from back-end server */
+    GlyphSet *glyphSets;  /**< Glyph Set IDs from back-end server */
 } dmxGlyphPrivRec, *dmxGlyphPrivPtr;
 
-
 extern void dmxInitRender(void);
 extern void dmxResetRender(void);
 
 extern Bool dmxPictureInit(ScreenPtr pScreen,
-			   PictFormatPtr formats, int nformats);
+                           PictFormatPtr formats, int nformats);
 
 extern void dmxCreatePictureList(WindowPtr pWindow);
 extern Bool dmxDestroyPictureList(WindowPtr pWindow);
@@ -68,68 +65,52 @@ extern Bool dmxDestroyPictureList(WindowPtr pWindow);
 extern int dmxCreatePicture(PicturePtr pPicture);
 extern void dmxDestroyPicture(PicturePtr pPicture);
 extern int dmxChangePictureClip(PicturePtr pPicture, int clipType,
-				pointer value, int n);
+                                void *value, int n);
 extern void dmxDestroyPictureClip(PicturePtr pPicture);
 extern void dmxChangePicture(PicturePtr pPicture, Mask mask);
 extern void dmxValidatePicture(PicturePtr pPicture, Mask mask);
 extern void dmxComposite(CARD8 op,
-			 PicturePtr pSrc, PicturePtr pMask, PicturePtr pDst,
-			 INT16 xSrc, INT16 ySrc,
-			 INT16 xMask, INT16 yMask,
-			 INT16 xDst, INT16 yDst,
-			 CARD16 width, CARD16 height);
+                         PicturePtr pSrc, PicturePtr pMask, PicturePtr pDst,
+                         INT16 xSrc, INT16 ySrc,
+                         INT16 xMask, INT16 yMask,
+                         INT16 xDst, INT16 yDst, CARD16 width, CARD16 height);
 extern void dmxGlyphs(CARD8 op,
-		      PicturePtr pSrc, PicturePtr pDst,
-		      PictFormatPtr maskFormat,
-		      INT16 xSrc, INT16 ySrc,
-		      int nlists, GlyphListPtr lists, GlyphPtr *glyphs);
+                      PicturePtr pSrc, PicturePtr pDst,
+                      PictFormatPtr maskFormat,
+                      INT16 xSrc, INT16 ySrc,
+                      int nlists, GlyphListPtr lists, GlyphPtr * glyphs);
 extern void dmxCompositeRects(CARD8 op,
-			      PicturePtr pDst,
-			      xRenderColor *color,
-			      int nRect, xRectangle *rects);
+                              PicturePtr pDst,
+                              xRenderColor * color,
+                              int nRect, xRectangle *rects);
 extern Bool dmxInitIndexed(ScreenPtr pScreen, PictFormatPtr pFormat);
 extern void dmxCloseIndexed(ScreenPtr pScreen, PictFormatPtr pFormat);
 extern void dmxUpdateIndexed(ScreenPtr pScreen, PictFormatPtr pFormat,
-			     int ndef, xColorItem *pdef);
+                             int ndef, xColorItem * pdef);
 extern void dmxTrapezoids(CARD8 op,
-			  PicturePtr pSrc, PicturePtr pDst,
-			  PictFormatPtr maskFormat,
-			  INT16 xSrc, INT16 ySrc,
-			  int ntrap, xTrapezoid *traps);
+                          PicturePtr pSrc, PicturePtr pDst,
+                          PictFormatPtr maskFormat,
+                          INT16 xSrc, INT16 ySrc,
+                          int ntrap, xTrapezoid * traps);
 extern void dmxTriangles(CARD8 op,
-			 PicturePtr pSrc, PicturePtr pDst,
-			 PictFormatPtr maskFormat,
-			 INT16 xSrc, INT16 ySrc,
-			 int ntri, xTriangle *tris);
-extern void dmxTriStrip(CARD8 op,
-			PicturePtr pSrc, PicturePtr pDst,
-			PictFormatPtr maskFormat,
-			INT16 xSrc, INT16 ySrc,
-			int npoint, xPointFixed *points);
-extern void dmxTriFan(CARD8 op,
-		      PicturePtr pSrc, PicturePtr pDst,
-		      PictFormatPtr maskFormat,
-		      INT16 xSrc, INT16 ySrc,
-		      int npoint, xPointFixed *points);
+                         PicturePtr pSrc, PicturePtr pDst,
+                         PictFormatPtr maskFormat,
+                         INT16 xSrc, INT16 ySrc, int ntri, xTriangle * tris);
 
 extern int dmxBECreateGlyphSet(int idx, GlyphSetPtr glyphSet);
 extern Bool dmxBEFreeGlyphSet(ScreenPtr pScreen, GlyphSetPtr glyphSet);
 extern int dmxBECreatePicture(PicturePtr pPicture);
 extern Bool dmxBEFreePicture(PicturePtr pPicture);
 
-extern int dmxPictPrivateIndex;		/**< Index for picture private data */
-extern int dmxGlyphSetPrivateIndex;	/**< Index for glyphset private data */
-
-
 /** Get the picture private data given a picture pointer */
 #define DMX_GET_PICT_PRIV(_pPict)					\
-    (dmxPictPrivPtr)(_pPict)->devPrivates[dmxPictPrivateIndex].ptr
+    (dmxPictPrivPtr)dixLookupPrivate(&(_pPict)->devPrivates, dmxPictPrivateKey)
 
 /** Set the glyphset private data given a glyphset pointer */
 #define DMX_SET_GLYPH_PRIV(_pGlyph, _pPriv)				\
-    GlyphSetSetPrivate((_pGlyph), dmxGlyphSetPrivateIndex, (_pPriv))
+    GlyphSetSetPrivate((_pGlyph), dmxGlyphSetPrivateKey, (_pPriv))
 /** Get the glyphset private data given a glyphset pointer */
 #define DMX_GET_GLYPH_PRIV(_pGlyph)					\
-    (dmxGlyphPrivPtr)GlyphSetGetPrivate((_pGlyph), dmxGlyphSetPrivateIndex)
+    (dmxGlyphPrivPtr)GlyphSetGetPrivate((_pGlyph), dmxGlyphSetPrivateKey)
 
-#endif /* DMXPICT_H */
+#endif                          /* DMXPICT_H */
