@@ -26,6 +26,8 @@
 #ifndef _XORG_LIST_H_
 #define _XORG_LIST_H_
 
+#include <stddef.h> /* offsetof() */
+
 /**
  * @file Classic doubly-link circular list implementation.
  * For real usage examples of the linked list, see the file test/list.c
@@ -117,7 +119,7 @@ struct xorg_list {
  *
  * @param The list to initialized.
  */
-static void
+static inline void
 xorg_list_init(struct xorg_list *list)
 {
     list->next = list->prev = list;
@@ -125,8 +127,7 @@ xorg_list_init(struct xorg_list *list)
 
 static inline void
 __xorg_list_add(struct xorg_list *entry,
-	    struct xorg_list *prev,
-	    struct xorg_list *next)
+                struct xorg_list *prev, struct xorg_list *next)
 {
     next->prev = entry;
     entry->next = next;
@@ -176,7 +177,6 @@ xorg_list_append(struct xorg_list *entry, struct xorg_list *head)
     __xorg_list_add(entry, head->prev, head);
 }
 
-
 static inline void
 __xorg_list_del(struct xorg_list *prev, struct xorg_list *next)
 {
@@ -213,7 +213,7 @@ xorg_list_del(struct xorg_list *entry)
  *
  * @return True if the list contains one or more elements or False otherwise.
  */
-static inline Bool
+static inline int
 xorg_list_is_empty(struct xorg_list *head)
 {
     return head->next == head;
@@ -234,7 +234,7 @@ xorg_list_is_empty(struct xorg_list *head)
  */
 #ifndef container_of
 #define container_of(ptr, type, member) \
-    (type *)((char *)(ptr) - (char *) &((type *)0)->member)
+    (type *)((char *)(ptr) - offsetof(type, member))
 #endif
 
 /**
@@ -273,9 +273,20 @@ xorg_list_is_empty(struct xorg_list *head)
 #define xorg_list_last_entry(ptr, type, member) \
     xorg_list_entry((ptr)->prev, type, member)
 
-#define __container_of(ptr, sample, member)				\
-    (void *)((char *)(ptr)						\
-	     - ((char *)&(sample)->member - (char *)(sample)))
+#ifdef HAVE_TYPEOF
+#define __container_of(ptr, sample, member)			\
+    container_of(ptr, typeof(*sample), member)
+#else
+/* This implementation of __container_of has undefined behavior according
+ * to the C standard, but it works in many cases.  If your compiler doesn't
+ * support typeof() and fails with this implementation, please try a newer
+ * compiler.
+ */
+#define __container_of(ptr, sample, member)                            \
+    (void *)((char *)(ptr)                                             \
+            - ((char *)&(sample)->member - (char *)(sample)))
+#endif
+
 /**
  * Loop through the list given by head and set pos to struct in the list.
  *
@@ -311,8 +322,6 @@ xorg_list_is_empty(struct xorg_list *head)
 	 &pos->member != (head);					\
 	 pos = tmp, tmp = __container_of(pos->member.next, tmp, member))
 
-
-
 /* NULL-Terminated List Interface
  *
  * The interface below does _not_ use the struct xorg_list as described above.
@@ -349,7 +358,7 @@ xorg_list_is_empty(struct xorg_list *head)
  * struct foo *element = list;
  * while ((element = nt_list_next(element, next)) { }
  *
- * This macro is not safe for node deletion. Use xorg_list_for_each_entry_safe
+ * This macro is not safe for node deletion. Use nt_list_for_each_entry_safe
  * instead.
  *
  * @param list The list or current element.
@@ -391,7 +400,6 @@ xorg_list_is_empty(struct xorg_list *head)
 		_entry;							\
 		_entry = _tmp, _tmp = (_tmp) ? (_tmp)->_member: NULL)
 
-
 /**
  * Append the element to the end of the list. This macro may be used to
  * merge two lists.
@@ -458,7 +466,7 @@ xorg_list_is_empty(struct xorg_list *head)
 #define nt_list_del(_entry, _list, _type, _member)		\
 	do {							\
 		_type *__e = _entry;				\
-		if (__e == NULL) break;				\
+		if (__e == NULL || _list == NULL) break;        \
 		if ((_list) == __e) {				\
 		    _list = __e->_member;			\
 		} else {					\
@@ -477,10 +485,8 @@ xorg_list_is_empty(struct xorg_list *head)
  * list functions. Unfortunately, the xf86OptionRec uses it and we can't
  * easily get rid of it. Do not use for new code.
  */
-typedef struct generic_list_rec
-{
-	void *next;
-}
-GenericListRec, *GenericListPtr, *glp;
+typedef struct generic_list_rec {
+    void *next;
+} GenericListRec, *GenericListPtr, *glp;
 
 #endif
