@@ -42,22 +42,20 @@
 #define _XF86MODULE_H
 
 #include "misc.h"
+#include "extension.h"
 #ifndef NULL
 #define NULL ((void *)0)
 #endif
 
 typedef enum {
-    LD_RESOLV_IFDONE		= 0,	/* only check if no more
-					   delays pending */
-    LD_RESOLV_NOW		= 1,	/* finish one delay step */
-    LD_RESOLV_FORCE		= 2	/* force checking... */
+    LD_RESOLV_IFDONE = 0,       /* only check if no more
+                                   delays pending */
+    LD_RESOLV_NOW = 1,          /* finish one delay step */
+    LD_RESOLV_FORCE = 2         /* force checking... */
 } LoaderResolveOptions;
 
 #define DEFAULT_LIST ((char *)-1)
 
-/* This indicates a special module that doesn't have the usual entry point */
-#define EXTERN_MODULE ((pointer)-1)
-
 /* Built-in ABI classes.  These definitions must not be changed. */
 #define ABI_CLASS_NONE		NULL
 #define ABI_CLASS_ANSIC		"X.Org ANSI C Emulation"
@@ -82,9 +80,9 @@ typedef enum {
  * mask is 0xFFFF0000.
  */
 #define ABI_ANSIC_VERSION	SET_ABI_VERSION(0, 4)
-#define ABI_VIDEODRV_VERSION	SET_ABI_VERSION(5, 0)
-#define ABI_XINPUT_VERSION	SET_ABI_VERSION(4, 0)
-#define ABI_EXTENSION_VERSION	SET_ABI_VERSION(2, 0)
+#define ABI_VIDEODRV_VERSION	SET_ABI_VERSION(20, 0)
+#define ABI_XINPUT_VERSION	SET_ABI_VERSION(22, 1)
+#define ABI_EXTENSION_VERSION	SET_ABI_VERSION(9, 0)
 #define ABI_FONT_VERSION	SET_ABI_VERSION(0, 6)
 
 #define MODINFOSTRING1	0xef23fdc5
@@ -97,21 +95,21 @@ typedef enum {
 /* Error return codes for errmaj.  New codes must only be added at the end. */
 typedef enum {
     LDR_NOERROR = 0,
-    LDR_NOMEM,		/* memory allocation failed */
-    LDR_NOENT,		/* Module file does not exist */
-    LDR_NOSUBENT,	/* pre-requsite file to be sub-loaded does not exist */
-    LDR_NOSPACE,	/* internal module array full */
-    LDR_NOMODOPEN,	/* module file could not be opened (check errmin) */
-    LDR_UNKTYPE,	/* file is not a recognized module type */
-    LDR_NOLOAD,		/* type specific loader failed */
-    LDR_ONCEONLY,	/* Module should only be loaded once (not an error) */
-    LDR_NOPORTOPEN,	/* could not open port (check errmin) */
-    LDR_NOHARDWARE,	/* could not query/initialize the hardware device */
-    LDR_MISMATCH,	/* the module didn't match the spec'd requirments */
-    LDR_BADUSAGE,	/* LoadModule is called with bad arguments */
-    LDR_INVALID,	/* The module doesn't have a valid ModuleData object */
-    LDR_BADOS,		/* The module doesn't support the OS */
-    LDR_MODSPECIFIC	/* A module-specific error in the SetupProc */
+    LDR_NOMEM,                  /* memory allocation failed */
+    LDR_NOENT,                  /* Module file does not exist */
+    LDR_NOSUBENT,               /* pre-requsite file to be sub-loaded does not exist */
+    LDR_NOSPACE,                /* internal module array full */
+    LDR_NOMODOPEN,              /* module file could not be opened (check errmin) */
+    LDR_UNKTYPE,                /* file is not a recognized module type */
+    LDR_NOLOAD,                 /* type specific loader failed */
+    LDR_ONCEONLY,               /* Module should only be loaded once (not an error) */
+    LDR_NOPORTOPEN,             /* could not open port (check errmin) */
+    LDR_NOHARDWARE,             /* could not query/initialize the hardware device */
+    LDR_MISMATCH,               /* the module didn't match the spec'd requirments */
+    LDR_BADUSAGE,               /* LoadModule is called with bad arguments */
+    LDR_INVALID,                /* The module doesn't have a valid ModuleData object */
+    LDR_BADOS,                  /* The module doesn't support the OS */
+    LDR_MODSPECIFIC             /* A module-specific error in the SetupProc */
 } LoaderErrorCode;
 
 /*
@@ -129,19 +127,19 @@ typedef enum {
 
 /* This structure is expected to be returned by the initfunc */
 typedef struct {
-    const char * modname;	/* name of module, e.g. "foo" */
-    const char * vendor;	/* vendor specific string */
-    CARD32	 _modinfo1_;	/* constant MODINFOSTRING1/2 to find */
-    CARD32	 _modinfo2_;	/* infoarea with a binary editor or sign tool */
-    CARD32	 xf86version;	/* contains XF86_VERSION_CURRENT */
-    CARD8	 majorversion;	/* module-specific major version */
-    CARD8	 minorversion;	/* module-specific minor version */
-    CARD16	 patchlevel;	/* module-specific patch level */
-    const char * abiclass;	/* ABI class that the module uses */
-    CARD32	 abiversion;	/* ABI version */
-    const char * moduleclass;	/* module class description */
-    CARD32	 checksum[4];	/* contains a digital signature of the */
-				/* version info structure */
+    const char *modname;        /* name of module, e.g. "foo" */
+    const char *vendor;         /* vendor specific string */
+    CARD32 _modinfo1_;          /* constant MODINFOSTRING1/2 to find */
+    CARD32 _modinfo2_;          /* infoarea with a binary editor or sign tool */
+    CARD32 xf86version;         /* contains XF86_VERSION_CURRENT */
+    CARD8 majorversion;         /* module-specific major version */
+    CARD8 minorversion;         /* module-specific minor version */
+    CARD16 patchlevel;          /* module-specific patch level */
+    const char *abiclass;       /* ABI class that the module uses */
+    CARD32 abiversion;          /* ABI version */
+    const char *moduleclass;    /* module class description */
+    CARD32 checksum[4];         /* contains a digital signature of the */
+    /* version info structure */
 } XF86ModuleVersionInfo;
 
 /*
@@ -149,12 +147,12 @@ typedef struct {
  * specify version and/or ABI requirements.
  */
 typedef struct {
-    CARD8	 majorversion;	/* module-specific major version */
-    CARD8	 minorversion;	/* moudle-specific minor version */
-    CARD16	 patchlevel;	/* module-specific patch level */
-    const char * abiclass;	/* ABI class that the module uses */
-    CARD32	 abiversion;	/* ABI version */
-    const char * moduleclass;	/* module class */
+    CARD8 majorversion;         /* module-specific major version */
+    CARD8 minorversion;         /* moudle-specific minor version */
+    CARD16 patchlevel;          /* module-specific patch level */
+    const char *abiclass;       /* ABI class that the module uses */
+    CARD32 abiversion;          /* ABI version */
+    const char *moduleclass;    /* module class */
 } XF86ModReqInfo;
 
 /* values to indicate unspecified fields in XF86ModReqInfo. */
@@ -171,54 +169,31 @@ typedef struct {
 
 #define INITARGS void
 
-typedef void (*InitExtension)(INITARGS);
-
-typedef struct {
-    InitExtension	initFunc;
-    const char *	name;
-    Bool		*disablePtr;
-    InitExtension	setupFunc;	
-    const char **	initDependencies;
-} ExtensionModule;
-
-extern ExtensionModule *ExtensionModuleList;
-
 /* Prototypes for Loader functions that are exported to modules */
-#ifndef IN_LOADER
-/* Prototypes with opaque pointers for use by modules */
-pointer LoadSubModule(pointer, const char *, const char **,
-		      const char **, pointer, const XF86ModReqInfo *,
-		      int *, int *);
-pointer LoadSubModuleLocal(pointer, const char *, const char **,
-			   const char **, pointer, const XF86ModReqInfo *,
-			   int *, int *);
-void UnloadSubModule(pointer);
-void UnloadModule (pointer);
-#endif
-pointer LoaderSymbol(const char *);
-pointer LoaderSymbolLocal(pointer module, const char *);
-char **LoaderListDirs(const char **, const char **);
-void LoaderFreeDirList(char **);
-void LoaderErrorMsg(const char *, const char *, int, int);
-void LoadExtension(ExtensionModule *, Bool);
-void LoaderRefSymLists(const char **, ...);
-void LoaderRefSymbols(const char *, ...);
-void LoaderReqSymLists(const char **, ...);
-void LoaderReqSymbols(const char *, ...);
-int LoaderCheckUnresolved(int);
-void LoaderGetOS(const char **name, int *major, int *minor, int *teeny);
-Bool LoaderShouldIgnoreABI(void);
-int LoaderGetABIVersion(const char *abiclass);
+extern _X_EXPORT void *LoadSubModule(void *, const char *, const char **,
+                                       const char **, void *,
+                                       const XF86ModReqInfo *, int *, int *);
+extern _X_EXPORT void UnloadSubModule(void *);
+extern _X_EXPORT void UnloadModule(void *);
+extern _X_EXPORT void *LoaderSymbol(const char *);
+extern _X_EXPORT const char **LoaderListDirs(const char **, const char **);
+extern _X_EXPORT void LoaderFreeDirList(char **);
+extern _X_EXPORT void LoaderErrorMsg(const char *, const char *, int, int);
+extern _X_EXPORT void LoaderGetOS(const char **name, int *major, int *minor,
+                                  int *teeny);
+extern _X_EXPORT Bool LoaderShouldIgnoreABI(void);
+extern _X_EXPORT int LoaderGetABIVersion(const char *abiclass);
 
-typedef pointer (*ModuleSetupProc)(pointer, pointer, int *, int *);
-typedef void (*ModuleTearDownProc)(pointer);
-#define MODULESETUPPROTO(func) pointer func(pointer, pointer, int*, int*)
-#define MODULETEARDOWNPROTO(func) void func(pointer)
+typedef void *(*ModuleSetupProc) (void *, void *, int *, int *);
+typedef void (*ModuleTearDownProc) (void *);
+
+#define MODULESETUPPROTO(func) void *func(void *, void *, int*, int*)
+#define MODULETEARDOWNPROTO(func) void func(void *)
 
 typedef struct {
-    XF86ModuleVersionInfo *	vers;
-    ModuleSetupProc		setup;
-    ModuleTearDownProc		teardown;
+    XF86ModuleVersionInfo *vers;
+    ModuleSetupProc setup;
+    ModuleTearDownProc teardown;
 } XF86ModuleData;
 
-#endif /* _XF86STR_H */
+#endif                          /* _XF86STR_H */
