@@ -1,4 +1,3 @@
-/* $XFree86$ */
 /*
  * Copyright 2001-2003 Red Hat Inc., Durham, North Carolina.
  *
@@ -51,6 +50,10 @@
 #ifndef DMX_H
 #define DMX_H
 
+#if HAVE_DMX_CONFIG_H
+#include <dmx-config.h>
+#endif
+
 #include "gcstruct.h"
 
 /* Handle client-side include files in one place. */
@@ -206,8 +209,6 @@ typedef struct _DMXScreenInfo {
     UnrealizeWindowProcPtr         UnrealizeWindow;
     RestackWindowProcPtr           RestackWindow;
     WindowExposuresProcPtr         WindowExposures;
-    PaintWindowBackgroundProcPtr   PaintWindowBackground;
-    PaintWindowBorderProcPtr       PaintWindowBorder;
     CopyWindowProcPtr              CopyWindow;
 
     ResizeWindowProcPtr            ResizeWindow;
