@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2006-2020 Oracle Corporation
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
@@ -218,7 +228,7 @@ int dbgcBpExec(PDBGC pDbgc, RTUINT iBp)
 
     /* Execute the command. */
     pDbgc->pszScratch = pDbgc->pszScratch + pBp->cchCmd + 1;
-    int rc = dbgcEvalCommand(pDbgc, pszScratch, pBp->cchCmd, false /* fNoExecute */);
+    int rc = dbgcEvalCommands(pDbgc, pszScratch, pBp->cchCmd, false /* fNoExecute */);
 
     /* Restore the scratch state. */
     pDbgc->iArg         = iArg;
@@ -227,3 +237,141 @@ int dbgcBpExec(PDBGC pDbgc, RTUINT iBp)
     return rc;
 }
 
+
+
+
+//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//
+//
+//
+//      F l o w T r a c e   M a n a g e m e n t
+//
+//
+//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//
+
+
+
+/**
+ * Returns the trace flow module matching the given id or NULL if not found.
+ *
+ * @returns Pointer to the trace flow module or NULL if not found.
+ * @param   pDbgc         The DBGC instance.
+ * @param   iTraceFlowMod The trace flow module identifier.
+ */
+DECLHIDDEN(PDBGCTFLOW) dbgcFlowTraceModGet(PDBGC pDbgc, uint32_t iTraceFlowMod)
+{
+    PDBGCTFLOW pIt;
+    RTListForEach(&pDbgc->LstTraceFlowMods, pIt, DBGCTFLOW, NdTraceFlow)
+    {
+        if (pIt->iTraceFlowMod == iTraceFlowMod)
+            return pIt;
+    }
+
+    return NULL;
+}
+
+
+/**
+ * Inserts the given trace flow module into the list.
+ *
+ * @param   pDbgc         The DBGC instance.
+ * @param   pTraceFlow    The trace flow module.
+ */
+static void dbgcFlowTraceModInsert(PDBGC pDbgc, PDBGCTFLOW pTraceFlow)
+{
+    PDBGCTFLOW pIt = RTListGetLast(&pDbgc->LstTraceFlowMods, DBGCTFLOW, NdTraceFlow);
+
+    if (   !pIt
+        || pIt->iTraceFlowMod < pTraceFlow->iTraceFlowMod)
+        RTListAppend(&pDbgc->LstTraceFlowMods, &pTraceFlow->NdTraceFlow);
+    else
+    {
+        RTListForEach(&pDbgc->LstTraceFlowMods, pIt, DBGCTFLOW, NdTraceFlow)
+        {
+            if (pIt->iTraceFlowMod < pTraceFlow->iTraceFlowMod)
+            {
+                RTListNodeInsertBefore(&pIt->NdTraceFlow, &pTraceFlow->NdTraceFlow);
+                break;
+            }
+        }
+    }
+}
+
+
+/**
+ * Returns the smallest free flow trace mod identifier.
+ *
+ * @returns Free flow trace mod identifier.
+ * @param   pDbgc         The DBGC instance.
+ */
+static uint32_t dbgcFlowTraceModIdFindFree(PDBGC pDbgc)
+{
+    uint32_t iId = 0;
+
+    PDBGCTFLOW pIt;
+    RTListForEach(&pDbgc->LstTraceFlowMods, pIt, DBGCTFLOW, NdTraceFlow)
+    {
+        PDBGCTFLOW pNext = RTListGetNext(&pDbgc->LstTraceFlowMods, pIt, DBGCTFLOW, NdTraceFlow);
+        if (   (   pNext
+                && pIt->iTraceFlowMod + 1 != pNext->iTraceFlowMod)
+            || !pNext)
+        {
+            iId = pIt->iTraceFlowMod + 1;
+            break;
+        }
+    }
+
+    return iId;
+}
+
+
+/**
+ * Adds a flow trace module to the debugger console.
+ *
+ * @returns VBox status code.
+ * @param   pDbgc         The DBGC instance.
+ * @param   hFlowTraceMod The flow trace module to add.
+ * @param   hFlow         The control flow graph to add.
+ * @param   piId          Where to store the ID of the module on success.
+ */
+DECLHIDDEN(int) dbgcFlowTraceModAdd(PDBGC pDbgc, DBGFFLOWTRACEMOD hFlowTraceMod, DBGFFLOW hFlow, uint32_t *piId)
+{
+    /*
+     * Add the module.
+     */
+    PDBGCTFLOW pTraceFlow = (PDBGCTFLOW)RTMemAlloc(sizeof(DBGCTFLOW));
+    if (!pTraceFlow)
+        return VERR_NO_MEMORY;
+
+    pTraceFlow->hTraceFlowMod = hFlowTraceMod;
+    pTraceFlow->hFlow         = hFlow;
+    pTraceFlow->iTraceFlowMod = dbgcFlowTraceModIdFindFree(pDbgc);
+    dbgcFlowTraceModInsert(pDbgc, pTraceFlow);
+
+    *piId = pTraceFlow->iTraceFlowMod;
+
+    return VINF_SUCCESS;
+}
+
+
+/**
+ * Deletes a breakpoint.
+ *
+ * @returns VBox status code.
+ * @param   pDbgc       The DBGC instance.
+ * @param   iTraceFlowMod The trace flow module identifier.
+ */
+DECLHIDDEN(int) dbgcFlowTraceModDelete(PDBGC pDbgc, uint32_t iTraceFlowMod)
+{
+    int rc = VINF_SUCCESS;
+    PDBGCTFLOW pTraceFlow = dbgcFlowTraceModGet(pDbgc, iTraceFlowMod);
+    if (pTraceFlow)
+    {
+        RTListNodeRemove(&pTraceFlow->NdTraceFlow);
+        RTMemFree(pTraceFlow);
+    }
+    else
+        rc = VERR_DBGC_BP_NOT_FOUND;
+
+    return rc;
+}
+
