@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2022 Oracle and/or its affiliates.
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
  * This file is part of VirtualBox base platform packages, as
  * available from https://www.virtualbox.org.
@@ -63,7 +63,6 @@ static FNDBGCCMD dbgcCmdQuit;
 static FNDBGCCMD dbgcCmdStop;
 static FNDBGCCMD dbgcCmdDetect;
 static FNDBGCCMD dbgcCmdDmesg;
-extern FNDBGCCMD dbgcCmdDumpImage;
 static FNDBGCCMD dbgcCmdCpu;
 static FNDBGCCMD dbgcCmdInfo;
 static FNDBGCCMD dbgcCmdLog;
@@ -81,6 +80,7 @@ static FNDBGCCMD dbgcCmdSet;
 static FNDBGCCMD dbgcCmdUnset;
 static FNDBGCCMD dbgcCmdLoadVars;
 static FNDBGCCMD dbgcCmdShowVars;
+static FNDBGCCMD dbgcCmdSleep;
 static FNDBGCCMD dbgcCmdLoadPlugIn;
 static FNDBGCCMD dbgcCmdUnloadPlugIn;
 static FNDBGCCMD dbgcCmdHarakiri;
@@ -135,6 +135,7 @@ static const DBGCVARDESC    g_aArgDmesg[] =
 static const DBGCVARDESC    g_aArgDumpImage[] =
 {
     /* cTimesMin,   cTimesMax,  enmCategory,            fFlags,                         pszName,        pszDescription */
+    {  0,           ~0U,        DBGCVAR_CAT_OPTION_STRING, 0,                           "options",      "Options." },
     {  1,           ~0U,        DBGCVAR_CAT_POINTER,    0,                              "address",      "Address of image to dump." },
 };
 
@@ -229,6 +230,12 @@ static const DBGCVARDESC    g_aArgMultiStep[] =
     {  0,           1,          DBGCVAR_CAT_NUMBER_NO_RANGE, DBGCVD_FLAGS_DEP_PREV,     "stride",       "The length of each step, defaults to 1." },
 };
 
+/** ntrbtree arguments. */
+static const DBGCVARDESC    g_aArgNtRbTree[] =
+{
+    /* cTimesMin,   cTimesMax,  enmCategory,            fFlags,                         pszName,        pszDescription */
+    {  1,           1,          DBGCVAR_CAT_GC_POINTER, 0,                              "rootaddr",     "The address of _RTL_RB_TREE" },
+};
 
 /** loadplugin, unloadplugin. */
 static const DBGCVARDESC    g_aArgPlugIn[] =
@@ -246,6 +253,13 @@ static const DBGCVARDESC    g_aArgSet[] =
     {  1,           1,          DBGCVAR_CAT_ANY,        0,                              "value",        "Value to assign to the variable." },
 };
 
+/** 'sleep' arguments */
+static const DBGCVARDESC    g_aArgSleep[] =
+{
+    /* cTimesMin,   cTimesMax,  enmCategory,            fFlags,                         pszName,        pszDescription */
+    {  1,           1,          DBGCVAR_CAT_NUMBER,     0,                              "milliseconds", "The sleep interval in milliseconds (max 30000ms)." },
+};
+
 /** 'stop' arguments */
 static const DBGCVARDESC    g_aArgStop[] =
 {
@@ -318,12 +332,14 @@ const DBGCCMD    g_aDbgcCmds[] =
     { "logdest",    0,        1,        &g_aArgLogDest[0],   RT_ELEMENTS(g_aArgLogDest),   0, dbgcCmdLogDest,   "[dest string]",        "Displays or modifies the logging destination (VBOX_LOG_DEST)." },
     { "logflags",   0,        1,        &g_aArgLogFlags[0],  RT_ELEMENTS(g_aArgLogFlags),  0, dbgcCmdLogFlags,  "[flags string]",       "Displays or modifies the logging flags (VBOX_LOG_FLAGS)." },
     { "logflush",   0,        0,        NULL,                0,                            0, dbgcCmdLogFlush,  "",                     "Flushes the log buffers." },
-    { "multistep",  0,        2,        &g_aArgMultiStep[0], RT_ELEMENTS(g_aArgMultiStep), 0, dbgcCmdMultiStep, "[count [stride]",              "Performs the specified number of step-into operations. Stops early if non-step event occurs." },
+    { "multistep",  0,        2,        &g_aArgMultiStep[0], RT_ELEMENTS(g_aArgMultiStep), 0, dbgcCmdMultiStep, "[count [stride]",      "Performs the specified number of step-into operations. Stops early if non-step event occurs." },
+    { "ntrbtree",   1,        1,        &g_aArgNtRbTree[0],  RT_ELEMENTS(g_aArgNtRbTree),  0, dbgcCmdNtRbTree,  "<rb-tree-address>",    "Windows: Dumps a red-black tree." },
     { "quit",       0,        0,        NULL,                0,                            0, dbgcCmdQuit,      "",                     "Exits the debugger." },
     { "runscript",  1,        1,        &g_aArgFilename[0],  RT_ELEMENTS(g_aArgFilename),  0, dbgcCmdRunScript, "<filename>",           "Runs the command listed in the script. Lines starting with '#' "
                                                                                                                                         "(after removing blanks) are comment. blank lines are ignored. Stops on failure." },
     { "set",        2,        2,        &g_aArgSet[0],       RT_ELEMENTS(g_aArgSet),       0, dbgcCmdSet,       "<var> <value>",        "Sets a global variable." },
     { "showvars",   0,        0,        NULL,                0,                            0, dbgcCmdShowVars,  "",                     "List all the defined variables." },
+    { "sleep",      1,        1,        &g_aArgSleep[0],     RT_ELEMENTS(g_aArgSleep),     0, dbgcCmdSleep,     "<milliseconds>",       "Sleeps for the given number of milliseconds (max 30000)." },
     { "stop",       0,        1,        &g_aArgStop[0],      RT_ELEMENTS(g_aArgStop),      0, dbgcCmdStop,      "[idCpu]",              "Stop execution either of all or the specified CPU. (The latter is not recommended unless you know exactly what you're doing.)" },
     { "unload",     1,       ~0U,       &g_aArgUnload[0],    RT_ELEMENTS(g_aArgUnload),    0, dbgcCmdUnload,    "<modname1> [modname2..N]", "Unloads one or more modules in the current address space." },
     { "unloadplugin", 1,     ~0U,       &g_aArgPlugIn[0],    RT_ELEMENTS(g_aArgPlugIn),    0, dbgcCmdUnloadPlugIn, "<plugin1> [plugin2..N]", "Unloads one or more plugins." },
@@ -1303,7 +1319,7 @@ static DECLCALLBACK(int) dbgcCmdFormat(PCDBGCCMD pCmd, PDBGCCMDHLP pCmdHlp, PUVM
     {
         "none", "bytes", "elements"
     };
-    int rc;
+    int rc = VINF_SUCCESS;
 
     for (unsigned iArg = 0; iArg < cArgs; iArg++)
     {
@@ -1416,7 +1432,7 @@ static DECLCALLBACK(int) dbgcCmdFormat(PCDBGCCMD pCmd, PDBGCCMDHLP pCmdHlp, PUVM
     } /* arg loop */
 
     NOREF(pCmd); NOREF(pUVM);
-    return 0;
+    return rc;
 }
 
 
@@ -1861,6 +1877,18 @@ static DECLCALLBACK(int) dbgcCmdShowVars(PCDBGCCMD pCmd, PDBGCCMDHLP pCmdHlp, PU
 }
 
 
+/**
+ * @callback_method_impl{FNDBGCCMD, The 'sleep' command.}
+ */
+static DECLCALLBACK(int) dbgcCmdSleep(PCDBGCCMD pCmd, PDBGCCMDHLP pCmdHlp, PUVM pUVM, PCDBGCVAR paArgs, unsigned cArgs)
+{
+    RT_NOREF(pCmd, pCmdHlp, pUVM, cArgs);
+    /** @todo make this interruptible. For now the command is limited to 30 sec. */
+    RTThreadSleep(RT_MIN((RTMSINTERVAL)paArgs[0].u.u64Number, RT_MS_30SEC));
+    return 0;
+}
+
+
 /**
  * @callback_method_impl{FNDBGCCMD, The 'loadplugin' command.}
  */
