@@ -4,21 +4,31 @@
  */
 
 /*
- * Copyright (C) 2011 Oracle Corporation
+ * Copyright (C) 2011-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #define LOG_GROUP LOG_GROUP_DEV_PCI_RAW
 #include <VBox/log.h>
 #include <VBox/sup.h>
@@ -27,7 +37,7 @@
 #include <VBox/vmm/pdm.h>
 #include <VBox/vmm/gvm.h>
 #include <VBox/vmm/gvmm.h>
-#include <VBox/vmm/vm.h>
+#include <VBox/vmm/vmcc.h>
 
 #include <iprt/asm.h>
 #include <iprt/assert.h>
@@ -42,9 +52,9 @@
 #include <iprt/asm-amd64-x86.h>
 
 
-/*******************************************************************************
-*   Structures and Typedefs                                                    *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Structures and Typedefs                                                                                                      *
+*********************************************************************************************************************************/
 typedef struct PCIRAWSRVSTATE
 {
     /** Structure lock. */
@@ -114,7 +124,7 @@ static DECLCALLBACK(bool) pcirawr0Isr(void* pContext, int32_t iHostIrq)
 
     RTSpinlockAcquire(pThis->hSpinlock);
     pThis->iPendingIrq = iHostIrq;
-    RTSpinlockReleaseNoInts(pThis->hSpinlock);
+    RTSpinlockRelease(pThis->hSpinlock);
 
     /**
      * @todo RTSemEventSignal() docs claims that it's platform-dependent
@@ -169,46 +179,33 @@ PCIRAWR0DECL(void) PciRawR0Term(void)
 /**
  * Per-VM R0 module init.
  */
-PCIRAWR0DECL(int)  PciRawR0InitVM(PVM pVM)
+PCIRAWR0DECL(int)  PciRawR0InitVM(PGVM pGVM)
 {
     PRAWPCIFACTORY pFactory = NULL;
-    int            rc;
-
-    rc = SUPR0ComponentQueryFactory(pVM->pSession, "VBoxRawPci", RAWPCIFACTORY_UUID_STR, (void **)&pFactory);
-
+    int rc = SUPR0ComponentQueryFactory(pGVM->pSession, "VBoxRawPci", RAWPCIFACTORY_UUID_STR, (void **)&pFactory);
     if (RT_SUCCESS(rc))
     {
         if (pFactory)
         {
-            PGVM pGVM = NULL;
-            rc = GVMMR0ByVM(pVM, &pGVM);
-            if (RT_SUCCESS(rc))
-                rc = pFactory->pfnInitVm(pFactory, pVM, &pGVM->rawpci.s);
+            rc = pFactory->pfnInitVm(pFactory, pGVM, &pGVM->rawpci.s);
             pFactory->pfnRelease(pFactory);
         }
     }
-
     return VINF_SUCCESS;
 }
 
 /**
  * Per-VM R0 module termination routine.
  */
-PCIRAWR0DECL(void)  PciRawR0TermVM(PVM pVM)
+PCIRAWR0DECL(void)  PciRawR0TermVM(PGVM pGVM)
 {
     PRAWPCIFACTORY pFactory = NULL;
-    int            rc;
-
-    rc = SUPR0ComponentQueryFactory(pVM->pSession, "VBoxRawPci", RAWPCIFACTORY_UUID_STR, (void **)&pFactory);
-
+    int rc = SUPR0ComponentQueryFactory(pGVM->pSession, "VBoxRawPci", RAWPCIFACTORY_UUID_STR, (void **)&pFactory);
     if (RT_SUCCESS(rc))
     {
         if (pFactory)
         {
-            PGVM pGVM = NULL;
-            rc = GVMMR0ByVM(pVM, &pGVM);
-            if (RT_SUCCESS(rc))
-                pFactory->pfnDeinitVm(pFactory, pVM, &pGVM->rawpci.s);
+            pFactory->pfnDeinitVm(pFactory, pGVM, &pGVM->rawpci.s);
             pFactory->pfnRelease(pFactory);
         }
     }
@@ -270,7 +267,7 @@ typedef struct DUMMYRAWPCIINS
 typedef struct DUMMYRAWPCIINS *PDUMMYRAWPCIINS;
 
 #define DEVPORT_2_DUMMYRAWPCIINS(pPort) \
-    ( (PDUMMYRAWPCIINS)((uint8_t *)pPort - RT_OFFSETOF(DUMMYRAWPCIINS, DevPort)) )
+    ( (PDUMMYRAWPCIINS)((uint8_t *)pPort - RT_UOFFSETOF(DUMMYRAWPCIINS, DevPort)) )
 
 static uint8_t dummyPciGetByte(PDUMMYRAWPCIINS pThis, uint32_t iRegister)
 {
@@ -500,7 +497,8 @@ static PRAWPCIDEVPORT pcirawr0CreateDummyDevice(uint32_t HostDevice, uint32_t fF
 
     return &pNew->DevPort;
 }
-#endif
+
+#endif /* DEBUG_nike */
 
 static DECLCALLBACK(void) pcirawr0DevObjDestructor(void *pvObj, void *pvIns, void *pvUnused)
 {
@@ -517,29 +515,29 @@ static DECLCALLBACK(void) pcirawr0DevObjDestructor(void *pvObj, void *pvIns, voi
 }
 
 
-static int pcirawr0OpenDevice(PSUPDRVSESSION   pSession,
-                              PVM              pVM,
+static int pcirawr0OpenDevice(PGVM pGVM, PSUPDRVSESSION pSession,
                               uint32_t         HostDevice,
                               uint32_t         fFlags,
                               PCIRAWDEVHANDLE *pHandle,
                               uint32_t        *pfDevFlags)
 {
+
+    int rc = GVMMR0ValidateGVMandEMT(pGVM, 0 /*idCpu*/);
+    if (RT_FAILURE(rc))
+        return rc;
+
     /*
      * Query the factory we want, then use it create and connect the host device.
      */
-    PRAWPCIFACTORY pFactory = NULL;
-    PRAWPCIDEVPORT pDevPort = NULL;
-    int rc;
-    PPCIRAWDEV     pNew;
-
-    pNew = (PPCIRAWDEV)RTMemAllocZ(sizeof(*pNew));
+    PPCIRAWDEV pNew = (PPCIRAWDEV)RTMemAllocZ(sizeof(*pNew));
     if (!pNew)
         return VERR_NO_MEMORY;
 
-
+    PRAWPCIFACTORY pFactory = NULL;
     rc = SUPR0ComponentQueryFactory(pSession, "VBoxRawPci", RAWPCIFACTORY_UUID_STR, (void **)&pFactory);
     /* No host driver registered, provide some fake implementation
        for debugging purposes. */
+    PRAWPCIDEVPORT pDevPort = NULL;
 #ifdef DEBUG_nike
     if (rc == VERR_SUPDRV_COMPONENT_NOT_FOUND)
     {
@@ -558,16 +556,12 @@ static int pcirawr0OpenDevice(PSUPDRVSESSION   pSession,
     {
         if (pFactory)
         {
-             PGVM pGVM = NULL;
-             rc = GVMMR0ByVM(pVM, &pGVM);
-
-            if (RT_SUCCESS(rc))
-                rc = pFactory->pfnCreateAndConnect(pFactory,
-                                                   HostDevice,
-                                                   fFlags,
-                                                   &pGVM->rawpci.s,
-                                                   &pDevPort,
-                                                   pfDevFlags);
+            rc = pFactory->pfnCreateAndConnect(pFactory,
+                                               HostDevice,
+                                               fFlags,
+                                               &pGVM->rawpci.s,
+                                               &pDevPort,
+                                               pfDevFlags);
             pFactory->pfnRelease(pFactory);
         }
 
@@ -575,26 +569,32 @@ static int pcirawr0OpenDevice(PSUPDRVSESSION   pSession,
         {
             rc = RTSpinlockCreate(&pNew->hSpinlock, RTSPINLOCK_FLAGS_INTERRUPT_SAFE, "PciRaw");
             AssertRC(rc);
-            rc = RTSemEventCreate(&pNew->hIrqEvent);
-            AssertRC(rc);
-
-            pNew->pSession = pSession;
-            pNew->pPort    = pDevPort;
-            pNew->pvObj = SUPR0ObjRegister(pSession, SUPDRVOBJTYPE_RAW_PCI_DEVICE,
-                                           pcirawr0DevObjDestructor, pNew, NULL);
-
-            uint32_t hHandle = 0;
-            rc = RTHandleTableAllocWithCtx(g_State.hHtDevs, pNew, pSession, &hHandle);
             if (RT_SUCCESS(rc))
             {
-                pNew->hHandle = (PCIRAWDEVHANDLE)hHandle;
-                *pHandle = pNew->hHandle;
-            }
-            else
-            {
-                SUPR0ObjRelease(pNew->pvObj, pSession);
+                rc = RTSemEventCreate(&pNew->hIrqEvent);
+                AssertRC(rc);
+                if (RT_SUCCESS(rc))
+                {
+                    pNew->pSession = pSession;
+                    pNew->pPort    = pDevPort;
+                    pNew->pvObj    = SUPR0ObjRegister(pSession, SUPDRVOBJTYPE_RAW_PCI_DEVICE,
+                                                      pcirawr0DevObjDestructor, pNew, NULL);
+                    if (pNew->pvObj)
+                    {
+
+                        uint32_t hHandle = 0;
+                        rc = RTHandleTableAllocWithCtx(g_State.hHtDevs, pNew, pSession, &hHandle);
+                        if (RT_SUCCESS(rc))
+                        {
+                            pNew->hHandle = (PCIRAWDEVHANDLE)hHandle;
+                            *pHandle = pNew->hHandle;
+                            return rc;
+                        }
+                        SUPR0ObjRelease(pNew->pvObj, pSession);
+                    }
+                    RTSemEventDestroy(pNew->hIrqEvent);
+                }
                 RTSpinlockDestroy(pNew->hSpinlock);
-                RTSemEventDestroy(pNew->hIrqEvent);
             }
         }
     }
@@ -871,7 +871,7 @@ static int pcirawr0GetIrq(PSUPDRVSESSION    pSession,
     iPendingIrq = pDev->iPendingIrq;
     pDev->iPendingIrq = 0;
     fTerminate = pDev->fTerminate;
-    RTSpinlockReleaseNoInts(pDev->hSpinlock);
+    RTSpinlockRelease(pDev->hSpinlock);
 
     /* Block until new IRQs arrives */
     if (!fTerminate)
@@ -881,13 +881,13 @@ static int pcirawr0GetIrq(PSUPDRVSESSION    pSession,
             rc = RTSemEventWaitNoResume(pDev->hIrqEvent, iTimeout);
             if (RT_SUCCESS(rc))
             {
-                /** @todo: racy */
+                /** @todo racy */
                 if (!ASMAtomicReadBool(&pDev->fTerminate))
                 {
                     RTSpinlockAcquire(pDev->hSpinlock);
                     iPendingIrq = pDev->iPendingIrq;
                     pDev->iPendingIrq = 0;
-                    RTSpinlockReleaseNoInts(pDev->hSpinlock);
+                    RTSpinlockRelease(pDev->hSpinlock);
                 }
                 else
                     rc = VERR_INTERRUPTED;
@@ -925,7 +925,7 @@ static int pcirawr0PowerStateChange(PSUPDRVSESSION    pSession,
  *
  * @returns VBox status code.
  */
-PCIRAWR0DECL(int) PciRawR0ProcessReq(PSUPDRVSESSION pSession, PVM pVM, PPCIRAWSENDREQ pReq)
+PCIRAWR0DECL(int) PciRawR0ProcessReq(PGVM pGVM, PSUPDRVSESSION pSession, PPCIRAWSENDREQ pReq)
 {
     LogFlow(("PciRawR0ProcessReq: %d for %x\n", pReq->iRequest, pReq->TargetDevice));
     int rc = VINF_SUCCESS;
@@ -934,7 +934,7 @@ PCIRAWR0DECL(int) PciRawR0ProcessReq(PSUPDRVSESSION pSession, PVM pVM, PPCIRAWSE
     switch (pReq->iRequest)
     {
         case PCIRAWR0_DO_OPEN_DEVICE:
-            rc = pcirawr0OpenDevice(pSession, pVM,
+            rc = pcirawr0OpenDevice(pGVM, pSession,
                                     pReq->u.aOpenDevice.PciAddress,
                                     pReq->u.aOpenDevice.fFlags,
                                     &pReq->u.aOpenDevice.Device,
@@ -1038,3 +1038,4 @@ PCIRAWR0DECL(int) PciRawR0ProcessReq(PSUPDRVSESSION pSession, PVM pVM, PPCIRAWSE
     LogFlow(("PciRawR0ProcessReq: returns %Rrc\n", rc));
     return rc;
 }
+
