@@ -4,24 +4,34 @@
  */
 
 /*
- * Copyright (C) 2009-2019 Oracle Corporation
+ * Copyright (C) 2009-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 /*
@@ -311,10 +321,10 @@ VBoxVgaControllerDriverSupported (
   //
   //  if (((Pci.Hdr.Command & 0x01) == 0x01)) {
   //
-  // See if this is a VirtualBox VGA PCI controller
+  // See if this is a VirtualBox VGA or VMSVGA II PCI controller
   //
-  if (Pci.Hdr.VendorId == VBOX_VENDOR_ID) {
-    if (Pci.Hdr.DeviceId == VBOX_VGA_DEVICE_ID) {
+  if ( ((Pci.Hdr.VendorId == VBOX_VENDOR_ID) && (Pci.Hdr.DeviceId == VBOX_VGA_DEVICE_ID))
+    || ((Pci.Hdr.VendorId == VMSVGA_VENDOR_ID) && (Pci.Hdr.DeviceId == VMSVGA_II_DEVICE_ID))) {
 
       Status = EFI_SUCCESS;
       if (RemainingDevicePath != NULL) {
@@ -336,7 +346,6 @@ VBoxVgaControllerDriverSupported (
           }
         }
       }
-    }
   }
 
 Done:
@@ -374,6 +383,7 @@ VBoxVgaControllerDriverStart (
   BOOLEAN                         PciAttributesSaved;
   EFI_DEVICE_PATH_PROTOCOL        *ParentDevicePath;
   ACPI_ADR_DEVICE_PATH            AcpiDeviceNode;
+  PCI_TYPE00                      Pci;
 
   PciAttributesSaved = FALSE;
   //
@@ -406,6 +416,23 @@ VBoxVgaControllerDriverStart (
     goto Error;
   }
 
+  //
+  // Read the PCI Configuration Header from the PCI Device again to figure out the model.
+  //
+  Status = Private->PciIo->Pci.Read (
+                                   Private->PciIo,
+                                   EfiPciIoWidthUint32,
+                                   0,
+                                   sizeof (Pci) / sizeof (UINT32),
+                                   &Pci
+                                   );
+  if (EFI_ERROR (Status)) {
+    DEBUG((DEBUG_INFO, "%a:%d status:%r\n", __FILE__, __LINE__, Status));
+    goto Error;
+  }
+
+  Private->DeviceType = Pci.Hdr.DeviceId;
+
   //
   // Save original PCI attributes
   //
@@ -487,9 +514,56 @@ VBoxVgaControllerDriverStart (
   }
 
   //
-  // Get VRAM size, needed for constructing a correct video mode list
+  // Now do some model-specific setup.
   //
-  Private->VRAMSize = ASMInU32(VBE_DISPI_IOPORT_DATA);
+  if (Private->DeviceType == VMSVGA_II_DEVICE_ID) {
+      EFI_ACPI_ADDRESS_SPACE_DESCRIPTOR     *IOPortDesc;
+
+      // VMSVGA
+      Private->BarIndexFB = 1;
+
+      Private->PciIo->GetBarAttributes (
+                          Private->PciIo,
+                          0,    // BAR 0 is the I/O port space
+                          NULL,
+                          (VOID**) &IOPortDesc
+                          );
+      Private->IOBase = (UINT16)IOPortDesc->AddrRangeMin;
+
+      //
+      // Query the VRAM size (for proper mode filtering)
+      //
+      ASMOutU32(Private->IOBase + SVGA_INDEX_PORT, SVGA_REG_VRAM_SIZE);
+      Private->VRAMSize = ASMInU32(Private->IOBase + SVGA_VALUE_PORT);
+
+#if 0
+      // Not used because of buggy emulation(?) which is not fully compatible
+      // with the simple "legacy" VMSVGA II register interface.
+
+      // Enable the device, set initial mode
+      ASMOutU32(Private->IOBase + SVGA_INDEX_PORT, SVGA_REG_WIDTH);
+      ASMOutU32(Private->IOBase + SVGA_VALUE_PORT, 1024);
+      ASMOutU32(Private->IOBase + SVGA_INDEX_PORT, SVGA_REG_HEIGHT);
+      ASMOutU32(Private->IOBase + SVGA_VALUE_PORT, 768);
+      ASMOutU32(Private->IOBase + SVGA_INDEX_PORT, SVGA_REG_BYTES_PER_LINE);
+      ASMOutU32(Private->IOBase + SVGA_VALUE_PORT, 768 * 4);
+      ASMOutU32(Private->IOBase + SVGA_INDEX_PORT, SVGA_REG_BITS_PER_PIXEL);
+      ASMOutU32(Private->IOBase + SVGA_VALUE_PORT, 32);
+      ASMOutU32(Private->IOBase + SVGA_INDEX_PORT, SVGA_REG_CONFIG_DONE);
+      ASMOutU32(Private->IOBase + SVGA_VALUE_PORT, 1);
+
+      ASMOutU32(Private->IOBase + SVGA_INDEX_PORT, SVGA_REG_ENABLE);
+      ASMOutU32(Private->IOBase + SVGA_VALUE_PORT, SVGA_REG_ENABLE_ENABLE);
+#endif
+  } else {
+      // VBoxVGA / VBoxSVGA
+      Private->BarIndexFB = 0;
+      //
+      // Get VRAM size, needed for constructing a correct video mode list
+      //
+      Private->VRAMSize = ASMInU32(VBE_DISPI_IOPORT_DATA);
+  }
+
 
   //
   // Construct video mode list
@@ -895,7 +969,7 @@ ClearScreen (
   Private->PciIo->Mem.Write (
                         Private->PciIo,
                         EfiPciIoWidthFillUint32,
-                        0,
+                        Private->BarIndexFB,
                         0,
                           Private->ModeData[Private->CurrentMode].HorizontalResolution
                         * Private->ModeData[Private->CurrentMode].VerticalResolution,
@@ -1042,7 +1116,7 @@ GetFrameBufferInfo(IN  APPLE_FRAMEBUFFERINFO_PROTOCOL   *This,
 
     Private->PciIo->GetBarAttributes (
                         Private->PciIo,
-                        0,
+                        Private->BarIndexFB,
                         NULL,
                         (VOID**) &FrameBufDesc
                         );
