@@ -4,18 +4,28 @@
  */
 
 /*
- * Copyright (C) 2014-2016 Oracle Corporation
+ * Copyright (C) 2014-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
-#include <iprt/err.h>
+#include <iprt/errcore.h>
 #include <iprt/mem.h>
 #include <iprt/assert.h>
 #include <iprt/log.h>
@@ -45,7 +55,7 @@ extern void (*glXGetProcAddress(const GLubyte *procname))( void );
 
 #endif
 
-#undef GL_EXT_FUNCS_GEN 
+#undef GL_EXT_FUNCS_GEN
 #define GL_EXT_FUNCS_GEN \
     /* GL_ARB_shader_objects */ \
     USE_GL_FUNC(WINED3D_PFNGLGETOBJECTPARAMETERIVARBPROC, \
@@ -225,7 +235,12 @@ SHADERDECL(int) ShaderContextCreate(void **ppShaderContext)
     pContext->pDeviceContext->shader_backend = &glsl_shader_backend;
     pContext->pDeviceContext->ps_selected_mode = SHADER_GLSL;
     pContext->pDeviceContext->vs_selected_mode = SHADER_GLSL;
+#ifndef VBOX_WITH_VMSVGA
     pContext->render_offscreen = false;
+#else
+    /* VMSVGA always renders offscreen. */
+    pContext->render_offscreen = true;
+#endif
 
     list_init(&pContext->pDeviceContext->shaders);
 
@@ -308,7 +323,7 @@ SHADERDECL(int) ShaderContextDestroy(void *pShaderContext)
     return VINF_SUCCESS;
 }
 
-SHADERDECL(int) ShaderCreateVertexShader(void *pShaderContext, const uint32_t *pShaderData, void **pShaderObj)
+SHADERDECL(int) ShaderCreateVertexShader(void *pShaderContext, const uint32_t *pShaderData, uint32_t cbShaderData, void **pShaderObj)
 {
     IWineD3DDeviceImpl *This;
     IWineD3DVertexShaderImpl *object;
@@ -324,6 +339,8 @@ SHADERDECL(int) ShaderCreateVertexShader(void *pShaderContext, const uint32_t *p
         return VERR_NO_MEMORY;
     }
 
+    object->baseShader.functionLength = cbShaderData;
+
     hr = vertexshader_init(object, This, (DWORD const *)pShaderData, NULL, NULL, NULL);
     if (FAILED(hr))
     {
@@ -332,6 +349,14 @@ SHADERDECL(int) ShaderCreateVertexShader(void *pShaderContext, const uint32_t *p
         return VERR_INTERNAL_ERROR;
     }
 
+    /* Tweak the float constants limit to use a greater number of constants.
+     * Keep some space for the internal usage.
+     * The shader creation code artificially sets the limit according to D3D shader version.
+     * But the guest may use more constants and we are not required to strictly follow D3D specs.
+     */
+    object->baseShader.limits.constant_float = RT_MAX(g_adapter.gl_info.limits.glsl_vs_float_constants / 2,
+                                                      object->baseShader.limits.constant_float);
+
 #ifdef VBOX_WINE_WITH_SHADER_CACHE
     object = vertexshader_check_cached(This, object);
 #endif
@@ -342,7 +367,7 @@ SHADERDECL(int) ShaderCreateVertexShader(void *pShaderContext, const uint32_t *p
     return VINF_SUCCESS;
 }
 
-SHADERDECL(int) ShaderCreatePixelShader(void *pShaderContext, const uint32_t *pShaderData, void **pShaderObj)
+SHADERDECL(int) ShaderCreatePixelShader(void *pShaderContext, const uint32_t *pShaderData, uint32_t cbShaderData, void **pShaderObj)
 {
     IWineD3DDeviceImpl *This;
     IWineD3DPixelShaderImpl *object;
@@ -358,6 +383,8 @@ SHADERDECL(int) ShaderCreatePixelShader(void *pShaderContext, const uint32_t *pS
         return VERR_NO_MEMORY;
     }
 
+    object->baseShader.functionLength = cbShaderData;
+
     hr = pixelshader_init(object, This, (DWORD const *)pShaderData, NULL, NULL, NULL);
     if (FAILED(hr))
     {
@@ -366,6 +393,14 @@ SHADERDECL(int) ShaderCreatePixelShader(void *pShaderContext, const uint32_t *pS
         return VERR_INTERNAL_ERROR;
     }
 
+    /* Tweak the float constants limit to use a greater number of constants.
+     * Keep some space for the internal usage.
+     * The shader creation code artificially sets the limit according to D3D shader version.
+     * But the guest may use more constants and we are not required to strictly follow D3D specs.
+     */
+    object->baseShader.limits.constant_float = RT_MAX(g_adapter.gl_info.limits.glsl_ps_float_constants / 2,
+                                                      object->baseShader.limits.constant_float);
+
 #ifdef VBOX_WINE_WITH_SHADER_CACHE
     object = pixelshader_check_cached(This, object);
 #endif
@@ -383,7 +418,7 @@ SHADERDECL(int) ShaderDestroyVertexShader(void *pShaderContext, void *pShaderObj
     SHADER_SET_CURRENT_CONTEXT(pShaderContext);
 
     object->lpVtbl->Release((IWineD3DVertexShader *)object);
-	return VINF_SUCCESS;
+        return VINF_SUCCESS;
 }
 
 SHADERDECL(int) ShaderDestroyPixelShader(void *pShaderContext, void *pShaderObj)
@@ -394,7 +429,7 @@ SHADERDECL(int) ShaderDestroyPixelShader(void *pShaderContext, void *pShaderObj)
     SHADER_SET_CURRENT_CONTEXT(pShaderContext);
 
     object->lpVtbl->Release((IWineD3DPixelShader *)object);
-	return VINF_SUCCESS;
+        return VINF_SUCCESS;
 }
 
 SHADERDECL(int) ShaderSetVertexShader(void *pShaderContext, void *pShaderObj)
@@ -464,8 +499,7 @@ SHADERDECL(int) ShaderSetVertexShaderConstantB(void *pShaderContext, uint32_t st
     SHADER_SET_CURRENT_CONTEXT(pShaderContext);
     This = g_pCurrentContext->pDeviceContext;
 
-    Log(("(ShaderSetVertexShaderConstantB %p, srcData %p, start %d, count %d)\n",
-            srcData, start, count));
+    Log(("(ShaderSetVertexShaderConstantB %p, srcData %p, start %d, count %d)\n", pShaderContext, srcData, start, count));
 
     if (!srcData || start >= MAX_CONST_B)
     {
@@ -494,8 +528,7 @@ SHADERDECL(int) ShaderSetVertexShaderConstantI(void *pShaderContext, uint32_t st
     SHADER_SET_CURRENT_CONTEXT(pShaderContext);
     This = g_pCurrentContext->pDeviceContext;
 
-    Log(("(ShaderSetVertexShaderConstantI %p, srcData %p, start %d, count %d)\n",
-            srcData, start, count));
+    Log(("(ShaderSetVertexShaderConstantI %p, srcData %p, start %d, count %d)\n", pShaderContext, srcData, start, count));
 
     if (!srcData || start >= MAX_CONST_I)
     {
@@ -521,8 +554,7 @@ SHADERDECL(int) ShaderSetVertexShaderConstantF(void *pShaderContext, uint32_t st
     SHADER_SET_CURRENT_CONTEXT(pShaderContext);
     This = g_pCurrentContext->pDeviceContext;
 
-    Log(("(ShaderSetVertexShaderConstantF %p, srcData %p, start %d, count %d)\n",
-            srcData, start, count));
+    Log(("(ShaderSetVertexShaderConstantF %p, srcData %p, start %d, count %d)\n", pShaderContext, srcData, start, count));
 
     if (srcData == NULL || start + count > This->d3d_vshader_constantF || start > This->d3d_vshader_constantF)
     {
@@ -549,8 +581,7 @@ SHADERDECL(int) ShaderSetPixelShaderConstantB(void *pShaderContext, uint32_t sta
     SHADER_SET_CURRENT_CONTEXT(pShaderContext);
     This = g_pCurrentContext->pDeviceContext;
 
-    Log(("(ShaderSetPixelShaderConstantB %p, srcData %p, start %d, count %d)\n",
-            srcData, start, count));
+    Log(("(ShaderSetPixelShaderConstantB %p, srcData %p, start %d, count %d)\n", pShaderContext, srcData, start, count));
 
     if (!srcData || start >= MAX_CONST_B)
     {
@@ -579,8 +610,7 @@ SHADERDECL(int) ShaderSetPixelShaderConstantI(void *pShaderContext, uint32_t sta
     SHADER_SET_CURRENT_CONTEXT(pShaderContext);
     This = g_pCurrentContext->pDeviceContext;
 
-    Log(("(ShaderSetPixelShaderConstantI %p, srcData %p, start %d, count %d)\n",
-            srcData, start, count));
+    Log(("(ShaderSetPixelShaderConstantI %p, srcData %p, start %d, count %d)\n", pShaderContext, srcData, start, count));
 
     if (!srcData || start >= MAX_CONST_I)
     {
@@ -606,8 +636,7 @@ SHADERDECL(int) ShaderSetPixelShaderConstantF(void *pShaderContext, uint32_t sta
     SHADER_SET_CURRENT_CONTEXT(pShaderContext);
     This = g_pCurrentContext->pDeviceContext;
 
-    Log(("(ShaderSetPixelShaderConstantF %p, srcData %p, start %d, count %d)\n",
-            srcData, start, count));
+    Log(("(ShaderSetPixelShaderConstantF %p, srcData %p, start %d, count %d)\n", pShaderContext, srcData, start, count));
 
     if (srcData == NULL || start + count > This->d3d_pshader_constantF || start > This->d3d_pshader_constantF)
     {
@@ -639,7 +668,7 @@ SHADERDECL(int) ShaderSetPositionTransformed(void *pShaderContext, unsigned cxVi
         return VINF_SUCCESS;    /* no changes; nothing to do. */
 
     Log(("ShaderSetPositionTransformed viewport (%d,%d) fPreTransformed=%d\n", cxViewPort, cyViewPort, fPreTransformed));
-    
+
     if (fPreTransformed)
     {   /* In the pre-transformed vertex coordinate case we need to disable all transformations as we're already using screen coordinates. */
         /* Load the identity matrix for the model view */
@@ -678,7 +707,7 @@ SHADERDECL(int) ShaderUpdateState(void *pShaderContext, uint32_t rtHeight)
 
     pThis->rtHeight = rtHeight;
 
-    /* @todo missing state:
+    /** @todo missing state:
      * - fog enable (stateblock->renderState[WINED3DRS_FOGENABLE])
      * - fog mode (stateblock->renderState[WINED3DRS_FOGTABLEMODE])
      * - stateblock->vertexDecl->position_transformed
@@ -754,7 +783,7 @@ SHADERDECL(int) ShaderTransformProjection(unsigned cxViewPort, unsigned cyViewPo
 
     glTranslatef(xoffset, -yoffset, -1.0f);
 
-    if (fPretransformed) 
+    if (fPretransformed)
     {
         /* One world coordinate equals one screen pixel; y-inversion no longer an issue */
         glOrtho(0, cxViewPort, 0, cyViewPort, -1, 1);
