@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 /*
@@ -46,11 +56,12 @@
 # include <paths.h>
 #endif
 
-#include <VBox/err.h>
+#include <iprt/errcore.h>
 #include <VBox/vmm/dbgf.h>
 #include <VBox/vmm/pdmdrv.h>
 #include <iprt/assert.h>
 #include <iprt/file.h>
+#include <iprt/path.h>
 #ifndef RT_OS_WINDOWS
 # include <sys/ioctl.h>
 # include <poll.h>
@@ -58,7 +69,7 @@
 #else
 # include <Winnls.h>
 # define _WINSOCK2API_
-# include <IPHlpApi.h>
+# include <iprt/win/iphlpapi.h>
 #endif
 #include <alias.h>
 
@@ -235,7 +246,7 @@
 /*
  * Loging macros
  */
-#if VBOX_WITH_DEBUG_NAT_SOCKETS
+#ifdef VBOX_WITH_DEBUG_NAT_SOCKETS
 # if defined(RT_OS_WINDOWS)
 #  define  DO_LOG_NAT_SOCK(so, proto, winevent, r_fdset, w_fdset, x_fdset)             \
    do {                                                                                \
@@ -253,7 +264,7 @@
                     CHECK_FD_SET(so, ign, nval) ? "RDNVAL":""));                   \
    } while (0)
 # endif /* !RT_OS_WINDOWS */
-#else /* !VBOX_WITH_DEBUG_NAT_SOCKETS */
+#else  /* !VBOX_WITH_DEBUG_NAT_SOCKETS */
 # define DO_LOG_NAT_SOCK(so, proto, winevent, r_fdset, w_fdset, x_fdset) do {} while (0)
 #endif /* !VBOX_WITH_DEBUG_NAT_SOCKETS */
 
@@ -297,7 +308,7 @@ static int slirpVerifyAndFreeSocket(PNATState pData, struct socket *pSocket)
 
 int slirp_init(PNATState *ppData, uint32_t u32NetAddr, uint32_t u32Netmask,
                bool fPassDomain, bool fUseHostResolver, int i32AliasMode,
-               int iIcmpCacheLimit, void *pvUser)
+               int iIcmpCacheLimit, bool fLocalhostReachable, void *pvUser)
 {
     int rc;
     PNATState pData;
@@ -314,6 +325,7 @@ int slirp_init(PNATState *ppData, uint32_t u32NetAddr, uint32_t u32Netmask,
     pData->fPassDomain = !fUseHostResolver ? fPassDomain : false;
     pData->fUseHostResolver = fUseHostResolver;
     pData->fUseHostResolverPermanent = fUseHostResolver;
+    pData->fLocalhostReachable = fLocalhostReachable;
     pData->pvUser = pvUser;
     pData->netmask = u32Netmask;
 
@@ -340,10 +352,10 @@ int slirp_init(PNATState *ppData, uint32_t u32NetAddr, uint32_t u32Netmask,
 
         WSAStartup(MAKEWORD(2, 0), &Data);
 
-        rc = RTLdrLoadSystem("Iphlpapi.dll", /* :fNoUnload */ true, &hLdrMod);
+        rc = RTLdrLoadSystem("Iphlpapi.dll", true /*fNoUnload*/, &hLdrMod);
         if (RT_SUCCESS(rc))
         {
-            rc = RTLdrGetSymbol(hLdrMod, "GetAdaptersAddresses", (void **)&pData->pfGetAdaptersAddresses);
+            rc = RTLdrGetSymbol(hLdrMod, "GetAdaptersAddresses", (void **)&pData->pfnGetAdaptersAddresses);
             if (RT_FAILURE(rc))
                 LogRel(("NAT: Can't find GetAdapterAddresses in Iphlpapi.dll\n"));
 
@@ -372,7 +384,7 @@ int slirp_init(PNATState *ppData, uint32_t u32NetAddr, uint32_t u32Netmask,
     pData->special_addr.s_addr = u32NetAddr;
     pData->slirp_ethaddr = &special_ethaddr[0];
     alias_addr.s_addr = pData->special_addr.s_addr | RT_H2N_U32_C(CTL_ALIAS);
-    /* @todo: add ability to configure this staff */
+    /** @todo add ability to configure this staff */
 
     /*
      * Some guests won't reacquire DHCP lease on link flap when VM is
@@ -387,13 +399,18 @@ int slirp_init(PNATState *ppData, uint32_t u32NetAddr, uint32_t u32Netmask,
     inet_aton("127.0.0.1", &loopback_addr);
 
     rc = slirpTftpInit(pData);
-    AssertRCReturn(rc, VINF_NAT_DNS);
+    AssertRCReturn(rc, rc);
 
     if (i32AliasMode & ~(PKT_ALIAS_LOG|PKT_ALIAS_SAME_PORTS|PKT_ALIAS_PROXY_ONLY))
     {
-        Log(("NAT: alias mode %x is ignored\n", i32AliasMode));
+        LogRel(("NAT: bad alias mode 0x%x ignored\n", i32AliasMode));
         i32AliasMode = 0;
     }
+    else if (i32AliasMode != 0)
+    {
+        LogRel(("NAT: alias mode 0x%x\n", i32AliasMode));
+    }
+
     pData->i32AliasMode = i32AliasMode;
     getouraddr(pData);
     {
@@ -410,14 +427,14 @@ int slirp_init(PNATState *ppData, uint32_t u32NetAddr, uint32_t u32Netmask,
         flags |= PKT_ALIAS_PUNCH_FW;
 #endif
         flags |= pData->i32AliasMode; /* do transparent proxying */
-        flags = LibAliasSetMode(pData->proxy_alias, flags, ~0);
+        flags = LibAliasSetMode(pData->proxy_alias, flags, ~0U);
         proxy_addr.s_addr = RT_H2N_U32(RT_N2H_U32(pData->special_addr.s_addr) | CTL_ALIAS);
         LibAliasSetAddress(pData->proxy_alias, proxy_addr);
         ftp_alias_load(pData);
         nbt_alias_load(pData);
     }
 #ifdef VBOX_WITH_NAT_SEND2HOME
-    /* @todo: we should know all interfaces available on host. */
+    /** @todo we should know all interfaces available on host. */
     pData->pInSockAddrHomeAddress = RTMemAllocZ(sizeof(struct sockaddr));
     pData->cInHomeAddressSize = 1;
     inet_aton("192.168.1.25", &pData->pInSockAddrHomeAddress[0].sin_addr);
@@ -561,12 +578,20 @@ void slirp_term(PNATState pData)
         LIST_REMOVE(ac, list);
         RTMemFree(ac);
     }
+    while (!LIST_EMPTY(&pData->port_forward_rule_head))
+    {
+        struct port_forward_rule *rule = LIST_FIRST(&pData->port_forward_rule_head);
+        LIST_REMOVE(rule, list);
+        RTMemFree(rule);
+    }
     slirpTftpTerm(pData);
     bootp_dhcp_fini(pData);
     m_fini(pData);
 #ifdef RT_OS_WINDOWS
     WSACleanup();
 #endif
+    if (tftp_prefix)
+        RTStrFree((char *)tftp_prefix);
 #ifdef LOG_ENABLED
     Log(("\n"
          "NAT statistics\n"
@@ -674,10 +699,7 @@ void slirp_select_fill(PNATState pData, int *pnfds, struct pollfd *polls)
         so->so_poll_index = -1;
 #endif
         STAM_COUNTER_INC(&pData->StatTCP);
-#ifdef VBOX_WITH_NAT_UDP_SOCKET_CLONE
-        /* TCP socket can't be cloned */
-        Assert((!so->so_cloneOf));
-#endif
+
         /*
          * See if we need a tcp_fasttimo
          */
@@ -733,7 +755,7 @@ void slirp_select_fill(PNATState pData, int *pnfds, struct pollfd *polls)
          * Set for reading (and urgent data) if we are connected, can
          * receive more, and we have room for it XXX /2 ?
          */
-        /* @todo: vvl - check which predicat here will be more useful here in rerm of new sbufs. */
+        /** @todo vvl - check which predicat here will be more useful here in rerm of new sbufs. */
         if (   CONN_CANFRCV(so)
             && (SBUF_LEN(&so->so_snd) < (SBUF_SIZE(&so->so_snd)/2))
 #ifdef RT_OS_WINDOWS
@@ -786,10 +808,6 @@ void slirp_select_fill(PNATState pData, int *pnfds, struct pollfd *polls)
                 CONTINUE_NO_UNLOCK(udp);
             }
         }
-#ifdef VBOX_WITH_NAT_UDP_SOCKET_CLONE
-        if (so->so_cloneOf)
-                CONTINUE_NO_UNLOCK(udp);
-#endif
 
         /*
          * When UDP packets are received from over the link, they're
@@ -951,10 +969,6 @@ void slirp_select_poll(PNATState pData, struct pollfd *polls, int ndfs)
      */
     QSOCKET_FOREACH(so, so_next, tcp)
     /* { */
-        /* TCP socket can't be cloned */
-#ifdef VBOX_WITH_NAT_UDP_SOCKET_CLONE
-        Assert((!so->so_cloneOf));
-#endif
         Assert(!so->fUnderPolling);
         so->fUnderPolling = 1;
         if (slirpVerifyAndFreeSocket(pData, so))
@@ -1069,7 +1083,7 @@ void slirp_select_poll(PNATState pData, struct pollfd *polls, int ndfs)
                 /* Finish connection first */
                 /* should we ignore return value? */
                 bool fRet = slirpConnectOrWrite(pData, so, true);
-                LogFunc(("fRet:%RTbool\n", fRet));
+                LogFunc(("fRet:%RTbool\n", fRet)); NOREF(fRet);
                 if (slirpVerifyAndFreeSocket(pData, so))
                     CONTINUE(tcp);
             }
@@ -1127,7 +1141,11 @@ void slirp_select_poll(PNATState pData, struct pollfd *polls, int ndfs)
             }
 
             /* if socket freed ''so'' is PHANTOM and next socket isn't points on it */
-            if (so_next->so_prev == so)
+            if (so_next->so_prev != so)
+            {
+                CONTINUE(tcp);
+            }
+            else
             {
                 /* mark the socket for termination _after_ it was drained */
                 so->so_close = 1;
@@ -1138,9 +1156,6 @@ void slirp_select_poll(PNATState pData, struct pollfd *polls, int ndfs)
                     sofcantsendmore(so);
 #endif
             }
-            if (so_next->so_prev == so)
-                so->fUnderPolling = 0;
-            CONTINUE(tcp);
         }
 
         /*
@@ -1222,10 +1237,6 @@ void slirp_select_poll(PNATState pData, struct pollfd *polls, int ndfs)
      */
      QSOCKET_FOREACH(so, so_next, udp)
      /* { */
-#ifdef VBOX_WITH_NAT_UDP_SOCKET_CLONE
-        if (so->so_cloneOf)
-            CONTINUE_NO_UNLOCK(udp);
-#endif
 #if 0
         so->fUnderPolling = 1;
         if(slirpVerifyAndFreeSocket(pData, so));
@@ -1253,10 +1264,13 @@ done:
 struct arphdr
 {
     unsigned short  ar_hrd;             /* format of hardware address   */
+#define ARPHRD_ETHER    1               /* ethernet hardware format     */
     unsigned short  ar_pro;             /* format of protocol address   */
     unsigned char   ar_hln;             /* length of hardware address   */
     unsigned char   ar_pln;             /* length of protocol address   */
     unsigned short  ar_op;              /* ARP opcode (command)         */
+#define ARPOP_REQUEST   1               /* ARP request                  */
+#define ARPOP_REPLY     2               /* ARP reply                    */
 
     /*
      *      Ethernet looks like this : This bit is variable sized however...
@@ -1322,12 +1336,23 @@ static void arp_input(PNATState pData, struct mbuf *m)
 {
     struct ethhdr *pEtherHeader;
     struct arphdr *pARPHeader;
+    int ar_op;
     uint32_t ip4TargetAddress;
 
-    int ar_op;
+    /* drivers never return runt packets, so this should never happen */
+    if (RT_UNLIKELY((size_t)m->m_len
+                    < sizeof(struct ethhdr) + sizeof(struct arphdr)))
+        goto done;
+
     pEtherHeader = mtod(m, struct ethhdr *);
     pARPHeader = (struct arphdr *)&pEtherHeader[1];
 
+    if (RT_UNLIKELY(   pARPHeader->ar_hrd != RT_H2N_U16_C(ARPHRD_ETHER)
+                    || pARPHeader->ar_pro != RT_H2N_U16_C(ETH_P_IP)
+                    || pARPHeader->ar_hln != ETH_ALEN
+                    || pARPHeader->ar_pln != sizeof(RTNETADDRIPV4)))
+        goto done;
+
     ar_op = RT_N2H_U16(pARPHeader->ar_op);
     ip4TargetAddress = *(uint32_t*)pARPHeader->ar_tip;
 
@@ -1338,6 +1363,12 @@ static void arp_input(PNATState pData, struct mbuf *m)
                 || CTL_CHECK(ip4TargetAddress, CTL_ALIAS)
                 || CTL_CHECK(ip4TargetAddress, CTL_TFTP))
             {
+#if 0 /* Dropping ARP requests destined for CTL_ALIAS breaks all outgoing traffic completely, so don't do that... */
+                /* Don't reply to ARP requests for the hosts loopback interface if it is disabled. */
+                if (   CTL_CHECK(ip4TargetAddress, CTL_ALIAS)
+                    && !pData->fLocalhostReachable)
+                    break;
+#endif
                 slirp_update_guest_addr_guess(pData, *(uint32_t *)pARPHeader->ar_sip, "arp request");
                 arp_output(pData, pEtherHeader->h_source, pARPHeader, ip4TargetAddress);
                 break;
@@ -1364,6 +1395,7 @@ static void arp_input(PNATState pData, struct mbuf *m)
             break;
     }
 
+  done:
     m_freem(pData, m);
 }
 
@@ -1379,7 +1411,7 @@ void slirp_input(PNATState pData, struct mbuf *m, size_t cbBuf)
     static bool fWarnedIpv6;
     struct ethhdr *eh;
 
-    m->m_len = cbBuf;
+    m->m_len = (int)cbBuf; Assert((size_t)m->m_len == cbBuf);
     if (cbBuf < ETH_HLEN)
     {
         Log(("NAT: packet having size %d has been ignored\n", m->m_len));
@@ -1430,7 +1462,7 @@ void if_encap(PNATState pData, uint16_t eth_proto, struct mbuf *m, int flags)
 {
     struct ethhdr *eh;
     uint8_t *mbuf = NULL;
-    size_t mlen = 0;
+    int mlen;
     STAM_PROFILE_START(&pData->StatIF_encap, a);
     LogFlowFunc(("ENTER: pData:%p, eth_proto:%RX16, m:%p, flags:%d\n",
                 pData, eth_proto, m, flags));
@@ -1666,7 +1698,9 @@ void slirp_post_sent(PNATState pData, void *pvArg)
 void slirp_set_dhcp_TFTP_prefix(PNATState pData, const char *tftpPrefix)
 {
     Log2(("tftp_prefix: %s\n", tftpPrefix));
-    tftp_prefix = tftpPrefix;
+    if (tftp_prefix)
+        RTStrFree((char *)tftp_prefix);
+    tftp_prefix = RTPathAbsDup(tftpPrefix);
 }
 
 void slirp_set_dhcp_TFTP_bootfile(PNATState pData, const char *bootFile)
@@ -1686,12 +1720,32 @@ void slirp_set_dhcp_next_server(PNATState pData, const char *next_server)
 
 int slirp_set_binding_address(PNATState pData, char *addr)
 {
-    if (addr == NULL || (inet_aton(addr, &pData->bindIP) == 0))
+    int ok;
+
+    pData->bindIP.s_addr = INADDR_ANY;
+
+    if (addr == NULL || *addr == '\0')
+        return VINF_SUCCESS;
+
+    ok = inet_aton(addr, &pData->bindIP);
+    if (!ok)
     {
-        pData->bindIP.s_addr = INADDR_ANY;
-        return 1;
+        LogRel(("NAT: Unable to parse binding address: %s\n", addr));
+        return VERR_INVALID_PARAMETER;
     }
-    return 0;
+
+    if (pData->bindIP.s_addr == INADDR_ANY)
+        return VINF_SUCCESS;
+
+    if ((pData->bindIP.s_addr & RT_N2H_U32_C(0xe0000000)) == RT_N2H_U32_C(0xe0000000))
+    {
+        LogRel(("NAT: Ignoring multicast binding address %RTnaipv4\n", pData->bindIP.s_addr));
+        pData->bindIP.s_addr = INADDR_ANY;
+        return VERR_INVALID_PARAMETER;
+    }
+
+    LogRel(("NAT: Binding address %RTnaipv4\n", pData->bindIP.s_addr));
+    return VINF_SUCCESS;
 }
 
 void slirp_set_dhcp_dns_proxy(PNATState pData, bool fDNSProxy)
@@ -2006,7 +2060,7 @@ int slirp_host_network_configuration_change_strategy_selector(const PNATState pD
         struct rcp_state rcp_state;
         int rc;
 
-        rcp_state.rcps_flags |= RCPSF_IGNORE_IPV6;
+        rcp_state.rcps_flags = RCPSF_IGNORE_IPV6;
         rc = rcp_parse(&rcp_state, RESOLV_CONF_FILE);
         LogRelFunc(("NAT: rcp_parse:%Rrc old domain:%s new domain:%s\n",
                     rc, LIST_EMPTY(&pData->pDomainList)
