@@ -1,13 +1,28 @@
+/* $Id: biosint.h $ */
+/** @file
+ * PC BIOS - BIOS internal definitions.
+ */
+
 /*
- * Copyright (C) 2006-2012 Oracle Corporation
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  * --------------------------------------------------------------------
  *
  * This code is based on:
@@ -38,6 +53,20 @@
  *
  */
 
+/*
+ * Oracle LGPL Disclaimer: For the avoidance of doubt, except that if any license choice
+ * other than GPL or LGPL is available it will apply instead, Oracle elects to use only
+ * the Lesser General Public License version 2.1 (LGPLv2) at this time for any software where
+ * a choice of LGPL license versions is made available with the language indicating
+ * that LGPLv2 or any later version may be used, or where a choice of which version
+ * of the LGPL is applied is otherwise unspecified.
+ */
+
+#ifndef VBOX_INCLUDED_SRC_PC_BIOS_biosint_h
+#define VBOX_INCLUDED_SRC_PC_BIOS_biosint_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 /* Compile-time assertion macro. */
 #define ct_assert(a)    extern int ct_ass_arr[!!(a) == 1];
@@ -50,20 +79,54 @@
 #define BX_USE_PS2_MOUSE    1
 #define BX_APM              1
 
-#define DEBUG_ATA       0
-#define DEBUG_AHCI      0
-#define DEBUG_CD_BOOT   0
-#define DEBUG_ELTORITO  0
-#define DEBUG_INT13_HD  0
-#define DEBUG_INT13_FL  0
-#define DEBUG_INT13_CD  0
-#define DEBUG_INT15     0
-#define DEBUG_INT15_MS  0
-#define DEBUG_INT16     0
-#define DEBUG_INT1A     0
-#define DEBUG_INT74     0
-#define DEBUG_PCI       0
-#define DEBUG_APM       0
+#ifndef DEBUG_ATA
+# define DEBUG_ATA       0
+#endif
+#ifdef DEBUG_AHCI
+# define DEBUG_AHCI      0
+#endif
+#ifndef DEBUG_SCSI
+# define DEBUG_SCSI      0
+#endif
+#ifndef DEBUG_CD_BOOT
+# define DEBUG_CD_BOOT   0
+#endif
+#ifndef DEBUG_ELTORITO
+# define DEBUG_ELTORITO  0
+#endif
+#ifndef DEBUG_INT13_HD
+# define DEBUG_INT13_HD  0
+#endif
+#ifndef DEBUG_INT13_FL
+# define DEBUG_INT13_FL  0
+#endif
+#ifndef DEBUG_INT13_CD
+# define DEBUG_INT13_CD  0
+#endif
+#ifndef DEBUG_INT15
+# define DEBUG_INT15     0
+#endif
+#ifndef DEBUG_INT15_MS
+# define DEBUG_INT15_MS  0
+#endif
+#ifndef DEBUG_INT16
+# define DEBUG_INT16     0
+#endif
+#ifndef DEBUG_INT1A
+# define DEBUG_INT1A     0
+#endif
+#ifndef DEBUG_INT74
+# define DEBUG_INT74     0
+#endif
+#ifndef DEBUG_PCI
+# define DEBUG_PCI       0
+#endif
+#ifndef DEBUG_APM
+# define DEBUG_APM       0
+#endif
+#ifndef DEBUG_POST
+# define DEBUG_POST      0
+#endif
 
 #define FP_OFF(p)   ((unsigned)(p))
 #define FP_SEG(p)   ((unsigned)((unsigned long)(void __far*)(p) >> 16))
@@ -135,6 +198,7 @@ typedef struct {
     pusha_regs_t    gr;
     uint16_t        es;
     uint16_t        ds;
+    uint16_t        ifl;
     iret_addr_t     ra;
 } kbd_regs_t;
 
@@ -213,10 +277,12 @@ extern  void        put_str(uint16_t action, const char __far *s);
 extern  void        put_str_near(uint16_t action, const char __near *s);
 extern  uint8_t     inb_cmos(uint8_t cmos_reg);
 extern  void        outb_cmos(uint8_t cmos_reg, uint8_t val);
+extern  uint16_t    get_cmos_word(uint8_t idxFirst);
 extern  uint16_t    cdrom_boot(void);
 extern  void        show_logo(void);
 extern  void        delay_boot(uint16_t secs);
 extern  bx_bool     set_enable_a20(bx_bool val);
+extern uint16_t     conv_mem_alloc(int n_kb, int in_ebda);
 
 #define printf(...)  bios_printf(BIOS_PRINTF_SCREEN, __VA_ARGS__)
 
@@ -231,17 +297,13 @@ extern  bx_bool     set_enable_a20(bx_bool val);
 #endif
 #ifdef VBOX
 #define BX_INFO(...)    do { put_str(BIOS_PRINTF_INFO, bios_prefix_string); bios_printf(BIOS_PRINTF_INFO, __VA_ARGS__); } while (0)
+#define BX_INFO_CON(...)do { put_str(BIOS_PRINTF_INFO, bios_prefix_string); bios_printf(BIOS_PRINTF_ALL, __VA_ARGS__); } while (0)
 #else /* !VBOX */
 #define BX_INFO(...)    bios_printf(BIOS_PRINTF_INFO, __VA_ARGS__)
 #endif /* !VBOX */
 #define BX_PANIC(...)   bios_printf(BIOS_PRINTF_DEBHALT, __VA_ARGS__)
 
-/* Because we don't tell the recompiler when guest physical memory 
- * is written, it can incorrectly cache guest code overwritten by
- * DMA (bus master or not). We just re-write the memory block to flush
- * any of its caches. This is not exactly efficient, but works!
- */
-#define DMA_WORKAROUND      1
+uint16_t pci16_find_device(uint32_t search_item, uint16_t index, int search_class, int ignore_if);
 
 /* Random hardware-related definitions. */
 
@@ -252,3 +314,6 @@ extern  bx_bool     set_enable_a20(bx_bool val);
 #define PIC_CMD_EOI         0x20
 #define PIC_CMD_RD_ISR      0x0B
 #define PIC_CMD_INIT        0x11
+
+#endif /* !VBOX_INCLUDED_SRC_PC_BIOS_biosint_h */
+
