@@ -1,13 +1,28 @@
+/* $Id: disk.c $ */
+/** @file
+ * PC BIOS - ???
+ */
+
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  * --------------------------------------------------------------------
  *
  * This code is based on:
@@ -38,6 +53,15 @@
  *
  */
 
+/*
+ * Oracle LGPL Disclaimer: For the avoidance of doubt, except that if any license choice
+ * other than GPL or LGPL is available it will apply instead, Oracle elects to use only
+ * the Lesser General Public License version 2.1 (LGPLv2) at this time for any software where
+ * a choice of LGPL license versions is made available with the language indicating
+ * that LGPLv2 or any later version may be used, or where a choice of which version
+ * of the LGPL is applied is otherwise unspecified.
+ */
+
 
 #include <stdint.h>
 #include "biosint.h"
@@ -53,7 +77,7 @@
 #endif
 
 /* Generic disk read/write routine signature. */
-typedef __fastcall (* dsk_rw_func)(bio_dsk_t __far *bios_dsk);
+typedef int __fastcall (* dsk_rw_func)(bio_dsk_t __far *bios_dsk);
 
 /* Controller specific disk access routines. Declared as a union to reduce
  * the need for conditionals when choosing between read/write functions.
@@ -79,7 +103,7 @@ dsk_acc_t   dskacc[DSKTYP_CNT] = {
 };
 
 
-//@todo: put in a header
+/// @todo put in a header
 #define AX      r.gr.u.r16.ax
 #define BX      r.gr.u.r16.bx
 #define CX      r.gr.u.r16.cx
@@ -98,12 +122,14 @@ dsk_acc_t   dskacc[DSKTYP_CNT] = {
  * Phoenix EDD 3.0. This is used as a fallback to generate sane logical
  * geometry in case none was provided in CMOS.
  */
-void set_geom_lba(chs_t __far *lgeo, uint32_t nsectors)
+void set_geom_lba(chs_t __far *lgeo, uint64_t nsectors64)
 {
     uint32_t    limit = 8257536;    /* 1024 * 128 * 63 */
+    uint32_t    nsectors;
     unsigned    heads = 255;
     int         i;
 
+    nsectors = (nsectors64 >> 32) ? 0xFFFFFFFFL : (uint32_t)nsectors64;
     /* Start with ~4GB limit, go down to 504MB. */
     for (i = 0; i < 4; ++i) {
         if (nsectors <= limit)
@@ -118,6 +144,140 @@ void set_geom_lba(chs_t __far *lgeo, uint32_t nsectors)
     lgeo->spt       = 63;   /* Always 63 sectors per track, the maximum. */
 }
 
+int edd_fill_dpt(dpt_t __far *dpt, bio_dsk_t __far *bios_dsk, uint8_t device)
+{
+    uint16_t    ebda_seg = read_word(0x0040,0x000E);
+
+    /* Check if buffer is large enough. */
+    if (dpt->size < 0x1a)
+        return -1;
+
+    /* Fill in EDD 1.x table. */
+    if (dpt->size >= 0x1a) {
+        uint64_t    lba;
+
+        dpt->size      = 0x1a;
+        dpt->blksize   = bios_dsk->devices[device].blksize;
+
+        if (bios_dsk->devices[device].device == DSK_DEVICE_CDROM) {
+            dpt->infos         = 0x74;  /* Removable, media change, lockable, max values */
+            dpt->cylinders     = 0xffffffff;
+            dpt->heads         = 0xffffffff;
+            dpt->spt           = 0xffffffff;
+            dpt->sector_count1 = 0xffffffff;
+            dpt->sector_count2 = 0xffffffff;
+        } else {
+            dpt->infos     = 0x02;  // geometry is valid
+            dpt->cylinders = bios_dsk->devices[device].pchs.cylinders;
+            dpt->heads     = bios_dsk->devices[device].pchs.heads;
+            dpt->spt       = bios_dsk->devices[device].pchs.spt;
+            lba = bios_dsk->devices[device].sectors;
+            dpt->sector_count1 = lba;
+            dpt->sector_count2 = lba >> 32;
+        }
+    }
+
+    /* Fill in EDD 2.x table. */
+    if (dpt->size >= 0x1e) {
+        uint8_t     channel, irq, mode, checksum, i, xlation;
+        uint16_t    iobase1, iobase2, options;
+
+        dpt->size = 0x1e;
+        dpt->dpte_segment = ebda_seg;
+        dpt->dpte_offset  = (uint16_t)&EbdaData->bdisk.dpte;
+
+        // Fill in dpte
+        channel = device / 2;
+        iobase1 = bios_dsk->channels[channel].iobase1;
+        iobase2 = bios_dsk->channels[channel].iobase2;
+        irq     = bios_dsk->channels[channel].irq;
+        mode    = bios_dsk->devices[device].mode;
+        xlation = bios_dsk->devices[device].translation;
+
+        options  = (xlation == GEO_TRANSLATION_NONE ? 0 : 1 << 3);  /* CHS translation */
+        options |= (1 << 4);    /* LBA translation */
+        if (bios_dsk->devices[device].device == DSK_DEVICE_CDROM) {
+            options |= (1 << 5);    /* Removable device */
+            options |= (1 << 6);    /* ATAPI device */
+        }
+#if VBOX_BIOS_CPU >= 80386
+        options |= (mode == ATA_MODE_PIO32 ? 1 : 0 << 7);
+#endif
+        options |= (xlation == GEO_TRANSLATION_LBA ? 1 : 0 << 9);
+        options |= (xlation == GEO_TRANSLATION_RECHS ? 3 : 0 << 9);
+
+        bios_dsk->dpte.iobase1  = iobase1;
+        bios_dsk->dpte.iobase2  = iobase2;
+        bios_dsk->dpte.prefix   = (0xe | (device % 2)) << 4;
+        bios_dsk->dpte.unused   = 0xcb;
+        bios_dsk->dpte.irq      = irq;
+        bios_dsk->dpte.blkcount = 1;
+        bios_dsk->dpte.dma      = 0;
+        bios_dsk->dpte.pio      = 0;
+        bios_dsk->dpte.options  = options;
+        bios_dsk->dpte.reserved = 0;
+        bios_dsk->dpte.revision = 0x11;
+
+        checksum = 0;
+        for (i = 0; i < 15; ++i)
+            checksum += read_byte(ebda_seg, (uint16_t)&EbdaData->bdisk.dpte + i);
+        checksum = -checksum;
+        bios_dsk->dpte.checksum = checksum;
+    }
+
+    /* Fill in EDD 3.x table. */
+    if (dpt->size >= 0x42) {
+        uint8_t     channel, iface, checksum, i;
+        uint16_t    iobase1;
+
+        channel = device / 2;
+        iface   = bios_dsk->channels[channel].iface;
+        iobase1 = bios_dsk->channels[channel].iobase1;
+
+        dpt->size       = 0x42;
+        dpt->key        = 0xbedd;
+        dpt->dpi_length = 0x24;
+        dpt->reserved1  = 0;
+        dpt->reserved2  = 0;
+
+        if (iface == ATA_IFACE_ISA) {
+            dpt->host_bus[0] = 'I';
+            dpt->host_bus[1] = 'S';
+            dpt->host_bus[2] = 'A';
+            dpt->host_bus[3] = ' ';
+        }
+        else {
+            // FIXME PCI
+        }
+        dpt->iface_type[0] = 'A';
+        dpt->iface_type[1] = 'T';
+        dpt->iface_type[2] = 'A';
+        dpt->iface_type[3] = ' ';
+        dpt->iface_type[4] = ' ';
+        dpt->iface_type[5] = ' ';
+        dpt->iface_type[6] = ' ';
+        dpt->iface_type[7] = ' ';
+
+        if (iface == ATA_IFACE_ISA) {
+            ((uint16_t __far *)dpt->iface_path)[0] = iobase1;
+            ((uint16_t __far *)dpt->iface_path)[1] = 0;
+            ((uint32_t __far *)dpt->iface_path)[1] = 0;
+        }
+        else {
+            // FIXME PCI
+        }
+        ((uint16_t __far *)dpt->device_path)[0] = device & 1; // device % 2; @todo: correct?
+        ((uint16_t __far *)dpt->device_path)[1] = 0;
+        ((uint32_t __far *)dpt->device_path)[1] = 0;
+
+        checksum = 0;
+        for (i = 30; i < 64; i++)
+            checksum += ((uint8_t __far *)dpt)[i];
+        checksum = -checksum;
+        dpt->checksum = checksum;
+    }
+    return 0;
+}
 
 void BIOSCALL int13_harddisk(disk_regs_t r)
 {
@@ -130,6 +290,8 @@ void BIOSCALL int13_harddisk(disk_regs_t r)
 
     BX_DEBUG_INT13_HD("%s: AX=%04x BX=%04x CX=%04x DX=%04x ES=%04x\n", __func__, AX, BX, CX, DX, ES);
 
+    SET_IF();   /* INT 13h always returns with interrupts enabled. */
+
     bios_dsk = read_word(0x0040,0x000E) :> &EbdaData->bdisk;
     write_byte(0x0040, 0x008e, 0);  // clear completion flag
 
@@ -151,11 +313,9 @@ void BIOSCALL int13_harddisk(disk_regs_t r)
     switch (GET_AH()) {
 
     case 0x00: /* disk controller reset */
-#ifdef VBOX_WITH_SCSI
-        /* SCSI controller does not need a reset. */
-        if (!VBOX_IS_SCSI_DEVICE(device))
-#endif
-        ata_reset (device);
+        /* For ATA drives only. NB: This resets both drives on a channel. */
+        if (VBOX_IS_ATA_DEVICE(device))
+            ata_reset(device);
         goto int13_success;
         break;
 
@@ -200,14 +360,17 @@ void BIOSCALL int13_harddisk(disk_regs_t r)
             goto int13_success;
 
         /* If required, translate LCHS to LBA and execute command. */
-        //@todo: The IS_SCSI_DEVICE check should be redundant...
+        /// @todo The IS_SCSI_DEVICE check should be redundant...
         if (( (bios_dsk->devices[device].pchs.heads != nlh) || (bios_dsk->devices[device].pchs.spt != nlspt)) || VBOX_IS_SCSI_DEVICE(device)) {
             lba = ((((uint32_t)cylinder * (uint32_t)nlh) + (uint32_t)head) * (uint32_t)nlspt) + (uint32_t)sector - 1;
             sector = 0; // this forces the command to be lba
+            BX_DEBUG_INT13_HD("%s: %d sectors from lba %lu @ %04x:%04x\n", __func__,
+                              count, lba, ES, BX);
+        } else {
+            BX_DEBUG_INT13_HD("%s: %d sectors from C/H/S %u/%u/%u @ %04x:%04x\n", __func__,
+                              count, cylinder, head, sector, ES, BX);
         }
 
-        BX_DEBUG_INT13_HD("%s: %d sectors from lba %lu @ %04x:%04x\n", __func__,
-                          count, lba, ES, BX);
 
         /* Clear the count of transferred sectors/bytes. */
         bios_dsk->drqp.trsfsectors = 0;
@@ -217,7 +380,7 @@ void BIOSCALL int13_harddisk(disk_regs_t r)
         bios_dsk->drqp.lba      = lba;
         bios_dsk->drqp.buffer   = MK_FP(ES, BX);
         bios_dsk->drqp.nsect    = count;
-        bios_dsk->drqp.sect_sz  = 512;  //@todo: device specific?
+        bios_dsk->drqp.sect_sz  = 512;  /// @todo device specific?
         bios_dsk->drqp.cylinder = cylinder;
         bios_dsk->drqp.head     = head;
         bios_dsk->drqp.sector   = sector;
@@ -238,10 +401,9 @@ void BIOSCALL int13_harddisk(disk_regs_t r)
         break;
 
     case 0x05: /* format disk track */
-          BX_INFO("format disk track called\n");
-          goto int13_success;
-          return;
-          break;
+        BX_INFO("format disk track called\n");
+        goto int13_success;
+        break;
 
     case 0x08: /* read disk drive parameters */
 
@@ -252,6 +414,9 @@ void BIOSCALL int13_harddisk(disk_regs_t r)
 
         count = bios_dsk->hdcount;
         /* Maximum cylinder number is just one less than the number of cylinders. */
+        /* To make Windows 3.1x WDCTRL.386 happy, we'd have to subtract 2, not 1,
+         * to account for a diagnostic cylinder.
+         */
         nlc = nlc - 1; /* 0 based , last sector not used */
         SET_AL(0);
         SET_CH(nlc & 0xff);
@@ -260,7 +425,7 @@ void BIOSCALL int13_harddisk(disk_regs_t r)
         SET_DL(count); /* FIXME returns 0, 1, or n hard drives */
 
         // FIXME should set ES & DI
-        // @todo: Actually, the above comment is nonsense.
+        /// @todo Actually, the above comment is nonsense.
 
         goto int13_success;
         break;
@@ -268,6 +433,10 @@ void BIOSCALL int13_harddisk(disk_regs_t r)
     case 0x10: /* check drive ready */
         // should look at 40:8E also???
 
+#ifdef VBOX_WITH_SCSI
+        /* SCSI drives are always "ready". */
+        if (!VBOX_IS_SCSI_DEVICE(device)) {
+#endif
         // Read the status from controller
         status = inb(bios_dsk->channels[device/2].iobase1 + ATA_CB_STAT);
         if ( (status & ( ATA_CB_STAT_BSY | ATA_CB_STAT_RDY )) == ATA_CB_STAT_RDY ) {
@@ -276,6 +445,10 @@ void BIOSCALL int13_harddisk(disk_regs_t r)
             SET_AH(0xAA);
             goto int13_fail_noah;
         }
+#ifdef VBOX_WITH_SCSI
+        } else  /* It's not an ATA drive. */
+            goto int13_success;
+#endif
         break;
 
     case 0x15: /* read disk drive size */
@@ -330,16 +503,19 @@ int13_success_noah:
 
 void BIOSCALL int13_harddisk_ext(disk_regs_t r)
 {
-    uint32_t            lba;
-    uint16_t            ebda_seg = read_word(0x0040,0x000E);
+    uint64_t            lba;
     uint16_t            segment, offset;
-    uint16_t            npc, nph, npspt;
-    uint16_t            size, count;
     uint8_t             device, status;
+    uint16_t            count;
     uint8_t             type;
     bio_dsk_t __far     *bios_dsk;
     int13ext_t __far    *i13_ext;
+#if 0
+    uint16_t            ebda_seg = read_word(0x0040,0x000E);
+    uint16_t            npc, nph, npspt;
+    uint16_t            size;
     dpt_t __far         *dpt;
+#endif
 
     bios_dsk = read_word(0x0040,0x000E) :> &EbdaData->bdisk;
 
@@ -383,18 +559,13 @@ void BIOSCALL int13_harddisk_ext(disk_regs_t r)
         segment = i13_ext->segment;
         offset  = i13_ext->offset;
 
-        BX_DEBUG_INT13_HD("%s: %d sectors from lba %lu @ %04x:%04x\n", __func__,
-                          count, i13_ext->lba1, segment, offset);
-
-        // Can't use 64 bits lba
+        // Get 64 bits lba and check
         lba = i13_ext->lba2;
-        if (lba != 0L) {
-            BX_PANIC("%s: function %02x. Can't use 64bits lba\n", __func__, GET_AH());
-            goto int13x_fail;
-        }
+        lba <<= 32;
+        lba |= i13_ext->lba1;
 
-        // Get 32 bits lba and check
-        lba = i13_ext->lba1;
+        BX_DEBUG_INT13_HD("%s: %d sectors from LBA 0x%llx @ %04x:%04x\n", __func__,
+                          count, lba, segment, offset);
 
         type = bios_dsk->devices[device].type;
         if (lba >= bios_dsk->devices[device].sectors) {
@@ -414,7 +585,7 @@ void BIOSCALL int13_harddisk_ext(disk_regs_t r)
         bios_dsk->drqp.lba     = lba;
         bios_dsk->drqp.buffer  = MK_FP(segment, offset);
         bios_dsk->drqp.nsect   = count;
-        bios_dsk->drqp.sect_sz = 512;   //@todo: device specific?
+        bios_dsk->drqp.sect_sz = 512;   /// @todo device specific?
         bios_dsk->drqp.sector  = 0;     /* Indicate LBA. */
         bios_dsk->drqp.dev_id  = device;
 
@@ -443,128 +614,10 @@ void BIOSCALL int13_harddisk_ext(disk_regs_t r)
         break;
 
     case 0x48: // IBM/MS get drive parameters
-        dpt = DS :> (dpt_t *)SI;
-        size = dpt->size;
-
-        /* Check if buffer is large enough. */
-        if (size < 0x1a)
+        if (edd_fill_dpt(DS :> (dpt_t *)SI, bios_dsk, device))
             goto int13x_fail;
-
-        /* Fill in EDD 1.x table. */
-        if (size >= 0x1a) {
-            uint16_t   blksize;
-
-            npc     = bios_dsk->devices[device].pchs.cylinders;
-            nph     = bios_dsk->devices[device].pchs.heads;
-            npspt   = bios_dsk->devices[device].pchs.spt;
-            lba     = bios_dsk->devices[device].sectors;
-            blksize = bios_dsk->devices[device].blksize;
-
-            dpt->size      = 0x1a;
-            dpt->infos     = 0x02;  // geometry is valid
-            dpt->cylinders = npc;
-            dpt->heads     = nph;
-            dpt->spt       = npspt;
-            dpt->blksize   = blksize;
-            dpt->sector_count1 = lba;   // FIXME should be Bit64
-            dpt->sector_count2 = 0;
-        }
-
-        /* Fill in EDD 2.x table. */
-        if (size >= 0x1e) {
-            uint8_t     channel, irq, mode, checksum, i, translation;
-            uint16_t    iobase1, iobase2, options;
-
-            dpt->size = 0x1e;
-            dpt->dpte_segment = ebda_seg;
-            dpt->dpte_offset  = (uint16_t)&EbdaData->bdisk.dpte;
-
-            // Fill in dpte
-            channel = device / 2;
-            iobase1 = bios_dsk->channels[channel].iobase1;
-            iobase2 = bios_dsk->channels[channel].iobase2;
-            irq     = bios_dsk->channels[channel].irq;
-            mode    = bios_dsk->devices[device].mode;
-            translation = bios_dsk->devices[device].translation;
-
-            options  = (translation == GEO_TRANSLATION_NONE ? 0 : 1 << 3);  // chs translation
-            options |= (1 << 4);    // lba translation
-            options |= (mode == ATA_MODE_PIO32 ? 1 : 0 << 7);
-            options |= (translation == GEO_TRANSLATION_LBA ? 1 : 0 << 9);
-            options |= (translation == GEO_TRANSLATION_RECHS ? 3 : 0 << 9);
-
-            bios_dsk->dpte.iobase1  = iobase1;
-            bios_dsk->dpte.iobase2  = iobase2;
-            bios_dsk->dpte.prefix   = (0xe | (device % 2)) << 4;
-            bios_dsk->dpte.unused   = 0xcb;
-            bios_dsk->dpte.irq      = irq;
-            bios_dsk->dpte.blkcount = 1;
-            bios_dsk->dpte.dma      = 0;
-            bios_dsk->dpte.pio      = 0;
-            bios_dsk->dpte.options  = options;
-            bios_dsk->dpte.reserved = 0;
-            bios_dsk->dpte.revision = 0x11;
-
-            checksum = 0;
-            for (i = 0; i < 15; ++i)
-                checksum += read_byte(ebda_seg, (uint16_t)&EbdaData->bdisk.dpte + i);
-            checksum = -checksum;
-            bios_dsk->dpte.checksum = checksum;
-        }
-
-        /* Fill in EDD 3.x table. */
-        if(size >= 0x42) {
-            uint8_t     channel, iface, checksum, i;
-            uint16_t    iobase1;
-
-            channel = device / 2;
-            iface   = bios_dsk->channels[channel].iface;
-            iobase1 = bios_dsk->channels[channel].iobase1;
-
-            dpt->size       = 0x42;
-            dpt->key        = 0xbedd;
-            dpt->dpi_length = 0x24;
-            dpt->reserved1  = 0;
-            dpt->reserved2  = 0;
-
-            if (iface == ATA_IFACE_ISA) {
-                dpt->host_bus[0] = 'I';
-                dpt->host_bus[1] = 'S';
-                dpt->host_bus[2] = 'A';
-                dpt->host_bus[3] = ' ';
-            }
-            else {
-                // FIXME PCI
-            }
-            dpt->iface_type[0] = 'A';
-            dpt->iface_type[1] = 'T';
-            dpt->iface_type[2] = 'A';
-            dpt->iface_type[3] = ' ';
-            dpt->iface_type[4] = ' ';
-            dpt->iface_type[5] = ' ';
-            dpt->iface_type[6] = ' ';
-            dpt->iface_type[7] = ' ';
-
-            if (iface == ATA_IFACE_ISA) {
-                ((uint16_t __far *)dpt->iface_path)[0] = iobase1;
-                ((uint16_t __far *)dpt->iface_path)[1] = 0;
-                ((uint32_t __far *)dpt->iface_path)[1] = 0;
-            }
-            else {
-                // FIXME PCI
-            }
-            ((uint16_t __far *)dpt->device_path)[0] = device & 1; // device % 2; @todo: correct?
-            ((uint16_t __far *)dpt->device_path)[1] = 0;
-            ((uint32_t __far *)dpt->device_path)[1] = 0;
-
-            checksum = 0;
-            for (i = 30; i < 64; i++)
-                checksum += read_byte(DS, SI + i);
-            checksum = -checksum;
-            dpt->checksum = checksum;
-        }
-
-        goto int13x_success;
+        else
+            goto int13x_success;
         break;
 
     case 0x4e: // // IBM/MS set hardware configuration
