@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2004-2012 Oracle Corporation
+ * Copyright (C) 2004-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 #include <stdint.h>
@@ -30,7 +40,7 @@
 
 /**
  * Set video mode (VGA).
- * @params    New video mode.
+ * @param   mode    New video mode.
  */
 void set_mode(uint8_t mode);
 #pragma aux set_mode =      \
@@ -41,7 +51,7 @@ void set_mode(uint8_t mode);
 
 /**
  * Set VESA video mode.
- * @params    New video mode.
+ * @param   mode    New video mode.
  */
 uint16_t vesa_set_mode(uint16_t mode);
 #pragma aux vesa_set_mode = \
@@ -51,7 +61,7 @@ uint16_t vesa_set_mode(uint16_t mode);
 
 /**
  * Get current VESA video mode.
- * @params    New video mode.
+ * @param   mode    New video mode.
  */
 uint16_t vesa_get_mode(uint16_t __far *mode);
 #pragma aux vesa_get_mode = \
@@ -61,11 +71,24 @@ uint16_t vesa_get_mode(uint16_t __far *mode);
     parm [es di] modify [ax bx] nomemory;
 
 
+/**
+ * Set custom video mode.
+ * @param   xres    Requested width
+ * @param   yres    Requested height
+ * @param   bpp     Requested bits per pixel
+ */
+uint16_t custom_set_mode(uint16_t xres, uint16_t yres, uint8_t bpp);
+#pragma aux custom_set_mode = \
+    "mov    ax, 5642h"      \
+    "mov    bl, 0"          \
+    "int    10h"            \
+    parm [cx] [dx] [bh] modify [ax] nomemory;
+
 /**
  * Check for keystroke.
  * @returns    True if keystroke available, False if not.
  */
-//@todo: INT 16h should already be returning the right value in al; could also use setz
+/// @todo INT 16h should already be returning the right value in al; could also use setz
 uint8_t check_for_keystroke(void);
 #pragma aux check_for_keystroke =   \
     "mov    ax, 100h"               \
@@ -91,7 +114,7 @@ uint8_t get_keystroke(void);
     modify [ax] nomemory;
 
 
-//@todo: This whole business with reprogramming the PIT is rather suspect.
+/// @todo This whole business with reprogramming the PIT is rather suspect.
 // The BIOS already has waiting facilities in INT 15h (fn 83h, 86h) which
 // should be utilized instead.
 
@@ -106,7 +129,7 @@ void wait_init(void);
     "out    40h, al"    \
     modify [ax] nomemory;
 
-//@todo: using this private interface is not great
+/// @todo using this private interface is not great
 extern void rtc_post(void);
 #pragma aux rtc_post "*";
 
@@ -115,7 +138,8 @@ extern void rtc_post(void);
  * timer at WAIT_HZ for a while.
  */
 void wait_uninit(void);
-#pragma aux wait_uninit =   \
+#if VBOX_BIOS_CPU >= 80386
+# pragma aux wait_uninit =   \
     ".386"                  \
     "mov    al, 34h"        \
     "out    43h, al"        \
@@ -129,6 +153,21 @@ void wait_uninit(void);
     "pop    ds"             \
     "popad"                 \
     modify [ax] nomemory;
+#else
+# pragma aux wait_uninit = \
+    "mov    al, 34h" \
+    "out    43h, al" \
+    "xor    ax, ax" \
+    "out    40h, al" \
+    "out    40h, al" \
+    "push   bp" \
+    "push   ds" \
+    "mov    ds, ax" \
+    "call   rtc_post" \
+    "pop    ds" \
+    "pop    bp" \
+    modify [ax bx cx dx si di];
+#endif
 
 
 /**
@@ -234,7 +273,7 @@ void print_detected_harddisks(void)
         {
             if (sata_ctrl_printed == 0)
             {
-                printf("\n\nAHCI controller:\n");
+                printf("\n\n  AHCI controller:");
                 sata_ctrl_printed = 1;
             }
 
@@ -248,7 +287,7 @@ void print_detected_harddisks(void)
         {
             if (scsi_ctrl_printed == 0)
             {
-                printf("\n\nSCSI controller:\n");
+                printf("\n\n  SCSI controller:");
                 scsi_ctrl_printed = 1;
             }
 
@@ -261,7 +300,7 @@ void print_detected_harddisks(void)
 
             if ((device < 4) && (ide_ctrl_printed == 0))
             {
-                printf("IDE controller:\n");
+                printf("  IDE controller:");
                 ide_ctrl_printed = 1;
             }
             else if ((device >= 4) && (sata_ctrl_printed == 0))
@@ -330,7 +369,7 @@ void show_logo(void)
     uint16_t    old_mode;
 
 
-    // Set PIT to 1ms ticks
+    // Set PIT to 64hz.
     wait_init();
 
     // Get main signature
@@ -352,8 +391,11 @@ void show_logo(void)
     if (!is_fade_in && !is_fade_out && !logo_time)
         goto done;
 
-    // Set video mode #0x142 640x480x32bpp
-    vesa_set_mode(0x142);
+    /* Set video mode using private video BIOS interface. */
+    tmp = custom_set_mode(640, 480, 32);
+    /* If custom mode set failed, fall back to VBE. */
+    if (tmp != 0x4F)
+        vesa_set_mode(0x142);
 
     if (is_fade_in)
     {
@@ -393,6 +435,8 @@ void show_logo(void)
             }
         }
     }
+    else if (!f12_pressed)
+        outw(LOGO_IO_PORT, LOGO_CMD_SHOW_BMP | 0);
 
 done:
     // Clear forced boot drive setting.
