@@ -1,13 +1,28 @@
+/* $Id: print.c $ */
+/** @file
+ * PC BIOS - ???
+ */
+
 /*
- * Copyright (C) 2006-2011 Oracle Corporation
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  * --------------------------------------------------------------------
  *
  * This code is based on:
@@ -38,6 +53,15 @@
  *
  */
 
+/*
+ * Oracle LGPL Disclaimer: For the avoidance of doubt, except that if any license choice
+ * other than GPL or LGPL is available it will apply instead, Oracle elects to use only
+ * the Lesser General Public License version 2.1 (LGPLv2) at this time for any software where
+ * a choice of LGPL license versions is made available with the language indicating
+ * that LGPLv2 or any later version may be used, or where a choice of which version
+ * of the LGPL is applied is otherwise unspecified.
+ */
+
 
 #include <stdint.h>
 #include <stdarg.h>
@@ -144,7 +168,9 @@ void put_str_near(uint16_t action, const char __near *s)
 //
 //   Supports %[format_width][length]format
 //   where format can be x,X,u,d,s,S,c
-//   and the optional length modifier is l (ell)
+//   and the optional length modifier is l (ell, long 32-bit) or ll
+//   (long long, 64-bit).
+//   Only x,X work with ll
 //--------------------------------------------------------------------------
 void bios_printf(uint16_t action, const char *s, ...)
 {
@@ -152,13 +178,13 @@ void bios_printf(uint16_t action, const char *s, ...)
     bx_bool     in_format;
     int         i;
     uint16_t    arg, nibble, hibyte, format_width, hexadd;
-    va_list     args; 
-    
+    va_list     args;
+
     va_start( args, s );
-    
+
     in_format = 0;
     format_width = 0;
-    
+
     if ((action & BIOS_PRINTF_DEBHALT) == BIOS_PRINTF_DEBHALT) {
         bios_printf (BIOS_PRINTF_SCREEN, "FATAL: ");
     }
@@ -189,6 +215,32 @@ void bios_printf(uint16_t action, const char *s, ...)
                 else if (c == 'u') {
                     put_uint(action, arg, format_width, 0);
                 }
+                else if (c == 'l' && s[1] == 'l') {
+                    uint64_t llval;
+                    uint16_t __far *cp16;
+
+                    s += 2;
+                    c = *s;
+                    cp16 = (uint16_t __far *)&llval;
+                    cp16[0] = arg;
+                    cp16[1] = va_arg( args, uint16_t );
+                    cp16[2] = va_arg( args, uint16_t );
+                    cp16[3] = va_arg( args, uint16_t );
+                    if (c == 'x' || c == 'X') {
+                        if (format_width == 0)
+                            format_width = 16;
+                        if (c == 'x')
+                            hexadd = 'a';
+                        else
+                            hexadd = 'A';
+                        for (i=format_width-1; i>=0; i--) {
+                            nibble =  (llval >> (i * 4)) & 0x000f;
+                            send (action, (nibble<=9)? (nibble+'0') : (nibble-10+hexadd));
+                        }
+                    } else {
+                        BX_PANIC("bios_printf: unknown %ll format\n");
+                    }
+                }
                 else if (c == 'l') {
                     s++;
                     c = *s; /* is it ld,lx,lu? */
