@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2015 Oracle Corporation
+ * Copyright (C) 2015-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
@@ -113,7 +123,6 @@ typedef HBDMGRDACLBKARGS *PHBDMGRDACLBKARGS;
 /**
  * Unclaims the given block device and frees its state removing it from the list.
  *
- * @returns nothing.
  * @param   pDev           The block device to unclaim.
  */
 static void hbdMgrDevUnclaim(PHBDMGRDEV pDev)
@@ -200,14 +209,18 @@ static int hbdMgrDAReturn2VBoxStatus(DAReturn hReturn)
 }
 
 /**
- * Callback notifying us that the async DADiskClaim()/DADiskUnmount call has completed.
+ * Implements the OS X callback DADiskClaimCallback.
+ *
+ * This notifies us that the async DADiskClaim()/DADiskUnmount call has
+ * completed.
  *
  * @param   hDiskRef         The disk that was attempted claimed / unmounted.
  * @param   hDissenterRef    NULL on success, contains details on failure.
  * @param   pvContext        Pointer to the return code variable.
  */
-static DECLCALLBACK(void) hbdMgrDACallbackComplete(DADiskRef hDiskRef, DADissenterRef hDissenterRef, void *pvContext)
+static void hbdMgrDACallbackComplete(DADiskRef hDiskRef, DADissenterRef hDissenterRef, void *pvContext)
 {
+    RT_NOREF(hDiskRef);
     PHBDMGRDACLBKARGS pArgs = (PHBDMGRDACLBKARGS)pvContext;
     pArgs->pszErrDetail = NULL;
 
@@ -230,14 +243,16 @@ static DECLCALLBACK(void) hbdMgrDACallbackComplete(DADiskRef hDiskRef, DADissent
 }
 
 /**
- * Callback notifying us about any attempt to mount a volume. If we claimed the volume
- * or the complete disk containing the volume we will deny the attempt.
+ * Implements the OS X callback DADiskMountApprovalCallback.
+ *
+ * This notifies us about any attempt to mount a volume.  If we claimed the
+ * volume or the complete disk containing the volume we will deny the attempt.
  *
  * @returns Reference to a DADissenter object which contains the result.
  * @param   hDiskRef         The disk that is about to be mounted.
- * @param   pvCOntext        Pointer to the block device manager.
+ * @param   pvContext        Pointer to the block device manager.
  */
-static DECLCALLBACK(DADissenterRef) hbdMgrDAMountApprovalCallback(DADiskRef hDiskRef, void *pvContext)
+static DADissenterRef hbdMgrDAMountApprovalCallback(DADiskRef hDiskRef, void *pvContext)
 {
     PHBDMGRINT pThis = (PHBDMGRINT)pvContext;
     DADiskRef hDiskParentRef = DADiskCopyWholeDisk(hDiskRef);
@@ -274,14 +289,15 @@ static DECLCALLBACK(DADissenterRef) hbdMgrDAMountApprovalCallback(DADiskRef hDis
 
 
 /**
+ * Implements OS X callback CFRunLoopSourceContext::perform.
+ *
  * Dummy handler for the wakeup source to kick the worker thread.
  *
- * @returns nothing.
  * @param   pInfo            Opaque user data given during source creation, unused.
  */
-static DECLCALLBACK(void) hbdMgrDAPerformWakeup(void *pInfo)
+static void hbdMgrDAPerformWakeup(void *pInfo)
 {
-    return;
+    RT_NOREF(pInfo);
 }
 
 
