@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2006-2016 Oracle Corporation
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
@@ -21,7 +31,7 @@
 *********************************************************************************************************************************/
 #define LOG_GROUP LOG_GROUP_VSCSI
 #include <VBox/log.h>
-#include <VBox/err.h>
+#include <iprt/errcore.h>
 #include <VBox/types.h>
 #include <VBox/vscsi.h>
 #include <iprt/cdefs.h>
@@ -43,7 +53,7 @@ typedef struct VSCSILUNSBC
     /** Core LUN structure */
     VSCSILUNINT    Core;
     /** Sector size of the medium. */
-    uint32_t       cbSector;
+    uint64_t       cbSector;
     /** Size of the virtual disk. */
     uint64_t       cSectors;
     /** VPD page pool. */
@@ -55,18 +65,16 @@ typedef VSCSILUNSBC *PVSCSILUNSBC;
 static DECLCALLBACK(int) vscsiLunSbcInit(PVSCSILUNINT pVScsiLun)
 {
     PVSCSILUNSBC pVScsiLunSbc = (PVSCSILUNSBC)pVScsiLun;
-    uint64_t cbDisk = 0;
     int rc = VINF_SUCCESS;
     int cVpdPages = 0;
 
-    rc = vscsiLunMediumGetSectorSize(pVScsiLun, &pVScsiLunSbc->cbSector);
-    if (RT_SUCCESS(rc))
-    {
-        rc = vscsiLunMediumGetSize(pVScsiLun, &cbDisk);
-        if (RT_SUCCESS(rc))
-            pVScsiLunSbc->cSectors = cbDisk / pVScsiLunSbc->cbSector;
-    }
+    uint32_t cRegions = vscsiLunMediumGetRegionCount(pVScsiLun);
+    if (cRegions != 1)
+        rc = VERR_INVALID_PARAMETER;
 
+    if (RT_SUCCESS(rc))
+        rc = vscsiLunMediumQueryRegionProperties(pVScsiLun, 0, NULL, &pVScsiLunSbc->cSectors,
+                                                 &pVScsiLunSbc->cbSector, NULL);
     if (RT_SUCCESS(rc))
         rc = vscsiVpdPagePoolInit(&pVScsiLunSbc->VpdPagePool);
 
@@ -220,22 +228,39 @@ static DECLCALLBACK(int) vscsiLunSbcReqProcess(PVSCSILUNINT pVScsiLun, PVSCSIREQ
                                                  SCSI_ASC_INV_FIELD_IN_CMD_PACKET, 0x00);
             else
             {
-                SCSIINQUIRYDATA ScsiInquiryReply;
+                if (pVScsiReq->cbCDB >= 5)
+                {
+                    SCSIINQUIRYDATA ScsiInquiryReply;
 
-                memset(&ScsiInquiryReply, 0, sizeof(ScsiInquiryReply));
+                    memset(&ScsiInquiryReply, 0, sizeof(ScsiInquiryReply));
 
-                ScsiInquiryReply.cbAdditional           = 31;
-                ScsiInquiryReply.u5PeripheralDeviceType = SCSI_INQUIRY_DATA_PERIPHERAL_DEVICE_TYPE_DIRECT_ACCESS;
-                ScsiInquiryReply.u3PeripheralQualifier  = SCSI_INQUIRY_DATA_PERIPHERAL_QUALIFIER_CONNECTED;
-                ScsiInquiryReply.u3AnsiVersion          = 0x05; /* SPC-4 compliant */
-                ScsiInquiryReply.fCmdQue                = 1;    /* Command queuing supported. */
-                ScsiInquiryReply.fWBus16                = 1;
-                vscsiPadStr(ScsiInquiryReply.achVendorId, "VBOX", 8);
-                vscsiPadStr(ScsiInquiryReply.achProductId, "HARDDISK", 16);
-                vscsiPadStr(ScsiInquiryReply.achProductLevel, "1.0", 4);
+                    ScsiInquiryReply.cbAdditional           = 31;
+                    ScsiInquiryReply.u5PeripheralDeviceType = SCSI_INQUIRY_DATA_PERIPHERAL_DEVICE_TYPE_DIRECT_ACCESS;
+                    ScsiInquiryReply.u3PeripheralQualifier  = SCSI_INQUIRY_DATA_PERIPHERAL_QUALIFIER_CONNECTED;
+                    ScsiInquiryReply.u3AnsiVersion          = 0x05; /* SPC-4 compliant */
+                    ScsiInquiryReply.fCmdQue                = 1;    /* Command queuing supported. */
+                    ScsiInquiryReply.fWBus16                = 1;
 
-                RTSgBufCopyFromBuf(&pVScsiReq->SgBuf, (uint8_t *)&ScsiInquiryReply, sizeof(SCSIINQUIRYDATA));
-                rcReq = vscsiLunReqSenseOkSet(pVScsiLun, pVScsiReq);
+                    const char *pszVendorId = "VBOX";
+                    const char *pszProductId = "HARDDISK";
+                    const char *pszProductLevel = "1.0";
+                    int rcTmp = vscsiLunQueryInqStrings(pVScsiLun, &pszVendorId, &pszProductId, &pszProductLevel);
+                    Assert(RT_SUCCESS(rcTmp) || rcTmp == VERR_NOT_FOUND); RT_NOREF(rcTmp);
+
+                    scsiPadStrS(ScsiInquiryReply.achVendorId, pszVendorId, 8);
+                    scsiPadStrS(ScsiInquiryReply.achProductId, pszProductId, 16);
+                    scsiPadStrS(ScsiInquiryReply.achProductLevel, pszProductLevel, 4);
+
+                    uint16_t cbDataReq = scsiBE2H_U16(&pVScsiReq->pbCDB[3]);
+                    vscsiReqSetXferDir(pVScsiReq, VSCSIXFERDIR_T2I);
+                    vscsiReqSetXferSize(pVScsiReq, RT_MIN(sizeof(SCSIINQUIRYDATA), cbDataReq));
+
+
+                    RTSgBufCopyFromBuf(&pVScsiReq->SgBuf, (uint8_t *)&ScsiInquiryReply, sizeof(SCSIINQUIRYDATA));
+                    rcReq = vscsiLunReqSenseOkSet(pVScsiLun, pVScsiReq);
+                }
+                else
+                    rcReq = vscsiLunReqSenseErrorSet(pVScsiLun, pVScsiReq, SCSI_SENSE_ILLEGAL_REQUEST, SCSI_ASC_INVALID_MESSAGE, 0x00);
             }
             break;
         }
@@ -249,10 +274,14 @@ static DECLCALLBACK(int) vscsiLunSbcReqProcess(PVSCSILUNINT pVScsiLun, PVSCSIREQ
              * able to be stored in 4 bytes return 0xffffffff in this field
              */
             if (pVScsiLunSbc->cSectors > UINT32_C(0xffffffff))
-                vscsiH2BEU32(aReply, UINT32_C(0xffffffff));
+                scsiH2BE_U32(aReply, UINT32_C(0xffffffff));
             else
-                vscsiH2BEU32(aReply, pVScsiLunSbc->cSectors - 1);
-            vscsiH2BEU32(&aReply[4], pVScsiLunSbc->cbSector);
+                scsiH2BE_U32(aReply, pVScsiLunSbc->cSectors - 1);
+            scsiH2BE_U32(&aReply[4], (uint32_t)pVScsiLunSbc->cbSector);
+
+            vscsiReqSetXferDir(pVScsiReq, VSCSIXFERDIR_T2I);
+            vscsiReqSetXferSize(pVScsiReq, sizeof(aReply));
+
             RTSgBufCopyFromBuf(&pVScsiReq->SgBuf, aReply, sizeof(aReply));
             rcReq = vscsiLunReqSenseOkSet(pVScsiLun, pVScsiReq);
             break;
@@ -260,15 +289,15 @@ static DECLCALLBACK(int) vscsiLunSbcReqProcess(PVSCSILUNINT pVScsiLun, PVSCSIREQ
         case SCSI_MODE_SENSE_6:
         {
             uint8_t uModePage = pVScsiReq->pbCDB[2] & 0x3f;
-            uint8_t aReply[24];
+            uint8_t aReply[16];
             uint8_t *pu8ReplyPos;
             bool    fValid = false;
 
             memset(aReply, 0, sizeof(aReply));
-            aReply[0] = 4; /* Reply length 4. */
-            aReply[1] = 0; /* Default media type. */
-            aReply[2] = RT_BIT(4); /* Caching supported. */
-            aReply[3] = 0; /* Block descriptor length. */
+            aReply[0] = 4 - 1;      /* Reply length 4 (sans length byte). */
+            aReply[1] = 0;          /* Default media type. */
+            aReply[2] = RT_BIT(4);  /* Caching supported. */
+            aReply[3] = 0;          /* Block descriptor length. */
 
             if (pVScsiLun->fFeatures & VSCSI_LUN_FEATURE_READONLY)
                 aReply[2] |= RT_BIT(7); /* Set write protect bit */
@@ -277,10 +306,10 @@ static DECLCALLBACK(int) vscsiLunSbcReqProcess(PVSCSILUNINT pVScsiLun, PVSCSIREQ
 
             if ((uModePage == 0x08) || (uModePage == 0x3f))
             {
-                memset(pu8ReplyPos, 0, 20);
-                *pu8ReplyPos++ = 0x08; /* Page code. */
-                *pu8ReplyPos++ = 0x12; /* Size of the page. */
-                *pu8ReplyPos++ = 0x4;  /* Write cache enabled. */
+                *pu8ReplyPos++ = 0x08;  /* Caching Page code. */
+                *pu8ReplyPos++ = 10;    /* Size of the page minus header, SCSI-2 format. */
+                *pu8ReplyPos++ = 0x4;   /* Write cache enabled. */
+                aReply[0] += 12;        /* Add page size to total reply length. */
                 fValid = true;
             } else if (uModePage == 0) {
                 fValid = true;
@@ -288,6 +317,9 @@ static DECLCALLBACK(int) vscsiLunSbcReqProcess(PVSCSILUNINT pVScsiLun, PVSCSIREQ
 
             /* Querying unknown pages must fail. */
             if (fValid) {
+                vscsiReqSetXferDir(pVScsiReq, VSCSIXFERDIR_T2I);
+                vscsiReqSetXferSize(pVScsiReq, RT_MIN(sizeof(aReply), pVScsiReq->pbCDB[4]));
+
                 RTSgBufCopyFromBuf(&pVScsiReq->SgBuf, aReply, sizeof(aReply));
                 rcReq = vscsiLunReqSenseOkSet(pVScsiLun, pVScsiReq);
             } else {
@@ -301,6 +333,9 @@ static DECLCALLBACK(int) vscsiLunSbcReqProcess(PVSCSILUNINT pVScsiLun, PVSCSIREQ
             size_t  cbCopied;
             size_t  cbList = pVScsiReq->pbCDB[4];
 
+            vscsiReqSetXferDir(pVScsiReq, VSCSIXFERDIR_I2T);
+            vscsiReqSetXferSize(pVScsiReq, pVScsiReq->pbCDB[4]);
+
             /* Copy the parameters. */
             cbCopied = RTSgBufCopyToBuf(&pVScsiReq->SgBuf, &abParms[0], sizeof(abParms));
 
@@ -312,7 +347,7 @@ static DECLCALLBACK(int) vscsiLunSbcReqProcess(PVSCSILUNINT pVScsiLun, PVSCSIREQ
             {
                 uint32_t    cbBlock;
 
-                cbBlock = vscsiBE2HU24(&abParms[4 + 5]);
+                cbBlock = scsiBE2H_U24(&abParms[4 + 5]);
                 Log2(("SBC: set LOGICAL BLOCK LENGTH to %u\n", cbBlock));
                 if (cbBlock == 512) /* Fixed block size. */
                 {
@@ -331,27 +366,28 @@ static DECLCALLBACK(int) vscsiLunSbcReqProcess(PVSCSILUNINT pVScsiLun, PVSCSIREQ
                                         |  (pVScsiReq->pbCDB[2] <<  8)
                                         | ((pVScsiReq->pbCDB[1] & 0x1f) << 16));
             cSectorTransfer = pVScsiReq->pbCDB[4];
+            cSectorTransfer = cSectorTransfer ? cSectorTransfer : 256;  /* Zero blocks means 256 */
             break;
         }
         case SCSI_READ_10:
         {
             enmTxDir        = VSCSIIOREQTXDIR_READ;
-            uLbaStart       = vscsiBE2HU32(&pVScsiReq->pbCDB[2]);
-            cSectorTransfer = vscsiBE2HU16(&pVScsiReq->pbCDB[7]);
+            uLbaStart       = scsiBE2H_U32(&pVScsiReq->pbCDB[2]);
+            cSectorTransfer = scsiBE2H_U16(&pVScsiReq->pbCDB[7]);
             break;
         }
         case SCSI_READ_12:
         {
             enmTxDir        = VSCSIIOREQTXDIR_READ;
-            uLbaStart       = vscsiBE2HU32(&pVScsiReq->pbCDB[2]);
-            cSectorTransfer = vscsiBE2HU32(&pVScsiReq->pbCDB[6]);
+            uLbaStart       = scsiBE2H_U32(&pVScsiReq->pbCDB[2]);
+            cSectorTransfer = scsiBE2H_U32(&pVScsiReq->pbCDB[6]);
             break;
         }
         case SCSI_READ_16:
         {
             enmTxDir        = VSCSIIOREQTXDIR_READ;
-            uLbaStart       = vscsiBE2HU64(&pVScsiReq->pbCDB[2]);
-            cSectorTransfer = vscsiBE2HU32(&pVScsiReq->pbCDB[10]);
+            uLbaStart       = scsiBE2H_U64(&pVScsiReq->pbCDB[2]);
+            cSectorTransfer = scsiBE2H_U32(&pVScsiReq->pbCDB[10]);
             break;
         }
         case SCSI_WRITE_6:
@@ -361,27 +397,28 @@ static DECLCALLBACK(int) vscsiLunSbcReqProcess(PVSCSILUNINT pVScsiLun, PVSCSIREQ
                                         | (pVScsiReq->pbCDB[2] <<  8)
                                         | ((pVScsiReq->pbCDB[1] & 0x1f) << 16));
             cSectorTransfer = pVScsiReq->pbCDB[4];
+            cSectorTransfer = cSectorTransfer ? cSectorTransfer : 256;  /* Zero blocks means 256 */
             break;
         }
         case SCSI_WRITE_10:
         {
             enmTxDir        = VSCSIIOREQTXDIR_WRITE;
-            uLbaStart       = vscsiBE2HU32(&pVScsiReq->pbCDB[2]);
-            cSectorTransfer = vscsiBE2HU16(&pVScsiReq->pbCDB[7]);
+            uLbaStart       = scsiBE2H_U32(&pVScsiReq->pbCDB[2]);
+            cSectorTransfer = scsiBE2H_U16(&pVScsiReq->pbCDB[7]);
             break;
         }
         case SCSI_WRITE_12:
         {
             enmTxDir        = VSCSIIOREQTXDIR_WRITE;
-            uLbaStart       = vscsiBE2HU32(&pVScsiReq->pbCDB[2]);
-            cSectorTransfer = vscsiBE2HU32(&pVScsiReq->pbCDB[6]);
+            uLbaStart       = scsiBE2H_U32(&pVScsiReq->pbCDB[2]);
+            cSectorTransfer = scsiBE2H_U32(&pVScsiReq->pbCDB[6]);
             break;
         }
         case SCSI_WRITE_16:
         {
             enmTxDir        = VSCSIIOREQTXDIR_WRITE;
-            uLbaStart       = vscsiBE2HU64(&pVScsiReq->pbCDB[2]);
-            cSectorTransfer = vscsiBE2HU32(&pVScsiReq->pbCDB[10]);
+            uLbaStart       = scsiBE2H_U64(&pVScsiReq->pbCDB[2]);
+            cSectorTransfer = scsiBE2H_U32(&pVScsiReq->pbCDB[10]);
             break;
         }
         case SCSI_SYNCHRONIZE_CACHE:
@@ -407,6 +444,9 @@ static DECLCALLBACK(int) vscsiLunSbcReqProcess(PVSCSILUNINT pVScsiLun, PVSCSIREQ
                     /* We do not implement an echo buffer. */
                     memset(aReply, 0, sizeof(aReply));
 
+                    vscsiReqSetXferDir(pVScsiReq, VSCSIXFERDIR_T2I);
+                    vscsiReqSetXferSize(pVScsiReq, RT_MIN(sizeof(aReply), scsiBE2H_U24(&pVScsiReq->pbCDB[6])));
+
                     RTSgBufCopyFromBuf(&pVScsiReq->SgBuf, aReply, sizeof(aReply));
                     rcReq =  vscsiLunReqSenseOkSet(pVScsiLun, pVScsiReq);
                     break;
@@ -422,6 +462,8 @@ static DECLCALLBACK(int) vscsiLunSbcReqProcess(PVSCSILUNINT pVScsiLun, PVSCSIREQ
         case SCSI_VERIFY_10:
         case SCSI_START_STOP_UNIT:
         {
+            vscsiReqSetXferDir(pVScsiReq, VSCSIXFERDIR_NONE);
+            vscsiReqSetXferSize(pVScsiReq, 0);
             rcReq = vscsiLunReqSenseOkSet(pVScsiLun, pVScsiReq);
             break;
         }
@@ -442,12 +484,16 @@ static DECLCALLBACK(int) vscsiLunSbcReqProcess(PVSCSILUNINT pVScsiLun, PVSCSIREQ
                         aReply[1] = 0;
                         aReply[2] = 0;
                         aReply[3] = 0;
+
+                        vscsiReqSetXferDir(pVScsiReq, VSCSIXFERDIR_T2I);
+                        vscsiReqSetXferSize(pVScsiReq, RT_MIN(sizeof(aReply), scsiBE2H_U16(&pVScsiReq->pbCDB[7])));
+
                         RTSgBufCopyFromBuf(&pVScsiReq->SgBuf, aReply, sizeof(aReply));
                         rcReq = vscsiLunReqSenseOkSet(pVScsiLun, pVScsiReq);
                         break;
                     }
                 }
-                /* fall thru */
+                RT_FALL_THRU();
                 default:
                     rcReq = vscsiLunReqSenseErrorSet(pVScsiLun, pVScsiReq, SCSI_SENSE_ILLEGAL_REQUEST, SCSI_ASC_INV_FIELD_IN_CMD_PACKET, 0x00);
             }
@@ -462,12 +508,14 @@ static DECLCALLBACK(int) vscsiLunSbcReqProcess(PVSCSILUNINT pVScsiLun, PVSCSIREQ
                     uint8_t aReply[32];
 
                     memset(aReply, 0, sizeof(aReply));
-                    vscsiH2BEU64(aReply, pVScsiLunSbc->cSectors - 1);
-                    vscsiH2BEU32(&aReply[8], 512);
+                    scsiH2BE_U64(aReply, pVScsiLunSbc->cSectors - 1);
+                    scsiH2BE_U32(&aReply[8], 512);
                     if (pVScsiLun->fFeatures & VSCSI_LUN_FEATURE_UNMAP)
                         aReply[14] = 0x80; /* LPME enabled */
                     /* Leave the rest 0 */
 
+                    vscsiReqSetXferDir(pVScsiReq, VSCSIXFERDIR_T2I);
+                    vscsiReqSetXferSize(pVScsiReq, sizeof(aReply));
                     RTSgBufCopyFromBuf(&pVScsiReq->SgBuf, aReply, sizeof(aReply));
                     rcReq = vscsiLunReqSenseOkSet(pVScsiLun, pVScsiReq);
                     break;
@@ -483,9 +531,11 @@ static DECLCALLBACK(int) vscsiLunSbcReqProcess(PVSCSILUNINT pVScsiLun, PVSCSIREQ
             {
                 uint8_t abHdr[8];
                 size_t cbCopied;
-                size_t cbList = vscsiBE2HU16(&pVScsiReq->pbCDB[7]);
+                size_t cbList = scsiBE2H_U16(&pVScsiReq->pbCDB[7]);
 
                 /* Copy the header. */
+                vscsiReqSetXferDir(pVScsiReq, VSCSIXFERDIR_I2T);
+                vscsiReqSetXferSize(pVScsiReq, cbList);
                 cbCopied = RTSgBufCopyToBuf(&pVScsiReq->SgBuf, &abHdr[0], sizeof(abHdr));
 
                 /* Using the anchor bit is not supported. */
@@ -493,7 +543,7 @@ static DECLCALLBACK(int) vscsiLunSbcReqProcess(PVSCSILUNINT pVScsiLun, PVSCSIREQ
                     && cbCopied == sizeof(abHdr)
                     && cbList >= 8)
                 {
-                    uint32_t    cBlkDesc = vscsiBE2HU16(&abHdr[2]) / 16;
+                    uint32_t    cBlkDesc = scsiBE2H_U16(&abHdr[2]) / 16;
 
                     if (cBlkDesc)
                     {
@@ -511,8 +561,8 @@ static DECLCALLBACK(int) vscsiLunSbcReqProcess(PVSCSILUNINT pVScsiLun, PVSCSIREQ
                                     break;
                                 }
 
-                                paRanges[i].offStart = vscsiBE2HU64(&abBlkDesc[0]) * 512;
-                                paRanges[i].cbRange = vscsiBE2HU32(&abBlkDesc[8]) * 512;
+                                paRanges[i].offStart = scsiBE2H_U64(&abBlkDesc[0]) * 512;
+                                paRanges[i].cbRange = (size_t)scsiBE2H_U32(&abBlkDesc[8]) * 512;
                             }
 
                             if (rcReq == SCSI_STATUS_OK)
@@ -546,14 +596,18 @@ static DECLCALLBACK(int) vscsiLunSbcReqProcess(PVSCSILUNINT pVScsiLun, PVSCSIREQ
         LogFlow(("%s: uLbaStart=%llu cSectorTransfer=%u\n",
                  __FUNCTION__, uLbaStart, cSectorTransfer));
 
+        vscsiReqSetXferSize(pVScsiReq, cSectorTransfer * 512);
+
         if (RT_UNLIKELY(uLbaStart + cSectorTransfer > pVScsiLunSbc->cSectors))
         {
+            vscsiReqSetXferDir(pVScsiReq, VSCSIXFERDIR_NONE);
             rcReq = vscsiLunReqSenseErrorSet(pVScsiLun, pVScsiReq, SCSI_SENSE_ILLEGAL_REQUEST, SCSI_ASC_LOGICAL_BLOCK_OOR, 0x00);
             vscsiDeviceReqComplete(pVScsiLun->pVScsiDevice, pVScsiReq, rcReq, false, VINF_SUCCESS);
         }
         else if (!cSectorTransfer)
         {
             /* A 0 transfer length is not an error. */
+            vscsiReqSetXferDir(pVScsiReq, VSCSIXFERDIR_NONE);
             rcReq = vscsiLunReqSenseOkSet(pVScsiLun, pVScsiReq);
             vscsiDeviceReqComplete(pVScsiLun->pVScsiDevice, pVScsiReq, rcReq, false, VINF_SUCCESS);
         }
@@ -563,15 +617,23 @@ static DECLCALLBACK(int) vscsiLunSbcReqProcess(PVSCSILUNINT pVScsiLun, PVSCSIREQ
             if (   (   enmTxDir == VSCSIIOREQTXDIR_WRITE
                     || enmTxDir == VSCSIIOREQTXDIR_FLUSH)
                 && (pVScsiLun->fFeatures & VSCSI_LUN_FEATURE_READONLY))
+            {
                 rcReq = vscsiLunReqSenseErrorSet(pVScsiLun, pVScsiReq, SCSI_SENSE_DATA_PROTECT, SCSI_ASC_WRITE_PROTECTED, 0x00);
+                vscsiDeviceReqComplete(pVScsiLun->pVScsiDevice, pVScsiReq, rcReq, false, VINF_SUCCESS);
+            }
             else
+            {
+                vscsiReqSetXferDir(pVScsiReq, enmTxDir == VSCSIIOREQTXDIR_WRITE ? VSCSIXFERDIR_I2T : VSCSIXFERDIR_T2I);
                 rc = vscsiIoReqTransferEnqueue(pVScsiLun, pVScsiReq, enmTxDir,
                                                uLbaStart * 512, cSectorTransfer * 512);
+            }
         }
     }
-    else if (pVScsiReq->pbCDB[0] ==  SCSI_SYNCHRONIZE_CACHE)
+    else if (pVScsiReq->pbCDB[0] == SCSI_SYNCHRONIZE_CACHE)
     {
         /* Enqueue flush */
+        vscsiReqSetXferDir(pVScsiReq, VSCSIXFERDIR_NONE);
+        vscsiReqSetXferSize(pVScsiReq, 0);
         rc = vscsiIoReqFlushEnqueue(pVScsiLun, pVScsiReq);
     }
     else if (pVScsiReq->pbCDB[0] !=  SCSI_UNMAP) /* Request completed */
@@ -580,6 +642,92 @@ static DECLCALLBACK(int) vscsiLunSbcReqProcess(PVSCSILUNINT pVScsiLun, PVSCSIREQ
     return rc;
 }
 
+
+/**
+ * The supported operation codes for the SBC LUN type.
+ *
+ * @note This gives the minimum size required by our implementation
+ *       which may be smaller than what the spec defines (for example
+ *       we do not access the control byte at the end).
+ */
+static uint8_t s_acbCdbOpc[] =
+{
+    VSCSI_LUN_CDB_SZ_INVALID_X8,     /**< 0x00 - 0x07 Invalid */
+    5,                               /**< 0x08        READ (6) */
+    VSCSI_LUN_CDB_SZ_INVALID,        /**< 0x09        Invalid */
+    5,                               /**< 0x0a        WRITE (6) */
+    VSCSI_LUN_CDB_SZ_INVALID,        /**< 0x0b        Invalid */
+    VSCSI_LUN_CDB_SZ_INVALID_X4,     /**< 0x0c - 0x0f Invalid */
+
+    VSCSI_LUN_CDB_SZ_INVALID_X2,     /**< 0x10 - 0x11 Invalid */
+    3,                               /**< 0x12        INQUIRY (at least 3) */
+    VSCSI_LUN_CDB_SZ_INVALID_X2,     /**< 0x13 - 0x14 Invalid */
+    5,                               /**< 0x15        MODE SELECT (6) */
+    VSCSI_LUN_CDB_SZ_INVALID_X4,     /**< 0x16 - 0x19 Invalid */
+    5,                               /**< 0x1a        MODE SENSE (6) */
+    1,                               /**< 0x1b        START STOP UNIT */
+    VSCSI_LUN_CDB_SZ_INVALID_X4,     /**< 0x1c - 0x1f Invalid */
+
+    VSCSI_LUN_CDB_SZ_INVALID_X4,     /**< 0x20 - 0x23 Invalid */
+    VSCSI_LUN_CDB_SZ_INVALID,        /**< 0x24        Invalid */
+    1,                               /**< 0x25        READ CAPACITY */
+    VSCSI_LUN_CDB_SZ_INVALID_X2,     /**< 0x26 - 0x27 Invalid */
+    9,                               /**< 0x28        READ (10) */
+    VSCSI_LUN_CDB_SZ_INVALID,        /**< 0x29        Invalid */
+    9,                               /**< 0x2a        WRITE (10) */
+    VSCSI_LUN_CDB_SZ_INVALID_X4,     /**< 0x2b - 0x2e Invalid */
+    1,                               /**< 0x2f        VERIFY (10) */
+
+    VSCSI_LUN_CDB_SZ_INVALID_X4,     /**< 0x30 - 0x33 Invalid */
+    VSCSI_LUN_CDB_SZ_INVALID,        /**< 0x34        Invalid */
+    1,                               /**< 0x35        SYNCHRONIZE CACHE */
+    VSCSI_LUN_CDB_SZ_INVALID_X4,     /**< 0x36 - 0x39 Invalid */
+    VSCSI_LUN_CDB_SZ_INVALID_X2,     /**< 0x3a - 0x3b Invalid */
+    8,                               /**< 0x3c        READ BUFFER */
+    VSCSI_LUN_CDB_SZ_INVALID,        /**< 0x3d        Invalid */
+    VSCSI_LUN_CDB_SZ_INVALID_X2,     /**< 0x3e - 0x3f Invalid */
+
+    VSCSI_LUN_CDB_SZ_INVALID_X2,     /**< 0x40 - 0x41 Invalid */
+    9,                               /**< 0x42        UNMAP */
+    VSCSI_LUN_CDB_SZ_INVALID,        /**< 0x43        Invalid */
+    VSCSI_LUN_CDB_SZ_INVALID_X8,     /**< 0x44 - 0x4b Invalid */
+    VSCSI_LUN_CDB_SZ_INVALID,        /**< 0x4c        Invalid */
+    9,                               /**< 0x4d        LOG SENSE */
+    VSCSI_LUN_CDB_SZ_INVALID_X2,     /**< 0x4e - 0x4f Invalid */
+
+    VSCSI_LUN_CDB_SZ_INVALID_X16,    /**< 0x50 - 0x5f Invalid */
+    VSCSI_LUN_CDB_SZ_INVALID_X16,    /**< 0x60 - 0x6f Invalid */
+    VSCSI_LUN_CDB_SZ_INVALID_X16,    /**< 0x70 - 0x7f Invalid */
+
+    VSCSI_LUN_CDB_SZ_INVALID_X8,     /**< 0x80 - 0x87 Invalid */
+    14,                              /**< 0x88        READ (16) */
+    VSCSI_LUN_CDB_SZ_INVALID,        /**< 0x89        Invalid */
+    14,                              /**< 0x8a        WRITE (16) */
+    VSCSI_LUN_CDB_SZ_INVALID,        /**< 0x8b        Invalid */
+    VSCSI_LUN_CDB_SZ_INVALID_X4,     /**< 0x8c - 0x8f Invalid */
+
+    VSCSI_LUN_CDB_SZ_INVALID_X8,     /**< 0x90 - 0x97 Invalid */
+    VSCSI_LUN_CDB_SZ_INVALID_X4,     /**< 0x98 - 0x9b Invalid */
+    VSCSI_LUN_CDB_SZ_INVALID_X2,     /**< 0x9c - 0x9d Invalid */
+    2,                               /**< 0x9e        SERVICE ACTION IN (16) (at least 2). */
+    VSCSI_LUN_CDB_SZ_INVALID,        /**< 0x9f        Invalid */
+
+    VSCSI_LUN_CDB_SZ_INVALID_X8,     /**< 0xa0 - 0xa7 Invalid */
+    10,                              /**< 0xa8        READ (12) */
+    VSCSI_LUN_CDB_SZ_INVALID,        /**< 0xa9        Invalid */
+    10,                              /**< 0xaa        WRITE (12) */
+    VSCSI_LUN_CDB_SZ_INVALID,        /**< 0xab        Invalid */
+    VSCSI_LUN_CDB_SZ_INVALID_X4,     /**< 0xac - 0xaf Invalid */
+
+    VSCSI_LUN_CDB_SZ_INVALID_X16,    /**< 0xb0 - 0xbf Invalid */
+    VSCSI_LUN_CDB_SZ_INVALID_X16,    /**< 0xc0 - 0xcf Invalid */
+    VSCSI_LUN_CDB_SZ_INVALID_X16,    /**< 0xd0 - 0xdf Invalid */
+    VSCSI_LUN_CDB_SZ_INVALID_X16,    /**< 0xe0 - 0xef Invalid */
+    VSCSI_LUN_CDB_SZ_INVALID_X16     /**< 0xf0 - 0xff Invalid */
+};
+AssertCompileSize(s_acbCdbOpc, 256 * sizeof(uint8_t));
+
+
 VSCSILUNDESC g_VScsiLunTypeSbc =
 {
     /** enmLunType */
@@ -588,11 +736,23 @@ VSCSILUNDESC g_VScsiLunTypeSbc =
     "SBC",
     /** cbLun */
     sizeof(VSCSILUNSBC),
+    /** pacbCdbOpc */
+    &s_acbCdbOpc[0],
+    /** cSupOpcInfo */
+    0,
+    /** paSupOpcInfo */
+    NULL,
     /** pfnVScsiLunInit */
     vscsiLunSbcInit,
     /** pfnVScsiLunDestroy */
     vscsiLunSbcDestroy,
     /** pfnVScsiLunReqProcess */
-    vscsiLunSbcReqProcess
+    vscsiLunSbcReqProcess,
+    /** pfnVScsiLunReqFree */
+    NULL,
+    /** pfnVScsiLunMediumInserted */
+    NULL,
+    /** pfnVScsiLunMediumRemoved */
+    NULL
 };
 
