@@ -6,20 +6,31 @@
 /*
  * Includes contributions from Hans Petter Selasky
  *
- * Copyright (C) 2006-2007 Oracle Corporation
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #define LOG_GROUP LOG_GROUP_DRV_USBPROXY
 #ifdef VBOX
 # include <iprt/stdint.h>
@@ -52,6 +63,7 @@
 #include <iprt/asm.h>
 #include <iprt/string.h>
 #include <iprt/file.h>
+#include <iprt/pipe.h>
 #include "../USBProxyDevice.h"
 
 /** Maximum endpoints supported. */
@@ -63,9 +75,10 @@
 # define VUSB_DIR_TO_DEV        0x00
 #endif
 
-/*******************************************************************************
-*   Structures and Typedefs                                                    *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Structures and Typedefs                                                                                                      *
+*********************************************************************************************************************************/
 typedef struct USBENDPOINTFBSD
 {
     /** Flag whether it is opened. */
@@ -95,19 +108,24 @@ typedef struct USBPROXYDEVFBSD
 {
     /** The open file. */
     RTFILE                 hFile;
-    /** Software endpoint structures */
-    USBENDPOINTFBSD        aSwEndpoint[USBFBSD_MAXENDPOINTS];
     /** Flag whether an URB is cancelling. */
     bool                   fCancelling;
     /** Flag whether initialised or not */
     bool                   fInit;
+    /** Pipe handle for waking up - writing end. */
+    RTPIPE                 hPipeWakeupW;
+    /** Pipe handle for waking up - reading end. */
+    RTPIPE                 hPipeWakeupR;
+    /** Software endpoint structures */
+    USBENDPOINTFBSD        aSwEndpoint[USBFBSD_MAXENDPOINTS];
     /** Kernel endpoint structures */
     struct usb_fs_endpoint aHwEndpoint[USBFBSD_MAXENDPOINTS];
 } USBPROXYDEVFBSD, *PUSBPROXYDEVFBSD;
 
-/*******************************************************************************
-*   Internal Functions                                                         *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Internal Functions                                                                                                           *
+*********************************************************************************************************************************/
 static int usbProxyFreeBSDEndpointClose(PUSBPROXYDEV pProxyDev, int Endpoint);
 
 /**
@@ -127,7 +145,7 @@ static int usbProxyFreeBSDDoIoCtl(PUSBPROXYDEV pProxyDev, unsigned long iCmd,
                                   void *pvArg, bool fHandleNoDev)
 {
     int rc = VINF_SUCCESS;
-    PUSBPROXYDEVFBSD pDevFBSD = (PUSBPROXYDEVFBSD) pProxyDev->Backend.pv;
+    PUSBPROXYDEVFBSD pDevFBSD = USBPROXYDEV_2_DATA(pProxyDev, PUSBPROXYDEVFBSD);
 
     LogFlow(("usbProxyFreeBSDDoIoCtl: iCmd=%#x\n", iCmd));
 
@@ -158,7 +176,7 @@ static int usbProxyFreeBSDDoIoCtl(PUSBPROXYDEV pProxyDev, unsigned long iCmd,
 static int usbProxyFreeBSDFsInit(PUSBPROXYDEV pProxyDev)
 {
     struct usb_fs_init UsbFsInit;
-    PUSBPROXYDEVFBSD pDevFBSD = (PUSBPROXYDEVFBSD) pProxyDev->Backend.pv;
+    PUSBPROXYDEVFBSD pDevFBSD = USBPROXYDEV_2_DATA(pProxyDev, PUSBPROXYDEVFBSD);
     int rc;
 
     LogFlow(("usbProxyFreeBSDFsInit: pProxyDev=%p\n", (void *)pProxyDev));
@@ -189,7 +207,7 @@ static int usbProxyFreeBSDFsInit(PUSBPROXYDEV pProxyDev)
 static int usbProxyFreeBSDFsUnInit(PUSBPROXYDEV pProxyDev)
 {
     struct usb_fs_uninit UsbFsUninit;
-    PUSBPROXYDEVFBSD pDevFBSD = (PUSBPROXYDEVFBSD) pProxyDev->Backend.pv;
+    PUSBPROXYDEVFBSD pDevFBSD = USBPROXYDEV_2_DATA(pProxyDev, PUSBPROXYDEVFBSD);
     int rc;
 
     LogFlow(("usbProxyFreeBSDFsUnInit: ProxyDev=%p\n", (void *)pProxyDev));
@@ -240,7 +258,7 @@ static void usbProxyFreeBSDSetupReq(struct usb_device_request *pSetupData,
 
 static int usbProxyFreeBSDEndpointOpen(PUSBPROXYDEV pProxyDev, int Endpoint, bool fIsoc, int index)
 {
-    PUSBPROXYDEVFBSD pDevFBSD = (PUSBPROXYDEVFBSD) pProxyDev->Backend.pv;
+    PUSBPROXYDEVFBSD pDevFBSD = USBPROXYDEV_2_DATA(pProxyDev, PUSBPROXYDEVFBSD);
     PUSBENDPOINTFBSD pEndpointFBSD = NULL; /* shut up gcc */
     struct usb_fs_endpoint *pXferEndpoint;
     struct usb_fs_open UsbFsOpen;
@@ -315,7 +333,7 @@ static int usbProxyFreeBSDEndpointOpen(PUSBPROXYDEV pProxyDev, int Endpoint, boo
  */
 static int usbProxyFreeBSDEndpointClose(PUSBPROXYDEV pProxyDev, int Endpoint)
 {
-    PUSBPROXYDEVFBSD pDevFBSD = (PUSBPROXYDEVFBSD) pProxyDev->Backend.pv;
+    PUSBPROXYDEVFBSD pDevFBSD = USBPROXYDEV_2_DATA(pProxyDev, PUSBPROXYDEVFBSD);
     PUSBENDPOINTFBSD pEndpointFBSD = &pDevFBSD->aSwEndpoint[Endpoint];
     struct usb_fs_close UsbFsClose;
     int rc = VINF_SUCCESS;
@@ -357,11 +375,10 @@ static int usbProxyFreeBSDEndpointClose(PUSBPROXYDEV pProxyDev, int Endpoint)
  *                          the form "sysfs:<sysfs path>//device:<device node>".
  *                          In the second case, the two paths are guaranteed
  *                          not to contain the substring "//".
- * @param   pvBackend       Backend specific pointer, unused for the linux backend.
  */
-static int usbProxyFreeBSDOpen(PUSBPROXYDEV pProxyDev, const char *pszAddress,
-                               void *pvBackend)
+static DECLCALLBACK(int) usbProxyFreeBSDOpen(PUSBPROXYDEV pProxyDev, const char *pszAddress)
 {
+    PUSBPROXYDEVFBSD pDevFBSD = USBPROXYDEV_2_DATA(pProxyDev, PUSBPROXYDEVFBSD);
     int rc;
 
     LogFlow(("usbProxyFreeBSDOpen: pProxyDev=%p pszAddress=%s\n", pProxyDev, pszAddress));
@@ -374,15 +391,16 @@ static int usbProxyFreeBSDOpen(PUSBPROXYDEV pProxyDev, const char *pszAddress,
     if (RT_SUCCESS(rc))
     {
         /*
-         * Allocate and initialize the linux backend data.
+         * Initialize the FreeBSD backend data.
          */
-        PUSBPROXYDEVFBSD pDevFBSD = (PUSBPROXYDEVFBSD)RTMemAllocZ(sizeof(USBPROXYDEVFBSD));
-        if (pDevFBSD)
+        pDevFBSD->hFile = hFile;
+        rc = usbProxyFreeBSDFsInit(pProxyDev);
+        if (RT_SUCCESS(rc))
         {
-            pDevFBSD->hFile = hFile;
-            pProxyDev->Backend.pv = pDevFBSD;
-
-            rc = usbProxyFreeBSDFsInit(pProxyDev);
+            /*
+             * Create wakeup pipe.
+             */
+            rc = RTPipeCreate(&pDevFBSD->hPipeWakeupR, &pDevFBSD->hPipeWakeupW, 0);
             if (RT_SUCCESS(rc))
             {
                 LogFlow(("usbProxyFreeBSDOpen(%p, %s): returns successfully hFile=%RTfile iActiveCfg=%d\n",
@@ -390,11 +408,7 @@ static int usbProxyFreeBSDOpen(PUSBPROXYDEV pProxyDev, const char *pszAddress,
 
                 return VINF_SUCCESS;
             }
-
-            RTMemFree(pDevFBSD);
         }
-        else
-            rc = VERR_NO_MEMORY;
 
         RTFileClose(hFile);
     }
@@ -404,9 +418,6 @@ static int usbProxyFreeBSDOpen(PUSBPROXYDEV pProxyDev, const char *pszAddress,
     Log(("usbProxyFreeBSDOpen(%p, %s) failed, rc=%d!\n",
          pProxyDev, pszAddress, rc));
 
-    pProxyDev->Backend.pv = NULL;
-
-    NOREF(pvBackend);
     return rc;
 }
 
@@ -418,9 +429,9 @@ static int usbProxyFreeBSDOpen(PUSBPROXYDEV pProxyDev, const char *pszAddress,
  * @returns VINF_SUCCESS.
  * @param   pProxyDev       The proxy device.
  */
-static int usbProxyFreeBSDInit(PUSBPROXYDEV pProxyDev)
+static DECLCALLBACK(int) usbProxyFreeBSDInit(PUSBPROXYDEV pProxyDev)
 {
-    PUSBPROXYDEVFBSD pDevFBSD = (PUSBPROXYDEVFBSD) pProxyDev->Backend.pv;
+    PUSBPROXYDEVFBSD pDevFBSD = USBPROXYDEV_2_DATA(pProxyDev, PUSBPROXYDEVFBSD);
     int rc;
 
     LogFlow(("usbProxyFreeBSDInit: pProxyDev=%s\n",
@@ -448,9 +459,9 @@ static int usbProxyFreeBSDInit(PUSBPROXYDEV pProxyDev)
 /**
  * Closes the proxy device.
  */
-static void usbProxyFreeBSDClose(PUSBPROXYDEV pProxyDev)
+static DECLCALLBACK(void) usbProxyFreeBSDClose(PUSBPROXYDEV pProxyDev)
 {
-    PUSBPROXYDEVFBSD pDevFBSD = (PUSBPROXYDEVFBSD)pProxyDev->Backend.pv;
+    PUSBPROXYDEVFBSD pDevFBSD = USBPROXYDEV_2_DATA(pProxyDev, PUSBPROXYDEVFBSD);
 
     LogFlow(("usbProxyFreeBSDClose: pProxyDev=%s\n", pProxyDev->pUsbIns->pszName));
 
@@ -459,12 +470,12 @@ static void usbProxyFreeBSDClose(PUSBPROXYDEV pProxyDev)
 
     usbProxyFreeBSDFsUnInit(pProxyDev);
 
+    RTPipeClose(pDevFBSD->hPipeWakeupR);
+    RTPipeClose(pDevFBSD->hPipeWakeupW);
+
     RTFileClose(pDevFBSD->hFile);
     pDevFBSD->hFile = NIL_RTFILE;
 
-    RTMemFree(pDevFBSD);
-    pProxyDev->Backend.pv = NULL;
-
     LogFlow(("usbProxyFreeBSDClose: returns\n"));
 }
 
@@ -474,9 +485,9 @@ static void usbProxyFreeBSDClose(PUSBPROXYDEV pProxyDev)
  * @returns VBox status code.
  * @param   pDev    The device to reset.
  */
-static int usbProxyFreeBSDReset(PUSBPROXYDEV pProxyDev, bool fResetOnFreeBSD)
+static DECLCALLBACK(int) usbProxyFreeBSDReset(PUSBPROXYDEV pProxyDev, bool fResetOnFreeBSD)
 {
-    PUSBPROXYDEVFBSD pDevFBSD = (PUSBPROXYDEVFBSD) pProxyDev->Backend.pv;
+    PUSBPROXYDEVFBSD pDevFBSD = USBPROXYDEV_2_DATA(pProxyDev, PUSBPROXYDEVFBSD);
     int iParm;
     int rc = VINF_SUCCESS;
 
@@ -531,9 +542,9 @@ done:
  * @param   pProxyDev       The device instance data.
  * @param   iCfg            The configuration to set.
  */
-static int usbProxyFreeBSDSetConfig(PUSBPROXYDEV pProxyDev, int iCfg)
+static DECLCALLBACK(int) usbProxyFreeBSDSetConfig(PUSBPROXYDEV pProxyDev, int iCfg)
 {
-    PUSBPROXYDEVFBSD pDevFBSD = (PUSBPROXYDEVFBSD) pProxyDev->Backend.pv;
+    PUSBPROXYDEVFBSD pDevFBSD = USBPROXYDEV_2_DATA(pProxyDev, PUSBPROXYDEVFBSD);
     int iCfgIndex;
     int rc;
 
@@ -546,7 +557,7 @@ static int usbProxyFreeBSDSetConfig(PUSBPROXYDEV pProxyDev, int iCfg)
     {
         LogFlow(("usbProxyFreeBSDSetInterface: Freeing kernel resources "
                  "failed failed rc=%d\n", rc));
-        return false;
+        return rc;
     }
 
     if (iCfg == 0)
@@ -567,28 +578,24 @@ static int usbProxyFreeBSDSetConfig(PUSBPROXYDEV pProxyDev, int iCfg)
         {
             LogFlow(("usbProxyFreeBSDSetConfig: configuration "
                      "%d not found\n", iCfg));
-            return false;
+            return VERR_NOT_FOUND;
         }
     }
 
     /* Set the config */
     rc = usbProxyFreeBSDDoIoCtl(pProxyDev, USB_SET_CONFIG, &iCfgIndex, true);
     if (RT_FAILURE(rc))
-        return false;
+        return rc;
 
     /* Allocate kernel ressources again. */
-    rc = usbProxyFreeBSDFsInit(pProxyDev);
-    if (RT_FAILURE(rc))
-        return false;
-
-    return true;
+    return usbProxyFreeBSDFsInit(pProxyDev);
 }
 
 /**
  * Claims an interface.
  * @returns success indicator.
  */
-static int usbProxyFreeBSDClaimInterface(PUSBPROXYDEV pProxyDev, int iIf)
+static DECLCALLBACK(int) usbProxyFreeBSDClaimInterface(PUSBPROXYDEV pProxyDev, int iIf)
 {
     int rc;
 
@@ -599,32 +606,24 @@ static int usbProxyFreeBSDClaimInterface(PUSBPROXYDEV pProxyDev, int iIf)
      * Try to detach kernel driver on this interface, ignore any
      * failures
      */
-    rc = usbProxyFreeBSDDoIoCtl(pProxyDev, USB_IFACE_DRIVER_DETACH, &iIf, true);
+    usbProxyFreeBSDDoIoCtl(pProxyDev, USB_IFACE_DRIVER_DETACH, &iIf, true);
 
     /* Try to claim interface */
-    rc = usbProxyFreeBSDDoIoCtl(pProxyDev, USB_CLAIM_INTERFACE, &iIf, true);
-    if (RT_FAILURE(rc))
-        return false;
-
-    return true;
+    return usbProxyFreeBSDDoIoCtl(pProxyDev, USB_CLAIM_INTERFACE, &iIf, true);
 }
 
 /**
  * Releases an interface.
  * @returns success indicator.
  */
-static int usbProxyFreeBSDReleaseInterface(PUSBPROXYDEV pProxyDev, int iIf)
+static DECLCALLBACK(int) usbProxyFreeBSDReleaseInterface(PUSBPROXYDEV pProxyDev, int iIf)
 {
     int rc;
 
     LogFlow(("usbProxyFreeBSDReleaseInterface: pProxyDev=%s "
         "ifnum=%x\n", pProxyDev->pUsbIns->pszName, iIf));
 
-    rc = usbProxyFreeBSDDoIoCtl(pProxyDev, USB_RELEASE_INTERFACE, &iIf, true);
-    if (RT_FAILURE(rc))
-        return false;
-
-    return true;
+    return usbProxyFreeBSDDoIoCtl(pProxyDev, USB_RELEASE_INTERFACE, &iIf, true);
 }
 
 /**
@@ -632,10 +631,9 @@ static int usbProxyFreeBSDReleaseInterface(PUSBPROXYDEV pProxyDev, int iIf)
  *
  * @returns success indicator.
  */
-static int
-usbProxyFreeBSDSetInterface(PUSBPROXYDEV pProxyDev, int iIf, int iAlt)
+static DECLCALLBACK(int) usbProxyFreeBSDSetInterface(PUSBPROXYDEV pProxyDev, int iIf, int iAlt)
 {
-    PUSBPROXYDEVFBSD pDevFBSD = (PUSBPROXYDEVFBSD) pProxyDev->Backend.pv;
+    PUSBPROXYDEVFBSD pDevFBSD = USBPROXYDEV_2_DATA(pProxyDev, PUSBPROXYDEVFBSD);
     struct usb_alt_interface UsbIntAlt;
     int rc;
 
@@ -648,7 +646,7 @@ usbProxyFreeBSDSetInterface(PUSBPROXYDEV pProxyDev, int iIf, int iAlt)
     {
         LogFlow(("usbProxyFreeBSDSetInterface: Freeing kernel resources "
                  "failed failed rc=%d\n", rc));
-        return false;
+        return rc;
     }
     memset(&UsbIntAlt, 0, sizeof(UsbIntAlt));
     UsbIntAlt.uai_interface_index = iIf;
@@ -659,24 +657,17 @@ usbProxyFreeBSDSetInterface(PUSBPROXYDEV pProxyDev, int iIf, int iAlt)
     {
         LogFlow(("usbProxyFreeBSDSetInterface: Setting interface %d %d "
                  "failed rc=%d\n", iIf, iAlt, rc));
-        return false;
+        return rc;
     }
 
-    rc = usbProxyFreeBSDFsInit(pProxyDev);
-    if (RT_FAILURE(rc))
-        return false;
-
-    return true;
+    return usbProxyFreeBSDFsInit(pProxyDev);
 }
 
 /**
  * Clears the halted endpoint 'ep_num'.
  */
-static bool usbProxyFreeBSDClearHaltedEp(PUSBPROXYDEV pProxyDev, unsigned int ep_num)
+static DECLCALLBACK(int) usbProxyFreeBSDClearHaltedEp(PUSBPROXYDEV pProxyDev, unsigned int ep_num)
 {
-    struct usb_ctl_request Req;
-    int rc;
-
     LogFlow(("usbProxyFreeBSDClearHaltedEp: pProxyDev=%s ep_num=%u\n",
              pProxyDev->pUsbIns->pszName, ep_num));
 
@@ -685,31 +676,26 @@ static bool usbProxyFreeBSDClearHaltedEp(PUSBPROXYDEV pProxyDev, unsigned int ep
      * Just ignore it.
      */
     if ((ep_num & 0xF) == 0)
-        return true;
+        return VINF_SUCCESS;
 
+    struct usb_ctl_request Req;
     memset(&Req, 0, sizeof(Req));
-
     usbProxyFreeBSDSetupReq(&Req.ucr_request,
                             VUSB_DIR_TO_DEV | VUSB_TO_ENDPOINT,
                             VUSB_REQ_CLEAR_FEATURE, 0, ep_num, 0);
 
-    rc = usbProxyFreeBSDDoIoCtl(pProxyDev, USB_DO_REQUEST, &Req, true);
+    int rc = usbProxyFreeBSDDoIoCtl(pProxyDev, USB_DO_REQUEST, &Req, true);
 
     LogFlow(("usbProxyFreeBSDClearHaltedEp: rc=%Rrc\n", rc));
-
-    if (RT_FAILURE(rc))
-        return false;
-
-    return true;
+    return rc;
 }
 
 /**
- * @copydoc USBPROXYBACK::pfnUrbQueue
+ * @interface_method_impl{USBPROXYBACK,pfnUrbQueue}
  */
-static int usbProxyFreeBSDUrbQueue(PVUSBURB pUrb)
+static DECLCALLBACK(int) usbProxyFreeBSDUrbQueue(PUSBPROXYDEV pProxyDev, PVUSBURB pUrb)
 {
-    PUSBPROXYDEV pProxyDev = PDMINS_2_DATA(pUrb->pUsbIns, PUSBPROXYDEV);
-    PUSBPROXYDEVFBSD pDevFBSD = (PUSBPROXYDEVFBSD) pProxyDev->Backend.pv;
+    PUSBPROXYDEVFBSD pDevFBSD = USBPROXYDEV_2_DATA(pProxyDev, PUSBPROXYDEVFBSD);
     PUSBENDPOINTFBSD pEndpointFBSD;
     struct usb_fs_endpoint *pXferEndpoint;
     struct usb_fs_start UsbFsStart;
@@ -723,9 +709,10 @@ static int usbProxyFreeBSDUrbQueue(PVUSBURB pUrb)
              pUrb, (unsigned)pUrb->EndPt, (unsigned)pUrb->enmDir));
 
     ep_num = pUrb->EndPt;
-
-    if ((pUrb->enmType != VUSBXFERTYPE_MSG) && (pUrb->enmDir == VUSBDIRECTION_IN))
+    if ((pUrb->enmType != VUSBXFERTYPE_MSG) && (pUrb->enmDir == VUSBDIRECTION_IN)) {
+        /* set IN-direction bit */
         ep_num |= 0x80;
+    }
 
     index = 0;
 
@@ -736,7 +723,7 @@ retry:
                                         index);
 
     if (index < 0)
-        return false;
+        return VERR_INVALID_PARAMETER;
 
     pEndpointFBSD = &pDevFBSD->aSwEndpoint[index];
     pXferEndpoint = &pDevFBSD->aHwEndpoint[index];
@@ -834,12 +821,12 @@ retry:
             index++;
             goto retry;
         }
-        return false;
+        return rc;
     }
     pUrb->Dev.pvPrivate = (void *)(long)(index + 1);
     pEndpointFBSD->pUrb = pUrb;
 
-    return true;
+    return rc;
 }
 
 /**
@@ -850,14 +837,14 @@ retry:
  * @param   pProxyDev   The device.
  * @param   cMillies    Number of milliseconds to wait. Use 0 to not wait at all.
  */
-static PVUSBURB usbProxyFreeBSDUrbReap(PUSBPROXYDEV pProxyDev, RTMSINTERVAL cMillies)
+static DECLCALLBACK(PVUSBURB) usbProxyFreeBSDUrbReap(PUSBPROXYDEV pProxyDev, RTMSINTERVAL cMillies)
 {
     struct usb_fs_endpoint *pXferEndpoint;
-    PUSBPROXYDEVFBSD pDevFBSD = (PUSBPROXYDEVFBSD) pProxyDev->Backend.pv;
+    PUSBPROXYDEVFBSD pDevFBSD = USBPROXYDEV_2_DATA(pProxyDev, PUSBPROXYDEVFBSD);
     PUSBENDPOINTFBSD pEndpointFBSD;
     PVUSBURB pUrb;
     struct usb_fs_complete UsbFsComplete;
-    struct pollfd PollFd;
+    struct pollfd pfd[2];
     int rc;
 
     LogFlow(("usbProxyFreeBSDUrbReap: pProxyDev=%p, cMillies=%u\n",
@@ -981,23 +968,38 @@ repeat:
                  (unsigned)pEndpointFBSD->acbData[1]));
 
     }
-    else if (cMillies && rc == VERR_RESOURCE_BUSY)
+    else if (cMillies != 0 && rc == VERR_RESOURCE_BUSY)
     {
-        /* Poll for finished transfers */
-        PollFd.fd = RTFileToNative(pDevFBSD->hFile);
-        PollFd.events = POLLIN | POLLRDNORM;
-        PollFd.revents = 0;
+        for (;;)
+        {
+            pfd[0].fd = RTFileToNative(pDevFBSD->hFile);
+            pfd[0].events = POLLIN | POLLRDNORM;
+            pfd[0].revents = 0;
 
-        rc = poll(&PollFd, 1, (cMillies == RT_INDEFINITE_WAIT) ? INFTIM : cMillies);
-        if (rc >= 1)
-        {
-            goto repeat;
-        }
-        else
-        {
-            LogFlow(("usbProxyFreeBSDUrbReap: "
-                     "poll returned rc=%d\n", rc));
+            pfd[1].fd = RTPipeToNative(pDevFBSD->hPipeWakeupR);
+            pfd[1].events = POLLIN | POLLRDNORM;
+            pfd[1].revents = 0;
+
+            rc = poll(pfd, 2, (cMillies == RT_INDEFINITE_WAIT) ? INFTIM : cMillies);
+            if (rc > 0)
+            {
+                if (pfd[1].revents & POLLIN)
+                {
+                    /* Got woken up, drain pipe. */
+                    uint8_t bRead;
+                    size_t cbIgnored = 0;
+                    RTPipeRead(pDevFBSD->hPipeWakeupR, &bRead, 1, &cbIgnored);
+                    /* Make sure we return from this function */
+                    cMillies = 0;
+                }
+                break;
+            }
+            if (rc == 0)
+                return NULL;
+            if (errno != EAGAIN)
+                return NULL;
         }
+        goto repeat;
     }
     return pUrb;
 }
@@ -1006,19 +1008,27 @@ repeat:
  * Cancels the URB.
  * The URB requires reaping, so we don't change its state.
  */
-static void usbProxyFreeBSDUrbCancel(PVUSBURB pUrb)
+static DECLCALLBACK(int) usbProxyFreeBSDUrbCancel(PUSBPROXYDEV pProxyDev, PVUSBURB pUrb)
 {
-    PUSBPROXYDEV pProxyDev = PDMINS_2_DATA(pUrb->pUsbIns, PUSBPROXYDEV);
     int index;
 
     index = (int)(long)pUrb->Dev.pvPrivate - 1;
 
     if (index < 0 || index >= USBFBSD_MAXENDPOINTS)
-        return;
+        return VINF_SUCCESS; /* invalid index, pretend success. */
 
     LogFlow(("usbProxyFreeBSDUrbCancel: epindex=%u\n", (unsigned)index));
+    return usbProxyFreeBSDEndpointClose(pProxyDev, index);
+}
 
-    usbProxyFreeBSDEndpointClose(pProxyDev, index);
+static DECLCALLBACK(int) usbProxyFreeBSDWakeup(PUSBPROXYDEV pProxyDev)
+{
+    PUSBPROXYDEVFBSD pDevFBSD = USBPROXYDEV_2_DATA(pProxyDev, PUSBPROXYDEVFBSD);
+    size_t cbIgnored;
+
+    LogFlowFunc(("pProxyDev=%p\n", pProxyDev));
+
+    return RTPipeWrite(pDevFBSD->hPipeWakeupW, "", 1, &cbIgnored);
 }
 
 /**
@@ -1026,7 +1036,10 @@ static void usbProxyFreeBSDUrbCancel(PVUSBURB pUrb)
  */
 extern const USBPROXYBACK g_USBProxyDeviceHost =
 {
+    /* pszName */
     "host",
+    /* cbBackend */
+    sizeof(USBPROXYDEVFBSD),
     usbProxyFreeBSDOpen,
     usbProxyFreeBSDInit,
     usbProxyFreeBSDClose,
@@ -1039,6 +1052,7 @@ extern const USBPROXYBACK g_USBProxyDeviceHost =
     usbProxyFreeBSDUrbQueue,
     usbProxyFreeBSDUrbCancel,
     usbProxyFreeBSDUrbReap,
+    usbProxyFreeBSDWakeup,
     0
 };
 
