@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2006-2019 Oracle Corporation
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
@@ -84,24 +94,12 @@ static inline bool rtcsTrue() { return true; }
  */
 typedef struct USBPROXYURBLNX
 {
-    /** The kernel URB data. */
-#if RT_GNUC_PREREQ(6, 0)
-    /* gcc 6.2 complains about the [] member of KUrb */
-# pragma GCC diagnostic push
-# pragma GCC diagnostic ignored "-Wpedantic"
-#endif
-    struct usbdevfs_urb             KUrb;
-#if RT_GNUC_PREREQ(6, 0)
-# pragma GCC diagnostic pop
-#endif
-    /** Space filler for the isochronous packets. */
-    struct usbdevfs_iso_packet_desc aIsocPktsDonUseTheseUseTheOnesInKUrb[8];
     /** Node to link the URB in of the existing lists. */
     RTLISTNODE                      NodeList;
     /** If we've split the VUSBURB up into multiple linux URBs, this is points to the head. */
-    struct USBPROXYURBLNX           *pSplitHead;
+    struct USBPROXYURBLNX          *pSplitHead;
     /** The next linux URB if split up. */
-    struct USBPROXYURBLNX           *pSplitNext;
+    struct USBPROXYURBLNX          *pSplitNext;
     /** Don't report these back. */
     bool                            fCanceledBySubmit;
     /** This split element is reaped. */
@@ -110,6 +108,16 @@ typedef struct USBPROXYURBLNX
     bool                            fDiscarded;
     /** Size to transfer in remaining fragments of a split URB */
     uint32_t                        cbSplitRemaining;
+
+#if RT_GNUC_PREREQ(6, 0) /* gcc 6.2 complains about the [] member of KUrb */
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wpedantic"
+#endif
+    /** The kernel URB data (variable size array included). */
+    struct usbdevfs_urb             KUrb;
+#if RT_GNUC_PREREQ(6, 0)
+# pragma GCC diagnostic pop
+#endif
 } USBPROXYURBLNX, *PUSBPROXYURBLNX;
 
 /**
@@ -129,9 +137,9 @@ typedef struct USBPROXYDEVLNX
     RTLISTANCHOR        ListInFlight;
     /** Are we using sysfs to find the active configuration? */
     bool                fUsingSysfs;
-    /** Pipe handle for waiking up - writing end. */
+    /** Pipe handle for waking up - writing end. */
     RTPIPE              hPipeWakeupW;
-    /** Pipe handle for waiking up - reading end. */
+    /** Pipe handle for waking up - reading end. */
     RTPIPE              hPipeWakeupR;
     /** The device node/sysfs path of the device.
      * Used to figure out the configuration after a reset. */
@@ -251,7 +259,6 @@ static void usbProxyLinuxSetConnected(PUSBPROXYDEV pProxyDev, int iIf, bool fCon
 /**
  * Links the given URB into the in flight list.
  *
- * @returns nothing.
  * @param   pDevLnx         The proxy device instance - Linux specific data.
  * @param   pUrbLnx         The URB to link into the in flight list.
  */
@@ -263,9 +270,10 @@ static void usbProxyLinuxUrbLinkInFlight(PUSBPROXYDEVLNX pDevLnx, PUSBPROXYURBLN
     RTListAppend(&pDevLnx->ListInFlight, &pUrbLnx->NodeList);
 }
 
+
 /**
  * Unlinks the given URB from the in flight list.
- * @returns nothing.
+ *
  * @param   pDevLnx         The proxy device instance - Linux specific data.
  * @param   pUrbLnx         The URB to link into the in flight list.
  */
@@ -284,8 +292,10 @@ static void usbProxyLinuxUrbUnlinkInFlight(PUSBPROXYDEVLNX pDevLnx, PUSBPROXYURB
     RTCritSectLeave(&pDevLnx->CritSect);
 }
 
+
 /**
  * Allocates a linux URB request structure.
+ *
  * @returns Pointer to an active URB request.
  * @returns NULL on failure.
  * @param   pProxyDev       The proxy device instance.
@@ -312,7 +322,9 @@ static PUSBPROXYURBLNX usbProxyLinuxUrbAlloc(PUSBPROXYDEV pProxyDev, PUSBPROXYUR
     else
     {
         RTCritSectLeave(&pDevLnx->CritSect);
-        pUrbLnx = (PUSBPROXYURBLNX)RTMemAlloc(sizeof(*pUrbLnx));
+        PVUSBURB pVUrbDummy; RT_NOREF(pVUrbDummy);
+        pUrbLnx = (PUSBPROXYURBLNX)RTMemAlloc(RT_UOFFSETOF_DYN(USBPROXYURBLNX,
+                                                               KUrb.iso_frame_desc[RT_ELEMENTS(pVUrbDummy->aIsocPkts)]));
         if (!pUrbLnx)
             return NULL;
     }
@@ -586,9 +598,8 @@ RTDECL(int) USBProxyDeviceLinuxGetFD(PUSBPROXYDEV pProxyDev)
  *                          the form "sysfs:<sysfs path>//device:<device node>".
  *                          In the second case, the two paths are guaranteed
  *                          not to contain the substring "//".
- * @param   pvBackend       Backend specific pointer, unused for the linux backend.
  */
-static DECLCALLBACK(int) usbProxyLinuxOpen(PUSBPROXYDEV pProxyDev, const char *pszAddress, void *pvBackend)
+static DECLCALLBACK(int) usbProxyLinuxOpen(PUSBPROXYDEV pProxyDev, const char *pszAddress)
 {
     LogFlow(("usbProxyLinuxOpen: pProxyDev=%p pszAddress=%s\n", pProxyDev, pszAddress));
     const char *pszDevNode;
@@ -667,10 +678,7 @@ static DECLCALLBACK(int) usbProxyLinuxOpen(PUSBPROXYDEV pProxyDev, const char *p
     else if (rc == VERR_ACCESS_DENIED)
         rc = VERR_VUSB_USBFS_PERMISSION;
 
-    Log(("usbProxyLinuxOpen(%p, %s) failed, rc=%s!\n", pProxyDev, pszAddress,
-         RTErrGetShort(rc)));
-
-    NOREF(pvBackend);
+    Log(("usbProxyLinuxOpen(%p, %s) failed, rc=%Rrc!\n", pProxyDev, pszAddress, rc));
     return rc;
 }
 
@@ -830,8 +838,7 @@ static DECLCALLBACK(int) usbProxyLinuxReset(PUSBPROXYDEV pProxyDev, bool fResetO
     if (usbProxyLinuxDoIoCtl(pProxyDev, USBDEVFS_RESET, NULL, false, 10))
     {
         int rc = errno;
-        LogRel(("usb-linux: Reset failed, rc=%s errno=%d.\n",
-               RTErrGetShort(RTErrConvertFromErrno(rc)), rc));
+        LogRel(("usb-linux: Reset failed, rc=%Rrc errno=%d.\n", RTErrConvertFromErrno(rc), rc));
         pProxyDev->iActiveCfg = -1;
         return RTErrConvertFromErrno(rc);
     }
@@ -986,7 +993,8 @@ static void usbProxyLinuxUrbSwapSetup(PVUSBSETUP pSetup)
 /**
  * Clean up after a failed URB submit.
  */
-static void usbProxyLinuxCleanupFailedSubmit(PUSBPROXYDEV pProxyDev, PUSBPROXYURBLNX pUrbLnx, PUSBPROXYURBLNX pCur, PVUSBURB pUrb, bool *pfUnplugged)
+static void usbProxyLinuxCleanupFailedSubmit(PUSBPROXYDEV pProxyDev, PUSBPROXYURBLNX pUrbLnx, PUSBPROXYURBLNX pCur,
+                                             PVUSBURB pUrb, bool *pfUnplugged)
 {
     if (pUrb->enmType == VUSBXFERTYPE_MSG)
         usbProxyLinuxUrbSwapSetup((PVUSBSETUP)pUrb->abData);
@@ -1493,7 +1501,7 @@ static DECLCALLBACK(PVUSBURB) usbProxyLinuxUrbReap(PUSBPROXYDEV pProxyDev, RTMSI
                     Log(("usb-linux: Reap URB. errno=%d pProxyDev=%s\n", errno, usbProxyGetName(pProxyDev)));
                 return NULL;
             }
-        pUrbLnx = (PUSBPROXYURBLNX)pKUrb;
+        pUrbLnx = RT_FROM_MEMBER(pKUrb, USBPROXYURBLNX, KUrb);
 
         /* split list: Is the entire split list done yet? */
         if (pUrbLnx->pSplitHead)
