@@ -5,22 +5,33 @@
 
 /*
  * Contributed by Ivo Smits <Ivo@UFO-Net.nl>, Howard Su and
- * Christophe Devriese <christophe.devriese@gmail.com>
+ * Christophe Devriese <christophe.devriese@gmail.com>.
  *
- * Copyright (C) 2011-2012 Oracle Corporation
+ * Copyright (C) 2011-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #define LOG_GROUP LOG_GROUP_VRDE
 #include <VBox/log.h>
 
@@ -36,7 +47,7 @@
 #include <iprt/thread.h>
 #include <iprt/cpp/utils.h>
 
-#include <VBox/err.h>
+#include <iprt/errcore.h>
 #include <VBox/RemoteDesktop/VRDEOrders.h>
 #include <VBox/RemoteDesktop/VRDE.h>
 
@@ -48,9 +59,9 @@
 #endif
 
 
-/*******************************************************************************
-*   Defined Constants And Macros                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Defined Constants And Macros                                                                                                 *
+*********************************************************************************************************************************/
 #define VNC_SIZEOFRGBA          4
 #define VNC_PASSWORDSIZE        20
 #define VNC_ADDRESSSIZE         60
@@ -58,14 +69,15 @@
 #define VNC_ADDRESS_OPTION_MAX  500
 
 
-/*******************************************************************************
-*   Structures and Typedefs                                                    *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Structures and Typedefs                                                                                                      *
+*********************************************************************************************************************************/
 class VNCServerImpl
 {
 public:
     VNCServerImpl()
     {
+        mVNCServer = NULL;
         mFrameBuffer = NULL;
         mScreenBuffer = NULL;
         mCursor = NULL;
@@ -79,6 +91,8 @@ public:
         if (mCursor)
             rfbFreeCursor(mCursor);
         RT_ZERO(szVNCPassword);
+        if (mVNCServer)
+            rfbScreenCleanup(mVNCServer);
     }
 
     int Init(const VRDEINTERFACEHDR *pCallbacks, void *pvCallback);
@@ -99,8 +113,8 @@ private:
     unsigned char *mScreenBuffer;
     unsigned char *mFrameBuffer;
     uint32_t uClients;
-    static DECLCALLBACK(enum rfbNewClientAction) rfbNewClientEvent(rfbClientPtr cl);
-    static DECLCALLBACK(void) vncMouseEvent(int buttonMask, int x, int y, rfbClientPtr cl);
+    static enum rfbNewClientAction rfbNewClientEvent(rfbClientPtr cl);
+    static void vncMouseEvent(int buttonMask, int x, int y, rfbClientPtr cl);
     static void vncKeyboardEvent(rfbBool down, rfbKeySym keySym, rfbClientPtr cl);
     static void clientGoneHook(rfbClientPtr cl);
 
@@ -249,6 +263,7 @@ int VNCServerImpl::queryVrdeFeature(const char *pszName, char *pszValue, size_t
  */
 DECLCALLBACK(int) VNCServerImpl::VRDEEnableConnections(HVRDESERVER hServer, bool fEnable)
 {
+    RT_NOREF(fEnable);
     VNCServerImpl *instance = (VNCServerImpl *)hServer;
 
 #ifdef LOG_ENABLED
@@ -307,7 +322,7 @@ DECLCALLBACK(int) VNCServerImpl::VRDEEnableConnections(HVRDESERVER hServer, bool
     else
     {
         const char szFeatName[] = "Property/TCP/Ports";
-        const uint32_t featLen = sizeof(VRDEFEATURE) + RT_MAX(sizeof(VNC_PORTSSIZE), sizeof(szFeatName)) - 1;
+        const uint32_t featLen = sizeof(VRDEFEATURE) + RT_MAX(VNC_PORTSSIZE, sizeof(szFeatName)) - 1;
         VRDEFEATURE *feature = (VRDEFEATURE *)RTMemTmpAlloc(featLen);
         feature->u32ClientId = 0;
         RTStrCopy(feature->achInfo, featLen - sizeof(VRDEFEATURE) + 1, szFeatName);
@@ -690,9 +705,9 @@ DECLCALLBACK(int) VNCServerImpl::VRDEEnableConnections(HVRDESERVER hServer, bool
  *
  * @return IPRT status code.
  */
-DECLCALLBACK(void) VNCServerImpl::VRDEDisconnect(HVRDESERVER hServer, uint32_t u32ClientId,
-                                                 bool fReconnect)
+DECLCALLBACK(void) VNCServerImpl::VRDEDisconnect(HVRDESERVER hServer, uint32_t u32ClientId, bool fReconnect)
 {
+    RT_NOREF(hServer, u32ClientId, fReconnect);
 }
 
 static inline void convert15To32bpp(uint8_t msb, uint8_t lsb, uint8_t &r, uint8_t &g, uint8_t &b)
@@ -778,9 +793,9 @@ DECLCALLBACK(void) VNCServerImpl::VRDEResize(HVRDESERVER hServer)
  * @param pvUpdate  Pointer to VBoxGuest.h::VRDEORDERHDR structure with extra data.
  * @param cbUpdate  Size of the update data.
  */
-DECLCALLBACK(void) VNCServerImpl::VRDEUpdate(HVRDESERVER hServer, unsigned uScreenId,
-                                             void *pvUpdate,uint32_t cbUpdate)
+DECLCALLBACK(void) VNCServerImpl::VRDEUpdate(HVRDESERVER hServer, unsigned uScreenId, void *pvUpdate,uint32_t cbUpdate)
 {
+    RT_NOREF(uScreenId);
     char *ptr = (char *)pvUpdate;
     VNCServerImpl *instance = (VNCServerImpl *)hServer;
     VRDEORDERHDR *order = (VRDEORDERHDR *)ptr;
@@ -811,7 +826,17 @@ DECLCALLBACK(void) VNCServerImpl::VRDEUpdate(HVRDESERVER hServer, unsigned uScre
                         solidrect->x + solidrect->w, solidrect->y + solidrect->h, RGB2BGR(solidrect->rgb));
                     return;
                 }
-            ///@todo: more orders
+            /// @todo more orders
+            }
+        }
+
+        if (!instance->mScreenBuffer)
+        {
+            VRDEResize(hServer);
+            if (!instance->mScreenBuffer)
+            {
+                LogRel(("VNCServerImpl::VRDEUpdate: Cannot get frame buffer"));
+                return;
             }
         }
 
@@ -921,10 +946,11 @@ DECLCALLBACK(void) VNCServerImpl::VRDEColorPointer(HVRDESERVER hServer,
  */
 DECLCALLBACK(void) VNCServerImpl::VRDEHidePointer(HVRDESERVER hServer)
 {
-    VNCServerImpl *instance = (VNCServerImpl *)hServer;
+    VNCServerImpl *pInstance = (VNCServerImpl *)hServer;
+    RT_NOREF(pInstance);
 
-    ///@todo: what's behavior for this. hide doesn't seems right
-    //rfbSetCursor(instance->mVNCServer, NULL);
+    /// @todo what's behavior for this. hide doesn't seems right
+    //rfbSetCursor(pInstance->mVNCServer, NULL);
 }
 
 /**
@@ -942,6 +968,7 @@ DECLCALLBACK(void) VNCServerImpl::VRDEAudioSamples(HVRDESERVER hServer,
                                                    uint32_t cSamples,
                                                    VRDEAUDIOFORMAT format)
 {
+    RT_NOREF(hServer, pvSamples, cSamples, format);
 }
 
 /**
@@ -957,6 +984,7 @@ DECLCALLBACK(void) VNCServerImpl::VRDEAudioVolume(HVRDESERVER hServer,
                                                   uint16_t u16Left,
                                                   uint16_t u16Right)
 {
+    RT_NOREF(hServer, u16Left, u16Right);
 }
 
 /**
@@ -977,6 +1005,7 @@ DECLCALLBACK(void) VNCServerImpl::VRDEUSBRequest(HVRDESERVER hServer,
                                                  void *pvParm,
                                                  uint32_t cbParm)
 {
+    RT_NOREF(hServer, u32ClientId, pvParm, cbParm);
 }
 
 /**
@@ -1002,6 +1031,7 @@ DECLCALLBACK(void) VNCServerImpl::VRDEClipboard(HVRDESERVER hServer,
                                                 uint32_t cbData,
                                                 uint32_t *pcbActualRead)
 {
+    RT_NOREF(hServer, u32Function, u32Format, pvData, cbData, pcbActualRead);
 }
 
 /**
@@ -1039,7 +1069,7 @@ DECLCALLBACK(void) VNCServerImpl::VRDEQueryInfo(HVRDESERVER hServer,
             }
             break;
         }
-        ///@todo lots more queries to implement
+        /// @todo lots more queries to implement
         default:
             break;
     }
@@ -1069,6 +1099,7 @@ DECLCALLBACK(void) VNCServerImpl::VRDERedirect(HVRDESERVER hServer,
                                                uint32_t u32SessionId,
                                                const char *pszCookie)
 {
+    RT_NOREF(hServer, u32ClientId, pszServer, pszUser, pszDomain, pszPassword, u32SessionId, pszCookie);
 }
 
 /**
@@ -1088,6 +1119,7 @@ DECLCALLBACK(void) VNCServerImpl::VRDEAudioInOpen(HVRDESERVER hServer,
                                                   VRDEAUDIOFORMAT audioFormat,
                                                   uint32_t u32SamplesPerBlock)
 {
+    RT_NOREF(hServer, pvCtx, u32ClientId, audioFormat, u32SamplesPerBlock);
 }
 
 /**
@@ -1098,9 +1130,9 @@ DECLCALLBACK(void) VNCServerImpl::VRDEAudioInOpen(HVRDESERVER hServer,
  *
  * @note Initialized to NULL when the VRDECallbackAudioIn callback is NULL.
  */
-DECLCALLBACK(void) VNCServerImpl::VRDEAudioInClose(HVRDESERVER hServer,
-                                                   uint32_t u32ClientId)
+DECLCALLBACK(void) VNCServerImpl::VRDEAudioInClose(HVRDESERVER hServer, uint32_t u32ClientId)
 {
+    RT_NOREF(hServer, u32ClientId);
 }
 
 
@@ -1115,7 +1147,7 @@ int VNCServerImpl::Init(const VRDEINTERFACEHDR *pCallbacks,
     }
     else if (pCallbacks->u64Version == VRDE_INTERFACE_VERSION_1)
     {
-        ///@todo: this is incorrect and it will cause crash if client call unsupport func.
+        /// @todo this is incorrect and it will cause crash if client call unsupport func.
         mCallbacks = (VRDECALLBACKS_4 *)pCallbacks;
         mCallback = pvCallback;
 
@@ -1286,7 +1318,7 @@ enum rfbNewClientAction VNCServerImpl::rfbNewClientEvent(rfbClientPtr cl)
 {
     VNCServerImpl *instance = static_cast<VNCServerImpl *>(cl->screen->screenData);
 
-    ///@todo: we need auth user here
+    /// @todo we need auth user here
 
     instance->mCallbacks->VRDECallbackClientConnect(instance->mCallback, (int)cl->sock);
     instance->uClients++;
