@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2013-2015 Oracle Corporation
+ * Copyright (C) 2013-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
@@ -22,17 +32,21 @@
 #include "VBoxManage.h"
 #include "VBoxManageGuestCtrl.h"
 
-#ifndef VBOX_ONLY_DOCS
-
 #include <VBox/com/com.h>
 #include <VBox/com/ErrorInfo.h>
 #include <VBox/com/errorprint.h>
 
+#include <iprt/semaphore.h>
 #include <iprt/time.h>
 
 #include <map>
 #include <vector>
 
+DECLARE_TRANSLATION_CONTEXT(GuestCtrlLsnr);
+
+
+/** Event semaphore we're using for notification. */
+extern RTSEMEVENT g_SemEventGuestCtrlCanceled;
 
 
 /*
@@ -83,7 +97,7 @@ STDMETHODIMP GuestFileEventListener::HandleEvent(VBoxEventType_T aType, IEvent *
     {
         case VBoxEventType_OnGuestFileStateChanged:
         {
-            HRESULT rc;
+            HRESULT hrc;
             do
             {
                 ComPtr<IGuestFileStateChangedEvent> pEvent = aEvent;
@@ -95,11 +109,11 @@ STDMETHODIMP GuestFileEventListener::HandleEvent(VBoxEventType_T aType, IEvent *
                 FileStatus_T fileSts;
                 CHECK_ERROR_BREAK(pEvent, COMGETTER(Status)(&fileSts));
                 Bstr strPath;
-                CHECK_ERROR_BREAK(pProcess, COMGETTER(FileName)(strPath.asOutParam()));
+                CHECK_ERROR_BREAK(pProcess, COMGETTER(Filename)(strPath.asOutParam()));
                 ULONG uID;
                 CHECK_ERROR_BREAK(pProcess, COMGETTER(Id)(&uID));
 
-                RTPrintf("File ID=%RU32 \"%s\" changed status to [%s]\n",
+                RTPrintf(GuestCtrlLsnr::tr("File ID=%RU32 \"%s\" changed status to [%s]\n"),
                          uID, Utf8Str(strPath).c_str(), gctlFileStatusToText(fileSts));
 
             } while (0);
@@ -139,7 +153,7 @@ STDMETHODIMP GuestProcessEventListener::HandleEvent(VBoxEventType_T aType, IEven
     {
         case VBoxEventType_OnGuestProcessStateChanged:
         {
-            HRESULT rc;
+            HRESULT hrc;
             do
             {
                 ComPtr<IGuestProcessStateChangedEvent> pEvent = aEvent;
@@ -155,7 +169,7 @@ STDMETHODIMP GuestProcessEventListener::HandleEvent(VBoxEventType_T aType, IEven
                 ULONG uPID;
                 CHECK_ERROR_BREAK(pProcess, COMGETTER(PID)(&uPID));
 
-                RTPrintf("Process PID=%RU32 \"%s\" changed status to [%s]\n",
+                RTPrintf(GuestCtrlLsnr::tr("Process PID=%RU32 \"%s\" changed status to [%s]\n"),
                          uPID, Utf8Str(strPath).c_str(), gctlProcessStatusToText(procSts));
 
             } while (0);
@@ -191,7 +205,7 @@ void GuestSessionEventListener::uninit(void)
     {
         if (!itProc->first.isNull())
         {
-            HRESULT rc;
+            HRESULT hrc;
             do
             {
                 /* Listener unregistration. */
@@ -200,7 +214,6 @@ void GuestSessionEventListener::uninit(void)
                 if (!pES.isNull())
                     CHECK_ERROR_BREAK(pES, UnregisterListener(itProc->second.mListener));
             } while (0);
-            itProc->first->Release();
         }
 
         ++itProc;
@@ -212,7 +225,7 @@ void GuestSessionEventListener::uninit(void)
     {
         if (!itFile->first.isNull())
         {
-            HRESULT rc;
+            HRESULT hrc;
             do
             {
                 /* Listener unregistration. */
@@ -221,7 +234,6 @@ void GuestSessionEventListener::uninit(void)
                 if (!pES.isNull())
                     CHECK_ERROR_BREAK(pES, UnregisterListener(itFile->second.mListener));
             } while (0);
-            itFile->first->Release();
         }
 
         ++itFile;
@@ -235,7 +247,7 @@ STDMETHODIMP GuestSessionEventListener::HandleEvent(VBoxEventType_T aType, IEven
     {
         case VBoxEventType_OnGuestFileRegistered:
         {
-            HRESULT rc;
+            HRESULT hrc;
             do
             {
                 ComPtr<IGuestFileRegisteredEvent> pEvent = aEvent;
@@ -247,15 +259,15 @@ STDMETHODIMP GuestSessionEventListener::HandleEvent(VBoxEventType_T aType, IEven
                 BOOL fRegistered;
                 CHECK_ERROR_BREAK(pEvent, COMGETTER(Registered)(&fRegistered));
                 Bstr strPath;
-                CHECK_ERROR_BREAK(pFile, COMGETTER(FileName)(strPath.asOutParam()));
+                CHECK_ERROR_BREAK(pFile, COMGETTER(Filename)(strPath.asOutParam()));
 
-                RTPrintf("File \"%s\" %s\n",
+                RTPrintf(GuestCtrlLsnr::tr("File \"%s\" %s\n"),
                          Utf8Str(strPath).c_str(),
-                         fRegistered ? "registered" : "unregistered");
+                         fRegistered ? GuestCtrlLsnr::tr("registered") : GuestCtrlLsnr::tr("unregistered"));
                 if (fRegistered)
                 {
                     if (mfVerbose)
-                        RTPrintf("Registering ...\n");
+                        RTPrintf(GuestCtrlLsnr::tr("Registering ...\n"));
 
                     /* Register for IGuestFile events. */
                     ComObjPtr<GuestFileEventListenerImpl> pListener;
@@ -278,7 +290,7 @@ STDMETHODIMP GuestSessionEventListener::HandleEvent(VBoxEventType_T aType, IEven
                     if (itFile != mFiles.end())
                     {
                         if (mfVerbose)
-                            RTPrintf("Unregistering file ...\n");
+                            RTPrintf(GuestCtrlLsnr::tr("Unregistering file ...\n"));
 
                         if (!itFile->first.isNull())
                         {
@@ -287,7 +299,6 @@ STDMETHODIMP GuestSessionEventListener::HandleEvent(VBoxEventType_T aType, IEven
                             CHECK_ERROR(itFile->first, COMGETTER(EventSource)(pES.asOutParam()));
                             if (!pES.isNull())
                                 CHECK_ERROR(pES, UnregisterListener(itFile->second.mListener));
-                            itFile->first->Release();
                         }
 
                         mFiles.erase(itFile);
@@ -300,7 +311,7 @@ STDMETHODIMP GuestSessionEventListener::HandleEvent(VBoxEventType_T aType, IEven
 
         case VBoxEventType_OnGuestProcessRegistered:
         {
-            HRESULT rc;
+            HRESULT hrc;
             do
             {
                 ComPtr<IGuestProcessRegisteredEvent> pEvent = aEvent;
@@ -314,13 +325,13 @@ STDMETHODIMP GuestSessionEventListener::HandleEvent(VBoxEventType_T aType, IEven
                 Bstr strPath;
                 CHECK_ERROR_BREAK(pProcess, COMGETTER(ExecutablePath)(strPath.asOutParam()));
 
-                RTPrintf("Process \"%s\" %s\n",
+                RTPrintf(GuestCtrlLsnr::tr("Process \"%s\" %s\n"),
                          Utf8Str(strPath).c_str(),
-                         fRegistered ? "registered" : "unregistered");
+                         fRegistered ? GuestCtrlLsnr::tr("registered") : GuestCtrlLsnr::tr("unregistered"));
                 if (fRegistered)
                 {
                     if (mfVerbose)
-                        RTPrintf("Registering ...\n");
+                        RTPrintf(GuestCtrlLsnr::tr("Registering ...\n"));
 
                     /* Register for IGuestProcess events. */
                     ComObjPtr<GuestProcessEventListenerImpl> pListener;
@@ -343,7 +354,7 @@ STDMETHODIMP GuestSessionEventListener::HandleEvent(VBoxEventType_T aType, IEven
                     if (itProc != mProcs.end())
                     {
                         if (mfVerbose)
-                            RTPrintf("Unregistering process ...\n");
+                            RTPrintf(GuestCtrlLsnr::tr("Unregistering process ...\n"));
 
                         if (!itProc->first.isNull())
                         {
@@ -352,7 +363,6 @@ STDMETHODIMP GuestSessionEventListener::HandleEvent(VBoxEventType_T aType, IEven
                             CHECK_ERROR(itProc->first, COMGETTER(EventSource)(pES.asOutParam()));
                             if (!pES.isNull())
                                 CHECK_ERROR(pES, UnregisterListener(itProc->second.mListener));
-                            itProc->first->Release();
                         }
 
                         mProcs.erase(itProc);
@@ -365,7 +375,7 @@ STDMETHODIMP GuestSessionEventListener::HandleEvent(VBoxEventType_T aType, IEven
 
         case VBoxEventType_OnGuestSessionStateChanged:
         {
-            HRESULT rc;
+            HRESULT hrc;
             do
             {
                 ComPtr<IGuestSessionStateChangedEvent> pEvent = aEvent;
@@ -381,7 +391,7 @@ STDMETHODIMP GuestSessionEventListener::HandleEvent(VBoxEventType_T aType, IEven
                 Bstr strName;
                 CHECK_ERROR_BREAK(pSession, COMGETTER(Name)(strName.asOutParam()));
 
-                RTPrintf("Session ID=%RU32 \"%s\" changed status to [%s]\n",
+                RTPrintf(GuestCtrlLsnr::tr("Session ID=%RU32 \"%s\" changed status to [%s]\n"),
                          uID, Utf8Str(strName).c_str(), gctlGuestSessionStatusToText(sessSts));
 
             } while (0);
@@ -417,7 +427,7 @@ void GuestEventListener::uninit(void)
     {
         if (!itSession->first.isNull())
         {
-            HRESULT rc;
+            HRESULT hrc;
             do
             {
                 /* Listener unregistration. */
@@ -427,7 +437,6 @@ void GuestEventListener::uninit(void)
                     CHECK_ERROR_BREAK(pES, UnregisterListener(itSession->second.mListener));
 
             } while (0);
-            itSession->first->Release();
         }
 
         ++itSession;
@@ -441,7 +450,7 @@ STDMETHODIMP GuestEventListener::HandleEvent(VBoxEventType_T aType, IEvent *aEve
     {
         case VBoxEventType_OnGuestSessionRegistered:
         {
-            HRESULT rc;
+            HRESULT hrc;
             do
             {
                 ComPtr<IGuestSessionRegisteredEvent> pEvent = aEvent;
@@ -457,13 +466,13 @@ STDMETHODIMP GuestEventListener::HandleEvent(VBoxEventType_T aType, IEvent *aEve
                 ULONG uID;
                 CHECK_ERROR_BREAK(pSession, COMGETTER(Id)(&uID));
 
-                RTPrintf("Session ID=%RU32 \"%s\" %s\n",
+                RTPrintf(GuestCtrlLsnr::tr("Session ID=%RU32 \"%s\" %s\n"),
                          uID, Utf8Str(strName).c_str(),
-                         fRegistered ? "registered" : "unregistered");
+                         fRegistered ? GuestCtrlLsnr::tr("registered") : GuestCtrlLsnr::tr("unregistered"));
                 if (fRegistered)
                 {
                     if (mfVerbose)
-                        RTPrintf("Registering ...\n");
+                        RTPrintf(GuestCtrlLsnr::tr("Registering ...\n"));
 
                     /* Register for IGuestSession events. */
                     ComObjPtr<GuestSessionEventListenerImpl> pListener;
@@ -475,6 +484,7 @@ STDMETHODIMP GuestEventListener::HandleEvent(VBoxEventType_T aType, IEvent *aEve
                     com::SafeArray<VBoxEventType_T> eventTypes;
                     eventTypes.push_back(VBoxEventType_OnGuestFileRegistered);
                     eventTypes.push_back(VBoxEventType_OnGuestProcessRegistered);
+                    eventTypes.push_back(VBoxEventType_OnGuestSessionStateChanged);
                     CHECK_ERROR_BREAK(es, RegisterListener(pListener, ComSafeArrayAsInParam(eventTypes),
                                                            true /* Active listener */));
 
@@ -487,7 +497,7 @@ STDMETHODIMP GuestEventListener::HandleEvent(VBoxEventType_T aType, IEvent *aEve
                     if (itSession != mSessions.end())
                     {
                         if (mfVerbose)
-                            RTPrintf("Unregistering ...\n");
+                            RTPrintf(GuestCtrlLsnr::tr("Unregistering ...\n"));
 
                         if (!itSession->first.isNull())
                         {
@@ -496,7 +506,6 @@ STDMETHODIMP GuestEventListener::HandleEvent(VBoxEventType_T aType, IEvent *aEve
                             CHECK_ERROR_BREAK(itSession->first, COMGETTER(EventSource)(pES.asOutParam()));
                             if (!pES.isNull())
                                 CHECK_ERROR_BREAK(pES, UnregisterListener(itSession->second.mListener));
-                            itSession->first->Release();
                         }
 
                         mSessions.erase(itSession);
@@ -514,5 +523,56 @@ STDMETHODIMP GuestEventListener::HandleEvent(VBoxEventType_T aType, IEvent *aEve
     return S_OK;
 }
 
-#endif /* !VBOX_ONLY_DOCS */
+/*
+ * GuestAdditionsRunlevelListener
+ * GuestAdditionsRunlevelListener
+ * GuestAdditionsRunlevelListener
+ */
 
+GuestAdditionsRunlevelListener::GuestAdditionsRunlevelListener(AdditionsRunLevelType_T enmRunLevel)
+    : mRunLevelTarget(enmRunLevel)
+{
+}
+
+GuestAdditionsRunlevelListener::~GuestAdditionsRunlevelListener(void)
+{
+}
+
+void GuestAdditionsRunlevelListener::uninit(void)
+{
+}
+
+STDMETHODIMP GuestAdditionsRunlevelListener::HandleEvent(VBoxEventType_T aType, IEvent *aEvent)
+{
+    Assert(mRunLevelTarget != AdditionsRunLevelType_None);
+
+    HRESULT hrc;
+
+    switch (aType)
+    {
+        case VBoxEventType_OnGuestAdditionsStatusChanged:
+        {
+            ComPtr<IGuestAdditionsStatusChangedEvent> pEvent = aEvent;
+            Assert(!pEvent.isNull());
+
+            AdditionsRunLevelType_T RunLevelCur = AdditionsRunLevelType_None;
+            CHECK_ERROR_BREAK(pEvent, COMGETTER(RunLevel)(&RunLevelCur));
+
+            if (mfVerbose)
+                RTPrintf(GuestCtrlLsnr::tr("Reached run level %RU32\n"), RunLevelCur);
+
+            if (RunLevelCur == mRunLevelTarget)
+            {
+                int vrc = RTSemEventSignal(g_SemEventGuestCtrlCanceled);
+                AssertRC(vrc);
+            }
+
+            break;
+        }
+
+        default:
+            AssertFailed();
+    }
+
+    return S_OK;
+}
