@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2006-2019 Oracle Corporation
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 /* GUI includes: */
@@ -20,12 +30,19 @@
 
 
 /* General: */
-const char *UIExtraDataDefs::GUI_EventHandlingType = "GUI/EventHandlingType";
+const char *UIExtraDataDefs::GUI_RestrictedDialogs = "GUI/RestrictedDialogs";
+const char *UIExtraDataDefs::GUI_ColorTheme = "GUI/ColorTheme";
 
 
 /* Messaging: */
 const char *UIExtraDataDefs::GUI_SuppressMessages = "GUI/SuppressMessages";
 const char *UIExtraDataDefs::GUI_InvertMessageOption = "GUI/InvertMessageOption";
+#ifdef VBOX_NOTIFICATION_CENTER_WITH_KEEP_BUTTON
+const char *UIExtraDataDefs::GUI_NotificationCenter_KeepSuccessfullProgresses = "GUI/NotificationCenter/KeepSuccessfullProgresses";
+#endif
+const char *UIExtraDataDefs::GUI_NotificationCenter_Alignment = "GUI/NotificationCenter/Alignment";
+const char *UIExtraDataDefs::GUI_NotificationCenter_Order = "GUI/NotificationCenter/Order";
+const char *UIExtraDataDefs::GUI_PreventBetaLabel = "GUI/PreventBetaLabel";
 #if !defined(VBOX_BLEEDING_EDGE) && !defined(DEBUG)
 const char *UIExtraDataDefs::GUI_PreventBetaWarning = "GUI/PreventBetaWarning";
 #endif
@@ -47,9 +64,7 @@ const char *UIExtraDataDefs::GUI_Progress_LegacyMode = "GUI/Progress/LegacyMode"
 const char *UIExtraDataDefs::GUI_Customizations = "GUI/Customizations";
 const char *UIExtraDataDefs::GUI_RestrictedGlobalSettingsPages = "GUI/RestrictedGlobalSettingsPages";
 const char *UIExtraDataDefs::GUI_RestrictedMachineSettingsPages = "GUI/RestrictedMachineSettingsPages";
-
-/* Settings: General: */
-const char *UIExtraDataDefs::GUI_HostScreenSaverDisabled = "GUI/HostScreenSaverDisabled";
+const char *UIExtraDataDefs::GUI_Settings_ExpertMode = "GUI/Settings/ExpertMode";
 
 /* Settings: Language: */
 const char *UIExtraDataDefs::GUI_LanguageID = "GUI/LanguageID";
@@ -57,6 +72,7 @@ const char *UIExtraDataDefs::GUI_LanguageID = "GUI/LanguageID";
 /* Settings: Display: */
 const char *UIExtraDataDefs::GUI_MaxGuestResolution = "GUI/MaxGuestResolution";
 const char *UIExtraDataDefs::GUI_ActivateHoveredMachineWindow = "GUI/ActivateHoveredMachineWindow";
+const char *UIExtraDataDefs::GUI_DisableHostScreenSaver = "GUI/DisableHostScreenSaver";
 
 /* Settings: Keyboard: */
 const char *UIExtraDataDefs::GUI_Input_SelectorShortcuts = "GUI/Input/SelectorShortcuts";
@@ -72,11 +88,16 @@ const char *UIExtraDataDefs::GUI_ProxySettings = "GUI/ProxySettings";
 const char *UIExtraDataDefs::GUI_RecentFolderHD = "GUI/RecentFolderHD";
 const char *UIExtraDataDefs::GUI_RecentFolderCD = "GUI/RecentFolderCD";
 const char *UIExtraDataDefs::GUI_RecentFolderFD = "GUI/RecentFolderFD";
-const char *UIExtraDataDefs::GUI_RecentFolderVISOContent = "GUI/RecentFolderVISOContent";
 const char *UIExtraDataDefs::GUI_RecentListHD = "GUI/RecentListHD";
 const char *UIExtraDataDefs::GUI_RecentListCD = "GUI/RecentListCD";
 const char *UIExtraDataDefs::GUI_RecentListFD = "GUI/RecentListFD";
 
+/* Settings: Network: */
+const char *UIExtraDataDefs::GUI_RestrictedNetworkAttachmentTypes = "GUI/RestrictedNetworkAttachmentTypes";
+
+/* VISO Creator: */
+const char *UIExtraDataDefs::GUI_VISOCreator_RecentFolder   = "GUI/VISOCreator/RecentFolder";
+const char *UIExtraDataDefs::GUI_VISOCreator_DialogGeometry   = "GUI/VISOCreator/DialogGeometry";
 
 /* VirtualBox Manager: */
 const char *UIExtraDataDefs::GUI_LastSelectorWindowPosition = "GUI/LastWindowPosition";
@@ -86,6 +107,7 @@ const char *UIExtraDataDefs::GUI_Toolbar_Text = "GUI/Toolbar/Text";
 const char *UIExtraDataDefs::GUI_Toolbar_MachineTools_Order = "GUI/Toolbar/MachineTools/Order";
 const char *UIExtraDataDefs::GUI_Toolbar_GlobalTools_Order = "GUI/Toolbar/GlobalTools/Order";
 const char *UIExtraDataDefs::GUI_Tools_LastItemsSelected = "GUI/Tools/LastItemsSelected";
+const char *UIExtraDataDefs::GUI_Tools_Detached = "GUI/Tools/Detached";
 const char *UIExtraDataDefs::GUI_Statusbar = "GUI/Statusbar";
 const char *UIExtraDataDefs::GUI_GroupDefinitions = "GUI/GroupDefinitions";
 const char *UIExtraDataDefs::GUI_LastItemSelected = "GUI/LastItemSelected";
@@ -99,29 +121,35 @@ const char *UIExtraDataDefs::GUI_SnapshotManager_Details_Expanded = "GUI/Snapsho
 
 /* Virtual Media Manager: */
 const char *UIExtraDataDefs::GUI_VirtualMediaManager_Details_Expanded = "GUI/VirtualMediaManager/Details/Expanded";
+const char *UIExtraDataDefs::GUI_VirtualMediaManager_Search_Widget_Expanded = "GUI/VirtualMediaManager/SearchWidget/Expanded";
 
 /* Host Network Manager: */
 const char *UIExtraDataDefs::GUI_HostNetworkManager_Details_Expanded = "GUI/HostNetworkManager/Details/Expanded";
 
 /* Cloud Profile Manager: */
+const char *UIExtraDataDefs::GUI_CloudProfileManager_Restrictions = "GUI/CloudProfileManager/Restrictions";
 const char *UIExtraDataDefs::GUI_CloudProfileManager_Details_Expanded = "GUI/CloudProfileManager/Details/Expanded";
 
+/* Cloud Console Manager: */
+const char *UIExtraDataDefs::GUI_CloudConsoleManager_Application = "GUI/CloudConsoleManager/Application";
+const char *UIExtraDataDefs::GUI_CloudConsoleManager_Restrictions = "GUI/CloudConsoleManager/Restrictions";
+const char *UIExtraDataDefs::GUI_CloudConsoleManager_Details_Expanded = "GUI/CloudConsoleManager/Details/Expanded";
+
+/* Cloud Console: */
+const char *UIExtraDataDefs::GUI_CloudConsole_PublicKey_Path = "GUI/CloudConsole/PublicKey/Path";
+
 #ifdef VBOX_GUI_WITH_EXTRADATA_MANAGER_UI
 /* Extra-data Manager: */
 const char *UIExtraDataDefs::GUI_ExtraDataManager_Geometry = "GUI/ExtraDataManager/Geometry";
 const char *UIExtraDataDefs::GUI_ExtraDataManager_SplitterHints = "GUI/ExtraDataManager/SplitterHints";
 #endif /* VBOX_GUI_WITH_EXTRADATA_MANAGER_UI */
 
-/* Wizards: */
-const char *UIExtraDataDefs::GUI_HideDescriptionForWizards = "GUI/HideDescriptionForWizards";
-
 
 /* Virtual Machine: */
 const char *UIExtraDataDefs::GUI_HideFromManager = "GUI/HideFromManager";
 const char *UIExtraDataDefs::GUI_HideDetails = "GUI/HideDetails";
 const char *UIExtraDataDefs::GUI_PreventReconfiguration = "GUI/PreventReconfiguration";
 const char *UIExtraDataDefs::GUI_PreventSnapshotOperations = "GUI/PreventSnapshotOperations";
-const char *UIExtraDataDefs::GUI_FirstRun = "GUI/FirstRun";
 const char *UIExtraDataDefs::GUI_MachineWindowIcons = "GUI/MachineWindowIcons";
 #ifndef VBOX_WS_MAC
 const char *UIExtraDataDefs::GUI_MachineWindowNamePostfix = "GUI/MachineWindowNamePostfix";
@@ -150,22 +178,15 @@ const char *UIExtraDataDefs::GUI_RestrictedVisualStates = "GUI/RestrictedVisualS
 const char *UIExtraDataDefs::GUI_Fullscreen = "GUI/Fullscreen";
 const char *UIExtraDataDefs::GUI_Seamless = "GUI/Seamless";
 const char *UIExtraDataDefs::GUI_Scale = "GUI/Scale";
-#ifdef VBOX_WS_X11
+#ifdef VBOX_WS_NIX
 const char *UIExtraDataDefs::GUI_Fullscreen_LegacyMode = "GUI/Fullscreen/LegacyMode";
 const char *UIExtraDataDefs::GUI_DistinguishMachineWindowGroups = "GUI/DistinguishMachineWindowGroups";
-#endif /* VBOX_WS_X11 */
+#endif /* VBOX_WS_NIX */
 const char *UIExtraDataDefs::GUI_AutoresizeGuest = "GUI/AutoresizeGuest";
 const char *UIExtraDataDefs::GUI_LastVisibilityStatusForGuestScreen = "GUI/LastVisibilityStatusForGuestScreen";
 const char *UIExtraDataDefs::GUI_LastGuestSizeHint = "GUI/LastGuestSizeHint";
 const char *UIExtraDataDefs::GUI_VirtualScreenToHostScreen = "GUI/VirtualScreenToHostScreen";
 const char *UIExtraDataDefs::GUI_AutomountGuestScreens = "GUI/AutomountGuestScreens";
-#ifdef VBOX_WITH_VIDEOHWACCEL
-const char *UIExtraDataDefs::GUI_Accelerate2D_StretchLinear = "GUI/Accelerate2D/StretchLinear";
-const char *UIExtraDataDefs::GUI_Accelerate2D_PixformatYV12 = "GUI/Accelerate2D/PixformatYV12";
-const char *UIExtraDataDefs::GUI_Accelerate2D_PixformatUYVY = "GUI/Accelerate2D/PixformatUYVY";
-const char *UIExtraDataDefs::GUI_Accelerate2D_PixformatYUY2 = "GUI/Accelerate2D/PixformatYUY2";
-const char *UIExtraDataDefs::GUI_Accelerate2D_PixformatAYUV = "GUI/Accelerate2D/PixformatAYUV";
-#endif /* VBOX_WITH_VIDEOHWACCEL */
 #ifndef VBOX_WS_MAC
 const char *UIExtraDataDefs::GUI_ShowMiniToolBar = "GUI/ShowMiniToolBar";
 const char *UIExtraDataDefs::GUI_MiniToolBarAutoHide = "GUI/MiniToolBarAutoHide";
@@ -186,10 +207,10 @@ const char *UIExtraDataDefs::GUI_GuruMeditationHandler = "GUI/GuruMeditationHand
 const char *UIExtraDataDefs::GUI_HidLedsSync = "GUI/HidLedsSync";
 const char *UIExtraDataDefs::GUI_ScaleFactor = "GUI/ScaleFactor";
 const char *UIExtraDataDefs::GUI_Scaling_Optimization = "GUI/Scaling/Optimization";
+const char *UIExtraDataDefs::GUI_FontScaleFactor = "GUI/FontScaleFactor";
 
-/* Virtual Machine: Information dialog: */
-const char *UIExtraDataDefs::GUI_InformationWindowGeometry = "GUI/InformationWindowGeometry";
-const char *UIExtraDataDefs::GUI_InformationWindowElements = "GUI/InformationWindowElements";
+/* Virtual Machine: Session Information Dialog: */
+const char *UIExtraDataDefs::GUI_SessionInformationDialogGeometry = "GUI/SessionInformationDialogGeometry";
 
 /* Guest control UI: */
 const char *UIExtraDataDefs::GUI_GuestControl_FileManagerDialogGeometry = "GUI/GuestControl/FileManagerDialogGeometry";
@@ -197,6 +218,16 @@ const char *UIExtraDataDefs::GUI_GuestControl_FileManagerVisiblePanels = "GUI/Gu
 const char *UIExtraDataDefs::GUI_GuestControl_ProcessControlSplitterHints = "GUI/GuestControl/ProcessControlSplitterHints";
 const char *UIExtraDataDefs::GUI_GuestControl_ProcessControlDialogGeometry = "GUI/GuestControl/ProcessControlDialogGeometry";
 
+/* Soft Keyboard: */
+const char *UIExtraDataDefs::GUI_SoftKeyboard_DialogGeometry = "GUI/SoftKeyboardDialogGeometry";
+const char *UIExtraDataDefs::GUI_SoftKeyboard_ColorTheme  = "GUI/SoftKeyboardColorTheme";
+const char *UIExtraDataDefs::GUI_SoftKeyboard_SelectedColorTheme  = "GUI/SoftKeyboardSelectedColorTheme";
+const char *UIExtraDataDefs::GUI_SoftKeyboard_SelectedLayout  = "GUI/SoftKeyboardSelectedLayout";
+const char *UIExtraDataDefs::GUI_SoftKeyboard_Options  = "GUI/SoftKeyboardOptions";
+const char *UIExtraDataDefs::GUI_SoftKeyboard_HideNumPad = "GUI/SoftKeyboardHideNumPad";
+const char *UIExtraDataDefs::GUI_SoftKeyboard_HideOSMenuKeys = "GUI/SoftKeyboardHideOSMenuKeys";
+const char *UIExtraDataDefs::GUI_SoftKeyboard_HideMultimediaKeys = "GUI/SoftKeyboardHideMultimediaKeys";
+
 /* File Manager options: */
 const char *UIExtraDataDefs::GUI_GuestControl_FileManagerOptions = "GUI/GuestControl/FileManagerOptions";
 const char *UIExtraDataDefs::GUI_GuestControl_FileManagerListDirectoriesFirst = "ListDirectoriesFirst";
@@ -209,6 +240,7 @@ const char *UIExtraDataDefs::GUI_DefaultCloseAction = "GUI/DefaultCloseAction";
 const char *UIExtraDataDefs::GUI_RestrictedCloseActions = "GUI/RestrictedCloseActions";
 const char *UIExtraDataDefs::GUI_LastCloseAction = "GUI/LastCloseAction";
 const char *UIExtraDataDefs::GUI_CloseActionHook = "GUI/CloseActionHook";
+const char *UIExtraDataDefs::GUI_DiscardStateOnPowerOff = "GUI/DiscardStateOnPowerOff";
 
 #ifdef VBOX_WITH_DEBUGGER_GUI
 /* Virtual Machine: Debug UI: */
@@ -222,18 +254,32 @@ const char *UIExtraDataDefs::GUI_LogViewerOptions = "GUI/LogViewerOptions";
 const char *UIExtraDataDefs::GUI_LogViewerWrapLinesEnabled = "WrapLines";
 const char *UIExtraDataDefs::GUI_LogViewerShowLineNumbersDisabled = "showLineNumbersDisabled";
 const char *UIExtraDataDefs::GUI_LogViewerNoFontStyleName = "noFontStyleName";
-const char *UIExtraDataDefs::GUI_GuestControl_LogViewerVisiblePanels = "GUI/LogViewerVisiblePanels";
 
+/* Help Browser */
+const char *UIExtraDataDefs::GUI_HelpBrowser_LastURLList = "GUI/HelpBrowserLastURLList";
+const char *UIExtraDataDefs::GUI_HelpBrowser_DialogGeometry = "GUI/HelpBrowserDialogGeomety";
+const char *UIExtraDataDefs::GUI_HelpBrowser_Bookmarks = "GUI/HelpBrowserBookmarks";
+const char *UIExtraDataDefs::GUI_HelpBrowser_ZoomPercentage = "GUI/HelpBrowserZoomPercentage";
+
+/* VM Activity Overview: */
+const char *UIExtraDataDefs::GUI_VMActivityOverview_HiddenColumns = "GUI/VMActivityOverviewHiddenColumns";
+const char *UIExtraDataDefs::GUI_VMActivityOverview_ShowAllMachines = "GUI/VMActivityOverviewShowAllMachines";
+const char *UIExtraDataDefs::GUI_VMActivityMonitor_DataSeriesColors = "GUI/VMActivityMonitorDataSeriesColors";
+const char *UIExtraDataDefs::GUI_VMActivityMonitor_ShowVMExits = "GUI/VMActivityMonitorShowVMExits";
+const char *UIExtraDataDefs::GUI_MediumSelector_DialogGeometry = "GUI/MediumSelector/DialogGeometry";
+
+const int UIExtraDataDefs::iFontScaleMin = 40;
+const int UIExtraDataDefs::iFontScaleMax = 200;
 
 /* Obsolete keys: */
-QMap<QString, QString> UIExtraDataDefs::prepareObsoleteKeysMap()
+QMultiMap<QString, QString> UIExtraDataDefs::prepareObsoleteKeysMap()
 {
-    QMap<QString, QString> map;
-    map.insertMulti(GUI_Details_Elements, GUI_DetailsPageBoxes);
-    map.insertMulti(GUI_Details_Elements_Preview_UpdateInterval, GUI_PreviewUpdate);
+    QMultiMap<QString, QString> map;
+    map.insert(GUI_Details_Elements, GUI_DetailsPageBoxes);
+    map.insert(GUI_Details_Elements_Preview_UpdateInterval, GUI_PreviewUpdate);
     return map;
 }
-QMap<QString, QString> UIExtraDataDefs::g_mapOfObsoleteKeys = UIExtraDataDefs::prepareObsoleteKeysMap();
+QMultiMap<QString, QString> UIExtraDataDefs::g_mapOfObsoleteKeys = UIExtraDataDefs::prepareObsoleteKeysMap();
 
 
 bool UIToolStuff::isTypeOfClass(UIToolType enmType, UIToolClass enmClass)
@@ -245,9 +291,12 @@ bool UIToolStuff::isTypeOfClass(UIToolType enmType, UIToolClass enmClass)
             switch (enmType)
             {
                 case UIToolType_Welcome:
+                case UIToolType_Extensions:
                 case UIToolType_Media:
                 case UIToolType_Network:
                 case UIToolType_Cloud:
+                case UIToolType_CloudConsole:
+                case UIToolType_VMActivityOverview:
                     return true;
                 default:
                     break;
@@ -261,6 +310,8 @@ bool UIToolStuff::isTypeOfClass(UIToolType enmType, UIToolClass enmClass)
                 case UIToolType_Details:
                 case UIToolType_Snapshots:
                 case UIToolType_Logs:
+                case UIToolType_VMActivity:
+                case UIToolType_FileManager:
                     return true;
                 default:
                     break;
