@@ -360,6 +360,7 @@ UIIconPoolGeneral::UIIconPoolGeneral()
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("Windows2016"),           ":/os_win2k16.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("Windows2019"),           ":/os_win2k19.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("Windows2022"),           ":/os_win2k22.png");
+    m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("Windows2025"),           ":/os_win2k25.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X86("WindowsNT"),             ":/os_win_other.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("WindowsNT"),             ":/os_win_other.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X86("OS2Warp3"),              ":/os_os2warp3.png");
@@ -405,6 +406,8 @@ UIIconPoolGeneral::UIIconPoolGeneral()
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X86("Debian12"),              ":/os_debian.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("Debian12"),              ":/os_debian.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_A64("Debian12"),              ":/os_debian.png");
+    m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("Debian13"),              ":/os_debian.png");
+    m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_A64("Debian13"),              ":/os_debian.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X86("OpenSUSE"),              ":/os_opensuse.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("OpenSUSE"),              ":/os_opensuse.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("OpenSUSE_Leap"),         ":/os_opensuse.png");
@@ -438,8 +441,13 @@ UIIconPoolGeneral::UIIconPoolGeneral()
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X86("RedHat6"),               ":/os_redhat.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("RedHat6"),               ":/os_redhat.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("RedHat7"),               ":/os_redhat.png");
+    m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_A64("RedHat7"),               ":/os_redhat.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("RedHat8"),               ":/os_redhat.png");
+    m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_A64("RedHat8"),               ":/os_redhat.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("RedHat9"),               ":/os_redhat.png");
+    m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_A64("RedHat9"),               ":/os_redhat.png");
+    m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("RedHat10"),              ":/os_redhat.png");
+    m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_A64("RedHat10"),              ":/os_redhat.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X86("Turbolinux"),            ":/os_turbolinux.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("Turbolinux"),            ":/os_turbolinux.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X86("Ubuntu"),                ":/os_ubuntu.png");
@@ -484,8 +492,14 @@ UIIconPoolGeneral::UIIconPoolGeneral()
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_A64("Ubuntu22"),              ":/os_ubuntu.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("Ubuntu23"),              ":/os_ubuntu.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_A64("Ubuntu23"),              ":/os_ubuntu.png");
+    m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("Ubuntu231"),             ":/os_ubuntu.png");
+    m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_A64("Ubuntu231"),             ":/os_ubuntu.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("Ubuntu24_LTS"),          ":/os_ubuntu.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_A64("Ubuntu24_LTS"),          ":/os_ubuntu.png");
+    m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("Ubuntu24"),              ":/os_ubuntu.png");
+    m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_A64("Ubuntu24"),              ":/os_ubuntu.png");
+    m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("Ubuntu25"),              ":/os_ubuntu.png");
+    m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_A64("Ubuntu25"),              ":/os_ubuntu.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X86("Lubuntu"),               ":/os_ubuntu.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("Lubuntu"),               ":/os_ubuntu.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X86("Xubuntu"),               ":/os_ubuntu.png");
@@ -494,6 +508,7 @@ UIIconPoolGeneral::UIIconPoolGeneral()
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("Xandros"),               ":/os_xandros.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X86("Oracle"),                ":/os_oracle.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("Oracle"),                ":/os_oracle.png");
+    m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_A64("Oracle"),                ":/os_oracle.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X86("Oracle3"),               ":/os_oracle.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("Oracle3"),               ":/os_oracle.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X86("Oracle4"),               ":/os_oracle.png");
@@ -503,12 +518,13 @@ UIIconPoolGeneral::UIIconPoolGeneral()
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X86("Oracle6"),               ":/os_oracle.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("Oracle6"),               ":/os_oracle.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("Oracle7"),               ":/os_oracle.png");
+    m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_A64("Oracle7"),               ":/os_oracle.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("Oracle8"),               ":/os_oracle.png");
+    m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_A64("Oracle8"),               ":/os_oracle.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("Oracle9"),               ":/os_oracle.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_A64("Oracle9"),               ":/os_oracle.png");
-    m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X86("Oracle"),                ":/os_oracle.png");
-    m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("Oracle"),                ":/os_oracle.png");
-    m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_A64("Oracle"),                ":/os_oracle.png");
+    m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("Oracle10"),              ":/os_oracle.png");
+    m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_A64("Oracle10"),              ":/os_oracle.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X86("Linux"),                 ":/os_linux.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X64("Linux"),                 ":/os_linux.png");
     m_guestOSTypeIconNames.insert(GUEST_OS_ID_STR_X86("FreeBSD"),               ":/os_freebsd.png");
