@@ -4,65 +4,87 @@
  */
 
 /*
- * Copyright (C) 2006-2019 Oracle Corporation
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 /* Qt includes: */
 #include <QApplication>
 #include <QMessageBox>
-#ifdef VBOX_WS_X11
+#ifdef VBOX_WS_NIX
 # ifndef Q_OS_SOLARIS
 #  include <QFontDatabase>
 # endif
 #endif
 
 /* GUI includes: */
-#include "VBoxGlobal.h"
-#include "UIStarter.h"
+#include "UICommon.h"
+#include "UILoggingDefs.h"
 #include "UIModalWindowManager.h"
+#include "UIStarter.h"
 #ifdef VBOX_WS_MAC
 # include "VBoxUtils.h"
 # include "UICocoaApplication.h"
 #endif /* VBOX_WS_MAC */
+#ifdef VBOX_WS_NIX
+# include "UIVersion.h"
+#endif
 
 /* Other VBox includes: */
 #include <iprt/buildconfig.h>
 #include <iprt/stream.h>
 #include <VBox/err.h>
 #include <VBox/version.h>
-#ifdef VBOX_WITH_HARDENING
-# include <VBox/sup.h>
-#endif
-#if !defined(VBOX_WITH_HARDENING) || defined(VBOX_GUI_WITH_SHARED_LIBRARY) && !defined(VBOX_RUNTIME_UI)
+#include <VBox/sup.h>
+#if !defined(VBOX_WITH_HARDENING) || !defined(VBOX_RUNTIME_UI)
 # include <iprt/initterm.h>
 # ifdef VBOX_WS_MAC
 #  include <iprt/asm.h>
 # endif
 #endif
-#ifdef VBOX_WS_X11
+#ifdef VBOX_WS_NIX
 # include <iprt/env.h>
 #endif
 #ifdef VBOX_WITH_HARDENING
 # include <iprt/ctype.h>
 #endif
+#if defined(VBOX_RUNTIME_UI) && defined(VBOX_WS_MAC)
+# include <iprt/path.h>
+#endif
+#ifdef VBOX_WS_WIN
+# include <VBox/com/microatl.h>
+#endif
+#if defined(VBOX_WITH_HARDENING) || defined(VBOX_WS_WIN)
+# include <iprt/string.h>
+#endif
 
 /* Other includes: */
 #ifdef VBOX_WS_MAC
 # include <dlfcn.h>
 # include <sys/mman.h>
 #endif /* VBOX_WS_MAC */
-#ifdef VBOX_WS_X11
+#ifdef VBOX_WS_NIX
 # include <dlfcn.h>
 # include <unistd.h>
 # include <X11/Xlib.h>
+# include "VBoxUtils-nix.h"
 # if defined(RT_OS_LINUX) && defined(DEBUG)
 #  include <signal.h>
 #  include <execinfo.h>
@@ -76,7 +98,7 @@
 #   define REG_PC REG_EIP
 #  endif /* !RT_ARCH_AMD64 */
 # endif /* RT_OS_LINUX && DEBUG */
-#endif /* VBOX_WS_X11 */
+#endif /* VBOX_WS_NIX */
 
 
 /* XXX Temporarily. Don't rely on the user to hack the Makefile himself! */
@@ -86,11 +108,14 @@ QString g_QStrHintLinuxNoMemory = QApplication::tr(
     );
 
 QString g_QStrHintLinuxNoDriver = QApplication::tr(
-    "The VirtualBox Linux kernel driver (vboxdrv) is either not loaded or "
-    "there is a permission problem with /dev/vboxdrv. Please reinstall the kernel "
-    "module by executing<br/><br/>"
+    "The VirtualBox Linux kernel driver is either not loaded or not set "
+    "up correctly. Please try setting it up again by executing<br/><br/>"
     "  <font color=blue>'/sbin/vboxconfig'</font><br/><br/>"
-    "as root."
+    "as root.<br/><br/>"
+    "If your system has EFI Secure Boot enabled you may also need to sign "
+    "the kernel modules (vboxdrv, vboxnetflt, vboxnetadp, vboxpci) before "
+    "you can load them. Please see your Linux system's documentation for "
+    "more information."
     );
 
 QString g_QStrHintOtherWrongDriverVersion = QApplication::tr(
@@ -105,8 +130,8 @@ QString g_QStrHintLinuxWrongDriverVersion = QApplication::tr(
     "VirtualBox. The installation of VirtualBox was apparently not "
     "successful. Executing<br/><br/>"
     "  <font color=blue>'/sbin/vboxconfig'</font><br/><br/>"
-    "may correct this. Make sure that you do not mix the "
-    "OSE version and the PUEL version of VirtualBox."
+    "may correct this. Make sure that you are not mixing builds "
+    "of VirtualBox from different sources."
     );
 
 QString g_QStrHintOtherNoDriver = QApplication::tr(
@@ -119,27 +144,7 @@ QString g_QStrHintReinstall = QApplication::tr(
     );
 
 
-#ifdef VBOX_WS_MAC
-/**
- * Mac OS X: Really ugly hack to bypass a set-uid check in AppKit.
- *
- * This will modify the issetugid() function to always return zero.  This must
- * be done _before_ AppKit is initialized, otherwise it will refuse to play ball
- * with us as it distrusts set-uid processes since Snow Leopard.  We, however,
- * have carefully dropped all root privileges at this point and there should be
- * no reason for any security concern here.
- */
-static void HideSetUidRootFromAppKit()
-{
-    /* Find issetguid() and make it always return 0 by modifying the code: */
-    void *pvAddr = dlsym(RTLD_DEFAULT, "issetugid");
-    int rc = mprotect((void *)((uintptr_t)pvAddr & ~(uintptr_t)0xfff), 0x2000, PROT_WRITE | PROT_READ | PROT_EXEC);
-    if (!rc)
-        ASMAtomicWriteU32((volatile uint32_t *)pvAddr, 0xccc3c031); /* xor eax, eax; ret; int3 */
-}
-#endif /* VBOX_WS_MAC */
-
-#ifdef VBOX_WS_X11
+#ifdef VBOX_WS_NIX
 /** X11: For versions of Xlib which are aware of multi-threaded environments this function
   *      calls for XInitThreads() which initializes Xlib support for concurrent threads.
   * @returns @c non-zero unless it is unsafe to make multi-threaded calls to Xlib.
@@ -163,7 +168,7 @@ static Status MakeSureMultiThreadingIsSafe()
     return rc;
 }
 
-# if defined(RT_OS_LINUX) && defined(DEBUG)
+# if (defined(RT_OS_LINUX) && !defined(RT_ARCH_ARM64)) && defined(DEBUG)
 /** X11, Linux, Debug: The signal handler that prints out a backtrace of the call stack.
   * @remarks The code is taken from http://www.linuxjournal.com/article/6391. */
 static void BackTraceSignalHandler(int sig, siginfo_t *pInfo, void *pSecret)
@@ -210,64 +215,64 @@ static void InstallSignalHandler()
     sigaction(SIGUSR1, &sa, 0);
 }
 # endif /* RT_OS_LINUX && DEBUG */
-#endif /* VBOX_WS_X11 */
+#endif /* VBOX_WS_NIX */
 
 /** Qt5 message handler, function that prints out
   * debug, warning, critical, fatal and system error messages.
-  * @param  type        Holds the type of the message.
+  * @param  enmType     Holds the type of the message.
   * @param  context     Holds the message context.
   * @param  strMessage  Holds the message body. */
-static void QtMessageOutput(QtMsgType type, const QMessageLogContext &context, const QString &strMessage)
+static void QtMessageOutput(QtMsgType enmType, const QMessageLogContext &context, const QString &strMessage)
 {
     NOREF(context);
-# ifndef VBOX_WS_X11
+# ifndef VBOX_WS_NIX
     NOREF(strMessage);
 # endif
-    switch (type)
+    switch (enmType)
     {
         case QtDebugMsg:
             Log(("Qt DEBUG: %s\n", strMessage.toUtf8().constData()));
             break;
         case QtWarningMsg:
             Log(("Qt WARNING: %s\n", strMessage.toUtf8().constData()));
-# ifdef VBOX_WS_X11
+# ifdef VBOX_WS_NIX
             /* Needed for instance for the message ``cannot connect to X server'': */
             RTStrmPrintf(g_pStdErr, "Qt WARNING: %s\n", strMessage.toUtf8().constData());
 # endif
             break;
         case QtCriticalMsg:
             Log(("Qt CRITICAL: %s\n", strMessage.toUtf8().constData()));
-# ifdef VBOX_WS_X11
+# ifdef VBOX_WS_NIX
             /* Needed for instance for the message ``cannot connect to X server'': */
             RTStrmPrintf(g_pStdErr, "Qt CRITICAL: %s\n", strMessage.toUtf8().constData());
 # endif
             break;
         case QtFatalMsg:
             Log(("Qt FATAL: %s\n", strMessage.toUtf8().constData()));
-# ifdef VBOX_WS_X11
+# ifdef VBOX_WS_NIX
             /* Needed for instance for the message ``cannot connect to X server'': */
             RTStrmPrintf(g_pStdErr, "Qt FATAL: %s\n", strMessage.toUtf8().constData());
 # endif
-# if QT_VERSION >= QT_VERSION_CHECK(5, 5, 0)
+            break;
         case QtInfoMsg:
             /** @todo ignore? */
             break;
-# endif
     }
 }
 
 /** Shows all available command line parameters. */
 static void ShowHelp()
 {
-#if !defined(VBOX_GUI_WITH_SHARED_LIBRARY) || !defined(VBOX_RUNTIME_UI)
-    static const char s_szTitle[] = VBOX_PRODUCT " VM Selector";
+#ifndef VBOX_RUNTIME_UI
+    static const char s_szTitle[] = VBOX_PRODUCT " Manager";
 #else
-    static const char s_szTitle[] = VBOX_PRODUCT " VM Runner";
+    static const char s_szTitle[] = VBOX_PRODUCT " Runner";
 #endif
+
     static const char s_szUsage[] =
-#if !defined(VBOX_GUI_WITH_SHARED_LIBRARY) || defined(VBOX_RUNTIME_UI)
+#ifdef VBOX_RUNTIME_UI
         "Options:\n"
-        "  --startvm <vmname|UUID>    start a VM by specifying its UUID or name\n"
+        "  --startvm=<vmname|UUID>    start a VM by specifying its UUID or name\n"
         "  --separate                 start a separate VM process\n"
         "  --normal                   keep normal (windowed) mode during startup\n"
         "  --fullscreen               switch to fullscreen mode during startup\n"
@@ -275,32 +280,31 @@ static void ShowHelp()
         "  --scale                    switch to scale mode during startup\n"
         "  --no-startvm-errormsgbox   do not show a message box for VM start errors\n"
         "  --restore-current          restore the current snapshot before starting\n"
+        "  --restore-snapshot=<name>  restore the named snapshot before starting\n"
         "  --no-aggressive-caching    delays caching media info in VM processes\n"
-        "  --fda <image|none>         Mount the specified floppy image\n"
-        "  --dvd <image|none>         Mount the specified DVD image\n"
+        "  --fda=<image|none>         Mount the specified floppy image\n"
+        "  --dvd=<image|none>         Mount the specified DVD image\n"
 # ifdef VBOX_GUI_WITH_PIDFILE
-        "  --pidfile <file>           create a pidfile file when a VM is up and running\n"
+        "  --pidfile=<file>           create a pidfile file when a VM is up and running\n"
 # endif /* VBOX_GUI_WITH_PIDFILE */
 # ifdef VBOX_WITH_DEBUGGER_GUI
         "  --dbg                      enable the GUI debug menu\n"
         "  --debug                    like --dbg and show debug windows at VM startup\n"
         "  --debug-command-line       like --dbg and show command line window at VM startup\n"
         "  --debug-statistics         like --dbg and show statistics window at VM startup\n"
+        "  --statistics-expand=<pat>  expand the matching statistics (can be repeated)\n"
+        "  --statistics-filter=<pat>  statistics filter\n"
+        "  --statistics-config=<str>  statistics configuration\n"
         "  --no-debug                 disable the GUI debug menu and debug windows\n"
         "  --start-paused             start the VM in the paused state\n"
         "  --start-running            start the VM running (for overriding --debug*)\n"
 # endif /* VBOX_WITH_DEBUGGER_GUI */
         "\n"
         "Expert options:\n"
-        "  --disable-patm             disable code patching (ignored by AMD-V/VT-x)\n"
-        "  --disable-csam             disable code scanning (ignored by AMD-V/VT-x)\n"
-        "  --recompile-supervisor     recompiled execution of supervisor code (*)\n"
-        "  --recompile-user           recompiled execution of user code (*)\n"
-        "  --recompile-all            recompiled execution of all code, with disabled\n"
-        "                             code patching and scanning\n"
         "  --execute-all-in-iem       For debugging the interpreted execution mode.\n"
-        "  --warp-pct <pct>           time warp factor, 100%% (= 1.0) = normal speed\n"
-        "  (*) For AMD-V/VT-x setups the effect is --recompile-all.\n"
+        "  --driverless               Do not open the support driver (NEM or IEM mode).\n"
+        "  --warp-pct=<pct>           time warp factor, 100%% (= 1.0) = normal speed\n"
+        "  --no-keyboard-grabbing     Don't grab the keyboard on windows (for debugging).\n"
         "\n"
 # ifdef VBOX_WITH_DEBUGGER_GUI
         "The following environment (and extra data) variables are evaluated:\n"
@@ -319,13 +323,12 @@ static void ShowHelp()
         ;
 
     RTPrintf("%s v%s\n"
-             "(C) 2005-" VBOX_C_YEAR " " VBOX_VENDOR "\n"
-             "All rights reserved.\n"
+             "Copyright (C) 2005-" VBOX_C_YEAR " " VBOX_VENDOR "\n"
              "\n"
              "%s",
              s_szTitle, RTBldCfgVersion(), s_szUsage);
 
-#ifdef RT_OS_WINDOWS
+#ifdef VBOX_WS_WIN
     /* Show message box. Modify the option list a little
      * to better make it fit in the upcoming dialog. */
     char szTitleWithVersion[sizeof(s_szTitle) + 128];
@@ -382,12 +385,12 @@ static void ShowHelp()
     RTStrPrintf(szTitleWithVersion, sizeof(szTitleWithVersion), "%s v%s - Command Line Options", s_szTitle, RTBldCfgVersion());
     MessageBoxExA(NULL /*hwndOwner*/, szMsg, szTitleWithVersion, MB_OK | MB_ICONINFORMATION,
                   MAKELANGID(LANG_NEUTRAL, SUBLANG_NEUTRAL));
-#endif
+#endif /* VBOX_WS_WIN */
 }
 
 extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char ** /*envp*/)
 {
-#ifdef RT_OS_WINDOWS
+#ifdef VBOX_WS_WIN
     ATL::CComModule _Module; /* Required internally by ATL (constructor records instance in global variable). */
 #endif
 
@@ -400,16 +403,27 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char ** /*envp*/)
     /* Simulate try-catch block: */
     do
     {
-#ifdef VBOX_WS_MAC
-        /* Hide setuid root from AppKit: */
-        HideSetUidRootFromAppKit();
-#endif /* VBOX_WS_MAC */
-
-#ifdef VBOX_WS_X11
+#ifdef VBOX_WS_NIX
         /* Make sure multi-threaded environment is safe: */
         if (!MakeSureMultiThreadingIsSafe())
             break;
-#endif /* VBOX_WS_X11 */
+        VBGHDISPLAYSERVERTYPE enmDisplayServerType = VBGHDisplayServerTypeDetect();
+        /* Abort before instantiating QApplication in case no active display server can be found to
+         * prevent QApplication from aborting:  */
+        if (enmDisplayServerType == VBGHDISPLAYSERVERTYPE_NONE)
+        {
+            iResultCode = 1;
+            Log(("We could not detect an active display server. Exiting."));
+            RTStrmPrintf(g_pStdErr, "No active display server, X11 or Wayland, detected. Exiting.\n");
+            break;
+        }
+        if (VBGHDisplayServerTypeIsXAvailable(enmDisplayServerType))
+            /* Force using Qt platform plugin 'xcb', we have X11 specific code: */
+            RTEnvSet("QT_QPA_PLATFORM", "xcb");
+        else
+            /* Assume pure Wayland (without a X server):*/
+            RTEnvSet("QT_QPA_PLATFORM", "wayland");
+#endif /* VBOX_WS_NIX */
 
         /* Console help preprocessing: */
         bool fHelpShown = false;
@@ -437,38 +451,58 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char ** /*envp*/)
 #endif /* VBOX_WITH_HARDENING */
 
 #ifdef VBOX_WS_MAC
-        /* Apply font fixes (before QApplication get created and instantiated font-hints): */
-        switch (VBoxGlobal::determineOsRelease())
-        {
-            case MacOSXRelease_Mavericks: QFont::insertSubstitution(".Lucida Grande UI", "Lucida Grande"); break;
-            case MacOSXRelease_Yosemite:  QFont::insertSubstitution(".Helvetica Neue DeskInterface", "Helvetica Neue"); break;
-            case MacOSXRelease_ElCapitan: QFont::insertSubstitution(".SF NS Text", "Helvetica Neue"); break;
-            default: break;
-        }
-
+        /* Prevent AppNap for Runtime UI only: */
+        bool fPreventAppNap = false;
+# ifdef VBOX_RUNTIME_UI
+        fPreventAppNap = true;
+# endif
         /* Instantiate own NSApplication before QApplication do it for us: */
-        UICocoaApplication::instance();
+        UICocoaApplication::create(fPreventAppNap);
+
+# ifdef VBOX_RUNTIME_UI
+        /* If we're a helper app inside Resources in the main application bundle,
+         * we need to amend the library path so the platform plugin can be found.
+         * Note! This builds on the initIprtForDarwinHelperApp() hack. */
+        {
+            char szExecDir[RTPATH_MAX];
+            int vrc = RTPathExecDir(szExecDir, sizeof(szExecDir));
+            AssertRC(vrc);
+            RTPathStripTrailingSlash(szExecDir); /* .../Contents/MacOS */
+            RTPathStripFilename(szExecDir);      /* .../Contents */
+            RTPathAppend(szExecDir, sizeof(szExecDir), "plugins");      /* .../Contents/plugins */
+            QCoreApplication::addLibraryPath(QString::fromUtf8(szExecDir));
+        }
+# endif /* VBOX_RUNTIME_UI */
 #endif /* VBOX_WS_MAC */
 
-#ifdef VBOX_WS_X11
-# if defined(RT_OS_LINUX) && defined(DEBUG)
+#ifdef VBOX_WS_NIX
+# if (defined(RT_OS_LINUX) && !defined(RT_ARCH_ARM64)) && defined(DEBUG)
         /* Install signal handler to backtrace the call stack: */
         InstallSignalHandler();
 # endif /* RT_OS_LINUX && DEBUG */
-#endif /* VBOX_WS_X11 */
+#endif /* VBOX_WS_NIX */
 
         /* Install Qt console message handler: */
         qInstallMessageHandler(QtMessageOutput);
 
-        /* Enable HiDPI support: */
-        QApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
-#if (!defined(DEBUG_bird) || defined(RT_OS_DARWIN))
-        QApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
-#endif
+#ifdef VBOX_WS_NIX
+        // WORKAROUND:
+        // Some Desktop Environments forcing this variable which was useful in Qt5 case.
+        // But in Qt6 case this variable is additional multiplier to scale-factor
+        // calculated automatic way, which means for x2 the effective factor will be 2x2=4.
+        // No idea how to proceed, probably we will ignore this variable for now, by
+        // setting it to be always equal to 1.
+        //qputenv("QT_SCALE_FACTOR", "1");
+#endif /* VBOX_WS_NIX */
 
         /* Create application: */
         QApplication a(argc, argv);
 
+#ifdef VBOX_WS_MAC
+        /* Disable menu icons on MacOS X host: */
+        ::darwinDisableIconsInMenus();
+#endif /* VBOX_WS_MAC */
+
 #ifdef VBOX_WS_WIN
         /* Drag in the sound drivers and DLLs early to get rid of the delay taking
          * place when the main menu bar (or any action from that menu bar) is
@@ -479,12 +513,7 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char ** /*envp*/)
         PlaySound(NULL, NULL, 0);
 #endif /* VBOX_WS_WIN */
 
-#ifdef VBOX_WS_MAC
-        /* Disable menu icons on MacOS X host: */
-        ::darwinDisableIconsInMenus();
-#endif /* VBOX_WS_MAC */
-
-#ifdef VBOX_WS_X11
+#ifdef VBOX_WS_NIX
         /* Make all widget native.
          * We did it to avoid various Qt crashes while testing widget attributes or acquiring winIds.
          * Yes, we aware of note that alien widgets faster to draw but the only widget we need to be fast
@@ -507,49 +536,38 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char ** /*envp*/)
 # endif /* !Q_OS_SOLARIS */
 
         /* Qt version check (major.minor are sensitive, fix number is ignored): */
-        if (VBoxGlobal::qtRTVersion() < (VBoxGlobal::qtCTVersion() & 0xFFFF00))
+        if (UIVersionInfo::qtRTVersion() < (UIVersionInfo::qtCTVersion() & 0xFFFF00))
         {
             QString strMsg = QApplication::tr("Executable <b>%1</b> requires Qt %2.x, found Qt %3.")
                                               .arg(qAppName())
-                                              .arg(VBoxGlobal::qtCTVersionString().section('.', 0, 1))
-                                              .arg(VBoxGlobal::qtRTVersionString());
+                                              .arg(UIVersionInfo::qtCTVersionString().section('.', 0, 1))
+                                              .arg(UIVersionInfo::qtRTVersionString());
             QMessageBox::critical(0, QApplication::tr("Incompatible Qt Library Error"),
-                                  strMsg, QMessageBox::Abort, 0);
+                                  strMsg, QMessageBox::Abort, QMessageBox::NoButton);
             qFatal("%s", strMsg.toUtf8().constData());
             break;
         }
-#endif /* VBOX_WS_X11 */
+#endif /* VBOX_WS_NIX */
 
         /* Create modal-window manager: */
         UIModalWindowManager::create();
 
-        /* Create UI starter: */
-        UIStarter::create();
-#ifndef VBOX_GUI_WITH_SHARED_LIBRARY
-        /* Create global app instance: */
-        VBoxGlobal::create();
-#else
-# ifndef VBOX_RUNTIME_UI
+#ifndef VBOX_RUNTIME_UI
         /* Create global app instance for Selector UI: */
-        VBoxGlobal::create(VBoxGlobal::UIType_SelectorUI);
-# else
+        UICommon::create(UIType_ManagerUI);
+#else
         /* Create global app instance for Runtime UI: */
-        VBoxGlobal::create(VBoxGlobal::UIType_RuntimeUI);
-# endif
+        UICommon::create(UIType_RuntimeUI);
 #endif
 
         /* Simulate try-catch block: */
         do
         {
-            /* Exit if VBoxGlobal is not valid: */
-            if (!vboxGlobal().isValid())
+            /* Exit if UICommon is not valid: */
+            if (!uiCommon().isValid())
                 break;
-
-            /* Init link between UI starter and global app instance: */
-            gStarter->init();
-
-            /* Exit if VBoxGlobal pre-processed arguments: */
-            if (vboxGlobal().processArgs())
+            /* Exit if UICommon pre-processed arguments: */
+            if (uiCommon().processArgs())
                 break;
 
             // WORKAROUND:
@@ -561,23 +579,20 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char ** /*envp*/)
             // shown.  That can happen for example if window is not yet shown because blocked
             // by startup error message-box which is not treated as real window by some
             // reason.  So we are making application exit manual everywhere.
-            qApp->setQuitOnLastWindowClosed(false);
+            a.setQuitOnLastWindowClosed(false);
 
+            /* Create UI starter: */
+            UIStarter uiStarter;
             /* Request to Start UI _after_ QApplication executed: */
-            QMetaObject::invokeMethod(gStarter, "sltStartUI", Qt::QueuedConnection);
+            QMetaObject::invokeMethod(&uiStarter, "sltStartUI", Qt::QueuedConnection);
 
             /* Start application: */
             iResultCode = a.exec();
-
-            /* Break link between UI starter and global app instance: */
-            gStarter->deinit();
         }
         while (0);
 
         /* Destroy global app instance: */
-        VBoxGlobal::destroy();
-        /* Destroy UI starter: */
-        UIStarter::destroy();
+        UICommon::destroy();
 
         /* Destroy modal-window manager: */
         UIModalWindowManager::destroy();
@@ -592,41 +607,122 @@ extern "C" DECLEXPORT(int) TrustedMain(int argc, char **argv, char ** /*envp*/)
     return iResultCode;
 }
 
-#if !defined(VBOX_WITH_HARDENING) || (defined(VBOX_GUI_WITH_SHARED_LIBRARY) && !defined(VBOX_RUNTIME_UI))
+#if !defined(VBOX_WITH_HARDENING) || !defined(VBOX_RUNTIME_UI)
+
+# if defined(RT_OS_DARWIN) && defined(VBOX_RUNTIME_UI)
+
+extern "C" const char *_dyld_get_image_name(uint32_t);
+
+/** Init runtime with the executable path pointing into the
+ * VirtualBox.app/Contents/MacOS/ rather than
+ * VirtualBox.app/Contents/Resource/VirtualBoxVM.app/Contents/MacOS/.
+ *
+ * This is a HACK to make codesign and friends happy on OS X.   The idea is to
+ * improve and eliminate this over time.
+ */
+DECL_NO_INLINE(static, int) initIprtForDarwinHelperApp(int cArgs, char ***ppapszArgs, uint32_t fFlags)
+{
+    const char *pszImageName = _dyld_get_image_name(0);
+    AssertReturn(pszImageName, VERR_INTERNAL_ERROR);
+
+    char szTmpPath[PATH_MAX + 1];
+    const char *psz = realpath(pszImageName, szTmpPath);
+    int rc;
+    if (psz)
+    {
+        char *pszFilename = RTPathFilename(szTmpPath);
+        if (pszFilename)
+        {
+            char const chSavedFilename0 = *pszFilename;
+            *pszFilename = '\0';
+            RTPathStripTrailingSlash(szTmpPath); /* VirtualBox.app/Contents/Resources/VirtualBoxVM.app/Contents/MacOS */
+            RTPathStripFilename(szTmpPath);      /* VirtualBox.app/Contents/Resources/VirtualBoxVM.app/Contents/ */
+            RTPathStripFilename(szTmpPath);      /* VirtualBox.app/Contents/Resources/VirtualBoxVM.app */
+            RTPathStripFilename(szTmpPath);      /* VirtualBox.app/Contents/Resources */
+            RTPathStripFilename(szTmpPath);      /* VirtualBox.app/Contents */
+            char *pszDst = strchr(szTmpPath, '\0');
+            pszDst = (char *)memcpy(pszDst, RT_STR_TUPLE("/MacOS/")) + sizeof("/MacOS/") - 1; /** @todo where is mempcpy? */
+            *pszFilename = chSavedFilename0;
+            memmove(pszDst, pszFilename, strlen(pszFilename) + 1);
+
+            return RTR3InitEx(RTR3INIT_VER_CUR, fFlags, cArgs, ppapszArgs, szTmpPath);
+        }
+        rc = VERR_INVALID_NAME;
+    }
+    else
+        rc = RTErrConvertFromErrno(errno);
+    AssertMsgRCReturn(rc, ("rc=%Rrc pszLink=\"%s\"\nhex: %.*Rhxs\n", rc, pszImageName, strlen(pszImageName), pszImageName), rc);
+    return rc;
+}
+# endif
+
 
 int main(int argc, char **argv, char **envp)
 {
-# ifdef VBOX_WS_X11
+# ifdef VBOX_WS_NIX
     /* Make sure multi-threaded environment is safe: */
     if (!MakeSureMultiThreadingIsSafe())
         return 1;
-# endif /* VBOX_WS_X11 */
+# endif /* VBOX_WS_NIX */
+
+    /*
+     * Determine the IPRT/SUPLib initialization flags if runtime UI process.
+     * Only initialize SUPLib if about to start a VM in this process.
+     *
+     * Note! This must must match the corresponding parsing in hardenedmain.cpp
+     *       and UICommon.cpp exactly, otherwise there will be weird error messages.
+     */
+    /** @todo r=bird: We should consider just postponing this stuff till VM
+     *        creation, it shouldn't make too much of a difference GIP-wise. */
+    uint32_t fFlags = 0;
+# ifdef VBOX_RUNTIME_UI
+    unsigned cOptionsLeft     = 4;
+    bool     fStartVM         = false;
+    bool     fSeparateProcess = false;
+    bool     fExecuteAllInIem = false;
+    bool     fDriverless      = false;
+    for (int i = 1; i < argc && cOptionsLeft > 0; ++i)
+    {
+        if (   !strcmp(argv[i], "--startvm")
+            || !strcmp(argv[i], "-startvm"))
+        {
+            cOptionsLeft -= fStartVM == false;
+            fStartVM = true;
+            i++;
+        }
+        else if (   !strcmp(argv[i], "--separate")
+                 || !strcmp(argv[i], "-separate"))
+        {
+            cOptionsLeft -= fSeparateProcess == false;
+            fSeparateProcess = true;
+        }
+        else if (!strcmp(argv[i], "--execute-all-in-iem"))
+        {
+            cOptionsLeft -= fExecuteAllInIem == false;
+            fExecuteAllInIem = true;
+        }
+        else if (!strcmp(argv[i], "--driverless"))
+        {
+            cOptionsLeft -= fDriverless == false;
+            fDriverless = true;
+        }
+    }
+    if (fStartVM && !fSeparateProcess)
+    {
+        fFlags |= RTR3INIT_FLAGS_TRY_SUPLIB;
+        if (fExecuteAllInIem)
+            fFlags |= SUPR3INIT_F_DRIVERLESS_IEM_ALLOWED << RTR3INIT_FLAGS_SUPLIB_SHIFT;
+        if (fDriverless)
+            fFlags |= SUPR3INIT_F_DRIVERLESS << RTR3INIT_FLAGS_SUPLIB_SHIFT;
+    }
+# endif /* VBOX_RUNTIME_UI */
 
     /* Initialize VBox Runtime: */
-# if !defined(VBOX_GUI_WITH_SHARED_LIBRARY) || defined(VBOX_RUNTIME_UI)
-    /* Initialize the SUPLib as well only if we are really about to start a VM.
-     * Don't do this if we are only starting the selector window or a separate VM process. */
-    bool fStartVM = false;
-    bool fSeparateProcess = false;
-    for (int i = 1; i < argc && !(fStartVM && fSeparateProcess); ++i)
-    {
-        /* NOTE: the check here must match the corresponding check for the
-         * options to start a VM in hardenedmain.cpp and VBoxGlobal.cpp exactly,
-         * otherwise there will be weird error messages. */
-        if (   !::strcmp(argv[i], "--startvm")
-            || !::strcmp(argv[i], "-startvm"))
-            fStartVM = true;
-        else if (   !::strcmp(argv[i], "--separate")
-                 || !::strcmp(argv[i], "-separate"))
-            fSeparateProcess = true;
-    }
-
-    uint32_t fFlags = fStartVM && !fSeparateProcess ? RTR3INIT_FLAGS_SUPLIB : 0;
-#else
-    uint32_t fFlags = 0;
-#endif
+# if defined(RT_OS_DARWIN) && defined(VBOX_RUNTIME_UI)
+    int rc = initIprtForDarwinHelperApp(argc, &argv, fFlags);
+# else
     int rc = RTR3InitExe(argc, &argv, fFlags);
-
+# endif
     /* Initialization failed: */
     if (RT_FAILURE(rc))
     {
@@ -676,7 +772,7 @@ int main(int argc, char **argv, char **envp)
 
         /* Show the error-message: */
         QMessageBox::critical(0 /* parent */, strTitle, strText,
-                              QMessageBox::Abort /* 1st button */, 0 /* 2nd button */);
+                              QMessageBox::Abort /* 1st button */, QMessageBox::NoButton /* 2nd button */);
 
         /* Default error-result: */
         return 1;
@@ -686,7 +782,7 @@ int main(int argc, char **argv, char **envp)
     return TrustedMain(argc, argv, envp);
 }
 
-#endif /* !VBOX_WITH_HARDENING || (VBOX_GUI_WITH_SHARED_LIBRARY && !VBOX_RUNTIME_UI) */
+#endif /* !VBOX_WITH_HARDENING || !VBOX_RUNTIME_UI */
 
 #ifdef VBOX_WITH_HARDENING
 
@@ -703,11 +799,6 @@ int main(int argc, char **argv, char **envp)
  */
 extern "C" DECLEXPORT(void) TrustedError(const char *pszWhere, SUPINITOP enmWhat, int rc, const char *pszMsgFmt, va_list va)
 {
-# ifdef VBOX_WS_MAC
-    /* Hide setuid root from AppKit: */
-    HideSetUidRootFromAppKit();
-# endif /* VBOX_WS_MAC */
-
     char szMsgBuf[_16K];
 
     /*
@@ -787,11 +878,11 @@ extern "C" DECLEXPORT(void) TrustedError(const char *pszWhere, SUPINITOP enmWhat
             break;
     }
 
-# ifdef VBOX_WS_X11
+# ifdef VBOX_WS_NIX
     /* We have to to make sure that we display the error-message
      * after the parent displayed its own message. */
     sleep(2);
-# endif /* VBOX_WS_X11 */
+# endif /* VBOX_WS_NIX */
 
     /* Update strText with strDetails: */
     if (!strDetails.isEmpty())
