@@ -4,142 +4,115 @@
  */
 
 /*
- * Copyright (C) 2012-2019 Oracle Corporation
+ * Copyright (C) 2012-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 /* Qt includes: */
-#include <QGraphicsScene>
+#include <QApplication>
 #include <QGraphicsSceneMouseEvent>
 #include <QGraphicsView>
 #include <QPainter>
 #include <QStyleOptionGraphicsItem>
+#include <QWindow>
 
 /* GUI includes: */
-#include "VBoxGlobal.h"
 #include "UIChooserItemGlobal.h"
 #include "UIChooserModel.h"
+#include "UIChooserNodeGlobal.h"
 #include "UIIconPool.h"
+#include "UIImageTools.h"
+#include "UITranslationEventListener.h"
 #include "UIVirtualBoxManager.h"
 
-/* Other VBox includes: */
-#include "iprt/cpp/utils.h"
 
-
-UIChooserItemGlobal::UIChooserItemGlobal(UIChooserItem *pParent,
-                                         int iPosition /* = -1 */)
-    : UIChooserItem(pParent, pParent->isTemporary(), 0, 100)
-    , m_iDefaultLightnessMin(0)
-    , m_iDefaultLightnessMax(0)
-    , m_iHoverLightnessMin(0)
-    , m_iHoverLightnessMax(0)
-    , m_iHighlightLightnessMin(0)
-    , m_iHighlightLightnessMax(0)
+UIChooserItemGlobal::UIChooserItemGlobal(UIChooserItem *pParent, UIChooserNodeGlobal *pNode)
+    : UIChooserItem(pParent, pNode)
+#ifdef VBOX_WS_MAC
+    , m_iDefaultColorDeviation(0)
+#endif
+    , m_iHoverLightnessStart(0)
+    , m_iHoverLightnessFinal(0)
+    , m_iHighlightLightnessStart(0)
+    , m_iHighlightLightnessFinal(0)
     , m_iMinimumNameWidth(0)
     , m_iMaximumNameWidth(0)
     , m_iHeightHint(0)
 {
-    /* Prepare: */
     prepare();
-
-    /* Add item to the parent: */
-    AssertMsg(parentItem(), ("No parent set for global-item!"));
-    parentItem()->addItem(this, iPosition);
-    setZValue(parentItem()->zValue() + 1);
-
-    /* Configure connections: */
-    connect(gpManager, &UIVirtualBoxManager::sigWindowRemapped,
-            this, &UIChooserItemGlobal::sltHandleWindowRemapped);
-
-    /* Init: */
-    updatePixmaps();
-
-    /* Apply language settings: */
-    retranslateUi();
-}
-
-UIChooserItemGlobal::UIChooserItemGlobal(UIChooserItem *pParent,
-                                         UIChooserItemGlobal * ,
-                                         int iPosition /* = -1 */)
-    : UIChooserItem(pParent, pParent->isTemporary(), 0, 100)
-    , m_iDefaultLightnessMin(0)
-    , m_iDefaultLightnessMax(0)
-    , m_iHoverLightnessMin(0)
-    , m_iHoverLightnessMax(0)
-    , m_iHighlightLightnessMin(0)
-    , m_iHighlightLightnessMax(0)
-    , m_iMinimumNameWidth(0)
-    , m_iMaximumNameWidth(0)
-{
-    /* Prepare: */
-    prepare();
-
-    /* Add item to the parent: */
-    AssertMsg(parentItem(), ("No parent set for global-item!"));
-    parentItem()->addItem(this, iPosition);
-    setZValue(parentItem()->zValue() + 1);
-
-    /* Configure connections: */
-    connect(gpManager, &UIVirtualBoxManager::sigWindowRemapped,
-            this, &UIChooserItemGlobal::sltHandleWindowRemapped);
-
-    /* Init: */
-    updatePixmaps();
-
-    /* Apply language settings: */
-    retranslateUi();
 }
 
 UIChooserItemGlobal::~UIChooserItemGlobal()
 {
-    /* If that item is focused: */
-    if (model()->focusItem() == this)
-    {
-        /* Unset the focus: */
-        model()->setFocusItem(0);
-    }
-    /* If that item is in selection list: */
-    if (model()->currentItems().contains(this))
-    {
-        /* Remove item from the selection list: */
-        model()->removeFromCurrentItems(this);
-    }
-    /* If that item is in navigation list: */
-    if (model()->navigationList().contains(this))
-    {
-        /* Remove item from the navigation list: */
-        model()->removeFromNavigationList(this);
-    }
-
-    /* Remove item from the parent: */
-    AssertMsg(parentItem(), ("No parent set for global-item!"));
-    parentItem()->removeItem(this);
+    cleanup();
 }
 
-bool UIChooserItemGlobal::isToolsButtonArea(const QPoint &position, int iMarginMultiplier /* = 1 */) const
+UIChooserNodeGlobal *UIChooserItemGlobal::nodeToGlobalType() const
+{
+    return node() ? node()->toGlobalNode() : 0;
+}
+
+bool UIChooserItemGlobal::isToolButtonArea(const QPoint &position, int iMarginMultiplier /* = 1 */) const
 {
     const int iFullWidth = geometry().width();
     const int iFullHeight = geometry().height();
-    const int iMargin = data(GlobalItemData_Margin).toInt();
+    const int iMarginHR = data(GlobalItemData_MarginHR).toInt();
     const int iButtonMargin = data(GlobalItemData_ButtonMargin).toInt();
-    const int iToolsPixmapX = iFullWidth - iMargin - 1 - m_toolsPixmap.width() / m_toolsPixmap.devicePixelRatio();
-    const int iToolsPixmapY = (iFullHeight - m_toolsPixmap.height() / m_toolsPixmap.devicePixelRatio()) / 2;
-    QRect rect = QRect(iToolsPixmapX,
-                       iToolsPixmapY,
-                       m_toolsPixmap.width() / m_toolsPixmap.devicePixelRatio(),
-                       m_toolsPixmap.height() / m_toolsPixmap.devicePixelRatio());
+    const int iToolPixmapX = iFullWidth - iMarginHR - 1
+                           - m_toolPixmap.width() / m_toolPixmap.devicePixelRatio();
+    const int iToolPixmapY = (iFullHeight - m_toolPixmap.height() / m_toolPixmap.devicePixelRatio()) / 2;
+    QRect rect = QRect(iToolPixmapX,
+                       iToolPixmapY,
+                       m_toolPixmap.width() / m_toolPixmap.devicePixelRatio(),
+                       m_toolPixmap.height() / m_toolPixmap.devicePixelRatio());
     rect.adjust(-iMarginMultiplier * iButtonMargin, -iMarginMultiplier * iButtonMargin,
                  iMarginMultiplier * iButtonMargin,  iMarginMultiplier * iButtonMargin);
     return rect.contains(position);
 }
 
+bool UIChooserItemGlobal::isPinButtonArea(const QPoint &position, int iMarginMultiplier /* = 1 */) const
+{
+    const int iFullWidth = geometry().width();
+    const int iFullHeight = geometry().height();
+    const int iMarginHR = data(GlobalItemData_MarginHR).toInt();
+    const int iSpacing = data(GlobalItemData_Spacing).toInt();
+    const int iButtonMargin = data(GlobalItemData_ButtonMargin).toInt();
+    const int iPinPixmapX = iFullWidth - iMarginHR - 1
+                          - m_toolPixmap.width() / m_toolPixmap.devicePixelRatio()
+                          - iSpacing
+                          - m_pinPixmap.width() / m_pinPixmap.devicePixelRatio();
+    const int iPinPixmapY = (iFullHeight - m_pinPixmap.height() / m_pinPixmap.devicePixelRatio()) / 2;
+    QRect rect = QRect(iPinPixmapX,
+                       iPinPixmapY,
+                       m_pinPixmap.width() / m_pinPixmap.devicePixelRatio(),
+                       m_pinPixmap.height() / m_pinPixmap.devicePixelRatio());
+    rect.adjust(-iMarginMultiplier * iButtonMargin, -iMarginMultiplier * iButtonMargin,
+                 iMarginMultiplier * iButtonMargin,  iMarginMultiplier * iButtonMargin);
+    return rect.contains(position);
+}
+
+int UIChooserItemGlobal::heightHint() const
+{
+    return m_iHeightHint;
+}
+
 void UIChooserItemGlobal::setHeightHint(int iHint)
 {
     /* Remember a new hint: */
@@ -150,18 +123,8 @@ void UIChooserItemGlobal::setHeightHint(int iHint)
     model()->updateLayout();
 }
 
-void UIChooserItemGlobal::retranslateUi()
+void UIChooserItemGlobal::sltRetranslateUI()
 {
-    /* Update description: */
-    m_strName = tr("Tools");
-    m_strDescription = m_strName;
-
-    /* Update linked values: */
-    updateMinimumNameWidth();
-    updateVisibleName();
-
-    /* Update tool-tip: */
-    updateToolTip();
 }
 
 void UIChooserItemGlobal::showEvent(QShowEvent *pEvent)
@@ -210,37 +173,45 @@ void UIChooserItemGlobal::paint(QPainter *pPainter, const QStyleOptionGraphicsIt
     paintGlobalInfo(pPainter, rectangle);
 }
 
+void UIChooserItemGlobal::setFavorite(bool fFavorite)
+{
+    /* Call to base-class: */
+    UIChooserItem::setFavorite(fFavorite);
+
+    /* Update pin-pixmap: */
+    updatePinPixmap();
+}
+
 void UIChooserItemGlobal::startEditing()
 {
     AssertMsgFailed(("Global graphics item do NOT support editing yet!"));
 }
 
+void UIChooserItemGlobal::updateItem()
+{
+    /* Update this global-item: */
+    updatePixmaps();
+    updateMinimumNameWidth();
+    updateVisibleName();
+    updateToolTip();
+    update();
+
+    /* Update parent group-item: */
+    parentItem()->updateToolTip();
+    parentItem()->update();
+}
+
 void UIChooserItemGlobal::updateToolTip()
 {
-//    setToolTip(toolTipText());
+    // Nothing for now..
 }
 
-QString UIChooserItemGlobal::name() const
+QList<UIChooserItem*> UIChooserItemGlobal::items(UIChooserNodeType) const
 {
-    return m_strName;
+    AssertMsgFailedReturn(("Global graphics item do NOT support children!"), QList<UIChooserItem*>());
 }
 
-QString UIChooserItemGlobal::description() const
-{
-    return m_strDescription;
-}
-
-QString UIChooserItemGlobal::fullName() const
-{
-    return m_strName;
-}
-
-QString UIChooserItemGlobal::definition() const
-{
-    return QString("n=%1").arg("GLOBAL");
-}
-
-void UIChooserItemGlobal::addItem(UIChooserItem *, int)
+void UIChooserItemGlobal::addItem(UIChooserItem *, bool, int)
 {
     AssertMsgFailed(("Global graphics item do NOT support children!"));
 }
@@ -250,46 +221,10 @@ void UIChooserItemGlobal::removeItem(UIChooserItem *)
     AssertMsgFailed(("Global graphics item do NOT support children!"));
 }
 
-void UIChooserItemGlobal::setItems(const QList<UIChooserItem*> &, UIChooserItemType)
+UIChooserItem *UIChooserItemGlobal::searchForItem(const QString &, int iSearchFlags)
 {
-    AssertMsgFailed(("Global graphics item do NOT support children!"));
-}
-
-QList<UIChooserItem*> UIChooserItemGlobal::items(UIChooserItemType) const
-{
-    AssertMsgFailedReturn(("Global graphics item do NOT support children!"), QList<UIChooserItem*>());
-}
-
-bool UIChooserItemGlobal::hasItems(UIChooserItemType) const
-{
-    AssertMsgFailedReturn(("Global graphics item do NOT support children!"), false);
-}
-
-void UIChooserItemGlobal::clearItems(UIChooserItemType)
-{
-    AssertMsgFailed(("Global graphics item do NOT support children!"));
-}
-
-void UIChooserItemGlobal::updateAllItems(const QUuid &)
-{
-    /* Update this global-item: */
-    updatePixmaps();
-    updateToolTip();
-
-    /* Update parent group-item: */
-    parentItem()->updateToolTip();
-    parentItem()->update();
-}
-
-void UIChooserItemGlobal::removeAllItems(const QUuid &)
-{
-    // Just do nothing ..
-}
-
-UIChooserItem *UIChooserItemGlobal::searchForItem(const QString &, int iItemSearchFlags)
-{
-    /* Ignoring if we are not searching for the global-item? */
-    if (!(iItemSearchFlags & UIChooserItemSearchFlag_Global))
+    /* Ignore if we are not searching for the global-item: */
+    if (!(iSearchFlags & UIChooserItemSearchFlag_Global))
         return 0;
 
     /* Returning this: */
@@ -301,42 +236,31 @@ UIChooserItem *UIChooserItemGlobal::firstMachineItem()
     return 0;
 }
 
-void UIChooserItemGlobal::sortItems()
-{
-    AssertMsgFailed(("Global graphics item do NOT support children!"));
-}
-
 void UIChooserItemGlobal::updateLayout()
 {
     // Just do nothing ..
 }
 
-QSizeF UIChooserItemGlobal::sizeHint(Qt::SizeHint which, const QSizeF &constraint /* = QSizeF() */) const
-{
-    /* If Qt::MinimumSize requested: */
-    if (which == Qt::MinimumSize)
-        return QSizeF(minimumWidthHint(), minimumHeightHint());
-    /* Else call to base-class: */
-    return UIChooserItem::sizeHint(which, constraint);
-}
-
 int UIChooserItemGlobal::minimumWidthHint() const
 {
     /* Prepare variables: */
-    const int iMargin = data(GlobalItemData_Margin).toInt();
+    const int iMarginHL = data(GlobalItemData_MarginHL).toInt();
+    const int iMarginHR = data(GlobalItemData_MarginHR).toInt();
     const int iSpacing = data(GlobalItemData_Spacing).toInt();
 
     /* Calculating proposed width: */
     int iProposedWidth = 0;
 
     /* Two margins: */
-    iProposedWidth += 2 * iMargin;
+    iProposedWidth += iMarginHL + iMarginHR;
     /* And global-item content width: */
     iProposedWidth += (m_pixmapSize.width() +
                        iSpacing +
                        m_iMinimumNameWidth +
                        iSpacing +
-                       m_toolsPixmapSize.width());
+                       m_toolPixmapSize.width() +
+                       iSpacing +
+                       m_pinPixmapSize.width());
 
     /* Return result: */
     return iProposedWidth;
@@ -345,14 +269,15 @@ int UIChooserItemGlobal::minimumWidthHint() const
 int UIChooserItemGlobal::minimumHeightHint() const
 {
     /* Prepare variables: */
-    const int iMargin = data(GlobalItemData_Margin).toInt();
+    const int iMarginV = data(GlobalItemData_MarginV).toInt();
 
     /* Calculating proposed height: */
     int iProposedHeight = 0;
 
     /* Global-item content height: */
     int iContentHeight = qMax(m_pixmapSize.height(), m_visibleNameSize.height());
-    iContentHeight = qMax(iContentHeight, m_toolsPixmapSize.height());
+    iContentHeight = qMax(iContentHeight, m_toolPixmapSize.height());
+    iContentHeight = qMax(iContentHeight, m_pinPixmapSize.height());
 
     /* If we have height hint: */
     if (m_iHeightHint)
@@ -364,7 +289,7 @@ int UIChooserItemGlobal::minimumHeightHint() const
     else
     {
         /* Two margins: */
-        iProposedHeight += 2 * iMargin;
+        iProposedHeight += 2 * iMarginV;
         /* And content height: */
         iProposedHeight += iContentHeight;
     }
@@ -373,25 +298,27 @@ int UIChooserItemGlobal::minimumHeightHint() const
     return iProposedHeight;
 }
 
-QPixmap UIChooserItemGlobal::toPixmap()
+QSizeF UIChooserItemGlobal::sizeHint(Qt::SizeHint which, const QSizeF &constraint /* = QSizeF() */) const
 {
-    /* Ask item to paint itself into pixmap: */
-    const QSize minimumSize = minimumSizeHint().toSize();
-    QPixmap pixmap(minimumSize);
-    QPainter painter(&pixmap);
-    QStyleOptionGraphicsItem options;
-    options.rect = QRect(QPoint(0, 0), minimumSize);
-    paint(&painter, &options);
-    return pixmap;
+    /* If Qt::MinimumSize requested: */
+    if (which == Qt::MinimumSize)
+        return QSizeF(minimumWidthHint(), minimumHeightHint());
+    /* Else call to base-class: */
+    return UIChooserItem::sizeHint(which, constraint);
 }
 
-bool UIChooserItemGlobal::isDropAllowed(QGraphicsSceneDragDropEvent *, DragToken) const
+QPixmap UIChooserItemGlobal::toPixmap()
+{
+    AssertFailedReturn(QPixmap());
+}
+
+bool UIChooserItemGlobal::isDropAllowed(QGraphicsSceneDragDropEvent *, UIChooserItemDragToken) const
 {
     /* No drops at all: */
     return false;
 }
 
-void UIChooserItemGlobal::processDrop(QGraphicsSceneDragDropEvent *, UIChooserItem *, DragToken)
+void UIChooserItemGlobal::processDrop(QGraphicsSceneDragDropEvent *, UIChooserItem *, UIChooserItemDragToken)
 {
     /* Nothing to process: */
 }
@@ -414,22 +341,24 @@ void UIChooserItemGlobal::sltHandleWindowRemapped()
 
 void UIChooserItemGlobal::prepare()
 {
-    /* Colors: */
-#ifdef VBOX_WS_MAC
-    m_iHighlightLightnessMin = 105;
-    m_iHighlightLightnessMax = 115;
-    m_iHoverLightnessMin = 115;
-    m_iHoverLightnessMax = 125;
-    m_iDefaultLightnessMin = 125;
-    m_iDefaultLightnessMax = 130;
-#else /* VBOX_WS_MAC */
-    m_iHighlightLightnessMin = 130;
-    m_iHighlightLightnessMax = 160;
-    m_iHoverLightnessMin = 160;
-    m_iHoverLightnessMax = 190;
-    m_iDefaultLightnessMin = 160;
-    m_iDefaultLightnessMax = 190;
-#endif /* !VBOX_WS_MAC */
+    /* Color tones: */
+#if defined(VBOX_WS_MAC)
+    m_iDefaultColorDeviation = 105;
+    m_iHoverLightnessStart = 125;
+    m_iHoverLightnessFinal = 115;
+    m_iHighlightLightnessStart = 115;
+    m_iHighlightLightnessFinal = 105;
+#elif defined(VBOX_WS_WIN)
+    m_iHoverLightnessStart = 220;
+    m_iHoverLightnessFinal = 210;
+    m_iHighlightLightnessStart = 190;
+    m_iHighlightLightnessFinal = 180;
+#else /* !VBOX_WS_MAC && !VBOX_WS_WIN */
+    m_iHoverLightnessStart = 125;
+    m_iHoverLightnessFinal = 115;
+    m_iHighlightLightnessStart = 110;
+    m_iHighlightLightnessFinal = 100;
+#endif /* !VBOX_WS_MAC && !VBOX_WS_WIN */
 
     /* Fonts: */
     m_nameFont = font();
@@ -438,6 +367,48 @@ void UIChooserItemGlobal::prepare()
     /* Sizes: */
     m_iMinimumNameWidth = 0;
     m_iMaximumNameWidth = 0;
+
+    /* Add item to the parent: */
+    AssertPtrReturnVoid(parentItem());
+    parentItem()->addItem(this, isFavorite(), position());
+
+    /* Configure connections: */
+    connect(gpManager, &UIVirtualBoxManager::sigWindowRemapped,
+            this, &UIChooserItemGlobal::sltHandleWindowRemapped);
+
+    /* Init: */
+    updatePixmaps();
+
+    /* Apply language settings: */
+    sltRetranslateUI();
+    connect(&translationEventListener(), &UITranslationEventListener::sigRetranslateUI,
+            this, &UIChooserItemGlobal::sltRetranslateUI);
+}
+
+void UIChooserItemGlobal::cleanup()
+{
+    /* If that item is current: */
+    if (model()->currentItem() == this)
+    {
+        /* Unset current-item: */
+        model()->setCurrentItem(0);
+    }
+    /* If that item is in selection list: */
+    if (model()->selectedItems().contains(this))
+    {
+        /* Remove item from the selection list: */
+        model()->removeFromSelectedItems(this);
+    }
+    /* If that item is in navigation list: */
+    if (model()->navigationItems().contains(this))
+    {
+        /* Remove item from the navigation list: */
+        model()->removeFromNavigationItems(this);
+    }
+
+    /* Remove item from the parent: */
+    AssertPtrReturnVoid(parentItem());
+    parentItem()->removeItem(this);
 }
 
 QVariant UIChooserItemGlobal::data(int iKey) const
@@ -446,7 +417,9 @@ QVariant UIChooserItemGlobal::data(int iKey) const
     switch (iKey)
     {
         /* Layout hints: */
-        case GlobalItemData_Margin:       return QApplication::style()->pixelMetric(QStyle::PM_SmallIconSize) / 3 * 2;
+        case GlobalItemData_MarginHL:     return QApplication::style()->pixelMetric(QStyle::PM_SmallIconSize);
+        case GlobalItemData_MarginHR:     return QApplication::style()->pixelMetric(QStyle::PM_SmallIconSize) / 4 * 5;
+        case GlobalItemData_MarginV:      return QApplication::style()->pixelMetric(QStyle::PM_SmallIconSize) / 4 * 3;
         case GlobalItemData_Spacing:      return QApplication::style()->pixelMetric(QStyle::PM_SmallIconSize) / 2;
         case GlobalItemData_ButtonMargin: return QApplication::style()->pixelMetric(QStyle::PM_SmallIconSize) / 4;
 
@@ -460,8 +433,10 @@ void UIChooserItemGlobal::updatePixmaps()
 {
     /* Update pixmap: */
     updatePixmap();
-    /* Update tools-pixmap: */
-    updateToolsPixmap();
+    /* Update tool-pixmap: */
+    updateToolPixmap();
+    /* Update pin-pixmap: */
+    updatePinPixmap();
 }
 
 void UIChooserItemGlobal::updatePixmap()
@@ -472,7 +447,8 @@ void UIChooserItemGlobal::updatePixmap()
 
     /* Create new icon, then acquire pixmap: */
     const QIcon icon = UIIconPool::iconSet(":/tools_global_32px.png");
-    const QPixmap pixmap = icon.pixmap(gpManager->windowHandle(), pixmapSize);
+    const qreal fDevicePixelRatio = gpManager->windowHandle() ? gpManager->windowHandle()->devicePixelRatio() : 1;
+    const QPixmap pixmap = icon.pixmap(pixmapSize, fDevicePixelRatio);
 
     /* Update linked values: */
     if (m_pixmapSize != pixmapSize)
@@ -488,24 +464,48 @@ void UIChooserItemGlobal::updatePixmap()
     }
 }
 
-void UIChooserItemGlobal::updateToolsPixmap()
+void UIChooserItemGlobal::updateToolPixmap()
 {
     /* Determine icon metric: */
     const int iIconMetric = QApplication::style()->pixelMetric(QStyle::PM_LargeIconSize) * .75;
-    /* Create new tools-pixmap and tools-pixmap size: */
-    const QIcon toolsIcon = UIIconPool::iconSet(":/tools_menu_24px.png");
-    AssertReturnVoid(!toolsIcon.isNull());
-    const QSize toolsPixmapSize = QSize(iIconMetric, iIconMetric);
-    const QPixmap toolsPixmap = toolsIcon.pixmap(gpManager->windowHandle(), toolsPixmapSize);
+    /* Create new tool-pixmap and tool-pixmap size: */
+    const QIcon toolIcon = UIIconPool::iconSet(":/tools_menu_24px.png");
+    AssertReturnVoid(!toolIcon.isNull());
+    const QSize toolPixmapSize = QSize(iIconMetric, iIconMetric);
+    const qreal fDevicePixelRatio = gpManager->windowHandle() ? gpManager->windowHandle()->devicePixelRatio() : 1;
+    const QPixmap toolPixmap = toolIcon.pixmap(toolPixmapSize, fDevicePixelRatio);
     /* Update linked values: */
-    if (m_toolsPixmapSize != toolsPixmapSize)
+    if (m_toolPixmapSize != toolPixmapSize)
     {
-        m_toolsPixmapSize = toolsPixmapSize;
+        m_toolPixmapSize = toolPixmapSize;
         updateGeometry();
     }
-    if (m_toolsPixmap.toImage() != toolsPixmap.toImage())
+    if (m_toolPixmap.toImage() != toolPixmap.toImage())
     {
-        m_toolsPixmap = toolsPixmap;
+        m_toolPixmap = toolPixmap;
+        update();
+    }
+}
+
+void UIChooserItemGlobal::updatePinPixmap()
+{
+    /* Determine icon metric: */
+    const int iIconMetric = QApplication::style()->pixelMetric(QStyle::PM_LargeIconSize) * .75;
+    /* Create new tool-pixmap and tool-pixmap size: */
+    const QIcon pinIcon = UIIconPool::iconSet(isFavorite() ? ":/favorite_pressed_24px.png" : ":/favorite_24px.png");
+    AssertReturnVoid(!pinIcon.isNull());
+    const QSize pinPixmapSize = QSize(iIconMetric, iIconMetric);
+    const qreal fDevicePixelRatio = gpManager->windowHandle() ? gpManager->windowHandle()->devicePixelRatio() : 1;
+    const QPixmap pinPixmap = pinIcon.pixmap(pinPixmapSize, fDevicePixelRatio);
+    /* Update linked values: */
+    if (m_pinPixmapSize != pinPixmapSize)
+    {
+        m_pinPixmapSize = pinPixmapSize;
+        updateGeometry();
+    }
+    if (m_pinPixmap.toImage() != pinPixmap.toImage())
+    {
+        m_pinPixmap = pinPixmap;
         update();
     }
 }
@@ -515,7 +515,8 @@ void UIChooserItemGlobal::updateMinimumNameWidth()
     /* Calculate new minimum name width: */
     QPaintDevice *pPaintDevice = model()->paintDevice();
     const QFontMetrics fm(m_nameFont, pPaintDevice);
-    const int iMinimumNameWidth = fm.width(compressText(m_nameFont, pPaintDevice, m_strName, textWidth(m_nameFont, pPaintDevice, 15)));
+    const int iMinimumNameWidth = fm.horizontalAdvance(compressText(m_nameFont, pPaintDevice, name(),
+                                                                    textWidth(m_nameFont, pPaintDevice, 15)));
 
     /* Is there something changed? */
     if (m_iMinimumNameWidth == iMinimumNameWidth)
@@ -529,15 +530,16 @@ void UIChooserItemGlobal::updateMinimumNameWidth()
 void UIChooserItemGlobal::updateMaximumNameWidth()
 {
     /* Prepare variables: */
-    const int iMargin = data(GlobalItemData_Margin).toInt();
+    const int iMarginHL = data(GlobalItemData_MarginHL).toInt();
+    const int iMarginHR = data(GlobalItemData_MarginHR).toInt();
     const int iSpacing = data(GlobalItemData_Spacing).toInt();
 
     /* Calculate new maximum name width: */
     int iMaximumNameWidth = (int)geometry().width();
-    iMaximumNameWidth -= iMargin; /* left margin */
+    iMaximumNameWidth -= iMarginHL; /* left margin */
     iMaximumNameWidth -= m_pixmapSize.width(); /* pixmap width */
     iMaximumNameWidth -= iSpacing; /* spacing between pixmap and name */
-    iMaximumNameWidth -= iMargin; /* right margin */
+    iMaximumNameWidth -= iMarginHR; /* right margin */
 
     /* Is there something changed? */
     if (m_iMaximumNameWidth == iMaximumNameWidth)
@@ -554,7 +556,7 @@ void UIChooserItemGlobal::updateVisibleName()
     QPaintDevice *pPaintDevice = model()->paintDevice();
 
     /* Calculate new visible name and name-size: */
-    const QString strVisibleName = compressText(m_nameFont, pPaintDevice, m_strName, m_iMaximumNameWidth);
+    const QString strVisibleName = compressText(m_nameFont, pPaintDevice, name(), m_iMaximumNameWidth);
     const QSize visibleNameSize = textSize(m_nameFont, pPaintDevice, strVisibleName);
 
     /* Update linked values: */
@@ -570,23 +572,23 @@ void UIChooserItemGlobal::updateVisibleName()
     }
 }
 
-void UIChooserItemGlobal::paintBackground(QPainter *pPainter, const QRect &rectangle) const
+void UIChooserItemGlobal::paintBackground(QPainter *pPainter, const QRect &rectangle)
 {
     /* Save painter: */
     pPainter->save();
 
     /* Prepare color: */
-    const QPalette pal = palette();
+    const QPalette pal = QApplication::palette();
 
-    /* Selection background: */
-    if (model()->currentItems().contains(unconst(this)))
+    /* Selected-item background: */
+    if (model()->selectedItems().contains(this))
     {
         /* Prepare color: */
         const QColor backgroundColor = pal.color(QPalette::Active, QPalette::Highlight);
         /* Draw gradient: */
         QLinearGradient bgGrad(rectangle.topLeft(), rectangle.bottomLeft());
-        bgGrad.setColorAt(0, backgroundColor.lighter(m_iHighlightLightnessMax));
-        bgGrad.setColorAt(1, backgroundColor.lighter(m_iHighlightLightnessMin));
+        bgGrad.setColorAt(0, backgroundColor.lighter(m_iHighlightLightnessStart));
+        bgGrad.setColorAt(1, backgroundColor.lighter(m_iHighlightLightnessFinal));
         pPainter->fillRect(rectangle, bgGrad);
 
         if (isHovered())
@@ -600,7 +602,7 @@ void UIChooserItemGlobal::paintBackground(QPainter *pPainter, const QRect &recta
             animationColor1.setAlpha(30);
 #endif
             animationColor2.setAlpha(0);
-            /* Draw hovering animated gradient: */
+            /* Draw hovered-item animated gradient: */
             QRect animatedRect = rectangle;
             animatedRect.setWidth(animatedRect.height());
             const int iLength = 2 * animatedRect.width() + rectangle.width();
@@ -615,15 +617,15 @@ void UIChooserItemGlobal::paintBackground(QPainter *pPainter, const QRect &recta
             pPainter->fillRect(rectangle, bgAnimatedGrad);
         }
     }
-    /* Hovering background: */
+    /* Hovered-item background: */
     else if (isHovered())
     {
         /* Prepare color: */
         const QColor backgroundColor = pal.color(QPalette::Active, QPalette::Highlight);
         /* Draw gradient: */
         QLinearGradient bgGrad(rectangle.topLeft(), rectangle.bottomLeft());
-        bgGrad.setColorAt(0, backgroundColor.lighter(m_iHoverLightnessMax));
-        bgGrad.setColorAt(1, backgroundColor.lighter(m_iHoverLightnessMin));
+        bgGrad.setColorAt(0, backgroundColor.lighter(m_iHoverLightnessStart));
+        bgGrad.setColorAt(1, backgroundColor.lighter(m_iHoverLightnessFinal));
         pPainter->fillRect(rectangle, bgGrad);
 
         /* Prepare color: */
@@ -635,7 +637,7 @@ void UIChooserItemGlobal::paintBackground(QPainter *pPainter, const QRect &recta
         animationColor1.setAlpha(50);
 #endif
         animationColor2.setAlpha(0);
-        /* Draw hovering animated gradient: */
+        /* Draw hovered-item animated gradient: */
         QRect animatedRect = rectangle;
         animatedRect.setWidth(animatedRect.height());
         const int iLength = 2 * animatedRect.width() + rectangle.width();
@@ -654,17 +656,15 @@ void UIChooserItemGlobal::paintBackground(QPainter *pPainter, const QRect &recta
     {
 #ifdef VBOX_WS_MAC
         /* Prepare color: */
-        const QColor backgroundColor = pal.color(QPalette::Active, QPalette::Mid);
+        const QColor backgroundColor = pal.color(QPalette::Active, QPalette::Window);
         /* Draw gradient: */
         QLinearGradient bgGrad(rectangle.topLeft(), rectangle.bottomLeft());
-        bgGrad.setColorAt(0, backgroundColor.lighter(m_iDefaultLightnessMax));
-        bgGrad.setColorAt(1, backgroundColor.lighter(m_iDefaultLightnessMin));
+        bgGrad.setColorAt(0, backgroundColor.lighter(m_iDefaultColorDeviation));
+        bgGrad.setColorAt(1, backgroundColor.darker(m_iDefaultColorDeviation));
         pPainter->fillRect(rectangle, bgGrad);
 #else
-        /* Prepare color: */
-        QColor backgroundColor = pal.color(QPalette::Active, QPalette::Mid).lighter(160);
-        /* Draw gradient: */
-        pPainter->fillRect(rectangle, backgroundColor);
+        /* Draw simple background: */
+        pPainter->fillRect(rectangle, pal.color(QPalette::Active, QPalette::Window));
 #endif
     }
 
@@ -672,28 +672,25 @@ void UIChooserItemGlobal::paintBackground(QPainter *pPainter, const QRect &recta
     pPainter->restore();
 }
 
-void UIChooserItemGlobal::paintFrame(QPainter *pPainter, const QRect &rectangle) const
+void UIChooserItemGlobal::paintFrame(QPainter *pPainter, const QRect &rectangle)
 {
-    /* Only chosen and/or hovered item should have a frame: */
-    if (!model()->currentItems().contains(unconst(this)) && !isHovered())
+    /* Only selected and/or hovered item should have a frame: */
+    if (!model()->selectedItems().contains(this) && !isHovered())
         return;
 
     /* Save painter: */
     pPainter->save();
 
     /* Prepare color: */
-    const QPalette pal = palette();
+    const QPalette pal = QApplication::palette();
     QColor strokeColor;
 
-    /* Selection frame: */
-    if (model()->currentItems().contains(unconst(this)))
-        strokeColor = pal.color(QPalette::Active, QPalette::Highlight).lighter(m_iHighlightLightnessMin - 40);
-    /* Hovering frame: */
+    /* Selected-item frame: */
+    if (model()->selectedItems().contains(this))
+        strokeColor = pal.color(QPalette::Active, QPalette::Highlight).lighter(m_iHighlightLightnessStart - 40);
+    /* Hovered-item frame: */
     else if (isHovered())
-        strokeColor = pal.color(QPalette::Active, QPalette::Highlight).lighter(m_iHoverLightnessMin - 50);
-    /* Default frame: */
-    else
-        strokeColor = pal.color(QPalette::Active, QPalette::Mid).lighter(m_iDefaultLightnessMin);
+        strokeColor = pal.color(QPalette::Active, QPalette::Highlight).lighter(m_iHoverLightnessStart - 40);
 
     /* Create/assign pen: */
     QPen pen(strokeColor);
@@ -709,36 +706,35 @@ void UIChooserItemGlobal::paintFrame(QPainter *pPainter, const QRect &rectangle)
     pPainter->restore();
 }
 
-void UIChooserItemGlobal::paintGlobalInfo(QPainter *pPainter, const QRect &rectangle) const
+void UIChooserItemGlobal::paintGlobalInfo(QPainter *pPainter, const QRect &rectangle)
 {
     /* Prepare variables: */
     const int iFullWidth = rectangle.width();
     const int iFullHeight = rectangle.height();
-    const int iMargin = data(GlobalItemData_Margin).toInt();
+    const int iMarginHL = data(GlobalItemData_MarginHL).toInt();
+    const int iMarginHR = data(GlobalItemData_MarginHR).toInt();
     const int iSpacing = data(GlobalItemData_Spacing).toInt();
     const int iButtonMargin = data(GlobalItemData_ButtonMargin).toInt();
 
-    /* Selected item foreground: */
-    if (model()->currentItems().contains(unconst(this)))
+    /* Selected or hovered item foreground: */
+    if (model()->selectedItems().contains(this) || isHovered())
     {
-        const QPalette pal = palette();
-        pPainter->setPen(pal.color(QPalette::HighlightedText));
-    }
-    /* Hovered item foreground: */
-    else if (isHovered())
-    {
-        /* Prepare color: */
-        const QPalette pal = palette();
+        /* Prepare palette: */
+        const QPalette pal = QApplication::palette();
+
+        /* Get background color: */
         const QColor highlight = pal.color(QPalette::Active, QPalette::Highlight);
-        const QColor hhl = highlight.lighter(m_iHoverLightnessMax);
-        if (hhl.value() - hhl.saturation() > 0)
-            pPainter->setPen(pal.color(QPalette::Active, QPalette::Text));
-        else
-            pPainter->setPen(pal.color(QPalette::Active, QPalette::HighlightedText));
+        const QColor background = model()->selectedItems().contains(this)
+                                ? highlight.lighter(m_iHighlightLightnessStart)
+                                : highlight.lighter(m_iHoverLightnessStart);
+
+        /* Gather foreground color for background one: */
+        const QColor foreground = suitableForegroundColor(pal, background);
+        pPainter->setPen(foreground);
     }
 
     /* Calculate indents: */
-    int iLeftColumnIndent = iMargin;
+    int iLeftColumnIndent = iMarginHL;
 
     /* Paint left column: */
     {
@@ -780,29 +776,31 @@ void UIChooserItemGlobal::paintGlobalInfo(QPainter *pPainter, const QRect &recta
     }
 
     /* Calculate indents: */
-    int iRightColumnIndent = iFullWidth - iMargin - 1 - m_toolsPixmap.width() / m_toolsPixmap.devicePixelRatio();
+    QGraphicsView *pView = model()->scene()->views().first();
+    const QPointF sceneCursorPosition = pView->mapToScene(pView->mapFromGlobal(QCursor::pos()));
+    const QPoint itemCursorPosition = mapFromScene(sceneCursorPosition).toPoint();
+    int iRightColumnIndent = iFullWidth - iMarginHR - 1 - m_toolPixmap.width() / m_toolPixmap.devicePixelRatio();
 
     /* Paint right column: */
-    if (model()->currentItem() == this)
+    if (   model()->firstSelectedItem() == this
+        || isHovered())
     {
         /* Prepare variables: */
-        int iToolsPixmapX = iRightColumnIndent;
-        int iToolsPixmapY = (iFullHeight - m_toolsPixmap.height() / m_toolsPixmap.devicePixelRatio()) / 2;
-        QRect buttonRectangle = QRect(iToolsPixmapX,
-                                      iToolsPixmapY,
-                                      m_toolsPixmap.width() / m_toolsPixmap.devicePixelRatio(),
-                                      m_toolsPixmap.height() / m_toolsPixmap.devicePixelRatio());
-        buttonRectangle.adjust(- iButtonMargin, -iButtonMargin, iButtonMargin, iButtonMargin);
-        const QPoint sceneCursorPosition = model()->scene()->views().first()->mapFromGlobal(QCursor::pos());
-        const QPoint itemCursorPosition = mapFromScene(sceneCursorPosition).toPoint();
+        const int iToolPixmapX = iRightColumnIndent;
+        const int iToolPixmapY = (iFullHeight - m_toolPixmap.height() / m_toolPixmap.devicePixelRatio()) / 2;
+        QRect toolButtonRectangle = QRect(iToolPixmapX,
+                                          iToolPixmapY,
+                                          m_toolPixmap.width() / m_toolPixmap.devicePixelRatio(),
+                                          m_toolPixmap.height() / m_toolPixmap.devicePixelRatio());
+        toolButtonRectangle.adjust(- iButtonMargin, -iButtonMargin, iButtonMargin, iButtonMargin);
 
-        /* Paint flat button: */
+        /* Paint tool button: */
         if (   isHovered()
-            && isToolsButtonArea(itemCursorPosition, 4))
+            && isToolButtonArea(itemCursorPosition, 4))
             paintFlatButton(/* Painter: */
                             pPainter,
                             /* Button rectangle: */
-                            buttonRectangle,
+                            toolButtonRectangle,
                             /* Cursor position: */
                             itemCursorPosition);
 
@@ -810,8 +808,43 @@ void UIChooserItemGlobal::paintGlobalInfo(QPainter *pPainter, const QRect &recta
         paintPixmap(/* Painter: */
                     pPainter,
                     /* Point to paint in: */
-                    QPoint(iToolsPixmapX, iToolsPixmapY),
+                    QPoint(iToolPixmapX, iToolPixmapY),
                     /* Pixmap to paint: */
-                    m_toolsPixmap);
+                    m_toolPixmap);
+    }
+
+    /* Calculate indents: */
+    iRightColumnIndent = iRightColumnIndent - m_toolPixmap.width() / m_toolPixmap.devicePixelRatio() - iSpacing;
+
+    /* Paint right column: */
+    if (   model()->firstSelectedItem() == this
+        || isHovered())
+    {
+        /* Prepare variables: */
+        const int iPinPixmapX = iRightColumnIndent;
+        const int iPinPixmapY = (iFullHeight - m_pinPixmap.height() / m_pinPixmap.devicePixelRatio()) / 2;
+        QRect pinButtonRectangle = QRect(iPinPixmapX,
+                                         iPinPixmapY,
+                                         m_pinPixmap.width() / m_pinPixmap.devicePixelRatio(),
+                                         m_pinPixmap.height() / m_pinPixmap.devicePixelRatio());
+        pinButtonRectangle.adjust(- iButtonMargin, -iButtonMargin, iButtonMargin, iButtonMargin);
+
+        /* Paint pin button: */
+        if (   isHovered()
+            && isPinButtonArea(itemCursorPosition, 4))
+            paintFlatButton(/* Painter: */
+                            pPainter,
+                            /* Button rectangle: */
+                            pinButtonRectangle,
+                            /* Cursor position: */
+                            itemCursorPosition);
+
+        /* Paint pixmap: */
+        paintPixmap(/* Painter: */
+                    pPainter,
+                    /* Point to paint in: */
+                    QPoint(iPinPixmapX, iPinPixmapY),
+                    /* Pixmap to paint: */
+                    m_pinPixmap);
     }
 }
