@@ -4,19 +4,32 @@
  */
 
 /*
- * Copyright (C) 2009-2016 Oracle Corporation
+ * Copyright (C) 2009-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
-#ifndef ___UIAbstractDockIconPreview_h___
-#define ___UIAbstractDockIconPreview_h___
+#ifndef FEQT_INCLUDED_SRC_platform_darwin_UIAbstractDockIconPreview_h
+#define FEQT_INCLUDED_SRC_platform_darwin_UIAbstractDockIconPreview_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 /* System includes */
 #include <ApplicationServices/ApplicationServices.h>
@@ -25,14 +38,14 @@
 #include "VBoxUtils-darwin.h"
 
 class UIFrameBuffer;
-class UISession;
+class UIMachine;
 
 class QPixmap;
 
 class UIAbstractDockIconPreview
 {
 public:
-    UIAbstractDockIconPreview(UISession *pSession, const QPixmap& overlayImage);
+    UIAbstractDockIconPreview(UIMachine *pMachine, const QPixmap& overlayImage);
     virtual ~UIAbstractDockIconPreview() {}
 
     virtual void updateDockOverlay() = 0;
@@ -45,7 +58,7 @@ public:
 class UIAbstractDockIconPreviewHelper
 {
 public:
-    UIAbstractDockIconPreviewHelper(UISession *pSession, const QPixmap& overlayImage);
+    UIAbstractDockIconPreviewHelper(UIMachine *pMachine, const QPixmap& overlayImage);
     virtual ~UIAbstractDockIconPreviewHelper();
     void initPreviewImages();
     void drawOverlayIcons(CGContextRef context);
@@ -58,7 +71,7 @@ public:
     inline CGRect centerRectTo(CGRect rect, const CGRect& toRect) const { return ::darwinCenterRectTo(rect, toRect); }
 
     /* Private member vars */
-    UISession *m_pSession;
+    UIMachine *m_pMachine;
     const CGRect m_dockIconRect;
 
     CGImageRef m_overlayImage;
@@ -69,5 +82,5 @@ public:
     CGRect m_monitorRect;
 };
 
-#endif /* ___UIAbstractDockIconPreview_h___ */
+#endif /* !FEQT_INCLUDED_SRC_platform_darwin_UIAbstractDockIconPreview_h */
 
