@@ -4,25 +4,37 @@
  */
 
 /*
- * Copyright (C) 2011-2016 Oracle Corporation
+ * Copyright (C) 2011-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
-#ifndef ___UIDnDMIMEData_h___
-#define ___UIDnDMIMEData_h___
+#ifndef FEQT_INCLUDED_SRC_runtime_UIDnDMIMEData_h
+#define FEQT_INCLUDED_SRC_runtime_UIDnDMIMEData_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 /* Qt includes: */
 #include <QMimeData>
 
 /* COM includes: */
-#include "COMEnums.h"
 #include "CConsole.h"
 #include "CDnDSource.h"
 #include "CGuest.h"
@@ -74,7 +86,7 @@ signals:
      * @param vaType                Qt's variant type of the MIME data.
      * @param vaData                Reference to QVariant where to store the retrieved data.
      */
-    int sigGetData(Qt::DropAction dropAction, const QString &strMIMEType, QVariant::Type vaType, QVariant &vaData) const;
+    int sigGetData(Qt::DropAction dropAction, const QString &strMIMEType, QMetaType::Type vaType, QVariant &vaData) const;
 
 public slots:
 
@@ -87,11 +99,11 @@ public slots:
 protected:
     /** @name Overridden functions of QMimeData.
      * @{ */
-    virtual QStringList formats(void) const;
+    virtual QStringList formats(void) const RT_OVERRIDE;
 
-    virtual bool hasFormat(const QString &mimeType) const;
+    virtual bool hasFormat(const QString &mimeType) const RT_OVERRIDE;
 
-    virtual QVariant retrieveData(const QString &strMIMEType, QVariant::Type vaType) const;
+    virtual QVariant retrieveData(const QString &strMIMEType, QMetaType metaType) const RT_OVERRIDE;
     /** @}  */
 
 public:
@@ -100,12 +112,12 @@ public:
      * @{ */
 
     /**
-     * Returns the matching variant type of a given MIME type.
+     * Returns the matching meta type of a given MIME type.
      *
-     * @returns Variant type.
+     * @returns Meta type.
      * @param strMIMEType               MIME type to retrieve variant type for.
      */
-    static QVariant::Type getVariantType(const QString &strMIMEType);
+    static QMetaType::Type getMetaType(const QString &strMIMEType);
 
     /**
      * Fills a QVariant with data according to the given type and data.
@@ -117,7 +129,7 @@ public:
      * @param   vaData                  Variant holding the transformed result.
      *                                  Note: The variant's type might be different from the input vaType!
      */
-    static int getDataAsVariant(const QVector<uint8_t> &vecData, const QString &strMIMEType, QVariant::Type vaType, QVariant &vaData);
+    static int getDataAsVariant(const QVector<uint8_t> &vecData, const QString &strMIMEType, QMetaType::Type vaType, QVariant &vaData);
     /** @}  */
 
 protected:
@@ -136,5 +148,5 @@ protected:
     mutable State     m_enmState;
 };
 
-#endif /* ___UIDnDMIMEData_h___ */
+#endif /* !FEQT_INCLUDED_SRC_runtime_UIDnDMIMEData_h */
 
