@@ -4,49 +4,56 @@
  */
 
 /*
- * Copyright (C) 2010-2016 Oracle Corporation
+ * Copyright (C) 2010-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
-#ifdef VBOX_WITH_PRECOMPILED_HEADERS
-# include <precomp.h>
-#else  /* !VBOX_WITH_PRECOMPILED_HEADERS */
-
 /* Qt includes: */
-# include <QTimer>
+#include <QTimer>
 
 /* GUI includes: */
-# include "VBoxGlobal.h"
-# include "UIDesktopWidgetWatchdog.h"
-# include "UIMessageCenter.h"
-# include "UISession.h"
-# include "UIActionPoolRuntime.h"
-# include "UIMachineLogicFullscreen.h"
-# include "UIMachineWindowFullscreen.h"
-# include "UIMultiScreenLayout.h"
-# include "UIShortcutPool.h"
-# include "UIMachineView.h"
-# include "QIMenu.h"
-# ifdef VBOX_WS_MAC
-#  include "UICocoaApplication.h"
-#  include "UIExtraDataManager.h"
-#  include "VBoxUtils.h"
-#  include "UIFrameBuffer.h"
-#  include <Carbon/Carbon.h>
-# endif /* VBOX_WS_MAC */
+#include "QIMenu.h"
+#include "UIActionPoolRuntime.h"
+#include "UICommon.h"
+#include "UIDesktopWidgetWatchdog.h"
+#include "UILoggingDefs.h"
+#include "UIMachine.h"
+#include "UIMachineLogicFullscreen.h"
+#include "UIMachineView.h"
+#include "UIMachineWindowFullscreen.h"
+#include "UIMessageCenter.h"
+#include "UIMultiScreenLayout.h"
+#include "UIShortcutPool.h"
+#ifdef VBOX_WS_MAC
+# include "UICocoaApplication.h"
+# include "UIExtraDataManager.h"
+# include "VBoxUtils.h"
+# include <Carbon/Carbon.h>
+#endif /* VBOX_WS_MAC */
 
-#endif /* !VBOX_WITH_PRECOMPILED_HEADERS */
+/* COM includes: */
+#include "CGraphicsAdapter.h"
 
 
-UIMachineLogicFullscreen::UIMachineLogicFullscreen(QObject *pParent, UISession *pSession)
-    : UIMachineLogic(pParent, pSession, UIVisualStateType_Fullscreen)
+UIMachineLogicFullscreen::UIMachineLogicFullscreen(UIMachine *pMachine)
+    : UIMachineLogic(pMachine)
     , m_pPopupMenu(0)
 #ifdef VBOX_WS_MAC
     , m_fScreensHaveSeparateSpaces(darwinScreensHaveSeparateSpaces())
@@ -54,26 +61,36 @@ UIMachineLogicFullscreen::UIMachineLogicFullscreen(QObject *pParent, UISession *
 {
     /* Create multiscreen layout: */
     m_pScreenLayout = new UIMultiScreenLayout(this);
-    actionPool()->toRuntime()->setMultiScreenLayout(m_pScreenLayout);
 }
 
 UIMachineLogicFullscreen::~UIMachineLogicFullscreen()
 {
     /* Delete multiscreen layout: */
-    actionPool()->toRuntime()->unsetMultiScreenLayout(m_pScreenLayout);
     delete m_pScreenLayout;
 }
 
+int UIMachineLogicFullscreen::hostScreenForGuestScreen(int iScreenId) const
+{
+    return m_pScreenLayout->hostScreenForGuestScreen(iScreenId);
+}
+
+bool UIMachineLogicFullscreen::hasHostScreenForGuestScreen(int iScreenId) const
+{
+    return m_pScreenLayout->hasHostScreenForGuestScreen(iScreenId);
+}
+
 bool UIMachineLogicFullscreen::checkAvailability()
 {
     /* Check if there is enough physical memory to enter fullscreen: */
-    if (uisession()->isGuestSupportsGraphics())
+    if (uimachine()->isGuestSupportsGraphics())
     {
-        quint64 availBits = machine().GetVRAMSize() /* VRAM */ * _1M /* MiB to bytes */ * 8 /* to bits */;
-        quint64 usedBits = m_pScreenLayout->memoryRequirements();
-        if (availBits < usedBits)
+        ulong uVRAMSize = 0;
+        uimachine()->acquireVRAMSize(uVRAMSize);
+        quint64 uAvailBits = uVRAMSize * _1M /* MiB to bytes */ * 8 /* to bits */;
+        quint64 uUsedBits = m_pScreenLayout->memoryRequirements();
+        if (uAvailBits < uUsedBits)
         {
-            if (!msgCenter().cannotEnterFullscreenMode(0, 0, 0, (((usedBits + 7) / 8 + _1M - 1) / _1M) * _1M))
+            if (!msgCenter().cannotEnterFullscreenMode(0, 0, 0, (((uUsedBits + 7) / 8 + _1M - 1) / _1M) * _1M))
                 return false;
         }
     }
@@ -82,7 +99,7 @@ bool UIMachineLogicFullscreen::checkAvailability()
     const UIShortcut &shortcut =
             gShortcutPool->shortcut(actionPool()->shortcutsExtraDataID(),
                                     actionPool()->action(UIActionIndexRT_M_View_T_Fullscreen)->shortcutExtraDataID());
-    const QString strHotKey = QString("Host+%1").arg(shortcut.toString());
+    const QString strHotKey = QString("Host+%1").arg(shortcut.primaryToPortableText());
     if (!msgCenter().confirmGoingFullscreen(strHotKey))
         return false;
 
@@ -93,8 +110,7 @@ Qt::WindowFlags UIMachineLogicFullscreen::windowFlags(ulong uScreenId) const
 {
     Q_UNUSED(uScreenId);
 #ifdef VBOX_WS_MAC
-    return vboxGlobal().osRelease() <= MacOSXRelease_Lion ? Qt::FramelessWindowHint :
-           uScreenId == 0 || screensHaveSeparateSpaces() ? Qt::Window : Qt::FramelessWindowHint;
+    return uScreenId == 0 || screensHaveSeparateSpaces() ? Qt::Window : Qt::FramelessWindowHint;
 #else /* !VBOX_WS_MAC */
     return Qt::FramelessWindowHint;
 #endif /* !VBOX_WS_MAC */
@@ -108,15 +124,8 @@ void UIMachineLogicFullscreen::adjustMachineWindowsGeometry()
     m_pScreenLayout->rebuild();
 
 #ifdef VBOX_WS_MAC
-    /* For Lion and previous: */
-    if (vboxGlobal().osRelease() <= MacOSXRelease_Lion)
-    {
-        /* Make sure all machine-window(s) have proper geometry: */
-        foreach (UIMachineWindow *pMachineWindow, machineWindows())
-            pMachineWindow->showInNecessaryMode();
-    }
-    /* For ML and next revalidate native fullscreen: */
-    else revalidateNativeFullScreen();
+    /* Revalidate native fullscreen: */
+    revalidateNativeFullScreen();
 #else /* !VBOX_WS_MAC */
     /* Make sure all machine-window(s) have proper geometry: */
     foreach (UIMachineWindow *pMachineWindow, machineWindows())
@@ -124,22 +133,9 @@ void UIMachineLogicFullscreen::adjustMachineWindowsGeometry()
 #endif /* !VBOX_WS_MAC */
 }
 
-int UIMachineLogicFullscreen::hostScreenForGuestScreen(int iScreenId) const
-{
-    return m_pScreenLayout->hostScreenForGuestScreen(iScreenId);
-}
-
-bool UIMachineLogicFullscreen::hasHostScreenForGuestScreen(int iScreenId) const
-{
-    return m_pScreenLayout->hasHostScreenForGuestScreen(iScreenId);
-}
-
 #ifdef RT_OS_DARWIN
 void UIMachineLogicFullscreen::sltHandleNativeFullscreenWillEnter()
 {
-    /* Make sure this method is only used for ML and next: */
-    AssertReturnVoid(vboxGlobal().osRelease() > MacOSXRelease_Lion);
-
     /* Get sender machine-window: */
     UIMachineWindow *pMachineWindow = qobject_cast<UIMachineWindow*>(sender());
     AssertPtrReturnVoid(pMachineWindow);
@@ -150,9 +146,6 @@ void UIMachineLogicFullscreen::sltHandleNativeFullscreenWillEnter()
 
 void UIMachineLogicFullscreen::sltHandleNativeFullscreenDidEnter()
 {
-    /* Make sure this method is only used for ML and next: */
-    AssertReturnVoid(vboxGlobal().osRelease() > MacOSXRelease_Lion);
-
     /* Get sender machine-window: */
     UIMachineWindow *pMachineWindow = qobject_cast<UIMachineWindow*>(sender());
     AssertPtrReturnVoid(pMachineWindow);
@@ -172,9 +165,6 @@ void UIMachineLogicFullscreen::sltHandleNativeFullscreenDidEnter()
 
 void UIMachineLogicFullscreen::sltHandleNativeFullscreenWillExit()
 {
-    /* Make sure this method is only used for ML and next: */
-    AssertReturnVoid(vboxGlobal().osRelease() > MacOSXRelease_Lion);
-
     /* Get sender machine-window: */
     UIMachineWindow *pMachineWindow = qobject_cast<UIMachineWindow*>(sender());
     AssertPtrReturnVoid(pMachineWindow);
@@ -185,9 +175,6 @@ void UIMachineLogicFullscreen::sltHandleNativeFullscreenWillExit()
 
 void UIMachineLogicFullscreen::sltHandleNativeFullscreenDidExit()
 {
-    /* Make sure this method is only used for ML and next: */
-    AssertReturnVoid(vboxGlobal().osRelease() > MacOSXRelease_Lion);
-
     /* Get sender machine-window: */
     UIMachineWindow *pMachineWindow = qobject_cast<UIMachineWindow*>(sender());
     AssertPtrReturnVoid(pMachineWindow);
@@ -239,20 +226,17 @@ void UIMachineLogicFullscreen::sltHandleNativeFullscreenDidExit()
                     (int)pMachineWindow->screenId()));
 
             /* Change visual-state to requested: */
-            UIVisualStateType type = uisession()->requestedVisualState();
+            UIVisualStateType type = uimachine()->requestedVisualState();
             if (type == UIVisualStateType_Invalid)
                 type = UIVisualStateType_Normal;
-            uisession()->setRequestedVisualState(UIVisualStateType_Invalid);
-            uisession()->changeVisualState(type);
+            uimachine()->setRequestedVisualState(UIVisualStateType_Invalid);
+            uimachine()->asyncChangeVisualState(type);
         }
     }
 }
 
 void UIMachineLogicFullscreen::sltHandleNativeFullscreenFailToEnter()
 {
-    /* Make sure this method is only used for ML and next: */
-    AssertReturnVoid(vboxGlobal().osRelease() > MacOSXRelease_Lion);
-
     /* Get sender machine-window: */
     UIMachineWindow *pMachineWindow = qobject_cast<UIMachineWindow*>(sender());
     AssertReturnVoid(pMachineWindow);
@@ -279,82 +263,61 @@ void UIMachineLogicFullscreen::sltHandleNativeFullscreenFailToEnter()
                 (int)pMachineWindow->screenId()));
 
         /* Ask session to change 'fullscreen' mode to 'normal': */
-        uisession()->setRequestedVisualState(UIVisualStateType_Normal);
+        uimachine()->setRequestedVisualState(UIVisualStateType_Normal);
 
-        /* If session already initialized => push mode-change directly: */
-        if (uisession()->isInitialized())
+        /* If machine UI already initialized => push mode-change directly: */
+        if (uimachine()->isInitialized())
             sltCheckForRequestedVisualStateType();
     }
 }
 
 void UIMachineLogicFullscreen::sltChangeVisualStateToNormal()
 {
-    /* Base-class handling for Lion and previous: */
-    if (vboxGlobal().osRelease() <= MacOSXRelease_Lion)
-        UIMachineLogic::sltChangeVisualStateToNormal();
-    /* Special handling for ML and next: */
-    else
-    {
-        /* Request 'normal' (window) visual-state: */
-        uisession()->setRequestedVisualState(UIVisualStateType_Normal);
-        /* Ask window(s) to exit 'fullscreen' mode: */
-        emit sigNotifyAboutNativeFullscreenShouldBeExited();
-    }
+    /* Request 'normal' (window) visual-state: */
+    uimachine()->setRequestedVisualState(UIVisualStateType_Normal);
+    /* Ask window(s) to exit 'fullscreen' mode: */
+    emit sigNotifyAboutNativeFullscreenShouldBeExited();
 }
 
 void UIMachineLogicFullscreen::sltChangeVisualStateToSeamless()
 {
-    /* Base-class handling for Lion and previous: */
-    if (vboxGlobal().osRelease() <= MacOSXRelease_Lion)
-        UIMachineLogic::sltChangeVisualStateToSeamless();
-    /* Special handling for ML and next: */
-    else
-    {
-        /* Request 'seamless' visual-state: */
-        uisession()->setRequestedVisualState(UIVisualStateType_Seamless);
-        /* Ask window(s) to exit 'fullscreen' mode: */
-        emit sigNotifyAboutNativeFullscreenShouldBeExited();
-    }
+    /* Request 'seamless' visual-state: */
+    uimachine()->setRequestedVisualState(UIVisualStateType_Seamless);
+    /* Ask window(s) to exit 'fullscreen' mode: */
+    emit sigNotifyAboutNativeFullscreenShouldBeExited();
 }
 
 void UIMachineLogicFullscreen::sltChangeVisualStateToScale()
 {
-    /* Base-class handling for Lion and previous: */
-    if (vboxGlobal().osRelease() <= MacOSXRelease_Lion)
-        UIMachineLogic::sltChangeVisualStateToScale();
-    /* Special handling for ML and next: */
-    else
-    {
-        /* Request 'scale' visual-state: */
-        uisession()->setRequestedVisualState(UIVisualStateType_Scale);
-        /* Ask window(s) to exit 'fullscreen' mode: */
-        emit sigNotifyAboutNativeFullscreenShouldBeExited();
-    }
+    /* Request 'scale' visual-state: */
+    uimachine()->setRequestedVisualState(UIVisualStateType_Scale);
+    /* Ask window(s) to exit 'fullscreen' mode: */
+    emit sigNotifyAboutNativeFullscreenShouldBeExited();
 }
 
 void UIMachineLogicFullscreen::sltCheckForRequestedVisualStateType()
 {
     LogRel(("GUI: UIMachineLogicFullscreen::sltCheckForRequestedVisualStateType: Requested-state=%d, Machine-state=%d\n",
-            uisession()->requestedVisualState(), uisession()->machineState()));
+            uimachine()->requestedVisualState(), uimachine()->machineState()));
 
     /* Do not try to change visual-state type if machine was not started yet: */
-    if (!uisession()->isRunning() && !uisession()->isPaused())
+    if (!uimachine()->isRunning() && !uimachine()->isPaused())
         return;
 
     /* Do not try to change visual-state type in 'manual override' mode: */
-    if (isManualOverrideMode())
+    if (uimachine()->isManualOverrideMode())
         return;
 
     /* Check requested visual-state types: */
-    switch (uisession()->requestedVisualState())
+    switch (uimachine()->requestedVisualState())
     {
         /* If 'normal' visual-state type is requested: */
         case UIVisualStateType_Normal:
         {
             LogRel(("GUI: UIMachineLogicFullscreen::sltCheckForRequestedVisualStateType: "
                     "Going 'normal' as requested...\n"));
-            uisession()->setRequestedVisualState(UIVisualStateType_Invalid);
-            uisession()->changeVisualState(UIVisualStateType_Normal);
+            uimachine()->setRequestedVisualState(UIVisualStateType_Invalid);
+            uimachine()->asyncChangeVisualState(UIVisualStateType_Normal);
             break;
         }
         default:
@@ -369,14 +332,14 @@ void UIMachineLogicFullscreen::sltMachineStateChanged()
     UIMachineLogic::sltMachineStateChanged();
 
     /* If machine-state changed from 'paused' to 'running': */
-    if (uisession()->isRunning() && uisession()->wasPaused())
+    if (uimachine()->isRunning() && uimachine()->wasPaused())
     {
         LogRel(("GUI: UIMachineLogicFullscreen::sltMachineStateChanged:"
                 "Machine-state changed from 'paused' to 'running': "
                 "Adjust machine-window geometry...\n"));
 
         /* Make sure further code will be called just once: */
-        uisession()->forgetPreviousMachineState();
+        uimachine()->forgetPreviousMachineState();
         /* Adjust machine-window geometry if necessary: */
         adjustMachineWindowsGeometry();
     }
@@ -397,15 +360,8 @@ void UIMachineLogicFullscreen::sltScreenLayoutChanged()
     LogRel(("GUI: UIMachineLogicFullscreen::sltScreenLayoutChanged: Multi-screen layout changed\n"));
 
 #ifdef VBOX_WS_MAC
-    /* For Lion and previous: */
-    if (vboxGlobal().osRelease() <= MacOSXRelease_Lion)
-    {
-        /* Make sure all machine-window(s) have proper geometry: */
-        foreach (UIMachineWindow *pMachineWindow, machineWindows())
-            pMachineWindow->showInNecessaryMode();
-    }
-    /* Revalidate native fullscreen for ML and next: */
-    else revalidateNativeFullScreen();
+    /* Revalidate native fullscreen: */
+    revalidateNativeFullScreen();
 #else /* !VBOX_WS_MAC */
     /* Make sure all machine-window(s) have proper geometry: */
     foreach (UIMachineWindow *pMachineWindow, machineWindows())
@@ -421,11 +377,9 @@ void UIMachineLogicFullscreen::sltGuestMonitorChange(KGuestMonitorChangedEventTy
     m_pScreenLayout->rebuild();
 
 #ifdef VBOX_WS_MAC
-    /* Call to base-class for Lion and previous: */
-    if (vboxGlobal().osRelease() <= MacOSXRelease_Lion)
-        UIMachineLogic::sltGuestMonitorChange(changeType, uScreenId, screenGeo);
-    /* Revalidate native fullscreen for ML and next: */
-    else revalidateNativeFullScreen();
+    /* Revalidate native fullscreen: */
+    RT_NOREF(changeType, uScreenId, screenGeo);
+    revalidateNativeFullScreen();
 #else /* !VBOX_WS_MAC */
     /* Call to base-class: */
     UIMachineLogic::sltGuestMonitorChange(changeType, uScreenId, screenGeo);
@@ -440,11 +394,8 @@ void UIMachineLogicFullscreen::sltHostScreenCountChange()
     m_pScreenLayout->rebuild();
 
 #ifdef VBOX_WS_MAC
-    /* Call to base-class for Lion and previous: */
-    if (vboxGlobal().osRelease() <= MacOSXRelease_Lion)
-        UIMachineLogic::sltHostScreenCountChange();
-    /* Revalidate native fullscreen for ML and next: */
-    else revalidateNativeFullScreen();
+    /* Revalidate native fullscreen: */
+    revalidateNativeFullScreen();
 #else /* !VBOX_WS_MAC */
     /* Call to base-class: */
     UIMachineLogic::sltHostScreenCountChange();
@@ -500,12 +451,12 @@ void UIMachineLogicFullscreen::prepareActionConnections()
     UIMachineLogic::prepareActionConnections();
 
     /* Prepare 'View' actions connections: */
-    connect(actionPool()->action(UIActionIndexRT_M_View_T_Fullscreen), SIGNAL(triggered(bool)),
-            this, SLOT(sltChangeVisualStateToNormal()));
-    connect(actionPool()->action(UIActionIndexRT_M_View_T_Seamless), SIGNAL(triggered(bool)),
-            this, SLOT(sltChangeVisualStateToSeamless()));
-    connect(actionPool()->action(UIActionIndexRT_M_View_T_Scale), SIGNAL(triggered(bool)),
-            this, SLOT(sltChangeVisualStateToScale()));
+    connect(actionPool()->action(UIActionIndexRT_M_View_T_Fullscreen), &UIAction::triggered,
+            this, &UIMachineLogicFullscreen::sltChangeVisualStateToNormal);
+    connect(actionPool()->action(UIActionIndexRT_M_View_T_Seamless), &UIAction::triggered,
+            this, &UIMachineLogicFullscreen::sltChangeVisualStateToSeamless);
+    connect(actionPool()->action(UIActionIndexRT_M_View_T_Scale), &UIAction::triggered,
+            this, &UIMachineLogicFullscreen::sltChangeVisualStateToScale);
 }
 
 void UIMachineLogicFullscreen::prepareMachineWindows()
@@ -515,15 +466,11 @@ void UIMachineLogicFullscreen::prepareMachineWindows()
         return;
 
 #ifdef VBOX_WS_MAC
-    /* For ML and next: */
-    if (vboxGlobal().osRelease() > MacOSXRelease_Lion)
-    {
-        /* Register to native notifications: */
-        UICocoaApplication::instance()->registerToNotificationOfWorkspace("NSWorkspaceDidActivateApplicationNotification", this,
-                                                                          UIMachineLogicFullscreen::nativeHandlerForApplicationActivation);
-        UICocoaApplication::instance()->registerToNotificationOfWorkspace("NSWorkspaceActiveSpaceDidChangeNotification", this,
-                                                                          UIMachineLogicFullscreen::nativeHandlerForActiveSpaceChange);
-    }
+    /* Register to native notifications: */
+    UICocoaApplication::instance()->registerToNotificationOfWorkspace("NSWorkspaceDidActivateApplicationNotification", this,
+                                                                      UIMachineLogicFullscreen::nativeHandlerForApplicationActivation);
+    UICocoaApplication::instance()->registerToNotificationOfWorkspace("NSWorkspaceActiveSpaceDidChangeNotification", this,
+                                                                      UIMachineLogicFullscreen::nativeHandlerForActiveSpaceChange);
 
     /* We have to make sure that we are getting the front most process.
      * This is necessary for Qt versions > 4.3.3: */
@@ -533,59 +480,56 @@ void UIMachineLogicFullscreen::prepareMachineWindows()
     /* Update the multi-screen layout: */
     m_pScreenLayout->update();
 
+    /* Acquire monitor count: */
+    ulong cMonitorCount = 0;
+    uimachine()->acquireMonitorCount(cMonitorCount);
+
     /* Create machine-window(s): */
-    for (uint cScreenId = 0; cScreenId < machine().GetMonitorCount(); ++cScreenId)
+    for (uint cScreenId = 0; cScreenId < cMonitorCount; ++cScreenId)
         addMachineWindow(UIMachineWindow::create(this, cScreenId));
 
-    /* Listen for frame-buffer resize: */
-    foreach (UIMachineWindow *pMachineWindow, machineWindows())
-        connect(pMachineWindow, SIGNAL(sigFrameBufferResize()),
-                this, SIGNAL(sigFrameBufferResize()));
-    emit sigFrameBufferResize();
-
     /* Connect multi-screen layout change handler: */
-    connect(m_pScreenLayout, SIGNAL(sigScreenLayoutChange()),
-            this, SLOT(sltScreenLayoutChanged()));
+    connect(m_pScreenLayout, &UIMultiScreenLayout::sigScreenLayoutChange,
+            this, &UIMachineLogicFullscreen::sltScreenLayoutChanged);
 
 #ifdef VBOX_WS_MAC
-    /* For ML and next: */
-    if (vboxGlobal().osRelease() > MacOSXRelease_Lion)
+    /* Enable native fullscreen support: */
+    foreach (UIMachineWindow *pMachineWindow, machineWindows())
     {
-        /* Enable native fullscreen support: */
-        foreach (UIMachineWindow *pMachineWindow, machineWindows())
-        {
-            /* Logic => window signals: */
-            connect(this, SIGNAL(sigNotifyAboutNativeFullscreenShouldBeEntered(UIMachineWindow*)),
-                    pMachineWindow, SLOT(sltEnterNativeFullscreen(UIMachineWindow*)));
-            connect(this, SIGNAL(sigNotifyAboutNativeFullscreenShouldBeExited(UIMachineWindow*)),
-                    pMachineWindow, SLOT(sltExitNativeFullscreen(UIMachineWindow*)));
-            /* Window => logic signals: */
-            connect(pMachineWindow, SIGNAL(sigNotifyAboutNativeFullscreenWillEnter()),
-                    this, SLOT(sltHandleNativeFullscreenWillEnter()),
-                    Qt::QueuedConnection);
-            connect(pMachineWindow, SIGNAL(sigNotifyAboutNativeFullscreenDidEnter()),
-                    this, SLOT(sltHandleNativeFullscreenDidEnter()),
-                    Qt::QueuedConnection);
-            connect(pMachineWindow, SIGNAL(sigNotifyAboutNativeFullscreenWillExit()),
-                    this, SLOT(sltHandleNativeFullscreenWillExit()),
-                    Qt::QueuedConnection);
-            connect(pMachineWindow, SIGNAL(sigNotifyAboutNativeFullscreenDidExit()),
-                    this, SLOT(sltHandleNativeFullscreenDidExit()),
-                    Qt::QueuedConnection);
-            connect(pMachineWindow, SIGNAL(sigNotifyAboutNativeFullscreenFailToEnter()),
-                    this, SLOT(sltHandleNativeFullscreenFailToEnter()),
-                    Qt::QueuedConnection);
-        }
-        /* Revalidate native fullscreen: */
-        revalidateNativeFullScreen();
+        UIMachineWindowFullscreen *pMachineWindowFullscreen = qobject_cast<UIMachineWindowFullscreen*>(pMachineWindow);
+        if (!pMachineWindow)
+            continue;
+        /* Logic => window signals: */
+        connect(this, &UIMachineLogicFullscreen::sigNotifyAboutNativeFullscreenShouldBeEntered,
+                pMachineWindowFullscreen, &UIMachineWindowFullscreen::sltEnterNativeFullscreen);
+        connect(this, &UIMachineLogicFullscreen::sigNotifyAboutNativeFullscreenShouldBeExited,
+                pMachineWindowFullscreen, &UIMachineWindowFullscreen::sltExitNativeFullscreen);
+        /* Window => logic signals: */
+        connect(pMachineWindowFullscreen, &UIMachineWindowFullscreen::sigNotifyAboutNativeFullscreenWillEnter,
+                this, &UIMachineLogicFullscreen::sltHandleNativeFullscreenWillEnter,
+                 Qt::QueuedConnection);
+        connect(pMachineWindowFullscreen, &UIMachineWindowFullscreen::sigNotifyAboutNativeFullscreenDidEnter,
+                this, &UIMachineLogicFullscreen::sltHandleNativeFullscreenDidEnter,
+                Qt::QueuedConnection);
+        connect(pMachineWindowFullscreen, &UIMachineWindowFullscreen::sigNotifyAboutNativeFullscreenWillExit,
+                this, &UIMachineLogicFullscreen::sltHandleNativeFullscreenWillExit,
+                Qt::QueuedConnection);
+        connect(pMachineWindowFullscreen, &UIMachineWindowFullscreen::sigNotifyAboutNativeFullscreenDidExit,
+                this, &UIMachineLogicFullscreen::sltHandleNativeFullscreenDidExit,
+                Qt::QueuedConnection);
+        connect(pMachineWindowFullscreen, &UIMachineWindowFullscreen::sigNotifyAboutNativeFullscreenFailToEnter,
+                this, &UIMachineLogicFullscreen::sltHandleNativeFullscreenFailToEnter,
+                Qt::QueuedConnection);
     }
+    /* Revalidate native fullscreen: */
+    revalidateNativeFullScreen();
 #endif /* VBOX_WS_MAC */
 
     /* Mark machine-window(s) created: */
     setMachineWindowsCreated(true);
 
-#ifdef VBOX_WS_X11
-    switch (vboxGlobal().typeOfWindowManager())
+#ifdef VBOX_WS_NIX
+    switch (uiCommon().typeOfWindowManager())
     {
         case X11WMType_GNOMEShell:
         case X11WMType_Mutter:
@@ -600,7 +544,7 @@ void UIMachineLogicFullscreen::prepareMachineWindows()
         default:
             break;
     }
-#endif /* VBOX_WS_X11 */
+#endif /* VBOX_WS_NIX */
 }
 
 void UIMachineLogicFullscreen::prepareMenu()
@@ -629,13 +573,9 @@ void UIMachineLogicFullscreen::cleanupMachineWindows()
         return;
 
 #ifdef VBOX_WS_MAC
-    /* For ML and next: */
-    if (vboxGlobal().osRelease() > MacOSXRelease_Lion)
-    {
-        /* Unregister from native notifications: */
-        UICocoaApplication::instance()->unregisterFromNotificationOfWorkspace("NSWorkspaceDidActivateApplicationNotification", this);
-        UICocoaApplication::instance()->unregisterFromNotificationOfWorkspace("NSWorkspaceActiveSpaceDidChangeNotification", this);
-    }
+    /* Unregister from native notifications: */
+    UICocoaApplication::instance()->unregisterFromNotificationOfWorkspace("NSWorkspaceDidActivateApplicationNotification", this);
+    UICocoaApplication::instance()->unregisterFromNotificationOfWorkspace("NSWorkspaceActiveSpaceDidChangeNotification", this);
 #endif/* VBOX_WS_MAC */
 
     /* Mark machine-window(s) destroyed: */
@@ -649,12 +589,12 @@ void UIMachineLogicFullscreen::cleanupMachineWindows()
 void UIMachineLogicFullscreen::cleanupActionConnections()
 {
     /* "View" actions disconnections: */
-    disconnect(actionPool()->action(UIActionIndexRT_M_View_T_Fullscreen), SIGNAL(triggered(bool)),
-               this, SLOT(sltChangeVisualStateToNormal()));
-    disconnect(actionPool()->action(UIActionIndexRT_M_View_T_Seamless), SIGNAL(triggered(bool)),
-               this, SLOT(sltChangeVisualStateToSeamless()));
-    disconnect(actionPool()->action(UIActionIndexRT_M_View_T_Scale), SIGNAL(triggered(bool)),
-               this, SLOT(sltChangeVisualStateToScale()));
+    disconnect(actionPool()->action(UIActionIndexRT_M_View_T_Fullscreen), &QAction::triggered,
+               this, &UIMachineLogicFullscreen::sltChangeVisualStateToNormal);
+    disconnect(actionPool()->action(UIActionIndexRT_M_View_T_Seamless), &QAction::triggered,
+               this, &UIMachineLogicFullscreen::sltChangeVisualStateToSeamless);
+    disconnect(actionPool()->action(UIActionIndexRT_M_View_T_Scale), &QAction::triggered,
+               this, &UIMachineLogicFullscreen::sltChangeVisualStateToScale);
 
     /* Call to base-class: */
     UIMachineLogic::cleanupActionConnections();
@@ -738,7 +678,7 @@ void UIMachineLogicFullscreen::revalidateNativeFullScreen(UIMachineWindow *pMach
             /* If that window
              * 1. should really be shown and
              * 2. is mapped to some host-screen: */
-            if (   uisession()->isScreenVisible(uScreenID)
+            if (   uimachine()->isScreenVisible(uScreenID)
                 && hasHostScreenForGuestScreen(uScreenID))
             {
                 LogRel(("GUI: UIMachineLogicFullscreen::revalidateNativeFullScreen: "
@@ -766,15 +706,15 @@ void UIMachineLogicFullscreen::revalidateNativeFullScreen(UIMachineWindow *pMach
         {
             /* Variables to compare: */
             const int iWantedHostScreenIndex = hostScreenForGuestScreen((int)uScreenID);
-            const int iCurrentHostScreenIndex = gpDesktop->screenNumber(pMachineWindow);
-            const QSize frameBufferSize((int)uisession()->frameBuffer(uScreenID)->width(), (int)uisession()->frameBuffer(uScreenID)->height());
-            const QSize screenSize = gpDesktop->screenGeometry(iWantedHostScreenIndex).size();
+            const int iCurrentHostScreenIndex = UIDesktopWidgetWatchdog::screenNumber(pMachineWindow);
+            const QSize guestScreenSize = uimachine()->guestScreenSize(uScreenID);
+            const QSize hostScreenSize = gpDesktop->screenGeometry(iWantedHostScreenIndex).size();
 
             /* If that window
              * 1. shouldn't really be shown or
              * 2. isn't mapped to some host-screen or
              * 3. should be located on another host-screen than currently. */
-            if (   !uisession()->isScreenVisible(uScreenID)
+            if (   !uimachine()->isScreenVisible(uScreenID)
                 || !hasHostScreenForGuestScreen(uScreenID)
                 || iWantedHostScreenIndex != iCurrentHostScreenIndex)
             {
@@ -790,8 +730,8 @@ void UIMachineLogicFullscreen::revalidateNativeFullScreen(UIMachineWindow *pMach
             }
 
             /* If that window
-             * 1. have another frame-buffer size than actually should. */
-            else if (frameBufferSize != screenSize)
+             * 1. have another size than actually should. */
+            else if (guestScreenSize != hostScreenSize)
             {
                 LogRel(("GUI: UIMachineLogicFullscreen::revalidateNativeFullScreen: "
                         "Ask machine-window #%d to adjust guest geometry\n", (int)uScreenID));
@@ -883,4 +823,3 @@ void UIMachineLogicFullscreen::nativeHandlerForActiveSpaceChange(const QMap<QStr
             revalidateNativeFullScreen(pMachineWindow);
 }
 #endif /* VBOX_WS_MAC */
-
