@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2010-2023 Oracle and/or its affiliates.
+ * Copyright (C) 2010-2024 Oracle and/or its affiliates.
  *
  * This file is part of VirtualBox base platform packages, as
  * available from https://www.virtualbox.org.
@@ -34,10 +34,10 @@
 /* GUI includes: */
 #include "UIMachineWindow.h"
 
-#if defined(VBOX_WS_WIN) || defined(VBOX_WS_X11)
+#if defined(VBOX_WS_WIN) || defined(VBOX_WS_NIX)
 /* Forward declarations: */
 class UIMiniToolBar;
-#endif /* VBOX_WS_WIN || VBOX_WS_X11 */
+#endif /* VBOX_WS_WIN || VBOX_WS_NIX */
 
 /** UIMachineWindow subclass used as full-screen machine window implementation. */
 class UIMachineWindowFullscreen : public UIMachineWindow
@@ -67,16 +67,16 @@ protected:
 
 #ifdef VBOX_WS_MAC
     /** Mac OS X: Handles native notifications @a strNativeNotificationName for 'fullscreen' window. */
-    void handleNativeNotification(const QString &strNativeNotificationName);
+    void handleNativeNotification(const QString &strNativeNotificationName)  RT_OVERRIDE RT_FINAL;
     /** Mac OS X: Returns whether window is in 'fullscreen' transition. */
     bool isInFullscreenTransition() const { return m_fIsInFullscreenTransition; }
 #endif /* VBOX_WS_MAC */
 
 private slots:
 
-#if defined(VBOX_WS_WIN) || defined(VBOX_WS_X11)
+#if defined(VBOX_WS_WIN) || defined(VBOX_WS_NIX)
     /** Handles machine state change event. */
-    void sltMachineStateChanged();
+    void sltMachineStateChanged() RT_OVERRIDE RT_FINAL;
 
     /** Revokes window activation. */
     void sltRevokeWindowActivation();
@@ -84,7 +84,7 @@ private slots:
     /** Handles signal about mini-toolbar auto-hide toggled.
       * @param  fEnabled  Brings whether auto-hide is enabled. */
     void sltHandleMiniToolBarAutoHideToggled(bool fEnabled);
-#endif /* VBOX_WS_WIN || VBOX_WS_X11 */
+#endif /* VBOX_WS_WIN || VBOX_WS_NIX */
 
 #ifdef RT_OS_DARWIN
     /** Mac OS X: Commands @a pMachineWindow to enter native 'fullscreen' mode if possible. */
@@ -99,36 +99,36 @@ private slots:
 private:
 
     /** Prepare notification-center routine. */
-    void prepareNotificationCenter();
+    void prepareNotificationCenter()  RT_OVERRIDE RT_FINAL;
     /** Prepare visual-state routine. */
-    void prepareVisualState();
-#if defined(VBOX_WS_WIN) || defined(VBOX_WS_X11)
+    void prepareVisualState() RT_OVERRIDE RT_FINAL;
+#if defined(VBOX_WS_WIN) || defined(VBOX_WS_NIX)
     /** Prepare mini-toolbar routine. */
     void prepareMiniToolbar();
-#endif /* VBOX_WS_WIN || VBOX_WS_X11 */
+#endif /* VBOX_WS_WIN || VBOX_WS_NIX */
 
-#if defined(VBOX_WS_WIN) || defined(VBOX_WS_X11)
+#if defined(VBOX_WS_WIN) || defined(VBOX_WS_NIX)
     /** Cleanup mini-toolbar routine. */
     void cleanupMiniToolbar();
-#endif /* VBOX_WS_WIN || VBOX_WS_X11 */
+#endif /* VBOX_WS_WIN || VBOX_WS_NIX */
     /** Cleanup visual-state routine. */
-    void cleanupVisualState();
+    void cleanupVisualState() RT_OVERRIDE RT_FINAL;
     /** Cleanup notification-center routine. */
-    void cleanupNotificationCenter();
+    void cleanupNotificationCenter() RT_OVERRIDE RT_FINAL;
 
     /** Updates geometry according to visual-state. */
     void placeOnScreen();
     /** Updates visibility according to visual-state. */
-    void showInNecessaryMode();
+    void showInNecessaryMode() RT_OVERRIDE RT_FINAL;
 
-#if defined(VBOX_WS_WIN) || defined(VBOX_WS_X11)
+#if defined(VBOX_WS_WIN) || defined(VBOX_WS_NIX)
     /** Common update routine. */
-    void updateAppearanceOf(int iElement);
-#endif /* VBOX_WS_WIN || VBOX_WS_X11 */
+    void updateAppearanceOf(int iElement) RT_OVERRIDE RT_FINAL;
+#endif /* VBOX_WS_WIN || VBOX_WS_NIX */
 
-#ifdef VBOX_WS_X11
+#ifdef VBOX_WS_NIX
     /** X11: Handles @a pEvent about state change. */
-    void changeEvent(QEvent *pEvent);
+    void changeEvent(QEvent *pEvent) RT_OVERRIDE RT_FINAL;
 #endif
 
 #ifdef VBOX_WS_WIN
@@ -136,10 +136,10 @@ private:
     void showEvent(QShowEvent *pEvent);
 #endif
 
-#if defined(VBOX_WS_WIN) || defined(VBOX_WS_X11)
+#if defined(VBOX_WS_WIN) || defined(VBOX_WS_NIX)
     /** Holds the mini-toolbar instance. */
     UIMiniToolBar *m_pMiniToolBar;
-#endif /* VBOX_WS_WIN || VBOX_WS_X11 */
+#endif /* VBOX_WS_WIN || VBOX_WS_NIX */
 
 #ifdef VBOX_WS_MAC
     /** Mac OS X: Reflects whether window is in 'fullscreen' transition. */
@@ -151,7 +151,7 @@ private:
     /** Holds whether the window was minimized before became hidden.
       * Used to restore minimized state when the window shown again. */
     bool m_fWasMinimized;
-#ifdef VBOX_WS_X11
+#ifdef VBOX_WS_NIX
     /** X11: Holds whether the window minimization is currently requested.
       * Used to prevent accidentally restoring to full-screen state. */
     bool m_fIsMinimizationRequested;
