@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2009-2019 Oracle Corporation
+ * Copyright (C) 2009-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 /* Qt includes: */
@@ -26,18 +36,18 @@
 
 /* GUI includes: */
 #include "QIFileDialog.h"
-#include "VBoxGlobal.h"
+#include "UICommon.h"
 #include "UILineTextEdit.h"
-
+#include "UITranslationEventListener.h"
 
 ////////////////////////////////////////////////////////////////////////////////
 // UITextEditor
 
 UITextEditor::UITextEditor(QWidget *pParent /* = NULL */)
-  : QIWithRetranslateUI<QIDialog>(pParent)
+  : QIDialog(pParent)
 {
     QVBoxLayout *pMainLayout = new QVBoxLayout(this);
-    pMainLayout->setMargin(12);
+    pMainLayout->setContentsMargins(12, 12, 12, 12);
 
     /* We need a text editor */
     m_pTextEdit = new QTextEdit(this);
@@ -48,15 +58,17 @@ UITextEditor::UITextEditor(QWidget *pParent /* = NULL */)
     m_pButtonBox->addButton(m_pOpenButton, QDialogButtonBox::ActionRole);
     pMainLayout->addWidget(m_pButtonBox);
     /* Connect the buttons so that they are useful */
-    connect(m_pButtonBox, SIGNAL(accepted()),
-            this, SLOT(accept()));
-    connect(m_pButtonBox, SIGNAL(rejected()),
-            this, SLOT(reject()));
-    connect(m_pOpenButton, SIGNAL(clicked()),
-            this, SLOT(open()));
+    connect(m_pButtonBox, &QDialogButtonBox::accepted,
+            this, &UITextEditor::accept);
+    connect(m_pButtonBox, &QDialogButtonBox::rejected,
+            this, &UITextEditor::reject);
+    connect(m_pOpenButton, &QPushButton::clicked,
+            this, &UITextEditor::open);
 
     /* Applying language settings */
-    retranslateUi();
+    sltRetranslateUI();
+    connect(&translationEventListener(), &UITranslationEventListener::sigRetranslateUI,
+            this, &UITextEditor::sltRetranslateUI);
 }
 
 void UITextEditor::setText(const QString& strText)
@@ -69,7 +81,7 @@ QString UITextEditor::text() const
     return m_pTextEdit->toPlainText();
 }
 
-void UITextEditor::retranslateUi()
+void UITextEditor::sltRetranslateUI()
 {
     setWindowTitle(tr("Edit text"));
     m_pOpenButton->setText(tr("&Replace..."));
@@ -78,7 +90,7 @@ void UITextEditor::retranslateUi()
 
 void UITextEditor::open()
 {
-    QString fileName = QIFileDialog::getOpenFileName(vboxGlobal().documentsPath(), tr("Text (*.txt);;All (*.*)"), this, tr("Select a file to open..."));
+    QString fileName = QIFileDialog::getOpenFileName(uiCommon().documentsPath(), tr("Text (*.txt);;All (*.*)"), this, tr("Select a file to open..."));
     if (!fileName.isEmpty())
     {
         QFile file(fileName);
@@ -95,20 +107,22 @@ void UITextEditor::open()
 // UILineTextEdit
 
 UILineTextEdit::UILineTextEdit(QWidget *pParent /* = NULL */)
-  : QIWithRetranslateUI<QPushButton>(pParent)
+  : QPushButton(pParent)
 {
-    connect(this, SIGNAL(clicked()),
-            this, SLOT(edit()));
+    connect(this, &UILineTextEdit::clicked,
+            this, &UILineTextEdit::edit);
 
     /* Don't interpret the Enter Key. */
     setAutoDefault(false);
     setDefault(false);
 
     setFocusPolicy(Qt::StrongFocus);
-    retranslateUi();
+    sltRetranslateUI();
+    connect(&translationEventListener(), &UITranslationEventListener::sigRetranslateUI,
+        this, &UILineTextEdit::sltRetranslateUI);
 }
 
-void UILineTextEdit::retranslateUi()
+void UILineTextEdit::sltRetranslateUI()
 {
     QPushButton::setText(tr("&Edit"));
 }
@@ -124,4 +138,3 @@ void UILineTextEdit::edit()
         emit sigFinished(this);
     }
 }
-
