@@ -4,20 +4,40 @@
  */
 
 /*
- * Copyright (C) 2008-2012 Oracle Corporation
+ * Copyright (C) 2008-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * The contents of this file may alternatively be used under the terms
+ * of the Common Development and Distribution License Version 1.0
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
+ * CDDL are applicable instead of those of the GPL.
+ *
+ * You may elect to license modified versions of this file under the
+ * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #define LOG_GROUP LOG_GROUP_NET_ADP_DRV
 #include "../../../Runtime/r0drv/darwin/the-darwin-kernel.h"
 
@@ -49,16 +69,17 @@ RT_C_DECLS_END
 #include <sys/param.h>
 #include <sys/conf.h>
 #include <miscfs/devfs/devfs.h>
-extern "C" {
+RT_C_DECLS_BEGIN
 #include <net/bpf.h>
-}
+RT_C_DECLS_END
 
 #define VBOXNETADP_OS_SPECFIC 1
 #include "../VBoxNetAdpInternal.h"
 
-/*******************************************************************************
-*   Defined Constants And Macros                                               *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Defined Constants And Macros                                                                                                 *
+*********************************************************************************************************************************/
 /** The maximum number of SG segments.
  * Used to prevent stack overflow and similar bad stuff. */
 #define VBOXNETADP_DARWIN_MAX_SEGS       32
@@ -69,9 +90,10 @@ extern "C" {
 
 #define VBOXNETADP_FROM_IFACE(iface) ((PVBOXNETADP) ifnet_softc(iface))
 
-/*******************************************************************************
-*   Internal Functions                                                         *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Internal Functions                                                                                                           *
+*********************************************************************************************************************************/
 RT_C_DECLS_BEGIN
 static kern_return_t    VBoxNetAdpDarwinStart(struct kmod_info *pKModInfo, void *pvData);
 static kern_return_t    VBoxNetAdpDarwinStop(struct kmod_info *pKModInfo, void *pvData);
@@ -81,9 +103,10 @@ static int VBoxNetAdpDarwinOpen(dev_t Dev, int fFlags, int fDevType, struct proc
 static int VBoxNetAdpDarwinClose(dev_t Dev, int fFlags, int fDevType, struct proc *pProcess);
 static int VBoxNetAdpDarwinIOCtl(dev_t Dev, u_long iCmd, caddr_t pData, int fFlags, struct proc *pProcess);
 
-/*******************************************************************************
-*   Global Variables                                                           *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Global Variables                                                                                                             *
+*********************************************************************************************************************************/
 /**
  * Declare the module stuff.
  */
@@ -92,9 +115,9 @@ extern kern_return_t _start(struct kmod_info *pKModInfo, void *pvData);
 extern kern_return_t _stop(struct kmod_info *pKModInfo, void *pvData);
 
 KMOD_EXPLICIT_DECL(VBoxNetAdp, VBOX_VERSION_STRING, _start, _stop)
-DECLHIDDEN(kmod_start_func_t *) _realmain = VBoxNetAdpDarwinStart;
-DECLHIDDEN(kmod_stop_func_t  *) _antimain = VBoxNetAdpDarwinStop;
-DECLHIDDEN(int)                 _kext_apple_cc = __APPLE_CC__;
+DECL_HIDDEN_DATA(kmod_start_func_t *) _realmain = VBoxNetAdpDarwinStart;
+DECL_HIDDEN_DATA(kmod_stop_func_t  *) _antimain = VBoxNetAdpDarwinStop;
+DECL_HIDDEN_DATA(int)                 _kext_apple_cc = __APPLE_CC__;
 RT_C_DECLS_END
 
 /**
@@ -119,8 +142,8 @@ static struct cdevsw    g_ChDev =
     /*.d_select   = */eno_select,
     /*.d_mmap     = */eno_mmap,
     /*.d_strategy = */eno_strat,
-    /*.d_getc     = */eno_getc,
-    /*.d_putc     = */eno_putc,
+    /*.d_getc     = */(void *)(uintptr_t)&enodev, //eno_getc,
+    /*.d_putc     = */(void *)(uintptr_t)&enodev, //eno_putc,
     /*.d_type     = */0
 };
 
@@ -139,54 +162,20 @@ static void vboxNetAdpDarwinComposeUUID(PVBOXNETADP pThis, PRTUUID pUuid)
 
 static errno_t vboxNetAdpDarwinOutput(ifnet_t pIface, mbuf_t pMBuf)
 {
-    PVBOXNETADP pThis = VBOXNETADP_FROM_IFACE(pIface);
-    Assert(pThis);
-    if (pThis->u.s.nTapMode & BPF_MODE_OUTPUT)
-    {
-        Log2(("vboxnetadp: out len=%d\n%.*Rhxd\n", mbuf_len(pMBuf), 14, mbuf_data(pMBuf)));
-        bpf_tap_out(pIface, DLT_EN10MB, pMBuf, NULL, 0);
-    }
+    /*
+     * We are a dummy interface with all the real work done in
+     * VBoxNetFlt bridged networking filter.  If anything makes it
+     * this far, it must be a broadcast or a packet for an unknown
+     * guest that intnet didn't know where to dispatch.  In that case
+     * we must still do the BPF tap and stats.
+     */
+    bpf_tap_out(pIface, DLT_EN10MB, pMBuf, NULL, 0);
+    ifnet_stat_increment_out(pIface, 1, mbuf_len(pMBuf), 0);
+
     mbuf_freem_list(pMBuf);
     return 0;
 }
 
-static void vboxNetAdpDarwinAttachFamily(PVBOXNETADP pThis, protocol_family_t Family)
-{
-    u_int32_t i;
-    for (i = 0; i < VBOXNETADP_MAX_FAMILIES; i++)
-        if (pThis->u.s.aAttachedFamilies[i] == 0)
-        {
-            pThis->u.s.aAttachedFamilies[i] = Family;
-            break;
-        }
-}
-
-static void vboxNetAdpDarwinDetachFamily(PVBOXNETADP pThis, protocol_family_t Family)
-{
-    u_int32_t i;
-    for (i = 0; i < VBOXNETADP_MAX_FAMILIES; i++)
-        if (pThis->u.s.aAttachedFamilies[i] == Family)
-            pThis->u.s.aAttachedFamilies[i] = 0;
-}
-
-static errno_t vboxNetAdpDarwinAddProto(ifnet_t pIface, protocol_family_t Family, const struct ifnet_demux_desc *pDemuxDesc, u_int32_t nDesc)
-{
-    PVBOXNETADP pThis = VBOXNETADP_FROM_IFACE(pIface);
-    Assert(pThis);
-    vboxNetAdpDarwinAttachFamily(pThis, Family);
-    LogFlow(("vboxNetAdpAddProto: Family=%d.\n", Family));
-    return ether_add_proto(pIface, Family, pDemuxDesc, nDesc);
-}
-
-static errno_t vboxNetAdpDarwinDelProto(ifnet_t pIface, protocol_family_t Family)
-{
-    PVBOXNETADP pThis = VBOXNETADP_FROM_IFACE(pIface);
-    Assert(pThis);
-    LogFlow(("vboxNetAdpDelProto: Family=%d.\n", Family));
-    vboxNetAdpDarwinDetachFamily(pThis, Family);
-    return ether_del_proto(pIface, Family);
-}
-
 static void vboxNetAdpDarwinDetach(ifnet_t pIface)
 {
     PVBOXNETADP pThis = VBOXNETADP_FROM_IFACE(pIface);
@@ -200,31 +189,58 @@ static errno_t vboxNetAdpDarwinDemux(ifnet_t pIface, mbuf_t pMBuf,
                                      char *pFrameHeader,
                                      protocol_family_t *pProtocolFamily)
 {
-    PVBOXNETADP pThis = VBOXNETADP_FROM_IFACE(pIface);
-    Assert(pThis);
-    Log2(("vboxNetAdpDarwinDemux: mode=%d\n", pThis->u.s.nTapMode));
-    if (pThis->u.s.nTapMode & BPF_MODE_INPUT)
-    {
-        Log2(("vboxnetadp: in len=%d\n%.*Rhxd\n", mbuf_len(pMBuf), 14, pFrameHeader));
-        bpf_tap_in(pIface, DLT_EN10MB, pMBuf, pFrameHeader, ETHER_HDR_LEN);
-    }
+    /*
+     * Anything we get here comes from VBoxNetFlt bridged networking
+     * filter where it has already been accounted for and fed to bpf.
+     */
     return ether_demux(pIface, pMBuf, pFrameHeader, pProtocolFamily);
 }
 
-static errno_t vboxNetAdpDarwinBpfTap(ifnet_t pIface, u_int32_t uLinkType, bpf_tap_mode nMode)
-{
-    PVBOXNETADP pThis = VBOXNETADP_FROM_IFACE(pIface);
-    Assert(pThis);
-    Log2(("vboxNetAdpDarwinBpfTap: mode=%d\n", nMode));
-    pThis->u.s.nTapMode = nMode;
-    return 0;
-}
 
-static errno_t vboxNetAdpDarwinBpfSend(ifnet_t pIface, u_int32_t uLinkType, mbuf_t pMBuf)
+static errno_t vboxNetAdpDarwinIfIOCtl(ifnet_t pIface, unsigned long uCmd, void *pvData)
 {
-    LogRel(("vboxnetadp: BPF send function is not implemented (dlt=%d)\n", uLinkType));
-    mbuf_freem_list(pMBuf);
-    return 0;
+    errno_t error = 0;
+
+    if (pvData == NULL)
+    {
+        /*
+         * Common pattern in the kernel code is to make changes in the
+         * net layer and then notify the device driver by calling its
+         * ioctl function with NULL parameter, e.g.:
+         *
+         *   ifnet_set_flags(interface, ...);
+         *   ifnet_ioctl(interface, 0, SIOCSIFFLAGS, NULL);
+         *
+         * These are no-ops for us, so tell the caller we succeeded
+         * because some callers do check that return value.
+         */
+        switch (uCmd)
+        {
+            case SIOCSIFFLAGS:
+                Log2(("VBoxNetAdp: %s%d: SIOCSIFFLAGS (null): flags = 0x%04hx\n",
+                      ifnet_name(pIface), ifnet_unit(pIface),
+                      (uint16_t)ifnet_flags(pIface)));
+                return 0;
+
+            case SIOCADDMULTI:
+            case SIOCDELMULTI:
+                Log2(("VBoxNetAdp: %s%d: SIOC%sMULTI (null)\n",
+                      ifnet_name(pIface), ifnet_unit(pIface),
+                      uCmd == SIOCADDMULTI ? "ADD" : "DEL"));
+                return 0;
+        }
+    }
+
+    Log2(("VBoxNetAdp: %s%d: %c%c '%c' %u len %u\n",
+          ifnet_name(pIface), ifnet_unit(pIface),
+          uCmd & IOC_OUT ? '<' : '-',
+          uCmd & IOC_IN  ? '>' : '-',
+          IOCGROUP(uCmd),
+          uCmd & 0xff,
+          IOCPARM_LEN(uCmd)));
+
+    error = ether_ioctl(pIface, uCmd, pvData);
+    return error;
 }
 
 
@@ -243,8 +259,6 @@ int vboxNetAdpOsCreate(PVBOXNETADP pThis, PCRTMAC pMACAddress)
         return rc;
     }
 
-    pThis->u.s.nTapMode = BPF_MODE_DISABLED;
-
     mac.sdl_len = sizeof(mac);
     mac.sdl_family = AF_LINK;
     mac.sdl_alen = ETHER_ADDR_LEN;
@@ -262,12 +276,12 @@ int vboxNetAdpOsCreate(PVBOXNETADP pThis, PCRTMAC pMACAddress)
     Params.type = IFT_ETHER;
     Params.output = vboxNetAdpDarwinOutput;
     Params.demux = vboxNetAdpDarwinDemux;
-    Params.add_proto = vboxNetAdpDarwinAddProto;
-    Params.del_proto = vboxNetAdpDarwinDelProto;
+    Params.add_proto = ether_add_proto;
+    Params.del_proto = ether_del_proto;
     Params.check_multi = ether_check_multi;
     Params.framer = ether_frameout;
     Params.softc = pThis;
-    Params.ioctl = (ifnet_ioctl_func)ether_ioctl;
+    Params.ioctl = vboxNetAdpDarwinIfIOCtl;
     Params.set_bpf_tap = NULL;
     Params.detach = vboxNetAdpDarwinDetach;
     Params.event = NULL;
@@ -280,12 +294,8 @@ int vboxNetAdpOsCreate(PVBOXNETADP pThis, PCRTMAC pMACAddress)
         err = ifnet_attach(pThis->u.s.pIface, &mac);
         if (!err)
         {
-            err = bpf_attach(pThis->u.s.pIface, DLT_EN10MB, ETHER_HDR_LEN,
-                      vboxNetAdpDarwinBpfSend, vboxNetAdpDarwinBpfTap);
-            if (err)
-            {
-                LogRel(("vboxnetadp: bpf_attach failed with %d\n", err));
-            }
+            bpfattach(pThis->u.s.pIface, DLT_EN10MB, ETHER_HDR_LEN);
+
             err = ifnet_set_flags(pThis->u.s.pIface, IFF_RUNNING | IFF_BROADCAST | IFF_SIMPLEX | IFF_MULTICAST, 0xFFFF);
             if (!err)
             {
@@ -312,7 +322,6 @@ int vboxNetAdpOsCreate(PVBOXNETADP pThis, PCRTMAC pMACAddress)
 
 void vboxNetAdpOsDestroy(PVBOXNETADP pThis)
 {
-    u_int32_t i;
     /* Bring down the interface */
     int rc = VINF_SUCCESS;
     errno_t err;
@@ -324,10 +333,6 @@ void vboxNetAdpOsDestroy(PVBOXNETADP pThis)
     if (err)
         Log(("vboxNetAdpDarwinUnregisterDevice: Failed to bring down interface "
              "(err=%d).\n", err));
-    /* Detach all protocols. */
-    for (i = 0; i < VBOXNETADP_MAX_FAMILIES; i++)
-        if (pThis->u.s.aAttachedFamilies[i])
-            ifnet_detach_protocol(pThis->u.s.pIface, pThis->u.s.aAttachedFamilies[i]);
     err = ifnet_detach(pThis->u.s.pIface);
     if (err)
         Log(("vboxNetAdpDarwinUnregisterDevice: Failed to detach interface "
@@ -349,15 +354,20 @@ void vboxNetAdpOsDestroy(PVBOXNETADP pThis)
 /**
  * Device open. Called on open /dev/vboxnetctl
  *
- * @param   pInode      Pointer to inode info structure.
- * @param   pFilp       Associated file pointer.
+ * @param   Dev         The device number.
+ * @param   fFlags      ???.
+ * @param   fDevType    ???.
+ * @param   pProcess    The process issuing this request.
  */
 static int VBoxNetAdpDarwinOpen(dev_t Dev, int fFlags, int fDevType, struct proc *pProcess)
 {
+    RT_NOREF(Dev, fFlags, fDevType, pProcess);
+#ifdef LOG_ENABLED
     char szName[128];
     szName[0] = '\0';
     proc_name(proc_pid(pProcess), szName, sizeof(szName));
     Log(("VBoxNetAdpDarwinOpen: pid=%d '%s'\n", proc_pid(pProcess), szName));
+#endif
     return 0;
 }
 
@@ -366,6 +376,7 @@ static int VBoxNetAdpDarwinOpen(dev_t Dev, int fFlags, int fDevType, struct proc
  */
 static int VBoxNetAdpDarwinClose(dev_t Dev, int fFlags, int fDevType, struct proc *pProcess)
 {
+    RT_NOREF(Dev, fFlags, fDevType, pProcess);
     Log(("VBoxNetAdpDarwinClose: pid=%d\n", proc_pid(pProcess)));
     return 0;
 }
@@ -382,6 +393,7 @@ static int VBoxNetAdpDarwinClose(dev_t Dev, int fFlags, int fDevType, struct pro
  */
 static int VBoxNetAdpDarwinIOCtl(dev_t Dev, u_long iCmd, caddr_t pData, int fFlags, struct proc *pProcess)
 {
+    RT_NOREF(Dev, fFlags, pProcess);
     uint32_t cbReq = IOCPARM_LEN(iCmd);
     PVBOXNETADPREQ pReq = (PVBOXNETADPREQ)pData;
     int rc;
@@ -441,7 +453,6 @@ int  vboxNetAdpOsInit(PVBOXNETADP pThis)
      */
     pThis->u.s.pIface = NULL;
     pThis->u.s.hEvtDetached = NIL_RTSEMEVENT;
-    memset(pThis->u.s.aAttachedFamilies, 0, sizeof(pThis->u.s.aAttachedFamilies));
 
     return VINF_SUCCESS;
 }
@@ -451,6 +462,7 @@ int  vboxNetAdpOsInit(PVBOXNETADP pThis)
  */
 static kern_return_t    VBoxNetAdpDarwinStart(struct kmod_info *pKModInfo, void *pvData)
 {
+    RT_NOREF(pKModInfo, pvData);
     int rc;
 
     /*
@@ -503,6 +515,7 @@ static kern_return_t    VBoxNetAdpDarwinStart(struct kmod_info *pKModInfo, void
  */
 static kern_return_t VBoxNetAdpDarwinStop(struct kmod_info *pKModInfo, void *pvData)
 {
+    RT_NOREF(pKModInfo, pvData);
     Log(("VBoxNetAdpDarwinStop\n"));
 
     vboxNetAdpShutdown();
