@@ -6,7 +6,7 @@
 /*
  * Includes contributions from François Revol
  *
- * Copyright (C) 2008-2023 Oracle and/or its affiliates.
+ * Copyright (C) 2008-2024 Oracle and/or its affiliates.
  *
  * This file is part of VirtualBox base platform packages, as
  * available from https://www.virtualbox.org.
@@ -44,6 +44,8 @@
 #include <VBox/GuestHost/SharedClipboard.h>
 #include <VBox/GuestHost/clipboard-helper.h>
 
+#include "darwin-pasteboard.h"
+
 
 /*********************************************************************************************************************************
 *   Defined Constants And Macros                                                                                                 *
@@ -51,6 +53,8 @@
 #define WITH_HTML_H2G 1
 #define WITH_HTML_G2H 1
 
+RT_GCC_NO_WARN_DEPRECATED_BEGIN /* Much here is deprecated since 12.0 */
+
 /* For debugging */
 //#define SHOW_CLIPBOARD_CONTENT
 
@@ -62,7 +66,7 @@
  *
  * @returns IPRT status code.
  */
-int initPasteboard(PasteboardRef *pPasteboardRef)
+DECLHIDDEN(int) initPasteboard(PasteboardRef *pPasteboardRef)
 {
     int rc = VINF_SUCCESS;
 
@@ -77,7 +81,7 @@ int initPasteboard(PasteboardRef *pPasteboardRef)
  *
  * @param pPasteboardRef Reference to the global pasteboard.
  */
-void destroyPasteboard(PasteboardRef *pPasteboardRef)
+DECLHIDDEN(void) destroyPasteboard(PasteboardRef *pPasteboardRef)
 {
     CFRelease(*pPasteboardRef);
     *pPasteboardRef = NULL;
@@ -98,8 +102,8 @@ void destroyPasteboard(PasteboardRef *pPasteboardRef)
  *
  * @returns VINF_SUCCESS.
  */
-int queryNewPasteboardFormats(PasteboardRef hPasteboard, uint64_t idOwnership, void *hStrOwnershipFlavor,
-                              uint32_t *pfFormats, bool *pfChanged)
+DECLHIDDEN(int) queryNewPasteboardFormats(PasteboardRef hPasteboard, uint64_t idOwnership, void *hStrOwnershipFlavor,
+                                          uint32_t *pfFormats, bool *pfChanged)
 {
     OSStatus orc;
 
@@ -215,7 +219,7 @@ int queryNewPasteboardFormats(PasteboardRef hPasteboard, uint64_t idOwnership, v
  *
  * @returns IPRT status code.
  */
-int readFromPasteboard(PasteboardRef pPasteboard, uint32_t fFormat, void *pv, uint32_t cb, uint32_t *pcbActual)
+DECLHIDDEN(int) readFromPasteboard(PasteboardRef pPasteboard, uint32_t fFormat, void *pv, uint32_t cb, uint32_t *pcbActual)
 {
     Log(("readFromPasteboard: fFormat = %02X\n", fFormat));
 
@@ -471,8 +475,8 @@ int readFromPasteboard(PasteboardRef pPasteboard, uint32_t fFormat, void *pv, ui
  *          without needing to request any data first.  That might help on
  *          flavor priority.
  */
-int takePasteboardOwnership(PasteboardRef hPasteboard, uint64_t idOwnership, const char *pszOwnershipFlavor,
-                            const char *pszOwnershipValue, void **phStrOwnershipFlavor)
+DECLHIDDEN(int) takePasteboardOwnership(PasteboardRef hPasteboard, uint64_t idOwnership, const char *pszOwnershipFlavor,
+                                        const char *pszOwnershipValue, void **phStrOwnershipFlavor)
 {
     /*
      * Release the old string.
@@ -540,7 +544,7 @@ int takePasteboardOwnership(PasteboardRef hPasteboard, uint64_t idOwnership, con
  *
  * @returns IPRT status code.
  */
-int writeToPasteboard(PasteboardRef hPasteboard, uint64_t idOwnership, const void *pv, uint32_t cb, uint32_t fFormat)
+DECLHIDDEN(int) writeToPasteboard(PasteboardRef hPasteboard, uint64_t idOwnership, const void *pv, uint32_t cb, uint32_t fFormat)
 {
     int       rc;
     OSStatus  orc;
@@ -716,3 +720,4 @@ int writeToPasteboard(PasteboardRef hPasteboard, uint64_t idOwnership, const voi
     return rc;
 }
 
+RT_GCC_NO_WARN_DEPRECATED_END
