@@ -10,7 +10,7 @@
  */
 
 /*
- * Copyright (C) 2011-2023 Oracle and/or its affiliates.
+ * Copyright (C) 2011-2024 Oracle and/or its affiliates.
  *
  * This file is part of VirtualBox base platform packages, as
  * available from https://www.virtualbox.org.
@@ -113,7 +113,7 @@ static DECLCALLBACK(int) infoDeregister(void *pvInstance, const char *pszName)
  * service
  * @param  pTable the table to initialise
  */
-void initTable(VBOXHGCMSVCFNTABLE *pTable, VBOXHGCMSVCHELPERS *pHelpers)
+static void initTable(VBOXHGCMSVCFNTABLE *pTable, VBOXHGCMSVCHELPERS *pHelpers)
 {
     pTable->cbSize               = sizeof (VBOXHGCMSVCFNTABLE);
     pTable->u32Version           = VBOX_HGCM_SVC_VERSION;
@@ -419,7 +419,10 @@ extern int  testRTFileRead(RTFILE File, void *pvBuf, size_t cbToRead, size_t *pc
              LLUIFY(cbToRead)); */
     bufferFromPath((char *)pvBuf, cbToRead, g_testRTFileRead_pszData);
     if (pcbRead)
-        *pcbRead = RT_MIN(cbToRead, strlen(g_testRTFileRead_pszData) + 1);
+    {
+        size_t cchData = strlen(g_testRTFileRead_pszData) + 1;
+        *pcbRead = RT_MIN(cbToRead, cchData);
+    }
     g_testRTFileRead_pszData = 0;
     return VINF_SUCCESS;
 }
@@ -432,7 +435,10 @@ extern int  testRTFileReadAt(RTFILE hFile, uint64_t offFile, void *pvBuf, size_t
              LLUIFY(cbToRead)); */
     bufferFromPath((char *)pvBuf, cbToRead, g_testRTFileRead_pszData);
     if (pcbRead)
-        *pcbRead = RT_MIN(cbToRead, strlen(g_testRTFileRead_pszData) + 1);
+    {
+        size_t cchData = strlen(g_testRTFileRead_pszData) + 1;
+        *pcbRead = RT_MIN(cbToRead, cchData);
+    }
     g_testRTFileRead_pszData = 0;
     return VINF_SUCCESS;
 }
@@ -580,6 +586,13 @@ extern int testRTSymlinkRead(const char *pszSymlink, char *pszTarget, size_t cbT
     return 0;
 }
 
+extern int testRTSymlinkCreate(const char *pszSymlink, const char *pszTarget, RTSYMLINKTYPE enmType, uint32_t fCreate)
+{
+    if (g_fFailIfNotLowercase && !RTStrIsLowerCased(strpbrk(pszSymlink, "/\\")))
+        return VERR_FILE_NOT_FOUND;
+    RT_NOREF4(pszSymlink, pszTarget, enmType, fCreate);
+    return 0;
+}
 
 extern int testRTFileCopy(const char *pszSrc, const char *pszDst)
 {
@@ -697,7 +710,8 @@ static SHFLROOT initWithWritableMapping(RTTEST hTest,
                                         VBOXHGCMSVCHELPERS *psvcHelpers,
                                         const char *pcszFolderName,
                                         const char *pcszMapping,
-                                        bool fCaseSensitive = true)
+                                        bool fCaseSensitive = true,
+                                        SymlinkPolicy_T enmSymlinkPolicy = SymlinkPolicy_AllowedToAnyTarget)
 {
     VBOXHGCMSVCPARM aParms[RT_MAX(SHFL_CPARMS_ADD_MAPPING,
                                   SHFL_CPARMS_MAP_FOLDER)];
@@ -708,7 +722,8 @@ static SHFLROOT initWithWritableMapping(RTTEST hTest,
     int rc;
 
     initTable(psvcTable, psvcHelpers);
-    AssertReleaseRC(VBoxHGCMSvcLoad(psvcTable));
+    rc = VBoxHGCMSvcLoad(psvcTable);
+    AssertReleaseRC(rc);
     AssertRelease(  psvcTable->pvService
                   = RTTestGuardedAllocTail(hTest, psvcTable->cbClient));
     RT_BZERO(psvcTable->pvService, psvcTable->cbClient);
@@ -719,8 +734,9 @@ static SHFLROOT initWithWritableMapping(RTTEST hTest,
                                       + FolderName.string.u16Size);
     HGCMSvcSetPv(&aParms[1], &Mapping,   RT_UOFFSETOF(SHFLSTRING, String)
                                    + Mapping.string.u16Size);
-    HGCMSvcSetU32(&aParms[2], 1);
+    HGCMSvcSetU32(&aParms[2], SHFL_ADD_MAPPING_F_WRITABLE | SHFL_ADD_MAPPING_F_CREATE_SYMLINKS);
     HGCMSvcSetPv(&aParms[3], &AutoMountPoint, SHFLSTRING_HEADER_SIZE + AutoMountPoint.string.u16Size);
+    HGCMSvcSetU32(&aParms[4], enmSymlinkPolicy);
     rc = psvcTable->pfnHostCall(psvcTable->pvService, SHFL_FN_ADD_MAPPING,
                                 SHFL_CPARMS_ADD_MAPPING, aParms);
     AssertReleaseRC(rc);
@@ -741,7 +757,8 @@ static SHFLROOT initWithUnwritableMapping(RTTEST hTest,
                                           VBOXHGCMSVCHELPERS *psvcHelpers,
                                           const char *pcszFolderName,
                                           const char *pcszMapping,
-                                          bool fCaseSensitive = true)
+                                          bool fCaseSensitive = true,
+                                          SymlinkPolicy_T enmSymlinkPolicy = SymlinkPolicy_AllowedToAnyTarget)
 {
     VBOXHGCMSVCPARM aParms[RT_MAX(SHFL_CPARMS_ADD_MAPPING,
                                   SHFL_CPARMS_MAP_FOLDER)];
@@ -766,6 +783,7 @@ static SHFLROOT initWithUnwritableMapping(RTTEST hTest,
                                    + Mapping.string.u16Size);
     HGCMSvcSetU32(&aParms[2], SHFL_ADD_MAPPING_F_CREATE_SYMLINKS);
     HGCMSvcSetPv(&aParms[3], &AutoMountPoint, SHFLSTRING_HEADER_SIZE + AutoMountPoint.string.u16Size);
+    HGCMSvcSetU32(&aParms[4], enmSymlinkPolicy);
     rc = psvcTable->pfnHostCall(psvcTable->pvService, SHFL_FN_ADD_MAPPING,
                                 SHFL_CPARMS_ADD_MAPPING, aParms);
     AssertReleaseRC(rc);
@@ -1034,8 +1052,10 @@ void testCreateFileSimple(RTTEST hTest)
     RTTEST_CHECK_MSG(hTest, Result == SHFL_FILE_CREATED,
                      (hTest, "Result=%d\n", (int) Result));
     unmapAndRemoveMapping(hTest, &svcTable, Root, "testname");
-    AssertReleaseRC(svcTable.pfnDisconnect(NULL, 0, svcTable.pvService));
-    AssertReleaseRC(svcTable.pfnUnload(NULL));
+    rc = svcTable.pfnDisconnect(NULL, 0, svcTable.pvService);
+    AssertReleaseRC(rc);
+    rc = svcTable.pfnUnload(NULL);
+    AssertReleaseRC(rc);
     RTTestGuardedFree(hTest, svcTable.pvService);
     RTTEST_CHECK_MSG(hTest, g_testRTFileClose_hFile == hFile,
                      (hTest, "File=%u\n", (uintptr_t)g_testRTFileClose_hFile));
@@ -1069,8 +1089,10 @@ void testCreateFileSimpleCaseInsensitive(RTTEST hTest)
     RTTEST_CHECK_MSG(hTest, Result == SHFL_FILE_CREATED,
                      (hTest, "Result=%d\n", (int) Result));
     unmapAndRemoveMapping(hTest, &svcTable, Root, "testname");
-    AssertReleaseRC(svcTable.pfnDisconnect(NULL, 0, svcTable.pvService));
-    AssertReleaseRC(svcTable.pfnUnload(NULL));
+    rc = svcTable.pfnDisconnect(NULL, 0, svcTable.pvService);
+    AssertReleaseRC(rc);
+    rc = svcTable.pfnUnload(NULL);
+    AssertReleaseRC(rc);
     RTTestGuardedFree(hTest, svcTable.pvService);
     RTTEST_CHECK_MSG(hTest, g_testRTFileClose_hFile == hFile,
                      (hTest, "File=%u\n", (uintptr_t)g_testRTFileClose_hFile));
@@ -1105,8 +1127,10 @@ void testCreateDirSimple(RTTEST hTest)
     RTTEST_CHECK_MSG(hTest, Result == SHFL_FILE_CREATED,
                      (hTest, "Result=%d\n", (int) Result));
     unmapAndRemoveMapping(hTest, &svcTable, Root, "testname");
-    AssertReleaseRC(svcTable.pfnDisconnect(NULL, 0, svcTable.pvService));
-    AssertReleaseRC(svcTable.pfnUnload(NULL));
+    rc = svcTable.pfnDisconnect(NULL, 0, svcTable.pvService);
+    AssertReleaseRC(rc);
+    rc = svcTable.pfnUnload(NULL);
+    AssertReleaseRC(rc);
     RTTestGuardedFree(hTest, svcTable.pvService);
     RTTEST_CHECK_MSG(hTest, g_testRTDirClose_hDir == hDir, (hTest, "hDir=%p\n", g_testRTDirClose_hDir));
 }
@@ -1157,6 +1181,92 @@ void testCopyFileReadOnly(RTTEST hTest)
     RTTestGuardedFree(hTest, svcTable.pvService);
 }
 
+static int testSymlinkCreationForSpecificPolicy(RTTEST hTest, SymlinkPolicy_T enmSymlinkPolicy)
+{
+    VBOXHGCMSVCFNTABLE  svcTable;
+    VBOXHGCMSVCHELPERS  svcHelpers;
+    SHFLROOT Root;
+    const RTFILE hFile = (RTFILE) 0x10000;
+    SHFLCREATERESULT Result;
+    int rc;
+
+    Root = initWithWritableMapping(hTest, &svcTable, &svcHelpers,
+                                   "/test/mapping", "testname",
+                                   true, enmSymlinkPolicy);
+    g_testRTFileOpen_hFile = hFile;
+    rc = createFile(&svcTable, Root, "file", SHFL_CF_ACCESS_READ, NULL, &Result);
+    RTTEST_CHECK_RC_OK(hTest, rc);
+    RTTEST_CHECK_MSG(hTest,
+                     !strcmp(&g_testRTFileOpen_szName[RTPATH_STYLE == RTPATH_STR_F_STYLE_DOS ? 2 : 0],
+                             "/test/mapping/file"),
+                     (hTest, "pszFilename=%s\n", &g_testRTFileOpen_szName[RTPATH_STYLE == RTPATH_STR_F_STYLE_DOS ? 2 : 0]));
+    RTTEST_CHECK_MSG(hTest, g_testRTFileOpen_fOpen == 0x181,
+                     (hTest, "fOpen=%llu\n", LLUIFY(g_testRTFileOpen_fOpen)));
+    RTTEST_CHECK_MSG(hTest, Result == SHFL_FILE_CREATED,
+                     (hTest, "Result=%d\n", (int)Result));
+
+    /* regular symlink creation should succeed unless no symlinks allowed */
+    rc = createSymlink(&svcTable, Root, "file", "symlink");
+    if (enmSymlinkPolicy == SymlinkPolicy_Forbidden)
+        RTTEST_CHECK_MSG(hTest, rc == VERR_WRITE_PROTECT,
+                         (hTest, "enmSymlinkPolicy=SymlinkPolicy_Forbidden 'ln -s file symlink' failed: rc=%Rrc\n", rc));
+    else
+        RTTEST_CHECK_RC_OK(hTest, rc);
+
+    /* absolute path to symlink sources only allowed for the 'any' policy */
+    rc = createSymlink(&svcTable, Root, "/path/to/file", "abs-symlink");
+    if (enmSymlinkPolicy == SymlinkPolicy_AllowedToAnyTarget)
+        RTTEST_CHECK_RC_OK(hTest, rc);
+    else
+        RTTEST_CHECK_MSG(hTest, rc == VERR_WRITE_PROTECT,
+                         (hTest, "enmSymlinkPolicy=%d 'ln -s file /absolute/path/symlink' failed: rc=%Rrc\n",
+                         enmSymlinkPolicy, rc));
+
+    /* relative path symlink sources with '..' components allowed only with 'relative' policy */
+    rc = createSymlink(&svcTable, Root, "./directory/../file", "rel-symlink");
+    if (   enmSymlinkPolicy == SymlinkPolicy_Forbidden
+        || enmSymlinkPolicy == SymlinkPolicy_AllowedInShareSubtree)
+        RTTEST_CHECK_MSG(hTest, rc == VERR_WRITE_PROTECT,
+                         (hTest, "enmSymlinkPolicy=%d 'ln -s ./path/../symlink' failed: rc=%Rrc\n",
+                         enmSymlinkPolicy, rc));
+    else
+        RTTEST_CHECK_RC_OK(hTest, rc);
+
+    /* relative path symlink source with no '..' components always OK */
+    rc = createSymlink(&svcTable, Root, "./directory/file", "dotslash-symlink");
+    if (enmSymlinkPolicy == SymlinkPolicy_Forbidden)
+        RTTEST_CHECK_MSG(hTest, rc == VERR_WRITE_PROTECT,
+                         (hTest, "enmSymlinkPolicy=%d 'ln -s ./path/../symlink' failed: rc=%Rrc\n",
+                         enmSymlinkPolicy, rc));
+    else
+        RTTEST_CHECK_RC_OK(hTest, rc);
+
+    unmapAndRemoveMapping(hTest, &svcTable, Root, "testname");
+    rc = svcTable.pfnDisconnect(NULL, 0, svcTable.pvService);
+    AssertReleaseRC(rc);
+    rc = svcTable.pfnUnload(NULL);
+    AssertReleaseRC(rc);
+    RTTestGuardedFree(hTest, svcTable.pvService);
+    RTTEST_CHECK_MSG(hTest, g_testRTFileClose_hFile == hFile,
+                     (hTest, "File=%u\n", (uintptr_t)g_testRTFileClose_hFile));
+
+    return rc;
+}
+
+void testSymlinkCreation(RTTEST hTest)
+{
+    SymlinkPolicy_T aEnmSymlinkPolicy[4] = {
+        SymlinkPolicy_AllowedToAnyTarget,
+        SymlinkPolicy_AllowedInShareSubtree,
+        SymlinkPolicy_AllowedToRelativeTargets,
+        SymlinkPolicy_Forbidden
+    };
+
+    RTTestSub(hTest, "Create variety of symlinks with different symlink policies");
+    for (size_t i = 0; i < RT_ELEMENTS(aEnmSymlinkPolicy); i++)
+        testSymlinkCreationForSpecificPolicy(hTest, aEnmSymlinkPolicy[i]);
+}
+
 void testSymlinkReadOnlyCreation(RTTEST hTest)
 {
     VBOXHGCMSVCFNTABLE  svcTable;
@@ -1211,8 +1321,10 @@ void testReadFileSimple(RTTEST hTest)
                      (hTest, "cbRead=%llu\n", LLUIFY(cbRead)));
     unmapAndRemoveMapping(hTest, &svcTable, Root, "testname");
     RTTEST_CHECK_MSG(hTest, g_testRTFileClose_hFile == hFile, (hTest, "File=%u\n", g_testRTFileClose_hFile));
-    AssertReleaseRC(svcTable.pfnDisconnect(NULL, 0, svcTable.pvService));
-    AssertReleaseRC(svcTable.pfnUnload(NULL));
+    rc = svcTable.pfnDisconnect(NULL, 0, svcTable.pvService);
+    AssertReleaseRC(rc);
+    rc = svcTable.pfnUnload(NULL);
+    AssertReleaseRC(rc);
     RTTestGuardedFree(hTest, svcTable.pvService);
 }
 
@@ -1245,8 +1357,10 @@ void testWriteFileSimple(RTTEST hTest)
                      (hTest, "cbWritten=%llu\n", LLUIFY(cbWritten)));
     unmapAndRemoveMapping(hTest, &svcTable, Root, "testname");
     RTTEST_CHECK_MSG(hTest, g_testRTFileClose_hFile == hFile, (hTest, "File=%u\n", g_testRTFileClose_hFile));
-    AssertReleaseRC(svcTable.pfnDisconnect(NULL, 0, svcTable.pvService));
-    AssertReleaseRC(svcTable.pfnUnload(NULL));
+    rc = svcTable.pfnDisconnect(NULL, 0, svcTable.pvService);
+    AssertReleaseRC(rc);
+    rc = svcTable.pfnUnload(NULL);
+    AssertReleaseRC(rc);
     RTTestGuardedFree(hTest, svcTable.pvService);
 }
 
@@ -1270,8 +1384,10 @@ void testFlushFileSimple(RTTEST hTest)
     RTTEST_CHECK_RC_OK(hTest, rc);
     RTTEST_CHECK_MSG(hTest, g_testRTFileFlush_hFile == hFile, (hTest, "File=%u\n", g_testRTFileFlush_hFile));
     unmapAndRemoveMapping(hTest, &svcTable, Root, "testname");
-    AssertReleaseRC(svcTable.pfnDisconnect(NULL, 0, svcTable.pvService));
-    AssertReleaseRC(svcTable.pfnUnload(NULL));
+    rc = svcTable.pfnDisconnect(NULL, 0, svcTable.pvService);
+    AssertReleaseRC(rc);
+    rc = svcTable.pfnUnload(NULL);
+    AssertReleaseRC(rc);
     RTTestGuardedFree(hTest, svcTable.pvService);
     RTTEST_CHECK_MSG(hTest, g_testRTFileClose_hFile == hFile, (hTest, "File=%u\n", g_testRTFileClose_hFile));
 }
@@ -1304,8 +1420,10 @@ void testDirListEmpty(RTTEST hTest)
     RTTEST_CHECK_MSG(hTest, cFiles == 0,
                      (hTest, "cFiles=%llu\n", LLUIFY(cFiles)));
     unmapAndRemoveMapping(hTest, &svcTable, Root, "testname");
-    AssertReleaseRC(svcTable.pfnDisconnect(NULL, 0, svcTable.pvService));
-    AssertReleaseRC(svcTable.pfnUnload(NULL));
+    rc = svcTable.pfnDisconnect(NULL, 0, svcTable.pvService);
+    AssertReleaseRC(rc);
+    rc = svcTable.pfnUnload(NULL);
+    AssertReleaseRC(rc);
     RTTestGuardedFree(hTest, svcTable.pvService);
     RTTEST_CHECK_MSG(hTest, g_testRTDirClose_hDir == hDir, (hTest, "hDir=%p\n", g_testRTDirClose_hDir));
 }
@@ -1345,8 +1463,10 @@ void testFSInfoQuerySetFMode(RTTEST hTest)
     RTTEST_CHECK_MSG(hTest, g_testRTFileSet_fMode == fMode,
                      (hTest, "Size=%llu\n", LLUIFY(g_testRTFileSet_fMode)));
     unmapAndRemoveMapping(hTest, &svcTable, Root, "testname");
-    AssertReleaseRC(svcTable.pfnDisconnect(NULL, 0, svcTable.pvService));
-    AssertReleaseRC(svcTable.pfnUnload(NULL));
+    rc = svcTable.pfnDisconnect(NULL, 0, svcTable.pvService);
+    AssertReleaseRC(rc);
+    rc = svcTable.pfnUnload(NULL);
+    AssertReleaseRC(rc);
     RTTestGuardedFree(hTest, svcTable.pvService);
     RTTEST_CHECK_MSG(hTest, g_testRTFileClose_hFile == hFile, (hTest, "File=%u\n", g_testRTFileClose_hFile));
 }
@@ -1388,8 +1508,10 @@ void testFSInfoQuerySetDirATime(RTTEST hTest)
                      (hTest, "ATime=%llu\n",
                       LLUIFY(RTTimeSpecGetNano(&g_testRTDirSetTimes_ATime))));
     unmapAndRemoveMapping(hTest, &svcTable, Root, "testname");
-    AssertReleaseRC(svcTable.pfnDisconnect(NULL, 0, svcTable.pvService));
-    AssertReleaseRC(svcTable.pfnUnload(NULL));
+    rc = svcTable.pfnDisconnect(NULL, 0, svcTable.pvService);
+    AssertReleaseRC(rc);
+    rc = svcTable.pfnUnload(NULL);
+    AssertReleaseRC(rc);
     RTTestGuardedFree(hTest, svcTable.pvService);
     RTTEST_CHECK_MSG(hTest, g_testRTDirClose_hDir == hDir, (hTest, "hDir=%p\n", g_testRTDirClose_hDir));
 }
@@ -1431,8 +1553,10 @@ void testFSInfoQuerySetFileATime(RTTEST hTest)
                      (hTest, "ATime=%llu\n",
                       LLUIFY(RTTimeSpecGetNano(&g_testRTFileSetTimes_ATime))));
     unmapAndRemoveMapping(hTest, &svcTable, Root, "testname");
-    AssertReleaseRC(svcTable.pfnDisconnect(NULL, 0, svcTable.pvService));
-    AssertReleaseRC(svcTable.pfnUnload(NULL));
+    rc = svcTable.pfnDisconnect(NULL, 0, svcTable.pvService);
+    AssertReleaseRC(rc);
+    rc = svcTable.pfnUnload(NULL);
+    AssertReleaseRC(rc);
     RTTestGuardedFree(hTest, svcTable.pvService);
     RTTEST_CHECK_MSG(hTest, g_testRTFileClose_hFile == hFile, (hTest, "File=%u\n", g_testRTFileClose_hFile));
 }
@@ -1464,8 +1588,10 @@ void testFSInfoQuerySetEndOfFile(RTTEST hTest)
     RTTEST_CHECK_MSG(hTest, g_testRTFileSetSize_cbSize == cbNew,
                      (hTest, "Size=%llu\n", LLUIFY(g_testRTFileSetSize_cbSize)));
     unmapAndRemoveMapping(hTest, &svcTable, Root, "testname");
-    AssertReleaseRC(svcTable.pfnDisconnect(NULL, 0, svcTable.pvService));
-    AssertReleaseRC(svcTable.pfnUnload(NULL));
+    rc = svcTable.pfnDisconnect(NULL, 0, svcTable.pvService);
+    AssertReleaseRC(rc);
+    rc = svcTable.pfnUnload(NULL);
+    AssertReleaseRC(rc);
     RTTestGuardedFree(hTest, svcTable.pvService);
     RTTEST_CHECK_MSG(hTest, g_testRTFileClose_hFile == hFile, (hTest, "File=%u\n", g_testRTFileClose_hFile));
 }
@@ -1510,8 +1636,10 @@ void testLockFileSimple(RTTEST hTest)
                      (hTest, "Size=%llu\n", LLUIFY(g_testRTFileUnlock_cbLock)));
 #endif
     unmapAndRemoveMapping(hTest, &svcTable, Root, "testname");
-    AssertReleaseRC(svcTable.pfnDisconnect(NULL, 0, svcTable.pvService));
-    AssertReleaseRC(svcTable.pfnUnload(NULL));
+    rc = svcTable.pfnDisconnect(NULL, 0, svcTable.pvService);
+    AssertReleaseRC(rc);
+    rc = svcTable.pfnUnload(NULL);
+    AssertReleaseRC(rc);
     RTTestGuardedFree(hTest, svcTable.pvService);
     RTTEST_CHECK_MSG(hTest, g_testRTFileClose_hFile == hFile, (hTest, "File=%u\n", g_testRTFileClose_hFile));
 }
