@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2006-2019 Oracle Corporation
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
@@ -43,6 +53,7 @@
 #ifdef RT_OS_DARWIN
 # include <Carbon/Carbon.h>
 #endif
+#include <VBox/com/defs.h> /* For S_OK. */
 
 #ifdef UNITTEST
 # include "teststubs.h"
@@ -102,7 +113,7 @@ static int vbsfCorrectCasing(SHFLCLIENTDATA *pClient, char *pszFullPath, char *p
     /** @todo Use RTDirOpen here and drop the whole uncessary path copying? */
     int rc = RTPathJoinEx(pDirEntry->szName, cbDirEntry - RT_OFFSETOF(RTDIRENTRYEX, szName),
                           pszFullPath, cchParentDir,
-                          RT_STR_TUPLE("*"));
+                          RT_STR_TUPLE("*"), RTPATH_STR_F_STYLE_HOST);
     AssertRC(rc);
     if (RT_SUCCESS(rc))
     {
@@ -333,7 +344,7 @@ static int vbsfCorrectPathCasing(SHFLCLIENTDATA *pClient, char *pszFullPath, siz
  * @param ppwszDst Where to store the pointer to the resulting normalized string.
  * @param pcwcDst  Where to store length of the normalized string in characters (without the trailing nul).
  */
-static int vbsfNormalizeStringDarwin(const PRTUTF16 pwszSrc, uint32_t cwcSrc, PRTUTF16 *ppwszDst, uint32_t *pcwcDst)
+static int vbsfNormalizeStringDarwin(PCRTUTF16 pwszSrc, uint32_t cwcSrc, PRTUTF16 *ppwszDst, uint32_t *pcwcDst)
 {
     /** @todo This belongs in rtPathToNative or in the windows shared folder file system driver...
      * The question is simply whether the NFD normalization is actually applied on a (virtual) file
@@ -427,8 +438,41 @@ static bool vbsfPathIsWildcardChar(char c)
     return false;
 }
 
+/**
+ * Validate the symbolic link creation policy inside a guest operating in a shared folder.
+ *
+ * @returns S_OK or VERR_WRITE_PROTECT.
+ * @param pchSymlinkPath    The pathname of the symbolic link within the guest.
+ * @param enmSymlinkPolicy  The symbolic link creation policy being evaluated.
+ *
+ * The enmSymlinkPolicy symlink creation policies are:
+ *  - @a none:       no policy set
+ *  - @a any:        no restrictions
+ *  - @a forbidden:  no symlinks allowed
+ *  - @a relative:   relative paths only ('..' path components allowed)
+ *  - @a subtree:    relative paths only within the shared folder (no '..' path components allowed)
+ */
+static int vbsfPathEvalSymlinkPolicy(const char *pchSymlinkPath, SymlinkPolicy_T enmSymlinkPolicy)
+{
+    /* If no symlink policy has been set we continue the historical behaviour of applying no
+     * additional restrictions.  The "any" policy also has no symlink path limitations. */
+    if (   enmSymlinkPolicy == SymlinkPolicy_None
+        || enmSymlinkPolicy == SymlinkPolicy_AllowedToAnyTarget)
+        return S_OK;
+
+    /* No absolute paths allowed except for the "any" policy.  The symlink path can't
+     * contain '..' components if the "subtree" policy in effect. */
+    if (   RTPathStartsWithRoot(pchSymlinkPath)
+        || enmSymlinkPolicy == SymlinkPolicy_Forbidden
+        || (   enmSymlinkPolicy == SymlinkPolicy_AllowedInShareSubtree
+            && RTStrStr(pchSymlinkPath, "..")))
+        return VERR_WRITE_PROTECT;
+
+    return S_OK;
+}
+
 int vbsfPathGuestToHost(SHFLCLIENTDATA *pClient, SHFLROOT hRoot,
-                        PSHFLSTRING pGuestString, uint32_t cbGuestString,
+                        PCSHFLSTRING pGuestString, uint32_t cbGuestString,
                         char **ppszHostPath, uint32_t *pcbHostPathRoot,
                         uint32_t fu32Options,
                         uint32_t *pfu32PathFlags)
@@ -471,23 +515,21 @@ int vbsfPathGuestToHost(SHFLCLIENTDATA *pClient, SHFLROOT hRoot,
     {
         /* UTF-8 */
         cbGuestPath = pGuestString->u16Length;
-        pchGuestPath = (char *)&pGuestString->String.utf8[0];
+        pchGuestPath = pGuestString->String.ach;
     }
     else
     {
         /* UTF-16 */
-        uint32_t cwcSrc;
-        PRTUTF16 pwszSrc;
 
 #ifdef RT_OS_DARWIN /* Misplaced hack! See todo! */
-        cwcSrc = 0;
-        pwszSrc = NULL;
-        rc = vbsfNormalizeStringDarwin(&pGuestString->String.ucs2[0],
+        uint32_t cwcSrc  = 0;
+        PRTUTF16 pwszSrc = NULL;
+        rc = vbsfNormalizeStringDarwin(&pGuestString->String.utf16[0],
                                        pGuestString->u16Length / sizeof(RTUTF16),
                                        &pwszSrc, &cwcSrc);
 #else
-        cwcSrc  = pGuestString->u16Length / sizeof(RTUTF16);
-        pwszSrc = &pGuestString->String.ucs2[0];
+        uint32_t  const cwcSrc  = pGuestString->u16Length / sizeof(RTUTF16);
+        PCRTUTF16 const pwszSrc = &pGuestString->String.utf16[0];
 #endif
 
         if (RT_SUCCESS(rc))
@@ -556,12 +598,16 @@ int vbsfPathGuestToHost(SHFLCLIENTDATA *pClient, SHFLROOT hRoot,
                 uint32_t cbSrc = cbGuestPath;
                 const char *pchSrc = pchGuestPath;
 
-                /* Strip leading delimiters from the path the guest specified. */
-                while (   cbSrc > 0
-                       && *pchSrc == pClient->PathDelimiter)
+                /* when validating source file pathnames for symbolic links we don't modify the path */
+                if (!(fu32Options & VBSF_O_PATH_CHECK_SYMLINK_POLICY))
                 {
-                    ++pchSrc;
-                    --cbSrc;
+                    /* Strip leading delimiters from the path the guest specified. */
+                    while (   cbSrc > 0
+                           && *pchSrc == pClient->PathDelimiter)
+                    {
+                        ++pchSrc;
+                        --cbSrc;
+                    }
                 }
 
                 /*
@@ -606,8 +652,24 @@ int vbsfPathGuestToHost(SHFLCLIENTDATA *pClient, SHFLROOT hRoot,
                 {
                     *pchDst++ = 0;
 
-                    /* Construct the full host path removing '.' and '..'. */
-                    rc = vbsfPathAbs(pszRoot, pchVerifiedPath, pszFullPath, cbFullPathAlloc);
+                    /* check if a symbolic link creation policy has been set */
+                    if (fu32Options & VBSF_O_PATH_CHECK_SYMLINK_POLICY)
+                    {
+                        /* copy the verified symlink source file path to be returned to the caller */
+                        rc = RTStrCopy(pszFullPath, cbFullPathAlloc, pchVerifiedPath);
+                        if (RT_SUCCESS(rc))
+                        {
+                            SymlinkPolicy_T enmSymlinkPolicy;
+                            rc = vbsfMappingsQuerySymlinkPolicy(pClient, hRoot, &enmSymlinkPolicy);
+                            if (RT_SUCCESS(rc))
+                                rc = vbsfPathEvalSymlinkPolicy(pchVerifiedPath, enmSymlinkPolicy);
+                        }
+                    }
+                    else
+                    {
+                        /* Construct the full host path removing '.' and '..'. */
+                        rc = vbsfPathAbs(pszRoot, pchVerifiedPath, pszFullPath, cbFullPathAlloc);
+                    }
                     if (RT_SUCCESS(rc))
                     {
                         if (pfu32PathFlags && fLastComponentHasWildcard)
@@ -658,7 +720,10 @@ int vbsfPathGuestToHost(SHFLCLIENTDATA *pClient, SHFLROOT hRoot,
                     }
                     else
                     {
-                        LogFunc(("vbsfPathAbs %Rrc\n", rc));
+                        if (fu32Options & VBSF_O_PATH_CHECK_SYMLINK_POLICY)
+                            LogFunc(("vbsfPathEvalSymlinkPolicy() returns rc=%Rrc\n", rc));
+                        else
+                            LogFunc(("vbsfPathAbs %Rrc\n", rc));
                     }
                 }
 
