@@ -944,7 +944,7 @@ install_python_bindings()
     else
         errorprint "Failed to install bindings for Python $PYTHON_VER"
     fi
-    rm "$CONFIG_DIR/python-$CONFIG_FILES"
+    rm -f "$CONFIG_DIR/python-$CONFIG_FILES"
 
     # Remove files created by Python API setup.
     rm -rf "$MY_PYTHON_INSTALLER_PATH/build"
@@ -979,6 +979,7 @@ maybe_run_python_bindings_installer() {
                 python3.9  \
                 python3.10 \
                 python3.11 \
+                python3.13 \
                 python3    \
                 python     \
                 "
