@@ -4,37 +4,51 @@
  */
 
 /*
- * Copyright (C) 2006-2016 Oracle Corporation
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
-#ifndef ____H_DISPLAYIMPL
-#define ____H_DISPLAYIMPL
+#ifndef MAIN_INCLUDED_DisplayImpl_h
+#define MAIN_INCLUDED_DisplayImpl_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include "SchemaDefs.h"
 
 #include <iprt/semaphore.h>
 #include <VBox/vmm/pdmdrv.h>
-#include <VBox/VMMDev.h>
-#include <VBox/VBoxVideo.h>
+#include <VBoxVideo.h>
 #include <VBox/vmm/pdmifs.h>
+#include <VBox/VMMDev.h>  /* For struct VMMDevDisplayDef - why is it in that file? */
 #include "DisplayWrap.h"
 
-#ifdef VBOX_WITH_CROGL
-# include <VBox/HostServices/VBoxCrOpenGLSvc.h>
+#include "DisplaySourceBitmapWrap.h"
+#include "GuestScreenInfoWrap.h"
+
+#ifdef VBOX_WITH_RECORDING
+# include "RecordingInternals.h"
+class RecordingContext;
 #endif
 
-#include "DisplaySourceBitmapWrap.h"
-
 class Console;
-struct VIDEORECCONTEXT;
 
 typedef struct _DISPLAYFBINFO
 {
@@ -80,32 +94,17 @@ typedef struct _DISPLAYFBINFO
 #ifdef VBOX_WITH_HGSMI
     bool fVBVAEnabled;
     bool fVBVAForceResize;
-    bool fRenderThreadMode;
-    PVBVAHOSTFLAGS pVBVAHostFlags;
+    VBVAHOSTFLAGS RT_UNTRUSTED_VOLATILE_GUEST *pVBVAHostFlags;
 #endif /* VBOX_WITH_HGSMI */
 
-#ifdef VBOX_WITH_CROGL
-    struct
-    {
-        bool fPending;
-        ULONG x;
-        ULONG y;
-        ULONG width;
-        ULONG height;
-    } pendingViewportInfo;
-#endif /* VBOX_WITH_CROGL */
-
-#ifdef VBOX_WITH_VPX
-    struct
-    {
-        ComPtr<IDisplaySourceBitmap> pSourceBitmap;
-    } videoCapture;
-#endif
+    /** Description of the currently plugged monitor with preferred mode,
+     * a.k.a the last mode hint sent. */
+    struct VMMDevDisplayDef monitorDesc;
 } DISPLAYFBINFO;
 
 /* The legacy VBVA (VideoAccel) data.
  *
- * Backward compatibility with the guest additions 3.x or older.
+ * Backward compatibility with the Guest Additions 3.x or older.
  */
 typedef struct VIDEOACCEL
 {
@@ -115,7 +114,7 @@ typedef struct VIDEOACCEL
     uint8_t    *pu8VbvaPartial;
     uint32_t    cbVbvaPartial;
 
-    /* Old guest additions (3.x and older) use both VMMDev and DevVGA refresh timer
+    /* Old Guest Additions (3.x and older) use both VMMDev and DevVGA refresh timer
      * to process the VBVABUFFER memory. Therefore the legacy VBVA (VideoAccel) host
      * code can be executed concurrently by VGA refresh timer and the guest VMMDev
      * request in SMP VMs. The semaphore serialized this.
@@ -127,12 +126,13 @@ typedef struct VIDEOACCEL
 class DisplayMouseInterface
 {
 public:
+    virtual ~DisplayMouseInterface() { }
     virtual HRESULT i_getScreenResolution(ULONG cScreen, ULONG *pcx,
                                           ULONG *pcy, ULONG *pcBPP, LONG *pXOrigin, LONG *pYOrigin) = 0;
     virtual void i_getFramebufferDimensions(int32_t *px1, int32_t *py1,
                                             int32_t *px2, int32_t *py2) = 0;
     virtual HRESULT i_reportHostCursorCapabilities(uint32_t fCapabilitiesAdded, uint32_t fCapabilitiesRemoved) = 0;
-    virtual HRESULT i_reportHostCursorPosition(int32_t x, int32_t y) = 0;
+    virtual HRESULT i_reportHostCursorPosition(int32_t x, int32_t y, bool fOutOfRange) = 0;
     virtual bool i_isInputMappingSet(void) = 0;
 };
 
@@ -144,7 +144,7 @@ class ATL_NO_VTABLE Display :
 {
 public:
 
-    DECLARE_EMPTY_CTOR_DTOR(Display)
+    DECLARE_COMMON_CLASS_METHODS(Display)
 
     HRESULT FinalConstruct();
     void FinalRelease();
@@ -155,44 +155,26 @@ public:
     int  i_registerSSM(PUVM pUVM);
 
     // public methods only for internal purposes
-    int  i_handleDisplayResize(unsigned uScreenId, uint32_t bpp, void *pvVRAM, uint32_t cbLine,
-                               uint32_t w, uint32_t h, uint16_t flags);
+    unsigned i_getMonitorCount() { return mcMonitors; }
+    int i_handleDisplayResize(unsigned uScreenId, uint32_t bpp, void *pvVRAM,
+                              uint32_t cbLine, uint32_t w, uint32_t h, uint16_t flags,
+                              int32_t xOrigin, int32_t yOrigin, bool fVGAResize);
     void i_handleDisplayUpdate(unsigned uScreenId, int x, int y, int w, int h);
     void i_handleUpdateVMMDevSupportsGraphics(bool fSupportsGraphics);
     void i_handleUpdateGuestVBVACapabilities(uint32_t fNewCapabilities);
     void i_handleUpdateVBVAInputMapping(int32_t xOrigin, int32_t yOrigin, uint32_t cx, uint32_t cy);
 #ifdef VBOX_WITH_VIDEOHWACCEL
-    int  i_handleVHWACommandProcess(PVBOXVHWACMD pCommand);
+    int  i_handleVHWACommandProcess(int enmCmd, bool fGuestCmd, VBOXVHWACMD RT_UNTRUSTED_VOLATILE_GUEST *pCommand);
 #endif
-#ifdef VBOX_WITH_CRHGSMI
-    void i_handleCrHgsmiCommandCompletion(int32_t result, uint32_t u32Function, PVBOXHGCMSVCPARM pParam);
-    void i_handleCrHgsmiControlCompletion(int32_t result, uint32_t u32Function, PVBOXHGCMSVCPARM pParam);
-    void i_handleCrHgsmiCommandProcess(PVBOXVDMACMD_CHROMIUM_CMD pCmd, uint32_t cbCmd);
-    void i_handleCrHgsmiControlProcess(PVBOXVDMACMD_CHROMIUM_CTL pCtl, uint32_t cbCtl);
-#endif
-#if defined(VBOX_WITH_HGCM) && defined(VBOX_WITH_CROGL)
-    int  i_handleCrHgcmCtlSubmit(struct VBOXCRCMDCTL* pCmd, uint32_t cbCmd,
-                                 PFNCRCTLCOMPLETION pfnCompletion,
-                                 void *pvCompletion);
-    void  i_handleCrVRecScreenshotPerform(uint32_t uScreen,
-                                          uint32_t x, uint32_t y, uint32_t uPixelFormat, uint32_t uBitsPerPixel,
-                                          uint32_t uBytesPerLine, uint32_t uGuestWidth, uint32_t uGuestHeight,
-                                          uint8_t *pu8BufferAddress, uint64_t u64TimeStamp);
-    /** @todo r=bird: u64TimeStamp - using the 'u64' prefix add nothing.
-     *        However, using one of the prefixes indicating the timestamp unit
-     *        would be very valuable!  */
-    bool i_handleCrVRecScreenshotBegin(uint32_t uScreen, uint64_t u64TimeStamp);
-    void i_handleCrVRecScreenshotEnd(uint32_t uScreen, uint64_t u64TimeStamp);
-    void i_handleVRecCompletion();
-#endif
-
-    int i_notifyCroglResize(PCVBVAINFOVIEW pView, PCVBVAINFOSCREEN pScreen, void *pvVRAM);
+    int  i_handle3DNotifyProcess(VBOX3DNOTIFY *p3DNotify);
 
     int  i_saveVisibleRegion(uint32_t cRect, PRTRECT pRect);
     int  i_handleSetVisibleRegion(uint32_t cRect, PRTRECT pRect);
+    int  i_handleUpdateMonitorPositions(uint32_t cPositions, PCRTPOINT paPositions);
     int  i_handleQueryVisibleRegion(uint32_t *pcRects, PRTRECT paRects);
 
-    void i_VideoAccelVRDP(bool fEnable);
+    void i_VRDPConnectionEvent(bool fConnect);
+    void i_VideoAccelVRDP(bool fEnable, int c);
 
     /* Legacy video acceleration requests coming from the VGA refresh timer. */
     int  VideoAccelEnableVGA(bool fEnable, VBVAMEMORY *pVbvaMemory);
@@ -201,11 +183,16 @@ public:
     int  VideoAccelEnableVMMDev(bool fEnable, VBVAMEMORY *pVbvaMemory);
     void VideoAccelFlushVMMDev(void);
 
-    int  i_VideoCaptureStart();
-    void i_VideoCaptureStop();
-    int  i_VideoCaptureEnableScreens(ComSafeArrayIn(BOOL, aScreens));
-#ifdef VBOX_WITH_VPX
-    void videoCaptureScreenChanged(unsigned uScreenId);
+    void i_updateDeviceCursorCapabilities(void);
+
+#ifdef VBOX_WITH_RECORDING
+    int i_recordingStart(void);
+    int i_recordingStop(void);
+    int i_recordingInvalidate(void);
+    int i_recordingScreenChanged(unsigned uScreenId, const DISPLAYFBINFO *pFBInfo);
+    int i_recordingScreenUpdate(unsigned uScreenId, uint8_t *pauFramebuffer, size_t cbFramebuffer, uint32_t x, uint32_t y, uint32_t w, uint32_t h, uint32_t uBytesPerLine);
+    int i_recordingScreenUpdate(unsigned uScreenId, uint32_t x, uint32_t y, uint32_t w, uint32_t h);
+    int i_recordingCursorPositionChange(unsigned uScreenId, uint32_t fFlags, int32_t x, int32_t y);
 #endif
 
     void i_notifyPowerDown(void);
@@ -219,7 +206,7 @@ public:
     virtual void i_getFramebufferDimensions(int32_t *px1, int32_t *py1,
                                             int32_t *px2, int32_t *py2);
     virtual HRESULT i_reportHostCursorCapabilities(uint32_t fCapabilitiesAdded, uint32_t fCapabilitiesRemoved);
-    virtual HRESULT i_reportHostCursorPosition(int32_t x, int32_t y);
+    virtual HRESULT i_reportHostCursorPosition(int32_t x, int32_t y, bool fOutOfRange);
     virtual bool i_isInputMappingSet(void)
     {
         return cxInputMapping != 0 && cyInputMapping != 0;
@@ -253,7 +240,16 @@ private:
                                      LONG aOriginY,
                                      ULONG aWidth,
                                      ULONG aHeight,
-                                     ULONG aBitsPerPixel);
+                                     ULONG aBitsPerPixel,
+                                     BOOL aNotify);
+    virtual HRESULT getVideoModeHint(ULONG aDisplay,
+                                     BOOL *aEnabled,
+                                     BOOL *aChangeOrigin,
+                                     LONG *aOriginX,
+                                     LONG *aOriginY,
+                                     ULONG *aWidth,
+                                     ULONG *aHeight,
+                                     ULONG *aBitsPerPixel);
     virtual HRESULT setSeamlessMode(BOOL aEnabled);
     virtual HRESULT takeScreenShot(ULONG aScreenId,
                                    BYTE *aAddress,
@@ -287,6 +283,17 @@ private:
     virtual HRESULT notifyHiDPIOutputPolicyChange(BOOL fUnscaledHiDPI);
     virtual HRESULT setScreenLayout(ScreenLayoutMode_T aScreenLayoutMode,
                                     const std::vector<ComPtr<IGuestScreenInfo> > &aGuestScreenInfo);
+    virtual HRESULT detachScreens(const std::vector<LONG> &aScreenIds);
+    virtual HRESULT createGuestScreenInfo(ULONG aDisplay,
+                                          GuestMonitorStatus_T aStatus,
+                                          BOOL aPrimary,
+                                          BOOL aChangeOrigin,
+                                          LONG aOriginX,
+                                          LONG aOriginY,
+                                          ULONG aWidth,
+                                          ULONG aHeight,
+                                          ULONG aBitsPerPixel,
+                                          ComPtr<IGuestScreenInfo> &aGuestScreenInfo);
 
     // Wrapped IEventListener properties
 
@@ -300,19 +307,12 @@ private:
                                  ULONG aHeight,
                                  BitmapFormat_T aBitmapFormat,
                                  ULONG *pcbOut);
-
-#ifdef VBOX_WITH_CRHGSMI
-    void i_setupCrHgsmiData(void);
-    void i_destructCrHgsmiData(void);
-#endif
-
-#if defined(VBOX_WITH_HGCM) && defined(VBOX_WITH_CROGL)
-    int i_crViewportNotify(ULONG aScreenId, ULONG x, ULONG y, ULONG width, ULONG height);
-#endif
+    int processVBVAResize(PCVBVAINFOVIEW pView, PCVBVAINFOSCREEN pScreen, void *pvVRAM, bool fResetInputMapping);
 
     static DECLCALLBACK(void*) i_drvQueryInterface(PPDMIBASE pInterface, const char *pszIID);
     static DECLCALLBACK(int)   i_drvConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfg, uint32_t fFlags);
     static DECLCALLBACK(void)  i_drvDestruct(PPDMDRVINS pDrvIns);
+    static DECLCALLBACK(void)  i_drvPowerOff(PPDMDRVINS pDrvIns);
     static DECLCALLBACK(int)   i_displayResizeCallback(PPDMIDISPLAYCONNECTOR pInterface, uint32_t bpp, void *pvVRAM,
                                                        uint32_t cbLine, uint32_t cx, uint32_t cy);
     static DECLCALLBACK(void)  i_displayUpdateCallback(PPDMIDISPLAYCONNECTOR pInterface,
@@ -326,35 +326,19 @@ private:
                                                                    void *pvVRAM, unsigned uScreenId);
 
 #ifdef VBOX_WITH_VIDEOHWACCEL
-    static DECLCALLBACK(int)  i_displayVHWACommandProcess(PPDMIDISPLAYCONNECTOR pInterface, PVBOXVHWACMD pCommand);
+    static DECLCALLBACK(int)  i_displayVHWACommandProcess(PPDMIDISPLAYCONNECTOR pInterface, int enmCmd, bool fGuestCmd,
+                                                          VBOXVHWACMD RT_UNTRUSTED_VOLATILE_GUEST *pCommand);
 #endif
+    static DECLCALLBACK(int)  i_display3DNotifyProcess(PPDMIDISPLAYCONNECTOR pInterface,
+                                                       VBOX3DNOTIFY *p3DNotify);
 
-#ifdef VBOX_WITH_CRHGSMI
-    static DECLCALLBACK(void)  i_displayCrHgsmiCommandProcess(PPDMIDISPLAYCONNECTOR pInterface,
-                                                              PVBOXVDMACMD_CHROMIUM_CMD pCmd, uint32_t cbCmd);
-    static DECLCALLBACK(void)  i_displayCrHgsmiControlProcess(PPDMIDISPLAYCONNECTOR pInterface, PVBOXVDMACMD_CHROMIUM_CTL pCtl,
-                                                              uint32_t cbCtl);
-
-    static DECLCALLBACK(void)  i_displayCrHgsmiCommandCompletion(int32_t result, uint32_t u32Function, PVBOXHGCMSVCPARM pParam,
-                                                                 void *pvContext);
-    static DECLCALLBACK(void)  i_displayCrHgsmiControlCompletion(int32_t result, uint32_t u32Function, PVBOXHGCMSVCPARM pParam,
-                                                                 void *pvContext);
-#endif
-#if defined(VBOX_WITH_HGCM) && defined(VBOX_WITH_CROGL)
-    static DECLCALLBACK(int)  i_displayCrHgcmCtlSubmit(PPDMIDISPLAYCONNECTOR pInterface,
-                                                       struct VBOXCRCMDCTL* pCmd, uint32_t cbCmd,
-                                                       PFNCRCTLCOMPLETION pfnCompletion,
-                                                       void *pvCompletion);
-    static DECLCALLBACK(void) i_displayCrHgcmCtlSubmitCompletion(int32_t result, uint32_t u32Function, PVBOXHGCMSVCPARM pParam,
-                                                                 void *pvContext);
-#endif
 #ifdef VBOX_WITH_HGSMI
     static DECLCALLBACK(int)   i_displayVBVAEnable(PPDMIDISPLAYCONNECTOR pInterface, unsigned uScreenId,
-                                                   PVBVAHOSTFLAGS pHostFlags, bool fRenderThreadMode);
+                                                   VBVAHOSTFLAGS RT_UNTRUSTED_VOLATILE_GUEST *pHostFlags);
     static DECLCALLBACK(void)  i_displayVBVADisable(PPDMIDISPLAYCONNECTOR pInterface, unsigned uScreenId);
     static DECLCALLBACK(void)  i_displayVBVAUpdateBegin(PPDMIDISPLAYCONNECTOR pInterface, unsigned uScreenId);
     static DECLCALLBACK(void)  i_displayVBVAUpdateProcess(PPDMIDISPLAYCONNECTOR pInterface, unsigned uScreenId,
-                                                          PCVBVACMDHDR pCmd, size_t cbCmd);
+                                                          struct VBVACMDHDR const RT_UNTRUSTED_VOLATILE_GUEST *pCmd, size_t cbCmd);
     static DECLCALLBACK(void)  i_displayVBVAUpdateEnd(PPDMIDISPLAYCONNECTOR pInterface, unsigned uScreenId, int32_t x, int32_t y,
                                                       uint32_t cx, uint32_t cy);
     static DECLCALLBACK(int)   i_displayVBVAResize(PPDMIDISPLAYCONNECTOR pInterface, PCVBVAINFOVIEW pView,
@@ -367,33 +351,19 @@ private:
 
     static DECLCALLBACK(void)  i_displayVBVAInputMappingUpdate(PPDMIDISPLAYCONNECTOR pInterface, int32_t xOrigin, int32_t yOrigin,
                                                                uint32_t cx, uint32_t cy);
+    static DECLCALLBACK(void)  i_displayVBVAReportCursorPosition(PPDMIDISPLAYCONNECTOR pInterface, uint32_t fFlags, uint32_t uScreen, uint32_t x, uint32_t y);
 #endif
 
-#if defined(VBOX_WITH_HGCM) && defined(VBOX_WITH_CROGL)
-    static DECLCALLBACK(void) i_displayCrVRecScreenshotPerform(void *pvCtx, uint32_t uScreen,
-                                                               uint32_t x, uint32_t y,
-                                                               uint32_t uBitsPerPixel, uint32_t uBytesPerLine,
-                                                               uint32_t uGuestWidth, uint32_t uGuestHeight,
-                                                               uint8_t *pu8BufferAddress, uint64_t u64TimeStamp);
-    static DECLCALLBACK(bool) i_displayCrVRecScreenshotBegin(void *pvCtx, uint32_t uScreen, uint64_t u64TimeStamp);
-    static DECLCALLBACK(void) i_displayCrVRecScreenshotEnd(void *pvCtx, uint32_t uScreen, uint64_t u64TimeStamp);
-
-    static DECLCALLBACK(void) i_displayVRecCompletion(struct VBOXCRCMDCTL* pCmd, uint32_t cbCmd, int rc, void *pvCompletion);
-#endif
-    static DECLCALLBACK(void) i_displayCrCmdFree(struct VBOXCRCMDCTL* pCmd, uint32_t cbCmd, int rc, void *pvCompletion);
-
-    static DECLCALLBACK(void) i_displaySSMSaveScreenshot(PSSMHANDLE pSSM, void *pvUser);
-    static DECLCALLBACK(int)  i_displaySSMLoadScreenshot(PSSMHANDLE pSSM, void *pvUser, uint32_t uVersion, uint32_t uPass);
-    static DECLCALLBACK(void) i_displaySSMSave(PSSMHANDLE pSSM, void *pvUser);
-    static DECLCALLBACK(int)  i_displaySSMLoad(PSSMHANDLE pSSM, void *pvUser, uint32_t uVersion, uint32_t uPass);
+    static DECLCALLBACK(int)  i_displaySSMSaveScreenshot(PSSMHANDLE pSSM, PCVMMR3VTABLE pVMM, void *pvUser);
+    static DECLCALLBACK(int)  i_displaySSMLoadScreenshot(PSSMHANDLE pSSM, PCVMMR3VTABLE pVMM, void *pvUser,
+                                                         uint32_t uVersion, uint32_t uPass);
+    static DECLCALLBACK(int)  i_displaySSMSave(PSSMHANDLE pSSM, PCVMMR3VTABLE pVMM, void *pvUser);
+    static DECLCALLBACK(int)  i_displaySSMLoad(PSSMHANDLE pSSM, PCVMMR3VTABLE pVMM, void *pvUser,
+                                               uint32_t uVersion, uint32_t uPass);
 
     Console * const         mParent;
     /** Pointer to the associated display driver. */
     struct DRVMAINDISPLAY   *mpDrv;
-    /** Pointer to the device instance for the VMM Device. */
-    PPDMDEVINS              mpVMMDev;
-    /** Set after the first attempt to find the VMM Device. */
-    bool                    mfVMMDevInited;
 
     unsigned mcMonitors;
     /** Input mapping rectangle top left X relative to the first screen. */
@@ -423,26 +393,8 @@ private:
 
     bool        mfVideoAccelVRDP;
     uint32_t    mfu32SupportedOrders;
-    int32_t volatile mcVideoAccelVRDPRefs;
-
-    /** Accelerate3DEnabled = true && GraphicsControllerType == VBoxVGA. */
-    bool        mfIsCr3DEnabled;
-
-#ifdef VBOX_WITH_CROGL
-    bool        mfCrOglDataHidden;
-#endif
-
-#ifdef VBOX_WITH_CRHGSMI
-    /* for fast host hgcm calls */
-    HGCMCVSHANDLE mhCrOglSvc;
-    RTCRITSECTRW mCrOglLock;
-#endif
-#ifdef VBOX_WITH_CROGL
-    CR_MAIN_INTERFACE mCrOglCallbacks;
-    volatile uint32_t mfCrOglVideoRecState;
-    CRVBOXHGCMTAKESCREENSHOT mCrOglScreenshotData;
-    VBOXCRCMDCTL_HGCM mCrOglScreenshotCtl;
-#endif
+    /** Number of currently connected VRDP clients. */
+    int32_t volatile mcVRDPRefs;
 
     /* The legacy VBVA data and methods. */
     VIDEOACCEL mVideoAccelLegacy;
@@ -459,54 +411,63 @@ private:
     void processAdapterData(void *pvVRAM, uint32_t u32VRAMSize);
     void processDisplayData(void *pvVRAM, unsigned uScreenId);
 
-    /* Serializes access to mVideoAccelLegacy and mfVideoAccelVRDP, etc between VRDP and Display. */
-    RTCRITSECT mVideoAccelLock;
-#ifdef VBOX_WITH_VPX
-    /* Serializes access to video capture source bitmaps. */
-    RTCRITSECT mVideoCaptureLock;
+    /** Serializes access to mVideoAccelLegacy and mfVideoAccelVRDP, etc between VRDP and Display. */
+    RTCRITSECT           mVideoAccelLock;
+
+#ifdef VBOX_WITH_RECORDING
+    /** Struct which holds information and state about (video) recording. */
+    struct Recording
+    {
+        Recording()
+            : pCtx(NULL) { }
+
+        /** Recording context. Constant across lifetime.
+         *  Might be NULL if not being used. */
+        RecordingContext * const pCtx;
+    } Recording;
+#endif
+
+#ifdef VBOX_WITH_STATISTICS
+    /** Struct for keeping STAM values. */
+    struct
+    {
+        /** Profiling Display::i_displayRefreshCallback(). */
+        STAMPROFILE profileDisplayRefreshCallback;
+        /** Statistics for monitor N. */
+        struct
+        {
+            /** Statistics for recording of monitor N. */
+            struct
+            {
+                /** Profiling recording code of monitor N. */
+                STAMPROFILE profileRecording;
+            } Recording;
+        } Monitor[SchemaDefs::MaxGuestMonitors];
+        /** Statistics for audio/video recording. */
+        struct
+        {
+            /** Profiling recording code of all active monitors. */
+            STAMPROFILE profileRecording;
+        } Recording;
+    } Stats;
 #endif
 
 public:
 
-    static int i_displayTakeScreenshotEMT(Display *pDisplay, ULONG aScreenId, uint8_t **ppbData, size_t *pcbData,
-                                          uint32_t *pcx, uint32_t *pcy, bool *pfMemFree);
-#if defined(VBOX_WITH_HGCM) && defined(VBOX_WITH_CROGL)
-    static BOOL  i_displayCheckTakeScreenshotCrOgl(Display *pDisplay, ULONG aScreenId, uint8_t *pbData,
-                                                   uint32_t u32Width, uint32_t u32Height);
-    int i_crCtlSubmit(struct VBOXCRCMDCTL* pCmd, uint32_t cbCmd, PFNCRCTLCOMPLETION pfnCompletion, void *pvCompletion);
-    int i_crCtlSubmitSync(struct VBOXCRCMDCTL* pCmd, uint32_t cbCmd);
-    /* copies the given command and submits it asynchronously,
-     * i.e. the pCmd data may be discarded right after the call returns */
-    int i_crCtlSubmitAsyncCmdCopy(struct VBOXCRCMDCTL* pCmd, uint32_t cbCmd);
-    /* performs synchronous request processing if 3D backend has something to display
-     * this is primarily to work-around 3d<->main thread deadlocks on OSX
-     * in case of async completion, the command is coppied to the allocated buffer,
-     * freeded on command completion
-     * can be used for "notification" commands, when client is not interested in command result,
-     * that must synchronize with 3D backend only when some 3D data is displayed.
-     * The routine does NOT provide any info on whether command is processed asynchronously or not */
-    int i_crCtlSubmitSyncIfHasDataForScreen(uint32_t u32ScreenID, struct VBOXCRCMDCTL* pCmd, uint32_t cbCmd);
-#endif
+    static DECLCALLBACK(int) i_displayTakeScreenshotEMT(Display *pDisplay, ULONG aScreenId, uint8_t **ppbData, size_t *pcbData,
+                                                        uint32_t *pcx, uint32_t *pcy, bool *pfMemFree);
 
 private:
-    static int i_InvalidateAndUpdateEMT(Display *pDisplay, unsigned uId, bool fUpdateAll);
-    static int i_drawToScreenEMT(Display *pDisplay, ULONG aScreenId, BYTE *address, ULONG x, ULONG y, ULONG width, ULONG height);
+    static DECLCALLBACK(int) i_InvalidateAndUpdateEMT(Display *pDisplay, unsigned uId, bool fUpdateAll);
+    static DECLCALLBACK(int) i_drawToScreenEMT(Display *pDisplay, ULONG aScreenId, BYTE *address, ULONG x, ULONG y,
+                                               ULONG width, ULONG height);
 
     void i_updateGuestGraphicsFacility(void);
 
-#if defined(VBOX_WITH_HGCM) && defined(VBOX_WITH_CROGL)
-    int i_crOglWindowsShow(bool fShow);
-#endif
-
 #ifdef VBOX_WITH_HGSMI
     volatile uint32_t mu32UpdateVBVAFlags;
 #endif
 
-#ifdef VBOX_WITH_VPX
-    VIDEORECCONTEXT *mpVideoRecCtx;
-    bool maVideoRecEnabled[SchemaDefs::MaxGuestMonitors];
-#endif
-
 private:
     DECLARE_CLS_COPY_CTOR_ASSIGN_NOOP(Display); /* Shuts up MSC warning C4625. */
 };
@@ -540,7 +501,7 @@ class ATL_NO_VTABLE DisplaySourceBitmap:
 {
 public:
 
-    DECLARE_EMPTY_CTOR_DTOR(DisplaySourceBitmap)
+    DECLARE_COMMON_CLASS_METHODS(DisplaySourceBitmap)
 
     HRESULT FinalConstruct();
     void FinalRelease();
@@ -584,5 +545,51 @@ private:
     Data m;
 };
 
-#endif // ____H_DISPLAYIMPL
+class ATL_NO_VTABLE GuestScreenInfo:
+    public GuestScreenInfoWrap
+{
+public:
+
+    DECLARE_COMMON_CLASS_METHODS(GuestScreenInfo)
+
+    HRESULT FinalConstruct();
+    void FinalRelease();
+
+    /* Public initializer/uninitializer for internal purposes only. */
+    HRESULT init(ULONG aDisplay,
+                 GuestMonitorStatus_T aGuestMonitorStatus,
+                 BOOL aPrimary,
+                 BOOL aChangeOrigin,
+                 LONG aOriginX,
+                 LONG aOriginY,
+                 ULONG aWidth,
+                 ULONG aHeight,
+                 ULONG aBitsPerPixel);
+    void uninit();
+
+private:
+    // wrapped IGuestScreenInfo properties
+    virtual HRESULT getScreenId(ULONG *aScreenId);
+    virtual HRESULT getGuestMonitorStatus(GuestMonitorStatus_T *aGuestMonitorStatus);
+    virtual HRESULT getPrimary(BOOL *aPrimary);
+    virtual HRESULT getOrigin(BOOL *aOrigin);
+    virtual HRESULT getOriginX(LONG *aOriginX);
+    virtual HRESULT getOriginY(LONG *aOriginY);
+    virtual HRESULT getWidth(ULONG *aWidth);
+    virtual HRESULT getHeight(ULONG *aHeight);
+    virtual HRESULT getBitsPerPixel(ULONG *aBitsPerPixel);
+    virtual HRESULT getExtendedInfo(com::Utf8Str &aExtendedInfo);
+
+    ULONG mScreenId;
+    GuestMonitorStatus_T mGuestMonitorStatus;
+    BOOL  mPrimary;
+    BOOL  mOrigin;
+    LONG  mOriginX;
+    LONG  mOriginY;
+    ULONG mWidth;
+    ULONG mHeight;
+    ULONG mBitsPerPixel;
+};
+
+#endif /* !MAIN_INCLUDED_DisplayImpl_h */
 /* vi: set tabstop=4 shiftwidth=4 expandtab: */
