@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2014-2019 Oracle Corporation
+ * Copyright (C) 2014-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 #ifndef MAIN_INCLUDED_GuestDnDTargetImpl_h
@@ -27,20 +37,22 @@
 #include <VBox/GuestHost/DragAndDrop.h>
 #include <VBox/HostServices/DragAndDropSvc.h>
 
-struct SENDDATACTX;
-typedef struct SENDDATACTX *PSENDDATACTX;
-class SendDataTask;
+struct GuestDnDSendCtx;
+class GuestDnDSendDataTask;
 
 class ATL_NO_VTABLE GuestDnDTarget :
     public GuestDnDTargetWrap,
     public GuestDnDBase
 {
 public:
+    GuestDnDTarget(void);
+    virtual ~GuestDnDTarget(void);
+
     /** @name COM and internal init/term/mapping cruft.
      * @{ */
-    DECLARE_EMPTY_CTOR_DTOR(GuestDnDTarget)
+    DECLARE_TRANSLATE_METHODS(GuestDnDTarget);
 
-    int     init(const ComObjPtr<Guest>& pGuest);
+    HRESULT init(const ComObjPtr<Guest>& pGuest);
     void    uninit(void);
 
     HRESULT FinalConstruct(void);
@@ -55,8 +67,6 @@ private:
     HRESULT getFormats(GuestDnDMIMEList &aFormats);
     HRESULT addFormats(const GuestDnDMIMEList &aFormats);
     HRESULT removeFormats(const GuestDnDMIMEList &aFormats);
-
-    HRESULT getProtocolVersion(ULONG *aProtocolVersion);
     /** @}  */
 
     /** Private wrapped @name IDnDTarget methods.
@@ -74,38 +84,44 @@ protected:
     static Utf8Str i_guestErrorToString(int guestRc);
     static Utf8Str i_hostErrorToString(int hostRc);
 
-    /** @name Thread task workers.
-     * @{ */
-    static void i_sendDataThreadTask(SendDataTask *pTask);
-    /** @}  */
-
     /** @name Callbacks for dispatch handler.
      * @{ */
-    static DECLCALLBACK(int) i_sendURIDataCallback(uint32_t uMsg, void *pvParms, size_t cbParms, void *pvUser);
+    static DECLCALLBACK(int) i_sendTransferDataCallback(uint32_t uMsg, void *pvParms, size_t cbParms, void *pvUser);
     /** @}  */
 
 protected:
 
-    int i_sendData(PSENDDATACTX pCtx, RTMSINTERVAL msTimeout);
-    int i_sendDataBody(PSENDDATACTX pCtx, GuestDnDData *pData);
-    int i_sendDataHeader(PSENDDATACTX pCtx, GuestDnDData *pData, GuestDnDURIData *pURIData /* = NULL */);
-    int i_sendDirectory(PSENDDATACTX pCtx, GuestDnDURIObjCtx *pObjCtx, GuestDnDMsg *pMsg);
-    int i_sendFile(PSENDDATACTX pCtx, GuestDnDURIObjCtx *pObjCtx, GuestDnDMsg *pMsg);
-    int i_sendFileData(PSENDDATACTX pCtx, GuestDnDURIObjCtx *pObjCtx, GuestDnDMsg *pMsg);
-    int i_sendRawData(PSENDDATACTX pCtx, RTMSINTERVAL msTimeout);
-    int i_sendURIData(PSENDDATACTX pCtx, RTMSINTERVAL msTimeout);
-    int i_sendURIDataLoop(PSENDDATACTX pCtx, GuestDnDMsg *pMsg);
+    /** @name Implemented virtual functions.
+     * @{ */
+    void i_reset(void);
+    /** @}  */
+
+    int i_sendData(GuestDnDSendCtx *pCtx, RTMSINTERVAL msTimeout);
+
+    int i_sendMetaDataBody(GuestDnDSendCtx *pCtx);
+    int i_sendMetaDataHeader(GuestDnDSendCtx *pCtx);
+
+    int i_sendTransferData(GuestDnDSendCtx *pCtx, RTMSINTERVAL msTimeout);
+    int i_sendTransferListObject(GuestDnDSendCtx *pCtx,  PDNDTRANSFERLIST pList, GuestDnDMsg *pMsg);
+
+    int i_sendDirectory(GuestDnDSendCtx *pCtx, PDNDTRANSFEROBJECT pObj, GuestDnDMsg *pMsg);
+    int i_sendFile(GuestDnDSendCtx *pCtx, PDNDTRANSFEROBJECT pObj, GuestDnDMsg *pMsg);
+    int i_sendFileData(GuestDnDSendCtx *pCtx, PDNDTRANSFEROBJECT pObj, GuestDnDMsg *pMsg);
+
+    int i_sendRawData(GuestDnDSendCtx *pCtx, RTMSINTERVAL msTimeout);
 
 protected:
 
     struct
     {
-        bool     mfTransferIsPending;
         /** Maximum data block size (in bytes) the target can handle. */
-        uint32_t mcbBlockSize;
+        uint32_t        mcbBlockSize;
+        /** The context for sending data to the guest.
+         *  At the moment only one transfer at a time is supported. */
+        GuestDnDSendCtx mSendCtx;
     } mData;
 
-    friend class SendDataTask;
+    friend class GuestDnDSendDataTask;
 };
 
 #endif /* !MAIN_INCLUDED_GuestDnDTargetImpl_h */
