@@ -5,19 +5,37 @@
  */
 
 /*
- * Copyright (C) 2006-2013 Oracle Corporation
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
-#ifndef ____H_HOSTPOWER
-#define ____H_HOSTPOWER
+#ifndef MAIN_INCLUDED_HostPower_h
+#define MAIN_INCLUDED_HostPower_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
+
+#ifdef RT_OS_DARWIN /* first, so we can undef pVM in iprt/cdefs.h */
+# include <IOKit/pwr_mgt/IOPMLib.h>
+# include <Carbon/Carbon.h>
+#endif
 
 #include "VirtualBoxBase.h"
 
@@ -27,11 +45,6 @@
 # include <VBox/dbus.h>
 #endif
 
-#ifdef RT_OS_DARWIN
-# include <IOKit/pwr_mgt/IOPMLib.h>
-# include <Carbon/Carbon.h>
-#endif /* RT_OS_DARWIN */
-
 class HostPowerService
 {
   public:
@@ -44,7 +57,7 @@ class HostPowerService
     std::vector<ComPtr<IInternalSessionControl> > mSessionControls;
 };
 
-# ifdef RT_OS_WINDOWS
+# if defined(RT_OS_WINDOWS) || defined(DOXYGEN_RUNNING)
 /**
  * The Windows hosted Power Service.
  */
@@ -63,7 +76,8 @@ private:
     HWND        mHwnd;
     RTTHREAD    mThread;
 };
-#elif defined(RT_OS_LINUX)
+# endif
+# if defined(RT_OS_LINUX) || defined(DOXYGEN_RUNNING)
 /**
  * The Linux hosted Power Service.
  */
@@ -86,7 +100,8 @@ private:
     DBusConnection *mpConnection;
 };
 
-# elif defined(RT_OS_DARWIN) /* RT_OS_WINDOWS */
+# endif
+# if defined(RT_OS_DARWIN) || defined(DOXYGEN_RUNNING)
 /**
  * The Darwin hosted Power Service.
  */
@@ -115,7 +130,7 @@ private:
 
     bool mCritical; /* Indicate if the battery was in the critical state last checked */
 };
-# endif /* RT_OS_DARWIN */
+# endif
 
-#endif /* !____H_HOSTPOWER */
+#endif /* !MAIN_INCLUDED_HostPower_h */
 /* vi: set tabstop=4 shiftwidth=4 expandtab: */
