@@ -4,18 +4,32 @@
  */
 
 /*
- * Copyright (C) 2008-2015 Oracle Corporation
+ * Copyright (C) 2008-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
-#ifndef ___performance_h
-#define ___performance_h
+
+#ifndef MAIN_INCLUDED_Performance_h
+#define MAIN_INCLUDED_Performance_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include <VBox/com/defs.h>
 #include <VBox/com/ptr.h>
@@ -275,12 +289,12 @@ namespace pm
                          ULONG aAllocVMM, ULONG aFreeVMM,
                          ULONG aBalloonedVMM, ULONG aSharedVMM,
                          ULONG aVmNetRx, ULONG aVmNetTx);
-        int enable(ULONG mask);
-        int disable(ULONG mask);
+        HRESULT enable(ULONG mask);
+        HRESULT disable(ULONG mask);
 
-        int enqueueRequest(CollectorGuestRequest *aRequest);
+        HRESULT enqueueRequest(CollectorGuestRequest *aRequest);
         HRESULT enableInternal(ULONG mask);
-        int disableInternal(ULONG mask);
+        HRESULT disableInternal(ULONG mask);
 
         const com::Utf8Str& getVMName() const { return mMachineName; };
 
@@ -302,7 +316,7 @@ namespace pm
         ULONG getVmNetTx()      { return mVmNetTx; };
 
     private:
-        int enableVMMStats(bool mCollectVMMStats);
+        HRESULT enableVMMStats(bool mCollectVMMStats);
 
         CollectorGuestManager *mManager;
 
@@ -342,7 +356,7 @@ namespace pm
         CollectorGuest *getVMMStatsProvider() { return mVMMStatsProvider; };
         void preCollect(CollectorHints& hints, uint64_t iTick);
         void destroyUnregistered();
-        int enqueueRequest(CollectorGuestRequest *aRequest);
+        HRESULT enqueueRequest(CollectorGuestRequest *aRequest);
 
         CollectorGuest *getBlockedGuest() { return mGuestBeingCalled; };
 
@@ -413,8 +427,8 @@ namespace pm
 
         bool collectorBeat(uint64_t nowAt);
 
-        virtual int enable()  { mEnabled = true; return S_OK; };
-        virtual int disable() { mEnabled = false; return S_OK; };
+        virtual HRESULT enable()  { mEnabled = true; return S_OK; };
+        virtual HRESULT disable() { mEnabled = false; return S_OK; };
         void unregister() { mUnregistered = true; };
 
         bool isUnregistered() { return mUnregistered; };
@@ -577,7 +591,7 @@ namespace pm
         void init(ULONG period, ULONG length);
         void preCollect(CollectorHints& hints, uint64_t iTick);
         void collect();
-        const char *getUnit() { return "mB"; };
+        const char *getUnit() { return "MB"; };
         ULONG getMinValue() { return 0; };
         ULONG getMaxValue() { return INT32_MAX; };
         ULONG getScale() { return 1; }
@@ -598,7 +612,7 @@ namespace pm
         void init(ULONG period, ULONG length);
         void preCollect(CollectorHints& hints, uint64_t iTick);
         void collect();
-        const char *getUnit() { return "mB"; };
+        const char *getUnit() { return "MB"; };
         ULONG getMinValue() { return 0; };
         ULONG getMaxValue() { return INT32_MAX; };
         ULONG getScale() { return 1; }
@@ -644,8 +658,8 @@ namespace pm
         void init(ULONG period, ULONG length);
         void preCollect(CollectorHints& hints, uint64_t iTick);
         void collect();
-        int enable();
-        int disable();
+        HRESULT enable();
+        HRESULT disable();
         const char *getUnit() { return "kB"; };
         ULONG getMinValue() { return 0; };
         ULONG getMaxValue() { return INT32_MAX; };
@@ -729,7 +743,7 @@ namespace pm
         void init(ULONG period, ULONG length);
         void preCollect(CollectorHints& hints, uint64_t iTick);
         void collect();
-        const char *getUnit() { return "mB"; };
+        const char *getUnit() { return "MB"; };
         ULONG getMinValue() { return 0; };
         ULONG getMaxValue() { return INT32_MAX; };
         ULONG getScale() { return 1; }
@@ -754,8 +768,8 @@ namespace pm
         void init(ULONG period, ULONG length);
         void preCollect(CollectorHints& hints, uint64_t iTick);
         void collect();
-        int enable();
-        int disable();
+        HRESULT enable();
+        HRESULT disable();
         const char *getUnit() { return "B/s"; };
         ULONG getMinValue() { return 0; };
         ULONG getMaxValue() { return INT32_MAX; };
@@ -774,8 +788,8 @@ namespace pm
         void init(ULONG period, ULONG length);
         void preCollect(CollectorHints& hints, uint64_t iTick);
         void collect();
-        int enable();
-        int disable();
+        HRESULT enable();
+        HRESULT disable();
         const char *getUnit() { return "%"; };
         ULONG getMinValue() { return 0; };
         ULONG getMaxValue() { return PM_CPU_LOAD_MULTIPLIER; };
@@ -796,8 +810,8 @@ namespace pm
         void init(ULONG period, ULONG length);
         void preCollect(CollectorHints& hints, uint64_t iTick);
         void collect();
-        int enable();
-        int disable();
+        HRESULT enable();
+        HRESULT disable();
         const char *getUnit() { return "kB"; };
         ULONG getMinValue() { return 0; };
         ULONG getMaxValue() { return INT32_MAX; };
@@ -813,6 +827,7 @@ namespace pm
     public:
         virtual ULONG compute(ULONG *data, ULONG length) = 0;
         virtual const char *getName() = 0;
+        virtual ~Aggregate() {}
     };
 
     class AggregateAvg : public Aggregate
@@ -896,5 +911,5 @@ namespace pm
         void processMetricList(const com::Utf8Str &name, const ComPtr<IUnknown> object);
     };
 }
-#endif /* ___performance_h */
+#endif /* !MAIN_INCLUDED_Performance_h */
 /* vi: set tabstop=4 shiftwidth=4 expandtab: */
