@@ -31,9 +31,11 @@
 # pragma once
 #endif
 
-#include <VBox/err.h>
-#include <VBox/settings.h>
+#include <map>
 
+#include <VBox/err.h>
+
+#include "RecordingInternals.h"
 #include "RecordingStream.h"
 
 class Console;
@@ -132,14 +134,14 @@ public:
 
 public:
 
-    const settings::Recording &GetConfig(void) const;
+    const ComPtr<IRecordingSettings> &GetSettings(void) const;
     RecordingStream *GetStream(unsigned uScreen) const;
     size_t GetStreamCount(void) const;
 #ifdef VBOX_WITH_AUDIO_RECORDING
     PRECORDINGCODEC GetCodecAudio(void) { return &this->m_CodecAudio; }
 #endif
 
-    int Create(Console *pConsole, const settings::Recording &Settings, ComPtr<IProgress> &pProgress);
+    int Create(Console *pConsole, ComPtr<IProgress> &pProgress);
     void Destroy(void);
 
     int Start(void);
@@ -172,7 +174,7 @@ public:
 
 protected:
 
-    int createInternal(Console *ptrConsole, const settings::Recording &Settings, ComPtr<IProgress> &pProgress);
+    int createInternal(Console *ptrConsole, ComPtr<IProgress> &pProgress);
     void reset(void);
     int startInternal(void);
     int stopInternal(void);
@@ -191,7 +193,7 @@ protected:
 
     bool progressIsCanceled(void) const;
     bool progressIsCompleted(void) const;
-    int progressCreate(const settings::Recording &Settings, ComObjPtr<Progress> &pProgress);
+    int progressCreate(const ComPtr<IRecordingSettings> &Settings, ComObjPtr<Progress> &Progress);
     int progressNotifyComplete(HRESULT hrc = S_OK, IVirtualBoxErrorInfo *pErrorInfo = NULL);
     int progressSet(uint32_t uOp, const com::Bstr &strDesc);
     int progressSet(uint64_t msTimestamp);
@@ -202,7 +204,7 @@ protected:
 
 protected:
 
-    int audioInit(const settings::RecordingScreen &screenSettings);
+    int audioInit(const ComPtr<IRecordingScreenSettings> &ScreenSettings);
 
 protected:
 
@@ -216,8 +218,8 @@ protected:
 
     /** Pointer to the console object. */
     Console                     *m_pConsole;
-    /** Used recording settings. */
-    settings::Recording          m_Settings;
+    /** Recording settings being used. */
+    ComPtr<IRecordingSettings>   m_Settings;
     /** The current state. */
     RECORDINGSTS                 m_enmState;
     /** Callback table. */
