@@ -1,28 +1,38 @@
 /* $Id: EmulatedUSBImpl.cpp $ */
 /** @file
- *
  * Emulated USB manager implementation.
  */
 
 /*
- * Copyright (C) 2013-2016 Oracle Corporation
+ * Copyright (C) 2013-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
-#define LOG_GROUP_MAIN_OVERRIDE LOG_GROUP_MAIN_EMULATEDUSB
+#define LOG_GROUP LOG_GROUP_MAIN_EMULATEDUSB
+#include "LoggingNew.h"
 
 #include "EmulatedUSBImpl.h"
 #include "ConsoleImpl.h"
-#include "Logging.h"
 
 #include <VBox/vmm/pdmusb.h>
+#include <VBox/vmm/vmmr3vtable.h>
 
 
 /*
@@ -39,145 +49,138 @@ typedef enum EUSBDEVICESTATUS
 
 class EUSBWEBCAM /* : public EUSBDEVICE */
 {
-    private:
-        int32_t volatile mcRefs;
+private:
+    int32_t volatile mcRefs;
 
-        EmulatedUSB *mpEmulatedUSB;
+    EmulatedUSB *mpEmulatedUSB;
 
-        RTUUID mUuid;
-        char mszUuid[RTUUID_STR_LENGTH];
+    RTUUID mUuid;
+    char mszUuid[RTUUID_STR_LENGTH];
 
-        Utf8Str mPath;
-        Utf8Str mSettings;
+    Utf8Str mPath;
+    Utf8Str mSettings;
 
-        EUSBSettingsMap mDevSettings;
-        EUSBSettingsMap mDrvSettings;
+    EUSBSettingsMap mDevSettings;
+    EUSBSettingsMap mDrvSettings;
 
-        void *mpvObject;
+    void *mpvObject;
 
-        static DECLCALLBACK(int) emulatedWebcamAttach(PUVM pUVM, EUSBWEBCAM *pThis, const char *pszDriver);
-        static DECLCALLBACK(int) emulatedWebcamDetach(PUVM pUVM, EUSBWEBCAM *pThis);
+    static DECLCALLBACK(int) emulatedWebcamAttach(PUVM pUVM, PCVMMR3VTABLE pVMM, EUSBWEBCAM *pThis, const char *pszDriver);
+    static DECLCALLBACK(int) emulatedWebcamDetach(PUVM pUVM, PCVMMR3VTABLE pVMM, EUSBWEBCAM *pThis);
 
-        HRESULT settingsParse(void);
+    HRESULT settingsParse(void);
 
-        ~EUSBWEBCAM()
+    ~EUSBWEBCAM()
+    {
+    }
+
+public:
+    EUSBWEBCAM()
+        :
+        mcRefs(1),
+        mpEmulatedUSB(NULL),
+        mpvObject(NULL),
+        enmStatus(EUSBDEVICE_CREATED)
+    {
+        RT_ZERO(mUuid);
+        RT_ZERO(mszUuid);
+    }
+
+    int32_t AddRef(void)
+    {
+        return ASMAtomicIncS32(&mcRefs);
+    }
+
+    void Release(void)
+    {
+        int32_t c = ASMAtomicDecS32(&mcRefs);
+        if (c == 0)
         {
+            delete this;
         }
+    }
 
-    public:
-        EUSBWEBCAM()
-            :
-            mcRefs(1),
-            mpEmulatedUSB(NULL),
-            mpvObject(NULL),
-            enmStatus(EUSBDEVICE_CREATED)
-        {
-            RT_ZERO(mUuid);
-            RT_ZERO(mszUuid);
-        }
+    HRESULT Initialize(Console *pConsole,
+                       EmulatedUSB *pEmulatedUSB,
+                       const com::Utf8Str *aPath,
+                       const com::Utf8Str *aSettings,
+                       void *pvObject);
+    HRESULT Attach(Console *pConsole, PUVM pUVM, PCVMMR3VTABLE pVMM, const char *pszDriver);
+    HRESULT Detach(Console *pConsole, PUVM pUVM, PCVMMR3VTABLE pVMM);
 
-        int32_t AddRef(void)
-        {
-            return ASMAtomicIncS32(&mcRefs);
-        }
+    bool HasId(const char *pszId) { return RTStrCmp(pszId, mszUuid) == 0;}
 
-        void Release(void)
-        {
-            int32_t c = ASMAtomicDecS32(&mcRefs);
-            if (c == 0)
-            {
-                delete this;
-            }
-        }
+    void *getObjectPtr() { return mpvObject; }
 
-        HRESULT Initialize(Console *pConsole,
-                           EmulatedUSB *pEmulatedUSB,
-                           const com::Utf8Str *aPath,
-                           const com::Utf8Str *aSettings,
-                           void *pvObject);
-        HRESULT Attach(Console *pConsole,
-                       PUVM pUVM,
-                       const char *pszDriver);
-        HRESULT Detach(Console *pConsole,
-                       PUVM pUVM);
-
-        bool HasId(const char *pszId) { return RTStrCmp(pszId, mszUuid) == 0;}
-
-        EUSBDEVICESTATUS enmStatus;
+    EUSBDEVICESTATUS enmStatus;
 };
 
-static int emulatedWebcamInsertSettings(PCFGMNODE pConfig, EUSBSettingsMap *pSettings)
-{
-    int rc = VINF_SUCCESS;
 
-    EUSBSettingsMap::const_iterator it;
-    for (it = pSettings->begin(); it != pSettings->end(); ++it)
+static int emulatedWebcamInsertSettings(PCFGMNODE pConfig, PCVMMR3VTABLE pVMM, EUSBSettingsMap *pSettings)
+{
+    for (EUSBSettingsMap::const_iterator it = pSettings->begin(); it != pSettings->end(); ++it)
     {
         /* Convert some well known settings for backward compatibility. */
+        int vrc;
         if (   RTStrCmp(it->first.c_str(), "MaxPayloadTransferSize") == 0
             || RTStrCmp(it->first.c_str(), "MaxFramerate") == 0)
         {
             uint32_t u32 = 0;
-            rc = RTStrToUInt32Full(it->second.c_str(), 10, &u32);
-            if (rc == VINF_SUCCESS)
-            {
-                rc = CFGMR3InsertInteger(pConfig, it->first.c_str(), u32);
-            }
-            else
-            {
-                if (RT_SUCCESS(rc)) /* VWRN_* */
-                {
-                    rc = VERR_INVALID_PARAMETER;
-                }
-            }
+            vrc = RTStrToUInt32Full(it->second.c_str(), 10, &u32);
+            if (vrc == VINF_SUCCESS)
+                vrc = pVMM->pfnCFGMR3InsertInteger(pConfig, it->first.c_str(), u32);
+            else if (RT_SUCCESS(vrc)) /* VWRN_* */
+                vrc = VERR_INVALID_PARAMETER;
         }
         else
-        {
-            rc = CFGMR3InsertString(pConfig, it->first.c_str(), it->second.c_str());
-        }
-
-        if (RT_FAILURE(rc))
-        {
-            break;
-        }
+            vrc = pVMM->pfnCFGMR3InsertString(pConfig, it->first.c_str(), it->second.c_str());
+        if (RT_FAILURE(vrc))
+            return vrc;
     }
 
-    return rc;
+    return VINF_SUCCESS;
 }
 
-/* static */ DECLCALLBACK(int) EUSBWEBCAM::emulatedWebcamAttach(PUVM pUVM, EUSBWEBCAM *pThis, const char *pszDriver)
+/*static*/ DECLCALLBACK(int)
+EUSBWEBCAM::emulatedWebcamAttach(PUVM pUVM, PCVMMR3VTABLE pVMM, EUSBWEBCAM *pThis, const char *pszDriver)
 {
-    PCFGMNODE pInstance = CFGMR3CreateTree(pUVM);
+    PCFGMNODE pInstance = pVMM->pfnCFGMR3CreateTree(pUVM);
     PCFGMNODE pConfig;
-    CFGMR3InsertNode(pInstance,   "Config", &pConfig);
-    int rc = emulatedWebcamInsertSettings(pConfig, &pThis->mDevSettings);
-    if (RT_FAILURE(rc))
-        return rc;
+    int vrc = pVMM->pfnCFGMR3InsertNode(pInstance,   "Config", &pConfig);
+    AssertRCReturn(vrc, vrc);
+    vrc = emulatedWebcamInsertSettings(pConfig, pVMM, &pThis->mDevSettings);
+    AssertRCReturn(vrc, vrc);
+
     PCFGMNODE pEUSB;
-    CFGMR3InsertNode(pConfig,       "EmulatedUSB", &pEUSB);
-    CFGMR3InsertString(pEUSB,         "Id", pThis->mszUuid);
-    CFGMR3InsertInteger(pEUSB,        "pfnCallback", (uintptr_t)EmulatedUSB::i_eusbCallback);
-    CFGMR3InsertInteger(pEUSB,        "pvCallback", (uintptr_t)pThis->mpEmulatedUSB);
+    vrc = pVMM->pfnCFGMR3InsertNode(pConfig,       "EmulatedUSB", &pEUSB);
+    AssertRCReturn(vrc, vrc);
+    vrc = pVMM->pfnCFGMR3InsertString(pEUSB,         "Id", pThis->mszUuid);
+    AssertRCReturn(vrc, vrc);
 
     PCFGMNODE pLunL0;
-    CFGMR3InsertNode(pInstance,   "LUN#0", &pLunL0);
-    CFGMR3InsertString(pLunL0,      "Driver", pszDriver);
-    CFGMR3InsertNode(pLunL0,        "Config", &pConfig);
-    CFGMR3InsertString(pConfig,       "DevicePath", pThis->mPath.c_str());
-    CFGMR3InsertInteger(pConfig,      "Object", (uintptr_t)pThis->mpvObject);
-    rc = emulatedWebcamInsertSettings(pConfig, &pThis->mDrvSettings);
-    if (RT_FAILURE(rc))
-        return rc;
+    vrc = pVMM->pfnCFGMR3InsertNode(pInstance,   "LUN#0", &pLunL0);
+    AssertRCReturn(vrc, vrc);
+    vrc = pVMM->pfnCFGMR3InsertString(pLunL0,      "Driver", pszDriver);
+    AssertRCReturn(vrc, vrc);
+    vrc = pVMM->pfnCFGMR3InsertNode(pLunL0,        "Config", &pConfig);
+    AssertRCReturn(vrc, vrc);
+    vrc = pVMM->pfnCFGMR3InsertString(pConfig,       "DevicePath", pThis->mPath.c_str());
+    AssertRCReturn(vrc, vrc);
+    vrc = pVMM->pfnCFGMR3InsertString(pConfig,       "Id", pThis->mszUuid);
+    AssertRCReturn(vrc, vrc);
+    vrc = emulatedWebcamInsertSettings(pConfig, pVMM, &pThis->mDrvSettings);
+    AssertRCReturn(vrc, vrc);
 
     /* pInstance will be used by PDM and deallocated on error. */
-    rc = PDMR3UsbCreateEmulatedDevice(pUVM, "Webcam", pInstance, &pThis->mUuid, NULL);
-    LogRelFlowFunc(("PDMR3UsbCreateEmulatedDevice %Rrc\n", rc));
-    return rc;
+    vrc = pVMM->pfnPDMR3UsbCreateEmulatedDevice(pUVM, "Webcam", pInstance, &pThis->mUuid, NULL);
+    LogRelFlowFunc(("PDMR3UsbCreateEmulatedDevice %Rrc\n", vrc));
+    return vrc;
 }
 
-/* static */ DECLCALLBACK(int) EUSBWEBCAM::emulatedWebcamDetach(PUVM pUVM, EUSBWEBCAM *pThis)
+/*static*/ DECLCALLBACK(int)
+EUSBWEBCAM::emulatedWebcamDetach(PUVM pUVM, PCVMMR3VTABLE pVMM, EUSBWEBCAM *pThis)
 {
-    return PDMR3UsbDetachDevice(pUVM, &pThis->mUuid);
+    return pVMM->pfnPDMR3UsbDetachDevice(pUVM, &pThis->mUuid);
 }
 
 HRESULT EUSBWEBCAM::Initialize(Console *pConsole,
@@ -189,19 +192,16 @@ HRESULT EUSBWEBCAM::Initialize(Console *pConsole,
     HRESULT hrc = S_OK;
 
     int vrc = RTUuidCreate(&mUuid);
-    if (RT_SUCCESS(vrc))
-    {
-        RTStrPrintf(mszUuid, sizeof(mszUuid), "%RTuuid", &mUuid);
-        hrc = mPath.assignEx(*aPath);
-        if (SUCCEEDED(hrc))
-        {
-            hrc = mSettings.assignEx(*aSettings);
-        }
+    AssertRCReturn(vrc, pConsole->setError(vrc, EmulatedUSB::tr("Init emulated USB webcam (RTUuidCreate -> %Rrc)"), vrc));
 
+    RTStrPrintf(mszUuid, sizeof(mszUuid), "%RTuuid", &mUuid);
+    hrc = mPath.assignEx(*aPath);
+    if (SUCCEEDED(hrc))
+    {
+        hrc = mSettings.assignEx(*aSettings);
         if (SUCCEEDED(hrc))
         {
             hrc = settingsParse();
-
             if (SUCCEEDED(hrc))
             {
                 mpEmulatedUSB = pEmulatedUSB;
@@ -210,13 +210,6 @@ HRESULT EUSBWEBCAM::Initialize(Console *pConsole,
         }
     }
 
-    if (SUCCEEDED(hrc) && RT_FAILURE(vrc))
-    {
-        LogFlowThisFunc(("%Rrc\n", vrc));
-        hrc = pConsole->setError(VBOX_E_IPRT_ERROR,
-                                 "Init emulated USB webcam (%Rrc)", vrc);
-    }
-
     return hrc;
 }
 
@@ -247,18 +240,16 @@ HRESULT EUSBWEBCAM::settingsParse(void)
                 fDrv = false;
             }
 
-            char *pszEq = RTStrStr(pszSrc, "=");
+            char *pszEq = strchr(pszSrc, '=');
             if (!pszEq)
             {
                 hr = E_INVALIDARG;
                 break;
             }
 
-            char *pszEnd = RTStrStr(pszEq, ";");
+            char *pszEnd = strchr(pszEq, ';');
             if (!pszEnd)
-            {
                 pszEnd = pszEq + strlen(pszEq);
-            }
 
             *pszEq = 0;
             char chEnd = *pszEnd;
@@ -268,13 +259,9 @@ HRESULT EUSBWEBCAM::settingsParse(void)
             if (*pszSrc != 0 && pszEq[1] != 0)
             {
                 if (fDev)
-                {
                     mDevSettings[pszSrc] = &pszEq[1];
-                }
                 if (fDrv)
-                {
                     mDrvSettings[pszSrc] = &pszEq[1];
-                }
             }
 
             *pszEq = '=';
@@ -282,9 +269,7 @@ HRESULT EUSBWEBCAM::settingsParse(void)
 
             pszSrc = pszEnd;
             if (*pszSrc == ';')
-            {
                 pszSrc++;
-            }
         }
 
         if (SUCCEEDED(hr))
@@ -300,43 +285,24 @@ HRESULT EUSBWEBCAM::settingsParse(void)
     return hr;
 }
 
-HRESULT EUSBWEBCAM::Attach(Console *pConsole,
-                           PUVM pUVM,
-                           const char *pszDriver)
+HRESULT EUSBWEBCAM::Attach(Console *pConsole, PUVM pUVM, PCVMMR3VTABLE pVMM, const char *pszDriver)
 {
-    HRESULT hrc = S_OK;
-
-    int  vrc = VMR3ReqCallWaitU(pUVM, 0 /* idDstCpu (saved state, see #6232) */,
-                                (PFNRT)emulatedWebcamAttach, 3,
-                                pUVM, this, pszDriver);
-
-    if (SUCCEEDED(hrc) && RT_FAILURE(vrc))
-    {
-        LogFlowThisFunc(("%Rrc\n", vrc));
-        hrc = pConsole->setError(VBOX_E_IPRT_ERROR,
-                                 "Attach emulated USB webcam (%Rrc)", vrc);
-    }
-
-    return hrc;
+    int vrc = pVMM->pfnVMR3ReqCallWaitU(pUVM, 0 /* idDstCpu (saved state, see #6232) */, (PFNRT)emulatedWebcamAttach, 4,
+                                        pUVM, pVMM, this, pszDriver);
+    if (RT_SUCCESS(vrc))
+        return S_OK;
+    LogFlowThisFunc(("%Rrc\n", vrc));
+    return pConsole->setErrorBoth(VBOX_E_VM_ERROR, vrc, EmulatedUSB::tr("Attach emulated USB webcam (%Rrc)"), vrc);
 }
 
-HRESULT EUSBWEBCAM::Detach(Console *pConsole,
-                           PUVM pUVM)
+HRESULT EUSBWEBCAM::Detach(Console *pConsole, PUVM pUVM, PCVMMR3VTABLE pVMM)
 {
-    HRESULT hrc = S_OK;
-
-    int vrc = VMR3ReqCallWaitU(pUVM, 0 /* idDstCpu (saved state, see #6232) */,
-                              (PFNRT)emulatedWebcamDetach, 2,
-                              pUVM, this);
-
-    if (SUCCEEDED(hrc) && RT_FAILURE(vrc))
-    {
-        LogFlowThisFunc(("%Rrc\n", vrc));
-        hrc = pConsole->setError(VBOX_E_IPRT_ERROR,
-                                 "Detach emulated USB webcam (%Rrc)", vrc);
-    }
-
-    return hrc;
+    int vrc = pVMM->pfnVMR3ReqCallWaitU(pUVM, 0 /* idDstCpu (saved state, see #6232) */, (PFNRT)emulatedWebcamDetach, 3,
+                                        pUVM, pVMM, this);
+    if (RT_SUCCESS(vrc))
+        return S_OK;
+    LogFlowThisFunc(("%Rrc\n", vrc));
+    return pConsole->setErrorBoth(VBOX_E_VM_ERROR, vrc, EmulatedUSB::tr("Detach emulated USB webcam (%Rrc)"), vrc);
 }
 
 
@@ -374,6 +340,9 @@ HRESULT EmulatedUSB::init(ComObjPtr<Console> pConsole)
 
     m.pConsole = pConsole;
 
+    mEmUsbIf.pvUser = this;
+    mEmUsbIf.pfnQueryEmulatedUsbDataById = EmulatedUSB::i_QueryEmulatedUsbDataById;
+
     /* Confirm a successful initialization */
     autoInitSpan.setSucceeded();
 
@@ -391,13 +360,16 @@ void EmulatedUSB::uninit()
     m.pConsole.setNull();
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
-    WebcamsMap::iterator it = m.webcams.begin();
-    while (it != m.webcams.end())
+    for (WebcamsMap::iterator it = m.webcams.begin(); it != m.webcams.end(); ++it)
     {
         EUSBWEBCAM *p = it->second;
-        m.webcams.erase(it++);
-        p->Release();
+        if (p)
+        {
+            it->second = NULL;
+            p->Release();
+        }
     }
+    m.webcams.clear();
     alock.release();
 
     /* Enclose the state transition Ready->InUninit->NotReady */
@@ -436,6 +408,11 @@ HRESULT EmulatedUSB::getWebcams(std::vector<com::Utf8Str> &aWebcams)
     return hrc;
 }
 
+PEMULATEDUSBIF EmulatedUSB::i_getEmulatedUsbIf()
+{
+    return &mEmUsbIf;
+}
+
 static const Utf8Str s_pathDefault(".0");
 
 HRESULT EmulatedUSB::webcamAttach(const com::Utf8Str &aPath,
@@ -488,22 +465,13 @@ HRESULT EmulatedUSB::i_webcamAttachInternal(const com::Utf8Str &aPath,
             }
 
             if (SUCCEEDED(hrc))
-            {
-                hrc = p->Attach(m.pConsole, ptrVM.rawUVM(), pszDriver);
-            }
+                hrc = p->Attach(m.pConsole, ptrVM.rawUVM(), ptrVM.vtable(), pszDriver);
 
             AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
             if (SUCCEEDED(hrc))
-            {
                 p->enmStatus = EUSBDEVICE_ATTACHED;
-            }
-            else
-            {
-                if (p->enmStatus != EUSBDEVICE_CREATED)
-                {
-                    m.webcams.erase(path);
-                }
-            }
+            else if (p->enmStatus != EUSBDEVICE_CREATED)
+                m.webcams.erase(path);
             alock.release();
 
             p->Release();
@@ -551,7 +519,7 @@ HRESULT EmulatedUSB::i_webcamDetachInternal(const com::Utf8Str &aPath)
 
         if (p)
         {
-            hrc = p->Detach(m.pConsole, ptrVM.rawUVM());
+            hrc = p->Detach(m.pConsole, ptrVM.rawUVM(), ptrVM.vtable());
             p->Release();
         }
         else
@@ -567,98 +535,114 @@ HRESULT EmulatedUSB::i_webcamDetachInternal(const com::Utf8Str &aPath)
     return hrc;
 }
 
-/* static */ DECLCALLBACK(int) EmulatedUSB::eusbCallbackEMT(EmulatedUSB *pThis, char *pszId, uint32_t iEvent,
-                                                            void *pvData, uint32_t cbData)
+/*static*/ DECLCALLBACK(int)
+EmulatedUSB::eusbCallbackEMT(EmulatedUSB *pThis, char *pszId, uint32_t iEvent, void *pvData, uint32_t cbData)
 {
     LogRelFlowFunc(("id %s event %d, data %p %d\n", pszId, iEvent, pvData, cbData));
 
     NOREF(cbData);
 
-    int rc = VINF_SUCCESS;
+    int vrc = VINF_SUCCESS;
     if (iEvent == 0)
     {
         com::Utf8Str path;
-        HRESULT hr = pThis->webcamPathFromId(&path, pszId);
-        if (SUCCEEDED(hr))
+        HRESULT hrc = pThis->webcamPathFromId(&path, pszId);
+        if (SUCCEEDED(hrc))
         {
-            hr = pThis->webcamDetach(path);
-            if (FAILED(hr))
+            hrc = pThis->webcamDetach(path);
+            if (FAILED(hrc))
             {
-                rc = VERR_INVALID_STATE;
+                vrc = VERR_INVALID_STATE;
             }
         }
         else
         {
-            rc = VERR_NOT_FOUND;
+            vrc = VERR_NOT_FOUND;
         }
     }
     else
     {
-        rc = VERR_INVALID_PARAMETER;
+        vrc = VERR_INVALID_PARAMETER;
     }
 
     RTMemFree(pszId);
     RTMemFree(pvData);
 
-    LogRelFlowFunc(("rc %Rrc\n", rc));
-    return rc;
+    LogRelFlowFunc(("vrc %Rrc\n", vrc));
+    return vrc;
 }
 
-/* static */ DECLCALLBACK(int) EmulatedUSB::i_eusbCallback(void *pv, const char *pszId, uint32_t iEvent,
-                                                           const void *pvData, uint32_t cbData)
+/* static */ DECLCALLBACK(int)
+EmulatedUSB::i_eusbCallback(void *pv, const char *pszId, uint32_t iEvent, const void *pvData, uint32_t cbData)
 {
     /* Make a copy of parameters, forward to EMT and leave the callback to not hold any lock in the device. */
-    int rc = VINF_SUCCESS;
-
-    void *pvIdCopy = NULL;
+    int vrc = VINF_SUCCESS;
     void *pvDataCopy = NULL;
     if (cbData > 0)
     {
        pvDataCopy = RTMemDup(pvData, cbData);
        if (!pvDataCopy)
-       {
-           rc = VERR_NO_MEMORY;
-       }
+           vrc = VERR_NO_MEMORY;
     }
-
-    if (RT_SUCCESS(rc))
+    if (RT_SUCCESS(vrc))
     {
-        pvIdCopy = RTMemDup(pszId, strlen(pszId) + 1);
-        if (!pvIdCopy)
+        void *pvIdCopy = RTMemDup(pszId, strlen(pszId) + 1);
+        if (pvIdCopy)
         {
-            rc = VERR_NO_MEMORY;
-        }
-    }
-
-    if (RT_SUCCESS(rc))
-    {
-        EmulatedUSB *pThis = (EmulatedUSB *)pv;
-        Console::SafeVMPtr ptrVM(pThis->m.pConsole);
-        if (ptrVM.isOk())
-        {
-            /* No wait. */
-            rc = VMR3ReqCallNoWaitU(ptrVM.rawUVM(), 0 /* idDstCpu */,
-                                    (PFNRT)EmulatedUSB::eusbCallbackEMT, 5,
-                                    pThis, pvIdCopy, iEvent, pvDataCopy, cbData);
+            if (RT_SUCCESS(vrc))
+            {
+                EmulatedUSB *pThis = (EmulatedUSB *)pv;
+                Console::SafeVMPtr ptrVM(pThis->m.pConsole);
+                if (ptrVM.isOk())
+                {
+                    /* No wait. */
+                    vrc = ptrVM.vtable()->pfnVMR3ReqCallNoWaitU(ptrVM.rawUVM(), 0 /* idDstCpu */,
+                                                                (PFNRT)EmulatedUSB::eusbCallbackEMT, 5,
+                                                                pThis, pvIdCopy, iEvent, pvDataCopy, cbData);
+                    if (RT_SUCCESS(vrc))
+                        return vrc;
+                }
+                else
+                    vrc = VERR_INVALID_STATE;
+            }
+            RTMemFree(pvIdCopy);
         }
         else
+            vrc = VERR_NO_MEMORY;
+        RTMemFree(pvDataCopy);
+    }
+    return vrc;
+}
+
+/*static*/
+DECLCALLBACK(int) EmulatedUSB::i_QueryEmulatedUsbDataById(void *pvUser, const char *pszId, void **ppvEmUsbCb, void **ppvEmUsbCbData, void **ppvObject)
+{
+    EmulatedUSB *pEmUsb = (EmulatedUSB *)pvUser;
+
+    AutoReadLock alock(pEmUsb COMMA_LOCKVAL_SRC_POS);
+    WebcamsMap::const_iterator it;
+    for (it = pEmUsb->m.webcams.begin(); it != pEmUsb->m.webcams.end(); ++it)
+    {
+        EUSBWEBCAM *p = it->second;
+        if (p->HasId(pszId))
         {
-            rc = VERR_INVALID_STATE;
+            if (ppvEmUsbCb)
+                *ppvEmUsbCb = (void *)EmulatedUSB::i_eusbCallback;
+            if (ppvEmUsbCbData)
+                *ppvEmUsbCbData = pEmUsb;
+            if (ppvObject)
+                *ppvObject = p->getObjectPtr();
+
+            return VINF_SUCCESS;
         }
     }
 
-    if (RT_FAILURE(rc))
-    {
-        RTMemFree(pvIdCopy);
-        RTMemFree(pvDataCopy);
-    }
-
-    return rc;
+    return VERR_NOT_FOUND;
 }
 
 HRESULT EmulatedUSB::webcamPathFromId(com::Utf8Str *pPath, const char *pszId)
 {
-    HRESULT hr = S_OK;
+    HRESULT hrc = S_OK;
 
     Console::SafeVMPtr ptrVM(m.pConsole);
     if (ptrVM.isOk())
@@ -677,16 +661,16 @@ HRESULT EmulatedUSB::webcamPathFromId(com::Utf8Str *pPath, const char *pszId)
 
         if (it == m.webcams.end())
         {
-            hr = E_FAIL;
+            hrc = E_FAIL;
         }
         alock.release();
     }
     else
     {
-        hr = VBOX_E_INVALID_VM_STATE;
+        hrc = VBOX_E_INVALID_VM_STATE;
     }
 
-    return hr;
+    return hrc;
 }
 
 /* vi: set tabstop=4 shiftwidth=4 expandtab: */
