@@ -415,6 +415,9 @@ static DECLCALLBACK(int) recordingCodecVPXEncode(PRECORDINGCODEC pCodec, PRECORD
 
         case RECORDINGFRAME_TYPE_CURSOR_SHAPE:
         {
+            Log3Func(("RECORDINGFRAME_TYPE_CURSOR_SHAPE: w=%d, h=%d\n",
+                      pFrame->u.CursorShape.Info.uWidth, pFrame->u.CursorShape.Info.uHeight));
+
             RecordingVideoFrameFree(pCodec->Video.VPX.pCursorShape);
             pCodec->Video.VPX.pCursorShape = RecordingVideoFrameDup(&pFrame->u.CursorShape);
             AssertPtr(pCodec->Video.VPX.pCursorShape);
@@ -475,10 +478,10 @@ static DECLCALLBACK(int) recordingCodecVPXEncode(PRECORDINGCODEC pCodec, PRECORD
 
             /* Blit mouse cursor to front buffer. */
             if (RT_SUCCESS(vrc))
-                vrc = RecordingVideoFrameBlitRawAlpha(pFront, pPosNew->x, pPosNew->y,
-                                                      pCursor->pau8Buf, pCursor->cbBuf,
-                                                      0 /* uSrcX */, 0 /* uSrcY */, pCursor->Info.uWidth, pCursor->Info.uHeight,
-                                                      pCursor->Info.uBytesPerLine, pCursor->Info.uBPP, pCursor->Info.enmPixelFmt);
+                RecordingVideoFrameBlitRawAlpha(pFront, pPosNew->x, pPosNew->y,
+                                                pCursor->pau8Buf, pCursor->cbBuf,
+                                                0 /* uSrcX */, 0 /* uSrcY */, pCursor->Info.uWidth, pCursor->Info.uHeight,
+                                                pCursor->Info.uBytesPerLine, pCursor->Info.uBPP, pCursor->Info.enmPixelFmt);
 #if 0
             RecordingUtilsDbgDumpVideoFrameEx(pFront, "/tmp/recording", "cursor-alpha-front");
 #endif
@@ -499,25 +502,23 @@ static DECLCALLBACK(int) recordingCodecVPXEncode(PRECORDINGCODEC pCodec, PRECORD
         return VINF_SUCCESS;
     }
 
-    Log3Func(("Encoding video parameters: %RU16x%RU16 (%RU8 FPS), originX=%RI32, originY=%RI32\n",
-              pCodec->Parms.u.Video.uWidth, pCodec->Parms.u.Video.uHeight, pCodec->Parms.u.Video.uFPS,
-              pCodec->Parms.u.Video.Scaling.u.Crop.m_iOriginX, pCodec->Parms.u.Video.Scaling.u.Crop.m_iOriginY));
+    Log3Func(("Source: %RU32x%RU32 (Pos %RU32x%RU32)\n", sw, sh, sx, sy));
+    Log3Func(("Front : %RU32x%RU32\n", pFront->Info.uWidth, pFront->Info.uHeight));
+    Log3Func(("Codec : %RU32x%RU32\n", pCodec->Parms.u.Video.uWidth, pCodec->Parms.u.Video.uHeight));
 
     vrc = RecordingUtilsCoordsCropCenter(&pCodec->Parms, &sx, &sy, &sw, &sh, &dx, &dy);
     if (vrc == VINF_SUCCESS) /* vrc might be VWRN_RECORDING_ENCODING_SKIPPED to skip encoding. */
     {
         Log3Func(("Encoding source %RI32,%RI32 (%RI32x%RI32) to %RI32,%RI32 (%zu bytes)\n",
                   sx, sy, sw, sh, dx, dy, sw * sh * (pFront->Info.uBPP / 8)));
-#ifdef DEBUG
-        AssertReturn(sw      <= (int32_t)pFront->Info.uWidth,  VERR_INVALID_PARAMETER);
-        AssertReturn(sh      <= (int32_t)pFront->Info.uHeight, VERR_INVALID_PARAMETER);
-        AssertReturn(sx + sw <= (int32_t)pFront->Info.uWidth , VERR_INVALID_PARAMETER);
-        AssertReturn(sy + sh <= (int32_t)pFront->Info.uHeight, VERR_INVALID_PARAMETER);
-#endif
-
 #if 0
-        RecordingUtilsDbgDumpImageData(&pFront->pau8Buf[(sy * pFront->Info.uBytesPerLine) + (sx * (pFront->Info.uBPP / 8))], pFront->cbBuf,
-                                       "/tmp/recording", "cropped", sw, sh, pFront->Info.uBytesPerLine, pFront->Info.uBPP);
+        AssertReturn(sw + dx <= (int32_t)pCodec->Parms.u.Video.uWidth,  VERR_INVALID_PARAMETER);
+        AssertReturn(sy + sh <= (int32_t)pCodec->Parms.u.Video.uHeight, VERR_INVALID_PARAMETER);
+#endif
+#if 0
+        RecordingUtilsDbgDumpImageData(pFront->pau8Buf, pFront->cbBuf,
+                                       NULL /* Use default temp dir */, "cropped", sx, sy, sw, sh,
+                                       pFront->Info.uBytesPerLine, pFront->Info.uBPP);
 #endif
         /* Blit (and convert from BGRA 32) the changed parts of the front buffer to the YUV 420 surface of the codec. */
         RecordingUtilsConvBGRA32ToYUVI420Ex(/* Destination */
@@ -536,10 +537,7 @@ static DECLCALLBACK(int) recordingCodecVPXEncode(PRECORDINGCODEC pCodec, PRECORD
 /** @copydoc RECORDINGCODECOPS::pfnScreenChange */
 static DECLCALLBACK(int) recordingCodecVPXScreenChange(PRECORDINGCODEC pCodec, PRECORDINGSURFACEINFO pInfo)
 {
-    /* The VPX encoder only understands even frame sizes. */
-    if (   (pInfo->uWidth  % 2) != 0
-        || (pInfo->uHeight % 2) != 0)
-        return VERR_INVALID_PARAMETER;
+    LogFunc(("ENTER: w=%RU32, h=%RU32, bpp=%RU8\n", pInfo->uWidth, pInfo->uHeight, pInfo->uBPP));
 
     PRECORDINGCODECVPX pVPX = &pCodec->Video.VPX;
 
@@ -575,6 +573,7 @@ static DECLCALLBACK(int) recordingCodecVPXScreenChange(PRECORDINGCODEC pCodec, P
     if (RT_FAILURE(vrc))
         LogRel(("Recording: Codec error handling screen change notification: %Rrc\n", vrc));
 
+    LogFlowFuncLeaveRC(vrc);
     return vrc;
 
 }
@@ -794,22 +793,31 @@ static DECLCALLBACK(int) recordingCodecVorbisFinalize(PRECORDINGCODEC pCodec)
  * @returns VBox status code.
  * @param   pCodec              Codec instance to initialize.
  * @param   pCallbacks          Codec callback table to use for the codec.
- * @param   Settings            Screen settings to use for initialization.
+ * @param   ScreenSettings      Recording screen settings to use for initialization.
  */
-static int recordingCodecInitAudio(const PRECORDINGCODEC pCodec,
-                                   const PRECORDINGCODECCALLBACKS pCallbacks, const settings::RecordingScreen &Settings)
+static int recordingCodecInitAudio(const PRECORDINGCODEC pCodec, const PRECORDINGCODECCALLBACKS pCallbacks,
+                                   const ComPtr<IRecordingScreenSettings> &ScreenSettings)
 {
     AssertReturn(pCodec->Parms.enmType == RECORDINGCODECTYPE_AUDIO, VERR_INVALID_PARAMETER);
 
-    com::Utf8Str strCodec;
-    settings::RecordingScreen::audioCodecToString(pCodec->Parms.enmAudioCodec, strCodec);
-    LogRel(("Recording: Initializing audio codec '%s'\n", strCodec.c_str()));
+    LogRel(("Recording: Initializing audio codec '%s'\n", RecordingUtilsAudioCodecToStr(pCodec->Parms.enmAudioCodec)));
 
     const PPDMAUDIOPCMPROPS pPCMProps = &pCodec->Parms.u.Audio.PCMProps;
 
-    PDMAudioPropsInit(pPCMProps,
-                      Settings.Audio.cBits / 8,
-                      true /* fSigned */, Settings.Audio.cChannels, Settings.Audio.uHz);
+    ULONG uBits;
+    HRESULT hrc = ScreenSettings->COMGETTER(AudioBits)(&uBits);
+    AssertComRCReturn(hrc, VERR_RECORDING_INIT_FAILED);
+    ULONG cChannels;
+    hrc = ScreenSettings->COMGETTER(AudioChannels)(&cChannels);
+    AssertComRCReturn(hrc, VERR_RECORDING_INIT_FAILED);
+    ULONG uHz;
+    hrc = ScreenSettings->COMGETTER(AudioHz)(&uHz);
+    AssertComRCReturn(hrc, VERR_RECORDING_INIT_FAILED);
+    com::Bstr bstrOpts;
+    hrc = ScreenSettings->COMGETTER(Options)(bstrOpts.asOutParam());
+    AssertComRCReturn(hrc, VERR_RECORDING_INIT_FAILED);
+
+    PDMAudioPropsInit(pPCMProps, uBits / 8, true /* fSigned */, cChannels, uHz);
     pCodec->Parms.uBitrate = 0; /** @todo No bitrate management for audio yet. */
 
     if (pCallbacks)
@@ -818,7 +826,7 @@ static int recordingCodecInitAudio(const PRECORDINGCODEC pCodec,
     int vrc = VINF_SUCCESS;
 
     if (pCodec->Ops.pfnParseOptions)
-        vrc = pCodec->Ops.pfnParseOptions(pCodec, Settings.strOptions);
+        vrc = pCodec->Ops.pfnParseOptions(pCodec, com::Utf8Str(bstrOpts).c_str());
 
     if (RT_SUCCESS(vrc))
         vrc = pCodec->Ops.pfnInit(pCodec);
@@ -854,19 +862,30 @@ static int recordingCodecInitAudio(const PRECORDINGCODEC pCodec,
  * @returns VBox status code.
  * @param   pCodec              Codec instance to initialize.
  * @param   pCallbacks          Codec callback table to use for the codec.
- * @param   Settings            Screen settings to use for initialization.
+ * @param   ScreenSettings      Recording screen settings to use for initialization.
  */
-static int recordingCodecInitVideo(const PRECORDINGCODEC pCodec,
-                                   const PRECORDINGCODECCALLBACKS pCallbacks, const settings::RecordingScreen &Settings)
+static int recordingCodecInitVideo(const PRECORDINGCODEC pCodec, const PRECORDINGCODECCALLBACKS pCallbacks,
+                                   const ComPtr<IRecordingScreenSettings> &ScreenSettings)
 {
-    com::Utf8Str strTemp;
-    settings::RecordingScreen::videoCodecToString(pCodec->Parms.enmVideoCodec, strTemp);
-    LogRel(("Recording: Initializing video codec '%s'\n", strTemp.c_str()));
+    LogRel(("Recording: Initializing video codec '%s'\n", RecordingUtilsVideoCodecToStr(pCodec->Parms.enmVideoCodec)));
 
-    pCodec->Parms.uBitrate         = Settings.Video.ulRate;
-    pCodec->Parms.u.Video.uFPS     = Settings.Video.ulFPS;
-    pCodec->Parms.u.Video.uWidth   = Settings.Video.ulWidth;
-    pCodec->Parms.u.Video.uHeight  = Settings.Video.ulHeight;
+    ULONG uRate;
+    HRESULT hrc = ScreenSettings->COMGETTER(VideoRate)(&uRate);
+    AssertComRCReturn(hrc, VERR_RECORDING_INIT_FAILED);
+    ULONG uFPS;
+    hrc = ScreenSettings->COMGETTER(VideoFPS)(&uFPS);
+    AssertComRCReturn(hrc, VERR_RECORDING_INIT_FAILED);
+    ULONG uWidth;
+    hrc = ScreenSettings->COMGETTER(VideoWidth)(&uWidth);
+    AssertComRCReturn(hrc, VERR_RECORDING_INIT_FAILED);
+    ULONG uHeight;
+    hrc = ScreenSettings->COMGETTER(VideoHeight)(&uHeight);
+    AssertComRCReturn(hrc, VERR_RECORDING_INIT_FAILED);
+
+    pCodec->Parms.uBitrate         = uRate;
+    pCodec->Parms.u.Video.uFPS     = uFPS;
+    pCodec->Parms.u.Video.uWidth   = uWidth;
+    pCodec->Parms.u.Video.uHeight  = uHeight;
     pCodec->Parms.u.Video.uDelayMs = RT_MS_1SEC / pCodec->Parms.u.Video.uFPS;
 
     if (pCallbacks)
@@ -882,7 +901,13 @@ static int recordingCodecInitVideo(const PRECORDINGCODEC pCodec,
     int vrc = VINF_SUCCESS;
 
     if (pCodec->Ops.pfnParseOptions)
-        vrc = pCodec->Ops.pfnParseOptions(pCodec, Settings.strOptions);
+    {
+        com::Bstr bstrOptions;
+        hrc = ScreenSettings->COMGETTER(Options)(bstrOptions.asOutParam());
+        AssertComRCReturn(hrc, VERR_RECORDING_INIT_FAILED);
+
+        vrc = pCodec->Ops.pfnParseOptions(pCodec, com::Utf8Str(bstrOptions).c_str());
+    }
 
     if (   RT_SUCCESS(vrc)
         && pCodec->Ops.pfnInit)
@@ -1050,9 +1075,10 @@ int recordingCodecCreateVideo(PRECORDINGCODEC pCodec, RecordingVideoCodec_T enmV
  * @returns VBox status code.
  * @param   pCodec              Codec to initialize.
  * @param   pCallbacks          Codec callback table to use. Optional and may be NULL.
- * @param   Settings            Settings to use for initializing the codec.
+ * @param   ScreenSettings      Screen settings to use for initializing the codec.
  */
-int recordingCodecInit(const PRECORDINGCODEC pCodec, const PRECORDINGCODECCALLBACKS pCallbacks, const settings::RecordingScreen &Settings)
+int recordingCodecInit(const PRECORDINGCODEC pCodec, const PRECORDINGCODECCALLBACKS pCallbacks,
+                       const ComPtr<IRecordingScreenSettings> &ScreenSettings)
 {
     int vrc = RTCritSectInit(&pCodec->CritSect);
     AssertRCReturn(vrc, vrc);
@@ -1063,9 +1089,9 @@ int recordingCodecInit(const PRECORDINGCODEC pCodec, const PRECORDINGCODECCALLBA
     recordingCodecReset(pCodec);
 
     if (pCodec->Parms.enmType == RECORDINGCODECTYPE_AUDIO)
-        vrc = recordingCodecInitAudio(pCodec, pCallbacks, Settings);
+        vrc = recordingCodecInitAudio(pCodec, pCallbacks, ScreenSettings);
     else if (pCodec->Parms.enmType == RECORDINGCODECTYPE_VIDEO)
-        vrc = recordingCodecInitVideo(pCodec, pCallbacks, Settings);
+        vrc = recordingCodecInitVideo(pCodec, pCallbacks, ScreenSettings);
     else
         AssertFailedStmt(vrc = VERR_NOT_SUPPORTED);
 
@@ -1193,6 +1219,7 @@ int recordingCodecScreenChange(PRECORDINGCODEC pCodec, PRECORDINGSURFACEINFO pIn
     if (   !pInfo->uWidth
         || !pInfo->uHeight)
         return VERR_INVALID_PARAMETER;
+    AssertReturn(pInfo->enmPixelFmt == RECORDINGPIXELFMT_BRGA32 /* Only format we support for now */, VERR_INVALID_PARAMETER);
     AssertReturn(pInfo->uBPP % 8 == 0, VERR_INVALID_PARAMETER);
 
     return pCodec->Ops.pfnScreenChange(pCodec, pInfo);
